@@ -1,7 +1,7 @@
 ######################################################################
 # creates both tar files (source and documentation). call this script
 # in an otherwise empty directory as follows:
-# 
+#
 #   bash make_tarfile.sh 6 0 0
 #
 # to generate the tar files for release 6.0.0
@@ -21,16 +21,7 @@ tar czf deal.II-$MAJOR.$MINOR.$PATCH.tar.gz deal.II
 # Generate stripped tarfile
 
 mv deal.II/doc deal.II/examples .
-cd deal.II
-mkdir doc
-cd doc
-touch Makefile.in
-mkdir doxygen
-cd doxygen
-touch Makefile.in options.dox.in options.136.in header.html.in header.136.in header.tex.in
-cd ../../..
 tar czf deal.nodoc-$MAJOR.$MINOR.$PATCH.tar.gz deal.II
-rm -rf deal.II/doc
 mv doc examples deal.II
 
 # Generate documentation
