@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -35,9 +35,21 @@
 #include "spin_mutex.h"
 #include "cache_aligned_allocator.h"
 #include "tbb_exception.h"
-#include <iterator>
 #include <new>
 
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
+    #pragma warning (push)
+    #pragma warning (disable: 4530)
+#endif
+
+#include <iterator>
+
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    #pragma warning (pop)
+#endif
+
+
 namespace tbb {
 
 #if !__TBB_TEMPLATE_FRIENDS_BROKEN
@@ -64,8 +76,6 @@ using namespace tbb::internal;
 
 typedef size_t ticket;
 
-static void* invalid_page;
-
 template<typename T> class micro_queue ;
 template<typename T> class micro_queue_pop_finalizer ;
 template<typename T> class concurrent_queue_base_v3;
@@ -109,6 +119,10 @@ public:
     char pad3[NFS_MaxLineSize-sizeof(size_t)-sizeof(size_t)-sizeof(atomic<size_t>)];
 } ;
 
+inline bool is_valid_page(const concurrent_queue_rep_base::page* p) {
+    return uintptr_t(p)>1;
+}
+
 //! Abstract class to define interface for page allocation/deallocation
 /**
  * For internal use only.
@@ -145,16 +159,12 @@ class micro_queue : no_copy {
         ~destroyer() {my_value.~T();}          
     };
 
-    T& get_ref( page& page, size_t index ) {
-        return static_cast<T*>(static_cast<void*>(&page+1))[index];
-    }
-
     void copy_item( page& dst, size_t index, const void* src ) {
         new( &get_ref(dst,index) ) T(*static_cast<const T*>(src)); 
     }
 
     void copy_item( page& dst, size_t dindex, const page& src, size_t sindex ) {
-        new( &get_ref(dst,dindex) ) T( static_cast<const T*>(static_cast<const void*>(&src+1))[sindex] );
+        new( &get_ref(dst,dindex) ) T( get_ref(const_cast<page&>(src),sindex) );
     }
 
     void assign_and_destroy_item( void* dst, page& src, size_t index ) {
@@ -168,6 +178,19 @@ class micro_queue : no_copy {
 public:
     friend class micro_queue_pop_finalizer<T>;
 
+    struct padded_page: page {
+        //! Not defined anywhere - exists to quiet warnings.
+        padded_page(); 
+        //! Not defined anywhere - exists to quiet warnings.
+        void operator=( const padded_page& );
+        //! Must be last field.
+        T last;
+    };
+
+    static T& get_ref( page& p, size_t index ) {
+        return (&static_cast<padded_page*>(static_cast<void*>(&p))->last)[index];
+    }
+
     atomic<page*> head_page;
     atomic<ticket> head_counter;
 
@@ -184,7 +207,7 @@ public:
 
     page* make_copy( concurrent_queue_base_v3<T>& base, const page* src_page, size_t begin_in_page, size_t end_in_page, ticket& g_index ) ;
 
-    void make_invalid( ticket k ) ;
+    void invalidate_page_and_rethrow( ticket k ) ;
 };
 
 template<typename T>
@@ -192,9 +215,9 @@ void micro_queue<T>::spin_wait_until_my_turn( atomic<ticket>& counter, ticket k,
     atomic_backoff backoff;
     do {
         backoff.pause();
-        if( counter&0x1 ) {
+        if( counter&1 ) {
             ++rb.n_invalid_entries;
-            throw_bad_last_alloc_exception_v4();
+            throw_exception( eid_bad_last_alloc );
         }
     } while( counter!=k ) ;
 }
@@ -205,12 +228,12 @@ void micro_queue<T>::push( const void* item, ticket k, concurrent_queue_base_v3<
     page* p = NULL;
     size_t index = k/concurrent_queue_rep_base::n_queue & (base.my_rep->items_per_page-1);
     if( !index ) {
-        try {
+        __TBB_TRY {
             concurrent_queue_page_allocator& pa = base;
             p = pa.allocate_page();
-        } catch (...) {
+        } __TBB_CATCH (...) {
             ++base.my_rep->n_invalid_entries;
-            make_invalid( k );
+            invalidate_page_and_rethrow( k );
         }
         p->mask = 0;
         p->next = NULL;
@@ -220,7 +243,8 @@ void micro_queue<T>::push( const void* item, ticket k, concurrent_queue_base_v3<
         
     if( p ) {
         spin_mutex::scoped_lock lock( page_mutex );
-        if( page* q = tail_page )
+        page* q = tail_page;
+        if( is_valid_page(q) )
             q->next = p;
         else
             head_page = p; 
@@ -229,15 +253,15 @@ void micro_queue<T>::push( const void* item, ticket k, concurrent_queue_base_v3<
         p = tail_page;
     }
    
-    try {
+    __TBB_TRY {
         copy_item( *p, index, item );
         // If no exception was thrown, mark item as present.
         p->mask |= uintptr_t(1)<<index;
         tail_counter += concurrent_queue_rep_base::n_queue; 
-    } catch (...) {
+    } __TBB_CATCH (...) {
         ++base.my_rep->n_invalid_entries;
         tail_counter += concurrent_queue_rep_base::n_queue; 
-        throw;
+        __TBB_RETHROW();
     }
 }
 
@@ -269,9 +293,9 @@ micro_queue<T>& micro_queue<T>::assign( const micro_queue<T>& src, concurrent_qu
     page_mutex   = src.page_mutex;
 
     const page* srcp = src.head_page;
-    if( srcp ) {
+    if( is_valid_page(srcp) ) {
         ticket g_index = head_counter;
-        try {
+        __TBB_TRY {
             size_t n_items  = (tail_counter-head_counter)/concurrent_queue_rep_base::n_queue;
             size_t index = head_counter/concurrent_queue_rep_base::n_queue & (base.my_rep->items_per_page-1);
             size_t end_in_first_page = (index+n_items<base.my_rep->items_per_page)?(index+n_items):base.my_rep->items_per_page;
@@ -293,8 +317,8 @@ micro_queue<T>& micro_queue<T>::assign( const micro_queue<T>& src, concurrent_qu
                 cur_page = cur_page->next;
             }
             tail_page = cur_page;
-        } catch (...) {
-            make_invalid( g_index );
+        } __TBB_CATCH (...) {
+            invalidate_page_and_rethrow( g_index );
         }
     } else {
         head_page = tail_page = NULL;
@@ -303,20 +327,20 @@ micro_queue<T>& micro_queue<T>::assign( const micro_queue<T>& src, concurrent_qu
 }
 
 template<typename T>
-void micro_queue<T>::make_invalid( ticket k ) {
-    static page dummy = {static_cast<page*>((void*)1), 0};
-    // mark it so that no more pushes are allowed.
-    invalid_page = &dummy;
+void micro_queue<T>::invalidate_page_and_rethrow( ticket k ) {
+    // Append an invalid page at address 1 so that no more pushes are allowed.
+    page* invalid_page = (page*)uintptr_t(1);
     {
         spin_mutex::scoped_lock lock( page_mutex );
         tail_counter = k+concurrent_queue_rep_base::n_queue+1;
-        if( page* q = tail_page )
-            q->next = static_cast<page*>(invalid_page);
+        page* q = tail_page;
+        if( is_valid_page(q) )
+            q->next = invalid_page;
         else
-            head_page = static_cast<page*>(invalid_page); 
-        tail_page = static_cast<page*>(invalid_page);
+            head_page = invalid_page;
+        tail_page = invalid_page;
     }
-    throw;
+    __TBB_RETHROW();
 }
 
 template<typename T>
@@ -348,16 +372,16 @@ public:
 template<typename T>
 micro_queue_pop_finalizer<T>::~micro_queue_pop_finalizer() {
     page* p = my_page;
-    if( p ) {
+    if( is_valid_page(p) ) {
         spin_mutex::scoped_lock lock( my_queue.page_mutex );
         page* q = p->next;
         my_queue.head_page = q;
-        if( !q ) {
+        if( !is_valid_page(q) ) {
             my_queue.tail_page = NULL;
         }
     }
     my_queue.head_counter = my_ticket;
-    if( p ) {
+    if( is_valid_page(p) ) {
         allocator.deallocate_page( p );
     }
 }
@@ -407,15 +431,17 @@ protected:
     typedef typename concurrent_queue_rep<T>::page page;
 
 private:
+    typedef typename micro_queue<T>::padded_page padded_page;
+
     /* override */ virtual page *allocate_page() {
         concurrent_queue_rep<T>& r = *my_rep;
-        size_t n = sizeof(page) + r.items_per_page*r.item_size;
+        size_t n = sizeof(padded_page) + (r.items_per_page-1)*sizeof(T);
         return reinterpret_cast<page*>(allocate_block ( n ));
     }
 
     /* override */ virtual void deallocate_page( concurrent_queue_rep_base::page *p ) {
         concurrent_queue_rep<T>& r = *my_rep;
-        size_t n = sizeof(page) + r.items_per_page*r.item_size;
+        size_t n = sizeof(padded_page) + (r.items_per_page-1)*sizeof(T);
         deallocate_block( reinterpret_cast<void*>(p), n );
     }
 
@@ -426,12 +452,14 @@ private:
     virtual void deallocate_block( void *p, size_t n ) = 0;
 
 protected:
-    concurrent_queue_base_v3( size_t item_size ) ;
+    concurrent_queue_base_v3();
 
     /* override */ virtual ~concurrent_queue_base_v3() {
+#if __TBB_USE_ASSERT
         size_t nq = my_rep->n_queue;
         for( size_t i=0; i<nq; i++ )
             __TBB_ASSERT( my_rep->array[i].tail_page==NULL, "pages were not freed properly" );
+#endif /* __TBB_USE_ASSERT */
         cache_aligned_allocator<concurrent_queue_rep<T> >().deallocate(my_rep,1);
     }
 
@@ -456,9 +484,9 @@ protected:
     /* note that the name may be misleading, but it remains so due to a historical accident. */
     void internal_finish_clear() ;
 
-    //! throw an exception
+    //! Obsolete
     void internal_throw_exception() const {
-        throw std::bad_alloc();
+        throw_exception( eid_bad_alloc );
     }
 
     //! copy internal representation
@@ -466,7 +494,8 @@ protected:
 };
 
 template<typename T>
-concurrent_queue_base_v3<T>::concurrent_queue_base_v3( size_t item_size ) {
+concurrent_queue_base_v3<T>::concurrent_queue_base_v3() {
+    const size_t item_size = sizeof(T);
     my_rep = cache_aligned_allocator<concurrent_queue_rep<T> >().allocate(1);
     __TBB_ASSERT( (size_t)my_rep % NFS_GetLineSize()==0, "alignment error" );
     __TBB_ASSERT( (size_t)&my_rep->head_counter % NFS_GetLineSize()==0, "alignment error" );
@@ -538,11 +567,12 @@ void concurrent_queue_base_v3<T>::internal_finish_clear() {
     size_t nq = r.n_queue;
     for( size_t i=0; i<nq; ++i ) {
         page* tp = r.array[i].tail_page;
-        __TBB_ASSERT( r.array[i].head_page==tp, "at most one page should remain" );
-        if( tp!=NULL) {
-            if( tp!=invalid_page ) deallocate_page( tp );
+        if( is_valid_page(tp) ) {
+            __TBB_ASSERT( r.array[i].head_page==tp, "at most one page should remain" );
+            deallocate_page( tp );
             r.array[i].tail_page = NULL;
-        }
+        } else 
+            __TBB_ASSERT( !is_valid_page(r.array[i].head_page), "head page pointer corrupt?" );
     }
 }
 
@@ -568,6 +598,7 @@ template<typename Container, typename Value> class concurrent_queue_iterator;
 
 template<typename T>
 class concurrent_queue_iterator_rep: no_assign {
+    typedef typename micro_queue<T>::padded_page padded_page;
 public:
     ticket head_counter;
     const concurrent_queue_base_v3<T>& my_queue;
@@ -581,11 +612,11 @@ public:
     }
 
     //! Set item to point to kth element.  Return true if at end of queue or item is marked valid; false otherwise.
-    bool get_item( void*& item, size_t k ) ;
+    bool get_item( T*& item, size_t k ) ;
 };
 
 template<typename T>
-bool concurrent_queue_iterator_rep<T>::get_item( void*& item, size_t k ) {
+bool concurrent_queue_iterator_rep<T>::get_item( T*& item, size_t k ) {
     if( k==my_queue.my_rep->tail_counter ) {
         item = NULL;
         return true;
@@ -593,16 +624,16 @@ bool concurrent_queue_iterator_rep<T>::get_item( void*& item, size_t k ) {
         typename concurrent_queue_base_v3<T>::page* p = array[concurrent_queue_rep<T>::index(k)];
         __TBB_ASSERT(p,NULL);
         size_t i = k/concurrent_queue_rep<T>::n_queue & (my_queue.my_rep->items_per_page-1);
-        item = static_cast<unsigned char*>(static_cast<void*>(p+1)) + my_queue.my_rep->item_size*i;
+        item = &micro_queue<T>::get_ref(*p,i);
         return (p->mask & uintptr_t(1)<<i)!=0;
     }
 }
 
-//! Type-independent portion of concurrent_queue_iterator.
+//! Constness-independent portion of concurrent_queue_iterator.
 /** @ingroup containers */
 template<typename Value>
 class concurrent_queue_iterator_base_v3 : no_assign {
-    //! Concurrentconcurrent_queue over which we are iterating.
+    //! Represents concurrent_queue over which we are iterating.
     /** NULL if one past last element in queue. */
     concurrent_queue_iterator_rep<Value>* my_rep;
 
@@ -613,26 +644,25 @@ class concurrent_queue_iterator_base_v3 : no_assign {
     friend bool operator!=( const concurrent_queue_iterator<C,T>& i, const concurrent_queue_iterator<C,U>& j );
 protected:
     //! Pointer to current item
-    mutable void* my_item;
+    Value* my_item;
 
-public:
     //! Default constructor
     concurrent_queue_iterator_base_v3() : my_rep(NULL), my_item(NULL) {
 #if __GNUC__==4&&__GNUC_MINOR__==3
         // to get around a possible gcc 4.3 bug
-        __asm__ __volatile__("": : :"memory");
+        __TBB_release_consistency_helper();
 #endif
     }
 
     //! Copy constructor
-    concurrent_queue_iterator_base_v3( const concurrent_queue_iterator_base_v3& i ) : my_rep(NULL), my_item(NULL) {
+    concurrent_queue_iterator_base_v3( const concurrent_queue_iterator_base_v3& i )
+    : no_assign(), my_rep(NULL), my_item(NULL) {
         assign(i);
     }
 
     //! Construct iterator pointing to head of queue.
     concurrent_queue_iterator_base_v3( const concurrent_queue_base_v3<Value>& queue ) ;
 
-protected:
     //! Assignment
     void assign( const concurrent_queue_iterator_base_v3<Value>& other ) ;
 
@@ -675,7 +705,7 @@ void concurrent_queue_iterator_base_v3<Value>::advance() {
     size_t k = my_rep->head_counter;
     const concurrent_queue_base_v3<Value>& queue = my_rep->my_queue;
 #if TBB_USE_ASSERT
-    void* tmp;
+    Value* tmp;
     my_rep->get_item(tmp,k);
     __TBB_ASSERT( my_item==tmp, NULL );
 #endif /* TBB_USE_ASSERT */
@@ -689,17 +719,18 @@ void concurrent_queue_iterator_base_v3<Value>::advance() {
     if( !my_rep->get_item(my_item, k) ) advance();
 }
 
-template<typename T>
-static inline const concurrent_queue_iterator_base_v3<const T>& add_constness( const concurrent_queue_iterator_base_v3<T>& q )
-{
-    return *reinterpret_cast<const concurrent_queue_iterator_base_v3<const T> *>(&q) ;
-}
+//! Similar to C++0x std::remove_cv
+/** "tbb_" prefix added to avoid overload confusion with C++0x implementations. */
+template<typename T> struct tbb_remove_cv {typedef T type;};
+template<typename T> struct tbb_remove_cv<const T> {typedef T type;};
+template<typename T> struct tbb_remove_cv<volatile T> {typedef T type;};
+template<typename T> struct tbb_remove_cv<const volatile T> {typedef T type;};
 
 //! Meets requirements of a forward iterator for STL.
 /** Value is either the T or const T type of the container.
     @ingroup containers */
 template<typename Container, typename Value>
-class concurrent_queue_iterator: public concurrent_queue_iterator_base_v3<Value>,
+class concurrent_queue_iterator: public concurrent_queue_iterator_base_v3<typename tbb_remove_cv<Value>::type>,
         public std::iterator<std::forward_iterator_tag,Value> {
 #if !__TBB_TEMPLATE_FRIENDS_BROKEN
     template<typename T, class A>
@@ -709,28 +740,20 @@ public: // workaround for MSVC
 #endif 
     //! Construct iterator pointing to head of queue.
     concurrent_queue_iterator( const concurrent_queue_base_v3<Value>& queue ) :
-        concurrent_queue_iterator_base_v3<Value>(queue)
+        concurrent_queue_iterator_base_v3<typename tbb_remove_cv<Value>::type>(queue)
     {
     }
 
 public:
     concurrent_queue_iterator() {}
 
-    //! Copy constructor
-    concurrent_queue_iterator( const concurrent_queue_iterator<Container,Value>& other ) :
-        concurrent_queue_iterator_base_v3<Value>(other)
-    {
-    }
-
-    template<typename T>
-    concurrent_queue_iterator( const concurrent_queue_iterator<Container,T>& other ) :
-        concurrent_queue_iterator_base_v3<Value>(add_constness(other))
-    {
-    }
+    concurrent_queue_iterator( const concurrent_queue_iterator<Container,typename Container::value_type>& other ) :
+        concurrent_queue_iterator_base_v3<typename tbb_remove_cv<Value>::type>(other)
+    {}
 
     //! Iterator assignment
     concurrent_queue_iterator& operator=( const concurrent_queue_iterator& other ) {
-        assign(other);
+        this->assign(other);
         return *this;
     }
 
@@ -808,6 +831,19 @@ protected:
     //! Size of an item
     size_t item_size;
 
+#if __TBB_GCC_3_3_PROTECTED_BROKEN
+public:
+#endif
+    template<typename T>
+    struct padded_page: page {
+        //! Not defined anywhere - exists to quiet warnings.
+        padded_page(); 
+        //! Not defined anywhere - exists to quiet warnings.
+        void operator=( const padded_page& );
+        //! Must be last field.
+        T last;
+    };
+
 private:
     virtual void copy_item( page& dst, size_t index, const void* src ) = 0;
     virtual void assign_and_destroy_item( void* dst, page& src, size_t index ) = 0;
@@ -860,7 +896,7 @@ private:
 //! Type-independent portion of concurrent_queue_iterator.
 /** @ingroup containers */
 class concurrent_queue_iterator_base_v3 {
-    //! Concurrentconcurrent_queue over which we are iterating.
+    //! concurrent_queue over which we are iterating.
     /** NULL if one past last element in queue. */
     concurrent_queue_iterator_rep* my_rep;
 
@@ -869,9 +905,11 @@ class concurrent_queue_iterator_base_v3 {
 
     template<typename C, typename T, typename U>
     friend bool operator!=( const concurrent_queue_iterator<C,T>& i, const concurrent_queue_iterator<C,U>& j );
+
+    void initialize( const concurrent_queue_base_v3& queue, size_t offset_of_data );
 protected:
     //! Pointer to current item
-    mutable void* my_item;
+    void* my_item;
 
     //! Default constructor
     concurrent_queue_iterator_base_v3() : my_rep(NULL), my_item(NULL) {}
@@ -881,9 +919,13 @@ protected:
         assign(i);
     }
 
-    //! Construct iterator pointing to head of queue.
+    //! Obsolete entry point for constructing iterator pointing to head of queue.
+    /** Does not work correctly for SSE types. */
     __TBB_EXPORTED_METHOD concurrent_queue_iterator_base_v3( const concurrent_queue_base_v3& queue );
 
+    //! Construct iterator pointing to head of queue.
+    __TBB_EXPORTED_METHOD concurrent_queue_iterator_base_v3( const concurrent_queue_base_v3& queue, size_t offset_of_data );
+
     //! Assignment
     void __TBB_EXPORTED_METHOD assign( const concurrent_queue_iterator_base_v3& i );
 
@@ -902,6 +944,7 @@ typedef concurrent_queue_iterator_base_v3 concurrent_queue_iterator_base;
 template<typename Container, typename Value>
 class concurrent_queue_iterator: public concurrent_queue_iterator_base,
         public std::iterator<std::forward_iterator_tag,Value> {
+
 #if !defined(_MSC_VER) || defined(__INTEL_COMPILER)
     template<typename T, class A>
     friend class ::tbb::concurrent_bounded_queue;
@@ -913,7 +956,7 @@ public: // workaround for MSVC
 #endif 
     //! Construct iterator pointing to head of queue.
     concurrent_queue_iterator( const concurrent_queue_base_v3& queue ) :
-        concurrent_queue_iterator_base_v3(queue)
+        concurrent_queue_iterator_base_v3(queue,__TBB_offsetof(concurrent_queue_base_v3::padded_page<Value>,last))
     {
     }
 
