@@ -1,8 +1,8 @@
 //-------------------------------------------------------------------------
-//    $Id: glossary.h 22974 2010-12-16 18:40:04Z bangerth $
+//    $Id: glossary.h 24408 2011-09-25 19:06:31Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -339,7 +339,9 @@
  * <dd>The <i>direction flag</i> is used in triangulations embedded in a
  * higher dimensional space to denote the orientation of cells and make the
  * manifold oriented. It is accessed using CellAccessor::direction_flag()
- * and set by the Triangulation class upon creation of a triangulation.
+ * and set by the Triangulation class upon creation of a triangulation. You
+ * can change all direction flags of a triangulation using the
+ * Triangulation::flip_all_direction_flags() function.
  *
  * The flag is necessary to make cases like this work: assume we have a
  * one-dimensional mesh embedded in a two-dimensional space,
@@ -505,8 +507,8 @@
  * DoFHandler objects. In essence, it is a guide to the parallel::distributed
  * namespace and the techniques used in step-40.
  *
- * The paper is submitted to the ACM Transactions on Mathematical Software
- * (ACM TOMS). It is also available as
+ * The paper is accepted for publication in the ACM Transactions on
+ * Mathematical Software (ACM TOMS). It is also available as
  * <a href="http://iamcs.tamu.edu/file_dl.php?type=preprint&preprint_id=237">IAMCS
  * preprint 2011-187</a>. Its current BiBTex entry is
  * @code
@@ -518,7 +520,7 @@ TechReport{BBHK11,
   institution =  {Institute for Applied Mathematics and Computational Science,
                   Texas A&M University},
   year = 	 2010,
-  number = 	 {2011-187, submitted}
+  number = 	 {2011-187, accepted for publication in ACM Trans. Math. Softw.}
 }
  * @endcode
  *
@@ -659,6 +661,65 @@ Article{BK07,
  * @ref GlossSupport "support points".</dd>
  *
  *
+ * <dt class="glossary">@anchor GlossLocallyOwnedCell <b>Locally owned cell</b></dt>
+ * <dd>This concept identifies a subset of all cells when using
+ * distributed meshes, see the @ref distributed module. In such meshes, each
+ * cell is owned by exactly one processor. The locally owned ones are those
+ * owned by the current processor.
+ * </dd>
+ *
+ *
+ * <dt class="glossary">@anchor GlossLocallyOwnedDof <b>Locally owned degrees of freedom</b></dt>
+ * <dd>This concept identifies a subset of all degrees of freedom when using
+ * distributed meshes, see the @ref distributed module.  Locally owned degrees
+ * of freedom live on locally owned cells. Since degrees of freedom are owned
+ * by only one processor, degrees of freedom on interfaces between cells owned
+ * by different processors may be owned by one or the other, so not all
+ * degrees of freedom on a locally owned cell are also locally owned degrees
+ * of freedom.
+ *
+ * Locally owned DoFs are a subset of the
+ * @ref GlossLocallyActiveDof "locally active DoFs".</dd>
+ *
+ *
+ * <dt class="glossary">@anchor GlossLocallyActiveDof <b>Locally active degrees of freedom</b></dt>
+ * <dd>This concept identifies a subset of all degrees of freedom when using
+ * distributed meshes, see the @ref distributed module.  Locally active degrees
+ * of freedom are those that live on locally owned cells. Degrees of freedom
+ * on interfaces between cells owned by different processors therefore belong
+ * to the set of locally active degrees of freedom for more than one processor.
+ *
+ * Locally active DoFs are a superset of the
+ * @ref GlossLocallyOwnedDof "locally owned DoFs" and a subset of the
+ * @ref GlossLocallyRelevantDof "locally relevant DoFs".
+ * </dd>
+ *
+ *
+ * <dt class="glossary">@anchor GlossLocallyRelevantDof <b>Locally relevant degrees of freedom</b></dt>
+ * <dd>This concept identifies a subset of all degrees of freedom when using
+ * distributed meshes, see the @ref distributed module.  Locally relevant
+ * degrees of freedom are those that live on locally owned or ghost cells.
+ * Consequently, they may be owned by different processors.
+ *
+ * Locally relevant DoFs are a superset of the
+ * @ref GlossLocallyActiveDof "locally active DoFs."
+ * </dd>
+ *
+ *
+ * <dt class="glossary">@anchor GlossMaterialId <b>Material id</b></dt>
+ * <dd>Each cell of a triangulation has associated with it a property called
+ * "material id". It is commonly used in problems with heterogenous
+ * coefficients to identify which part of the domain a cell is in and,
+ * consequently, which value the coefficient should have on this particular
+ * cell. The material id is inherited from mother to child cell upon mesh
+ * refinement.
+ *
+ * The material id is set and queried using the CellAccessor::material_id,
+ * CellAccessor::set_material_id and CellAccessor::recursively_set_material_id
+ * functions.
+ * </dd>
+ *
+ *
  * <dt class="glossary">@anchor mg_paper <b>%Multigrid paper</b></dt>
  * <dd>The "multigrid paper" is a paper by B. Janssen and G. Kanschat, titled
  * "Adaptive multilevel methods with local smoothing", that
@@ -733,9 +794,30 @@ Article{JK10,
  * class provide non-primitive finite elements because there, each
  * vector-value shape function may have several non-zero components.</dd>
  *
+ *
  * <dt class="glossary">@anchor GlossReferenceCell <b>Reference cell</b></dt>
  * <dd>The hypercube [0,1]<sup>dim</sup>, on which all parametric finite
- * element shape functions are defined.</dd>
+ * element shape functions are defined. Many properties of the reference
+ * cell are described by the GeometryInfo class.</dd>
+ *
+ *
+ * <dt class="glossary">@anchor GlossSerialization <b>Serialization</b></dt>
+
+ * <dd>The term "serialization" refers to the process of writing the state of
+ * an object to a stream and later retrieve it again. A typical use case is to
+ * save the state of a program to disk for possible later resurrection, often
+ * in the context of checkpoint/restart strategies for long running
+ * computations or on computers that aren't very reliable (e.g. on very large
+ * clusters where individual nodes occasionally fail and then bring down an
+ * entire MPI job). In either case, one wants to occasionally save the state
+ * of the program so that, upon failure, one can restart it at that point
+ * rather than having to run it again from the beginning.
+ *
+ * deal.II implements serialization facilities by implementing the necessary
+ * interfaces for the <a
+ * href="http://www.boost.org/doc/libs/1_46_1/libs/serialization/doc/index.html"
+ * target="_top">BOOST serialization</a> library. See there for examples on
+ * how to save and restore objects. </dd>
  *
  *
  * <dt class="glossary">@anchor GlossShape <b>Shape functions</b></dt>
