@@ -1,8 +1,8 @@
 //-------------------------------------------------------------------------
-//    $Id: multithreading.h 22882 2010-11-29 20:10:43Z bangerth $
+//    $Id: multithreading.h 24582 2011-10-08 23:17:21Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2006, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2006, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -34,8 +34,8 @@
  * namespace. The MultithreadInfo class allows to query certain
  * properties of the system, such as the number of CPUs. These
  * facilities for %parallel computing are described in the
- * following. The step-9, step-13, step-14, step-32, step-35 and
- * step-37 tutorial programs also show their use in practice, with the
+ * following. The step-9, step-13, step-14, step-32 and step-35
+ * tutorial programs also show their use in practice, with the
  * ones starting with step-32 using a more modern style of doing
  * things in which essentially we describe <i>what</i> can be done in
  * %parallel, while the older tutorial programs describe <i>how</i>
@@ -98,11 +98,12 @@
  * identified with individual C++ statements, but often they more
  * generally coincide with entire code blocks.
  *
- * The point here is this: To exploit the independence of tasks 2 and 3, we
- * could start two threads and run each task on its own thread; we would then
- * wait for the two threads to finish (an operation called "joining a thread")
- * and go on with statement 4. As discussed in more detail below, code to
- * achieve this would look like this:
+ * The point here is this: If we wanted to use threads to exploit the
+ * independence of tasks 2 and 3, we would start two threads and run each of
+ * tasks 2 and 3 on its own thread; we would then wait for the two threads to
+ * finish (an operation called "joining a thread") and go on with statement
+ * 4. Code to achieve this would look like this (the actual syntax is
+ * explained in more detail below):
  * @code
    dof_handler.distribute_dofs (fe);
 
@@ -120,7 +121,7 @@
  * But what if
  * your computer has only one processor core, or if we have two but there is
  * already a different part of the program running in %parallel to the code
- * above? In that case, we could of course still start new threads, but the
+ * above? In that case, the code above would still start new threads, but the
  * program is not going to run faster since no additional compute resources
  * are available; rather, the program will run slower since threads have to be
  * created and destroyed, and the operating system has to schedule threads to
@@ -413,7 +414,7 @@
          *out = *in_1 + *in_2;
  * @endcode
  * The next C++ standard will contain a more elegant way to achieve the
- * same effect shown above using the Boost Lambda library, through a
+ * same effect shown above using the Boost library, through a
  * mechanism known as <i>lambda expressions</i> and <i>closures</i>.
  *
  * Note also that we have made sure that no CPU ever gets a chunk of
@@ -485,21 +486,21 @@
                               Vector       &dst) const
     {
       parallel::transform (dst.begin(), dst.end(),
-                           boost::bind (&SparseMatrix::vmult_one_row,
+                           std_cxx1x::bind (&SparseMatrix::vmult_one_row,
 			                this,
-					boost::cref(src),
-					boost::ref(dst),
-					_1),
+					std_cxx1x::cref(src),
+					std_cxx1x::ref(dst),
+					std_cxx1x::_1),
 			   200);
     }
  * @endcode
  * Note how we use <a
- * href="http://www.boost.org/doc/libs/1_37_0/libs/bind/bind.html">boost::bind</a>
+ * href="http://www.boost.org/doc/libs/1_37_0/libs/bind/bind.html">std_cxx1x::bind</a>
  * to <i>bind</i> certain arguments to the <code>vmult_one_row</code>
  * function, leaving one argument open and thus allowing the
  * parallel::transform function to consider the passed function argument as
  * unary. Also note that we need to make the source and destination vectors as
- * (const) references to prevent boost::bind from passing them by value
+ * (const) references to prevent std_cxx1x::bind from passing them by value
  * (implying a copy for <code>src</code> and writing the result into a
  * temporary copy of <code>dst</code>, neither of which is what we desired).
  * Finally, notice the grainsize of a minimum of 200 rows of a matrix that
@@ -541,11 +542,11 @@
                               Vector       &dst) const
     {
        parallel::apply_to_subranges (0, n_rows(),
-				     boost::bind (vmult_on_subrange,
+				     std_cxx1x::bind (vmult_on_subrange,
 						  this,
-						  _1, _2,
-						  boost::cref(src),
-						  boost::ref(dst)),
+						  std_cxx1x::_1, std_cxx1x::_2,
+						  std_cxx1x::cref(src),
+						  std_cxx1x::ref(dst)),
 				     200);
     }
  * @endcode
@@ -616,10 +617,10 @@
       return
         std::sqrt
 	(parallel::accumulate_from_subranges (0, n_rows(),
-				              boost::bind (mat_norm_sqr_on_subrange,
+				              std_cxx1x::bind (mat_norm_sqr_on_subrange,
 						           this,
-						           _1, _2,
-						           boost::cref(x)),
+						           std_cxx1x::_1, std_cxx1x::_2,
+						           std_cxx1x::cref(x)),
 				              200));
     }
  * @endcode
@@ -915,6 +916,13 @@
 		:
 		fe_values (fe, quadrature, update_flags)
        {}
+
+     ScratchData (const ScratchData &scratch)
+		:
+		fe_values (scratch.fe_values.get_fe(),
+		           scratch.fe_values.get_quadrature(),
+		           scratch.fe_values.get_update_flags())
+       {}
    }
  * @endcode
  * and then use this FEValues object in the assemble function:
@@ -928,8 +936,15 @@
      ...
    }
  * @endcode
+ *   Just as for the <code>PerTaskData</code> structure, we will create a
+ *   sample <code>ScratchData</code> object and pass it to the work stream
+ *   object, which will replicate it as many times as necessary. For this
+ *   to work <code>ScratchData</code> structures need to copyable. Since FEValues
+ *   objects are rather complex and cannot be copied implicitly, we provided
+ *   our own copy constructor for the <code>ScratchData</code> structure.
+ *
  *   The same approach, putting things into the <code>ScratchData</code>
- *   data structure should be used for everything that is expensive to
+ *   data structure, should be used for everything that is expensive to
  *   construct. This holds, in particular, for everything that needs to
  *   allocate memory upon construction; for example, if the values of a
  *   function need to be evaluated at quadrature points, then this is
@@ -956,9 +971,17 @@
                   const Quadrature<dim>    &quadrature,
                   const UpdateFlags         update_flags)
 		:
-		rhs_values (quadrature.n_quadrature_points),
+		rhs_values (quadrature.size()),
 		fe_values (fe, quadrature, update_flags)
        {}
+
+      ScratchData (const ScratchData &scratch)
+		:
+		rhs_values (scratch.rhs_values),
+		fe_values (scratch.fe_values.get_fe(),
+		           scratch.fe_values.get_quadrature(),
+		           scratch.fe_values.get_update_flags())
+       {}
    }
 
    template <int dim>
@@ -992,19 +1015,20 @@
      // ...is the same as:
      WorkStream::run (dof_handler.begin_active(),
 	              dof_handler.end(),
-		      boost::bind(&MyClass<dim>::assemble_on_one_cell, *this, _1, _2, _3),
-		      boost::bind(&MyClass<dim>::copy_local_to_global, *this, _1),
+		      std_cxx1x::bind(&MyClass<dim>::assemble_on_one_cell, *this,
+		                  std_cxx1x::_1, std_cxx1x::_2, std_cxx1x::_3),
+		      std_cxx1x::bind(&MyClass<dim>::copy_local_to_global, *this, std_cxx1x::_1),
 		      per_task_data);
  * @endcode
- * Note how <code>boost::bind</code> produces a function object that takes three
+ * Note how <code>std_cxx1x::bind</code> produces a function object that takes three
  * arguments by binding the member function to the <code>*this</code>
- * object. <code>_1, _2</code> and <code>_3</code> are placeholders for the first,
+ * object. <code>std_cxx1x::_1, std_cxx1x::_2</code> and <code>std_cxx1x::_3</code> are placeholders for the first,
  * second and third argument that can be specified later on. In other words, for
  * example if <code>p</code> is the result of the first call to
- * <code>boost::bind</code>, then the call <code>p(cell, scratch_data,
+ * <code>std_cxx1x::bind</code>, then the call <code>p(cell, scratch_data,
  * per_task_data)</code> will result in executing
  * <code>this-@>assemble_on_one_cell (cell, scratch_data, per_task_data)</code>,
- * i.e. <code>boost::bind</code> has bound the object to the function pointer
+ * i.e. <code>std_cxx1x::bind</code> has bound the object to the function pointer
  * but left the three arguments open for later.
  *
  * Similarly, let us assume that <code>MyClass::assemble_on_one_cell</code>
@@ -1026,25 +1050,25 @@
  * @code
      WorkStream::run (dof_handler.begin_active(),
 	              dof_handler.end(),
-		      boost::bind(&MyClass<dim>::assemble_on_one_cell,
+		      std_cxx1x::bind(&MyClass<dim>::assemble_on_one_cell,
 		                  *this,
 				  current_solution,
-				  _1,
-				  _2,
- 				  _3,
+				  std_cxx1x::_1,
+				  std_cxx1x::_2,
+ 				  std_cxx1x::_3,
 				  previous_time+time_step),
-		      boost::bind(&MyClass<dim>::copy_local_to_global,
-		                  *this, _1),
+		      std_cxx1x::bind(&MyClass<dim>::copy_local_to_global,
+		                  *this, std_cxx1x::_1),
 		      per_task_data);
  * @endcode
  * Here, we bind the object, the linearization point argument, and the
  * current time argument to the function before we hand it off to
  * WorkStream::run(). WorkStream::run() will then simply call the
  * function with the cell and scratch and per task objects which will be filled
- * in at the positions indicated by <code>_1, _2</code> and <code>_3</code>.
+ * in at the positions indicated by <code>std_cxx1x::_1, std_cxx1x::_2</code> and <code>std_cxx1x::_3</code>.
  *
  * To see the WorkStream class used in practice on tasks like the ones
- * outlined above, take a look at the step-32, step-35 or step-37
+ * outlined above, take a look at the step-32 or step-35
  * tutorial programs.
  *
  *
@@ -1095,12 +1119,11 @@
  * <h3>Thread-based parallelism</h3>
  *
  * Even though tasks are a higher-level way to describe things, there are
- * cases where they are poorly suited to a task. The main reason for not
- * using tasks even for computations that are independent are listed in the
- * section on
+ * cases that are poorly suited to a task (for a discussion of some of
+ * these cases see
  * @ref MTHow "How scheduling tasks works and when task-based programming is not efficient"
- * above. Primarily, jobs that are not able to fully utilize the CPU are bad
- * fits for tasks as discussed above.
+ * above). Generally, jobs that are not able to fully utilize the CPU are bad
+ * fits for tasks and good fits for threads.
  *
  * In a case like this, you can resort to explicitly start threads, rather
  * than tasks, using pretty much the same syntax as above. For example, if you
@@ -1131,15 +1154,12 @@
  * @endcode
  *
  * Here, Threads::new_thread starts the given function that writes to the
- * output file on a new thread that can run in %parallel to everything
- * else. Note that this function is actually pretty well parallelized: both
- * DataOut::build_patches() and KellyErrorEstimator::estimate() already use
- * WorkStream and will therefore utilize pretty much all available compute
- * resources. In %parallel to the KellyErrorEstimator::estimate() function, the
- * DataOut::write_vtk() function will run on a separate thread, independent of
- * the scheduler that takes care of the tasks, but that is not a problem
- * because writing lots of data to a file is not something that will keep a
- * CPU very busy.
+ * output file on a new thread that can run in %parallel to everything else:
+ * In %parallel to the KellyErrorEstimator::estimate() function, the
+ * DataOut::write_vtk() function will run on a separate thread. This execution
+ * is independent of the scheduler that takes care of tasks, but that is
+ * not a problem because writing lots of data to a file is not something that
+ * will keep a CPU very busy.
  *
  * Creating threads works pretty much the same way as tasks, i.e. you can wait
  * for the termination of a thread using Threads::Thread::join(), query the
