@@ -1,8 +1,8 @@
 //-------------------------------------------------------------------------
-//    $Id: vector_valued.h 20602 2010-02-13 17:44:17Z bangerth $
+//    $Id: vector_valued.h 23591 2011-04-13 19:06:22Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2008 by the deal.II authors
+//    Copyright (C) 2008, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -34,7 +34,7 @@
  *   <li>Complex-valued solutions consisting of real and imaginary parts, as
  *       discussed for example in step-29.
  * </ul>
- * 
+ *
  * This page gives an overview of how to implement such vector-valued problems
  * efficiently in deal.II.
  *
@@ -48,6 +48,7 @@
  *  <li> @ref VVAlternative "An alternative approach"
  *  <li> @ref VVBlockSolvers "Block solvers"
  *  <li> @ref VVExtracting "Extracting data from solutions"
+ *  <li> @ref VVOutput "Generating graphical output"
  * </ol> </td> </tr> </table>
  *
  *
@@ -180,7 +181,7 @@
  * Once we have settled on this description, we need to find a way to describe
  * the vector-valued finite element space from which we draw solution and test
  * functions. This is where the FESystem class comes in: it composes
- * vector-valued finite element spaces from simpler one. For example, if we
+ * vector-valued finite element spaces from simpler ones. For example, if we
  * were to attempt to use $Q_1$ elements for all <code>dim</code> components
  * of $\mathbf u$ and the one pressure component $p$, we could use the
  * following object:
@@ -217,7 +218,7 @@
  * cells; this is done using the FE_DGQ class. The combined element will then
  * be described by
  * @code
- *   FESystem<dim> finite_element (FERaviartThomas<dim>(1), 1,
+ *   FESystem<dim> finite_element (FE_RaviartThomas<dim>(1), 1,
  *                                 FE_DGQ<dim>(1),          1);
  * @endcode
  * i.e. we combine a single copy of the Raviart-Thomas element with a single
@@ -236,7 +237,7 @@
  * represent scalar fields (e.g. temperature, pressure, concentration) and/or
  * if any of its components are parts of vector fields (velocities,
  * displacements) or tensors (e.g. stresses). As a consequence, the FEValues
- * objects we use below to evaluate finite element shape functions at
+ * object we use below to evaluate finite element shape functions at
  * quadrature points only knows that it has a finite element with a number
  * of vector components, but doesn't know how to group them. We will show
  * how to give these components a logical connection using the
@@ -244,23 +245,23 @@
  *
  *
  * @anchor VVAssembling
- * <h3>Assembling linear systems</h3> 
+ * <h3>Assembling linear systems</h3>
  *
  * The next step is to assemble the linear system. How to do this for the
  * simple case of a scalar problem has been shown in many tutorial programs,
  * starting with step-3. Here we will show how to do it for
  * vector problems.
  *
- * How to do this is possibly best explained by showing an example
+ * The whole concept is probably best explained by showing an example
  * illustrating how the local contribution of a cell to the weak form of above
- * mixed Laplace equations could be assembled. This is essentially how @ref
- * step_20 "step-20" does it:
+ * mixed Laplace equations could be assembled. This is essentially how
+ * step-20 does it:
  * @code
   const FEValuesExtractors::Vector velocities (0);
   const FEValuesExtractors::Scalar pressure (dim);
 
   ...
-  
+
   typename DoFHandler<dim>::active_cell_iterator
     cell = dof_handler.begin_active(),
     endc = dof_handler.end();
@@ -272,8 +273,8 @@
 
       right_hand_side.value_list (fe_values.get_quadrature_points(),
                                   rhs_values);
-      
-      for (unsigned int q=0; q<n_q_points; ++q) 
+
+      for (unsigned int q=0; q<n_q_points; ++q)
         for (unsigned int i=0; i<dofs_per_cell; ++i)
           {
             for (unsigned int j=0; j<dofs_per_cell; ++j)
@@ -291,7 +292,7 @@
                               rhs_values[q] *
                               fe_values.JxW(q);
           }
- * @endcode 
+ * @endcode
  *
  * So here's what is happening:
  * <ul>
@@ -312,8 +313,8 @@
  *        and right hand side vector. Recall that the cell contributions
  *        to the bilinear form (i.e. neglecting boundary terms) looked as
  *        follows, based on shape functions
- *        $V_i=\left(\begin{array}{c}\mathbf v_i q_i\end{array}\right),
- *         V_j=\left(\begin{array}{c}\mathbf v_j q_j\end{array}\right)$:
+ *        $V_i=\left(\begin{array}{c}\mathbf v_i \\ q_i\end{array}\right),
+ *         V_j=\left(\begin{array}{c}\mathbf v_j \\ q_j\end{array}\right)$:
           @f{eqnarray*}
             (\mathbf v_i, \mathbf v_j)
 	    -
@@ -330,8 +331,9 @@
 				    fe_values[pressure].value (j, q)
                                     -
 				    fe_values[pressure].value (i, q) *
-				    fe_values[velocities].divergence (j, q)) *
-                                    fe_values.JxW(q);
+				    fe_values[velocities].divergence (j, q)
+				   ) *
+                                   fe_values.JxW(q);
  *        @endcode
  *        The similarities are pretty obvious.
  *
@@ -371,7 +373,21 @@
  * Other examples of using extractors and views are shown in tutorial programs
  * step-21,
  * step-22,
- * step-31 and a few other programs.
+ * step-31 and several other programs.
+ *
+ * @note In the current context, when we talk about a vector (for example in
+ * extracting the velocity components above), we mean the word in the sense
+ * physics uses it: it has <code>spacedim</code> components that behave in
+ * specific ways under coordinate system transformations. Examples include
+ * velocity or displacement fields. This is opposed to how mathematics uses
+ * the word "vector" (and how we use this word in other contexts in the
+ * library, for example in the Vector class), where it really stands for a
+ * collection of numbers. An example of this latter use of the word could be
+ * the set of concentrations of chemical species in a flame; however, these
+ * are really just a collection of scalar variables, since they do not change
+ * if the coordinate system is rotated, unlike the components of a velocity
+ * vector, and consequently, this FEValuesExtractors::Vector class should not
+ * be used for this case.
  *
  *
  * @anchor VVAlternative
@@ -404,13 +420,13 @@
  * parameters. Given our discussions above, the obvious way to implement this
  * bilinear form would be as follows, using an extractor object that interprets
  * all <code>dim</code> components of the finite element as single vector,
- * rather than disjoint components:
+ * rather than disjoint scalar components:
  *
  * @code
       const FEValuesExtractors::Vector displacements (0);
 
       ...
-      
+
       for (unsigned int q_point=0; q_point<n_q_points; ++q_point)
 	for (unsigned int i=0; i<dofs_per_cell; ++i)
 	  {
@@ -418,28 +434,28 @@
 	      = fe_values[displacements].gradient (i,q_point);
 	    const double phi_i_div
 	      = fe_values[displacements].divergence (i,q_point);
-	  
-	    for (unsigned int j=0; j<dofs_per_cell; ++j) 
+
+	    for (unsigned int j=0; j<dofs_per_cell; ++j)
 	      {
 		const Tensor<2,dim> phi_j_grad
 		  = fe_values[displacements].gradient (j,q_point);
 		const double phi_j_div
 		  = fe_values[displacements].divergence (j,q_point);
 
-		cell_matrix(i,j) 
-		  +=  (phi_i_div * phi_j_div *
-		       lambda_values[q_point]
+		cell_matrix(i,j)
+		  +=  (lambda_values[q_point] *
+		       phi_i_div * phi_j_div
 		       +
-		       scalar_product(phi_i_grad, phi_j_grad) *
-		       mu_values[q_point]
+		       mu_values[q_point] *
+		       scalar_product(phi_i_grad, phi_j_grad)
 		       +
-		       scalar_product(phi_i_grad, transpose(phi_j_grad)) *
-		       mu_values[q_point])
-		      *
+		       mu_values[q_point] *
+		       scalar_product(phi_i_grad, transpose(phi_j_grad))
+		      ) *
 		      fe_values.JxW(q_point);
 	      }
 	  }
- * @endcode 
+ * @endcode
  *
  * The scalar product between two tensors used in this bilinear form is
  * implemented as follows:
@@ -456,14 +472,14 @@ scalar_product (const Tensor<2,dim> &u,
       tmp += u[i][j] * v[i][j];
   return tmp;
 }
- * @endcode 
+ * @endcode
  *
  * Now, this is not the code used in step-8. In fact,
  * if one used the above code over the one implemented in that program,
  * it would run about 8 per cent slower. It can be improved (bringing
  * down the penalty to about 4 per cent) by taking a close look at the
  * bilinear form. In fact, we can transform it as follows:
-@f{eqnarray*} 
+@f{eqnarray*}
   a({\mathbf u}, {\mathbf v})
   &=&
   \left(
@@ -489,7 +505,7 @@ scalar_product (const Tensor<2,dim> &u,
   \sum_{i,j}
   \left(
     \mu \partial_i u_j, \frac 12[\partial_i v_j + \partial_j v_i]
-  \right)_\Omega  
+  \right)_\Omega
   \\
   &=&
   \left(
@@ -530,29 +546,28 @@ scalar_product (const Tensor<2,dim> &u,
 	      = fe_values[displacements].symmetric_gradient (i,q_point);
 	    const double phi_i_div
 	      = fe_values[displacements].divergence (i,q_point);
-	  
-	    for (unsigned int j=0; j<dofs_per_cell; ++j) 
+
+	    for (unsigned int j=0; j<dofs_per_cell; ++j)
 	      {
 		const SymmetricTensor<2,dim> phi_j_symmgrad
 		  = fe_values[displacements].symmetric_gradient (j,q_point);
 		const double phi_j_div
 		  = fe_values[displacements].divergence (j,q_point);
 
-		cell_matrix(i,j) 
+		cell_matrix(i,j)
 		  +=  (phi_i_div * phi_j_div *
 		       lambda_values[q_point]
 		       +
 		       2 *
 		       (phi_i_symmgrad * phi_j_symmgrad) *
-		       mu_values[q_point])
-		      *
+		       mu_values[q_point]) *
 		      fe_values.JxW(q_point));
 	      }
 	  }
- * @endcode 
+ * @endcode
  *
  * So if, again, this is not the code we use in step-8, what do
- * we do there? The answer rests on the finite element we use. There, we use the
+ * we do there? The answer rests on the finite element we use. In step-8, we use the
  * following element:
  * @code
  *   FESystem<dim> finite_element (FE_Q<dim>(1), dim);
@@ -562,7 +577,8 @@ scalar_product (const Tensor<2,dim> &u,
  * "primitive" element: an element that may be vector-valued but where each
  * shape function has exactly one non-zero component. In other words: if the
  * $x$-component of a displacement shape function is nonzero, then the $y$-
- * and $z$-components must be zero and so on. What this means is that also
+ * and $z$-components must be zero and similarly for the other components.
+ * What this means is that also
  * derived quantities based on shape functions inherit this sparsity property.
  * For example: the divergence
  * $\mathrm{div}\ \Phi(x,y,z)=\partial_x\varphi_x(x,y,z) +
@@ -671,7 +687,7 @@ scalar_product (const Tensor<2,dim> &u,
   =
   \left(
   \begin{array}{c} F \\ G \end{array}
-  \right).  
+  \right).
 @f}
  * What this system means, of course, is
 @f{eqnarray*}
@@ -722,16 +738,16 @@ scalar_product (const Tensor<2,dim> &u,
     system_matrix.block(0,1).vmult (tmp, solution.block(1));
     tmp *= -1;
     tmp += system_rhs.block(0);
-    
+
 
     SolverControl solver_control (solution.block(0).size(),
                                   1e-8*tmp.l2_norm());
     SolverCG<> cg (solver_control, vector_memory);
-  
+
     cg.solve (system_matrix.block(0,0),
               solution.block(0),
 	      tmp,
-              PreconditionIdentity());        
+              PreconditionIdentity());
  * @endcode
  *
  * What's happening here is that we allocate a temporary vector with as many
@@ -764,7 +780,7 @@ scalar_product (const Tensor<2,dim> &u,
  * @code
   std::vector<Vector<double> > local_solution_values (n_q_points,
                                                       Vector<double> (dim+1));
- 
+
   typename DoFHandler<dim>::active_cell_iterator
     cell = dof_handler.begin_active(),
     endc = dof_handler.end();
@@ -797,8 +813,8 @@ scalar_product (const Tensor<2,dim> &u,
        Tensor<1,dim> velocity;
        for (unsigned int d=0; d<dim; ++d)
          velocity[d] = local_solution_values[q](d);
-	 
-       ... do something with this velocity ...				  
+
+       ... do something with this velocity ...
  * @endcode
  * Note how we convert from a dealii::Vector (which is simply a collection
  * of vector elements) into a <code>Tensor@<1,dim@></code> because the
@@ -812,7 +828,7 @@ scalar_product (const Tensor<2,dim> &u,
   std::vector<Tensor<1,dim> > local_velocity_values (n_q_points);
 
   const FEValuesExtractors::Vector velocities (0);
-  
+
   typename DoFHandler<dim>::active_cell_iterator
     cell = dof_handler.begin_active(),
     endc = dof_handler.end();
@@ -839,7 +855,7 @@ scalar_product (const Tensor<2,dim> &u,
   std::vector<double> local_pressure_values (n_q_points);
 
   const FEValuesExtractors::Scalar pressure (dim);
-  
+
   typename DoFHandler<dim>::active_cell_iterator
     cell = dof_handler.begin_active(),
     endc = dof_handler.end();
@@ -861,7 +877,7 @@ scalar_product (const Tensor<2,dim> &u,
  * Likewise, to extract the gradients of scalar components,
  * FEValuesViews::Scalar::get_function_gradients and
  * FEValuesViews::Scalar::get_function_hessians do the job.
- * For vector-(tensor-)valued quantities, there are functions
+ * For vector- (tensor-)valued quantities, there are functions
  * FEValuesViews::Vector::get_function_gradients and
  * FEValuesViews::Vector::get_function_hessians, and in
  * addition
@@ -874,6 +890,67 @@ scalar_product (const Tensor<2,dim> &u,
  * FEValuesViews::Scalar::get_function_laplacians and
  * FEValuesViews::Vector::get_function_laplacians.
  *
+ *
+ * @anchor VVOutput
+ * <h3>Generating graphical output</h3>
+ *
+ * As mentioned above, an FESystem object may hold multiple vector components,
+ * but it doesn't have a notion what they actually mean. As an example, take
+ * the object
+ * @code
+ *   FESystem<dim> finite_element (FE_Q<dim>(1), dim+1);
+ * @endcode
+ * It has <code>dim+1</code> vector components, but what do they mean? Are they
+ * the <code>dim</code> components of a velocity vector plus one pressure? Are
+ * they the pressure plus the <code>dim</code> velocity components? Or are
+ * they a collection of scalars?
+ *
+ * The point is that the FESystem class doesn't care. The <i>interpretation</i>
+ * of what the components mean is up to the person who uses the element later,
+ * for example in assembling a linear form, or in extracting data solution
+ * components for a linearized system in the next Newton step. In almost
+ * all cases, this interpretation happens at the place where it is needed.
+ *
+ * There is one case where one has to be explicit, however, and that is in
+ * generating graphical output. The reason is that many file formats for
+ * visualization want data that represents vectors (e.g. velocities,
+ * displacements, etc) to be stored separately from scalars (pressures,
+ * densities, etc), and there often is no way to group a bunch of scalars into
+ * a vector field from within a visualization program.
+ *
+ * To achieve this, we need to let the DataOut class and friends know which
+ * components of the FESystem form vectors (with <code>dim</code> components)
+ * and which are scalars. This is shown, for example, in step-22 where we
+ * generate output as follows:
+ * @code
+  std::vector<std::string> solution_names (dim, "velocity");
+  solution_names.push_back ("pressure");
+
+  std::vector<DataComponentInterpretation::DataComponentInterpretation>
+    data_component_interpretation
+    (dim, DataComponentInterpretation::component_is_part_of_vector);
+  data_component_interpretation
+    .push_back (DataComponentInterpretation::component_is_scalar);
+
+  DataOut<dim> data_out;
+  data_out.attach_dof_handler (dof_handler);
+  data_out.add_data_vector (solution, solution_names,
+			    DataOut<dim>::type_dof_data,
+			    data_component_interpretation);
+  data_out.build_patches ();
+ * @endcode
+ * In other words, we here create an array of <code>dim+1</code> elements in
+ * which we store which elements of the finite element are vectors and which
+ * are scalars; the array is filled with <code>dim</code> copies of
+ * DataComponentInterpretation::component_is_part_of_vector and a single
+ * trailing element of DataComponentInterpretation::component_is_scalar . The
+ * array is then given as an extra argument to DataOut::add_data_vector to
+ * explain how the data in the given solution vector is to be interpreted.
+ * Visualization programs like Visit and Paraview will then offer to show
+ * these <code>dim</code> components as vector fields, rather than as
+ * individual scalar fields.
+ *
+ *
  * @ingroup feall feaccess
  */
- 
+
