@@ -11,9 +11,9 @@ use strict;
 my @steps = (1,2,3,4,5,6,7,8,9,
 	     10,11,12,13,14,15,16,17,18,19,
 	     20,21,22,23,24,25,   27,28,29,
-	     30,31,   33,34,35,36,   38,39,
-             40,45);
-;
+	     30,31,32,33,34,35,36,   38,39,
+             40,45,46);
+
 
 # List of additional node attributes to highlight purpose and state of the example
 my %style = (
