@@ -1,15 +1,16 @@
 /**
- * @page changes_after_7_0 Changes after Version 7.0
+ * @page changes_after_7_1 Changes after Version 7.1
 
 <p>
 This is the list of changes made after the release of
-deal.II version 7.0.0.
-All entries are signed with the names of the author. Regular
-contributor's names are abbreviated by WB (Wolfgang Bangerth), GK
-(Guido Kanschat), RH (Ralf Hartmann).
+deal.II version 7.1.0.
+All entries are signed with the names of the author.
 </p>
 
 
+
+<!-- ----------- INCOMPATIBILITIES ----------------- -->
+
 <a name="incompatible"></a>
 <h3 style="color:red">Incompatibilities</h3>
 
@@ -23,25 +24,27 @@ inconvenience this causes.
 </p>
 
 <ol>
-  <li>
+<li> None so far.
 </ol>
 
 
+<!-- ----------- GENERAL IMPROVEMENTS ----------------- -->
+
 <a name="general"></a>
 <h3>General</h3>
 
 <ol>
-  <li>
+<li> None so far.
 </ol>
 
 
+<!-- ----------- SPECIFIC IMPROVEMENTS ----------------- -->
 
 <a name="specific"></a>
 <h3>Specific improvements</h3>
 
-
 <ol>
-  <li>
+<li> None so far.
 </ol>
 
 
