@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: product_matrix.cc 21458 2010-07-07 14:56:48Z kanschat $
+//    $Id: product_matrix.cc 23709 2011-05-17 04:34:08Z bangerth $
 //
 //    Copyright (C) 2005, 2006, 2010 by the deal.II authors
 //
@@ -12,10 +12,10 @@
 
 // See documentation of ProductMatrix for documentation of this example
 
-#include <base/logstream.h>
-#include <lac/matrix_lib.h>
-#include <lac/full_matrix.h>
-#include <lac/vector.h>
+#include <deal.II/base/logstream.h>
+#include <deal.II/lac/matrix_lib.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/vector.h>
 
 using namespace dealii;
 
