@@ -1,9 +1,9 @@
-/* $Id: step-3.cc 22331 2010-10-13 14:39:46Z bangerth $ */
-/* Author: Wolfgang Bangerth, University of Heidelberg, 1999 */
+/* $Id: step-3.cc 24232 2011-09-02 09:47:37Z kronbichler $ */
+/* Author: Wolfgang Bangerth, 1999, Guido Kanschat, 2011 */
 
-/*    $Id: step-3.cc 22331 2010-10-13 14:39:46Z bangerth $       */
+/*    $Id: step-3.cc 24232 2011-09-02 09:47:37Z kronbichler $       */
 /*                                                                */
-/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2010 by the deal.II authors */
+/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2010, 2011 by the deal.II authors */
 /*                                                                */
 /*    This file is subject to QPL and may not be  distributed     */
 /*    without copyright and license information. Please refer     */
@@ -18,12 +18,12 @@
 				 // classes which handle
 				 // triangulations and enumeration of
 				 // degrees of freedom:
-#include <grid/tria.h>
-#include <dofs/dof_handler.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/dofs/dof_handler.h>
 				 // And this is the file in which the
 				 // functions are declared that
 				 // create grids:
-#include <grid/grid_generator.h>
+#include <deal.II/grid/grid_generator.h>
 
 				 // The next three files contain classes which
 				 // are needed for loops over all cells and to
@@ -33,34 +33,34 @@
 				 // cells; the last one is new and provides
 				 // information about the degrees of freedom
 				 // local to a cell:
-#include <grid/tria_accessor.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_accessor.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_accessor.h>
 
 				 // In this file contains the description of
 				 // the Lagrange interpolation finite element:
-#include <fe/fe_q.h>
+#include <deal.II/fe/fe_q.h>
 
 				 // And this file is needed for the
 				 // creation of sparsity patterns of
 				 // sparse matrices, as shown in
 				 // previous examples:
-#include <dofs/dof_tools.h>
+#include <deal.II/dofs/dof_tools.h>
 
 				 // The next two file are needed for
 				 // assembling the matrix using
 				 // quadrature on each cell. The
 				 // classes declared in them will be
 				 // explained below:
-#include <fe/fe_values.h>
-#include <base/quadrature_lib.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/base/quadrature_lib.h>
 
 				 // The following three include files
 				 // we need for the treatment of
 				 // boundary values:
-#include <base/function.h>
-#include <numerics/vectors.h>
-#include <numerics/matrices.h>
+#include <deal.II/base/function.h>
+#include <deal.II/numerics/vectors.h>
+#include <deal.II/numerics/matrices.h>
 
 				 // We're now almost to the end. The second to
 				 // last group of include files is for the
@@ -77,16 +77,16 @@
 				 // program, we use the identity
 				 // preconditioner which does nothing, but we
 				 // need to include the file anyway):
-#include <lac/vector.h>
-#include <lac/full_matrix.h>
-#include <lac/sparse_matrix.h>
-#include <lac/compressed_sparsity_pattern.h>
-#include <lac/solver_cg.h>
-#include <lac/precondition.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/compressed_sparsity_pattern.h>
+#include <deal.II/lac/solver_cg.h>
+#include <deal.II/lac/precondition.h>
 
 				 // Finally, this is for output to a
 				 // file and to the console:
-#include <numerics/data_out.h>
+#include <deal.II/numerics/data_out.h>
 #include <fstream>
 #include <iostream>
 
@@ -95,7 +95,7 @@
 				 // scope:
 using namespace dealii;
 
-                                 // @sect3{The <code>LaplaceProblem</code> class}
+                                 // @sect3{The <code>Step3</code> class}
 
 				 // Instead of the procedural programming of
 				 // previous examples, we encapsulate
@@ -117,20 +117,24 @@ using namespace dealii;
                                  // i.e. all the functions that actually do
                                  // anything, are in the private section of
                                  // the class:
-class LaplaceProblem
+class Step3
 {
   public:
-    LaplaceProblem ();
+    Step3 ();
 
     void run ();
 
 				     // Then there are the member functions
 				     // that mostly do what their names
-				     // suggest. Since they do not need to be
-				     // called from outside, they are made
-				     // private to this class.
+				     // suggest and whose have been discussed
+				     // in the introduction already. Since
+				     // they do not need to be called from
+				     // outside, they are made private to this
+				     // class.
+
   private:
-    void make_grid_and_dofs ();
+    void make_grid ();
+    void setup_system ();
     void assemble_system ();
     void solve ();
     void output_results () const;
@@ -163,7 +167,7 @@ class LaplaceProblem
     Vector<double>       system_rhs;
 };
 
-                                 // @sect4{LaplaceProblem::LaplaceProblem}
+                                 // @sect4{Step3::Step3}
 
 				 // Here comes the constructor. It does not
 				 // much more than first to specify that we
@@ -181,15 +185,16 @@ class LaplaceProblem
 				 // distribute degree of freedom on the mesh
 				 // using the distribute_dofs() function.) All
 				 // the other member variables of the
-				 // LaplaceProblem class have a default
+				 // Step3 class have a default
 				 // constructor which does all we want.
-LaplaceProblem::LaplaceProblem () :
+Step3::Step3 ()
+		:
                 fe (1),
 		dof_handler (triangulation)
 {}
 
 
-                                 // @sect4{LaplaceProblem::make_grid_and_dofs}
+                                 // @sect4{Step3::make_grid}
 
                                  // Now, the first thing we've got to
 				 // do is to generate the
@@ -197,12 +202,8 @@ LaplaceProblem::LaplaceProblem () :
 				 // like to do our computation and
 				 // number each vertex with a degree
 				 // of freedom. We have seen this in
-				 // the previous examples before. Then
-				 // we have to set up space for the
-				 // system matrix and right hand side
-				 // of the discretized problem. This
-				 // is what this function does:
-void LaplaceProblem::make_grid_and_dofs ()
+				 // the previous examples before.
+void Step3::make_grid ()
 {
 				   // First create the grid and refine
 				   // all cells five times. Since the
@@ -214,7 +215,7 @@ void LaplaceProblem::make_grid_and_dofs ()
   GridGenerator::hyper_cube (triangulation, -1, 1);
   triangulation.refine_global (5);
 				   // Unsure that 1024 is the correct number?
-				   // Let's see: n_active_cells return the
+				   // Let's see: n_active_cells returns the
 				   // number of active cells:
   std::cout << "Number of active cells: "
 	    << triangulation.n_active_cells()
@@ -236,18 +237,25 @@ void LaplaceProblem::make_grid_and_dofs ()
 	    << std::endl;
 				   // Note the distinction between
 				   // n_active_cells() and n_cells().
+}
 
-				   // Next we enumerate all the degrees of
-				   // freedom. This is done by using the
-				   // distribute_dofs function, as we have
-				   // seen in the step-2 example. Since we use
-				   // the <code>FE_Q</code> class with a polynomial
-				   // degree of 1, i.e. bilinear elements,
-				   // this associates one degree of freedom
-				   // with each vertex. While we're at
-				   // generating output, let us also take a
-				   // look at how many degrees of freedom are
-				   // generated:
+                                 // @sect4{Step3::setup_system}
+
+				 // Next we enumerate all the degrees of
+				 // freedom and set up matrix and vector
+				 // objects to hold the system
+				 // data. Enumerating is done by using
+				 // DoFHandler::distribute_dofs(), as we have
+				 // seen in the step-2 example. Since we use
+				 // the FE_Q class and have set the polynomial
+				 // degree to 1 in the constructor,
+				 // i.e. bilinear elements, this associates
+				 // one degree of freedom with each
+				 // vertex. While we're at generating output,
+				 // let us also take a look at how many
+				 // degrees of freedom are generated:
+void Step3::setup_system ()
+{
   dof_handler.distribute_dofs (fe);
   std::cout << "Number of degrees of freedom: "
 	    << dof_handler.n_dofs()
@@ -258,40 +266,36 @@ void LaplaceProblem::make_grid_and_dofs ()
 				   // should be 33 times 33, or 1089.
 
 				   // As we have seen in the previous example,
-				   // we set up a sparsity pattern for the
-				   // system matrix and tag those entries that
-				   // might be nonzero.
+				   // we set up a sparsity pattern by first
+				   // creating a temporary structure, tagging
+				   // those entries that might be nonzero, and
+				   // then copying the data over to the
+				   // SparsityPattern object that can then be
+				   // used by the system matrix.
   CompressedSparsityPattern c_sparsity(dof_handler.n_dofs());
   DoFTools::make_sparsity_pattern (dof_handler, c_sparsity);
   sparsity_pattern.copy_from(c_sparsity);
 
-				   // Now the sparsity pattern is
-				   // built, you
-				   // can't add nonzero entries
-				   // anymore. The sparsity pattern is
-				   // `sealed', so to say, and we can
-				   // initialize the matrix itself
-				   // with it. Note that the
+				   // Note that the
 				   // SparsityPattern object does
 				   // not hold the values of the
 				   // matrix, it only stores the
 				   // places where entries are. The
-				   // entries are themselves stored in
+				   // entries themselves are stored in
 				   // objects of type SparseMatrix, of
 				   // which our variable system_matrix
 				   // is one.
 				   //
-				   // The distinction between sparsity
-				   // pattern and matrix was made to
-				   // allow several matrices to use
-				   // the same sparsity pattern. This
-				   // may not seem relevant, but when
-				   // you consider the size which
-				   // matrices can have, and that it
-				   // may take some time to build the
-				   // sparsity pattern, this becomes
-				   // important in large-scale
-				   // problems.
+				   // The distinction between sparsity pattern
+				   // and matrix was made to allow several
+				   // matrices to use the same sparsity
+				   // pattern. This may not seem relevant
+				   // here, but when you consider the size
+				   // which matrices can have, and that it may
+				   // take some time to build the sparsity
+				   // pattern, this becomes important in
+				   // large-scale problems if you have to
+				   // store several matrices in your program.
   system_matrix.reinit (sparsity_pattern);
 
 				   // The last thing to do in this
@@ -303,29 +307,28 @@ void LaplaceProblem::make_grid_and_dofs ()
   system_rhs.reinit (dof_handler.n_dofs());
 }
 
-                                 // @sect4{LaplaceProblem::assemble_system}
+                                 // @sect4{Step3::assemble_system}
 
 
-				 // Now comes the difficult part:
-				 // assembling matrices and
-				 // vectors. In fact, this is not
-				 // overly difficult, but it is
-				 // something that the library can't
-				 // do for you as for most of the
-				 // other things in the functions
-				 // above and below.
+				 // The next step is to compute the entries of
+				 // the matrix and right hand side that form
+				 // the linear system from which we compute
+				 // the solution. This is the central function
+				 // of each finite element program and we have
+				 // discussed the primary steps in the
+				 // introduction already.
 				 //
-				 // The general way to assemble matrices and
-				 // vectors is to loop over all cells, and on
-				 // each cell compute the contribution of that
-				 // cell to the global matrix and right hand
-				 // side by quadrature. The point to realize
-				 // now is that we need the values of the
-				 // shape functions at the locations of
+				 // The general approach to assemble matrices
+				 // and vectors is to loop over all cells, and
+				 // on each cell compute the contribution of
+				 // that cell to the global matrix and right
+				 // hand side by quadrature. The point to
+				 // realize now is that we need the values of
+				 // the shape functions at the locations of
 				 // quadrature points on the real
 				 // cell. However, both the finite element
 				 // shape functions as well as the quadrature
-				 // points are only defined on the unit
+				 // points are only defined on the reference
 				 // cell. They are therefore of little help to
 				 // us, and we will in fact hardly ever query
 				 // information about finite element shape
@@ -333,17 +336,18 @@ void LaplaceProblem::make_grid_and_dofs ()
 				 // objects directly.
 				 //
 				 // Rather, what is required is a way to map
-				 // this data from the unit cell to the real
-				 // cell. Classes that can do that are derived
-				 // from the Mapping class, though one again
-				 // often does not have to deal with them
-				 // directly: many functions in the library
-				 // can take a mapping object as argument, but
-				 // when it is omitted they simply resort to
-				 // the standard bilinear Q1 mapping. We will
-				 // go this route, and not bother with it for
-				 // the moment (we come back to this in
-				 // step-10, step-11, and step-12).
+				 // this data from the reference cell to the
+				 // real cell. Classes that can do that are
+				 // derived from the Mapping class, though one
+				 // again often does not have to deal with
+				 // them directly: many functions in the
+				 // library can take a mapping object as
+				 // argument, but when it is omitted they
+				 // simply resort to the standard bilinear Q1
+				 // mapping. We will go this route, and not
+				 // bother with it for the moment (we come
+				 // back to this in step-10, step-11, and
+				 // step-12).
 				 //
 				 // So what we now have is a collection of
 				 // three classes to deal with: finite
@@ -351,17 +355,18 @@ void LaplaceProblem::make_grid_and_dofs ()
 				 // objects. That's too much, so there is one
 				 // type of class that orchestrates
 				 // information exchange between these three:
-				 // the <code>FEValues</code> class. If given one
-				 // instance of each three of these objects,
-				 // it will be able to provide you with
-				 // information about values and gradients of
-				 // shape functions at quadrature points on a
-				 // real cell.
+				 // the FEValues class. If given one instance
+				 // of each three of these objects (or two,
+				 // and an implicit linear mapping), it will
+				 // be able to provide you with information
+				 // about values and gradients of shape
+				 // functions at quadrature points on a real
+				 // cell.
                                  //
                                  // Using all this, we will assemble the
                                  // linear system for this problem in the
                                  // following function:
-void LaplaceProblem::assemble_system ()
+void Step3::assemble_system ()
 {
 				   // Ok, let's start: we need a quadrature
 				   // formula for the evaluation of the
@@ -387,37 +392,37 @@ void LaplaceProblem::assemble_system ()
 				   // what we want it to compute on each cell:
 				   // we need the values of the shape
 				   // functions at the quadrature points (for
-				   // the right hand side (f,phi)), their
-				   // gradients (for the matrix entries (grad
-				   // phi_i, grad phi_j)), and also the
+				   // the right hand side $(\varphi,f)$), their
+				   // gradients (for the matrix entries $(\nabla
+				   // \varphi_i, \nabla \varphi_j)$), and also the
 				   // weights of the quadrature points and the
 				   // determinants of the Jacobian
-				   // transformations from the unit cell to
-				   // the real cells.
+				   // transformations from the reference cell
+				   // to the real cells.
 				   //
 				   // This list of what kind of information we
-				   // actually need is given as a bitwise
-				   // connection of flags as the third
+				   // actually need is given as a
+				   // collection of flags as the third
 				   // argument to the constructor of
-				   // <code>FEValues</code>. Since these values have to
+				   // FEValues. Since these values have to
 				   // be recomputed, or updated, every time we
 				   // go to a new cell, all of these flags
 				   // start with the prefix <code>update_</code> and
 				   // then indicate what it actually is that
 				   // we want updated. The flag to give if we
 				   // want the values of the shape functions
-				   // computed is <code>update_values</code>; for the
+				   // computed is #update_values; for the
 				   // gradients it is
-				   // <code>update_gradients</code>. The determinants
+				   // #update_gradients. The determinants
 				   // of the Jacobians and the quadrature
 				   // weights are always used together, so
 				   // only the products (Jacobians times
 				   // weights, or short <code>JxW</code>) are computed;
 				   // since we need them, we have to list
-				   // <code>update_JxW_values</code> as well:
+				   // #update_JxW_values as well:
   FEValues<2> fe_values (fe, quadrature_formula,
 			 update_values | update_gradients | update_JxW_values);
-                                   // The advantage of this proceeding is that
+                                   // The advantage of this approach is that
 				   // we can specify what kind of information
 				   // we actually need on each cell. It is
 				   // easily understandable that this approach
@@ -520,8 +525,8 @@ void LaplaceProblem::assemble_system ()
 				       // gradients of the shape functions be
 				       // computed, as well as the
 				       // determinants of the Jacobian
-				       // matrices of the mapping between unit
-				       // cell and true cell, at the
+				       // matrices of the mapping between
+				       // reference cell and true cell, at the
 				       // quadrature points. Since all these
 				       // values depend on the geometry of the
 				       // cell, we have to have the FEValues
@@ -547,9 +552,9 @@ void LaplaceProblem::assemble_system ()
 				       // at the quadrature point times the
 				       // weight of this quadrature point. You
 				       // can get the gradient of shape
-				       // function i at quadrature point
+				       // function $i$ at quadrature point
 				       // q_point by using
-				       // fe_values.shape_grad(i,q_point);
+				       // <code>fe_values.shape_grad(i,q_point)</code>;
 				       // this gradient is a 2-dimensional
 				       // vector (in fact it is of type
 				       // Tensor@<1,dim@>, with here dim=2) and
@@ -561,9 +566,9 @@ void LaplaceProblem::assemble_system ()
 				       // determinant and the quadrature point
 				       // weight (that one gets together by
 				       // the call to
-				       // <code>fe_values.JxW</code>). Finally, this is
+				       // FEValues::JxW() ). Finally, this is
 				       // repeated for all shape functions
-				       // phi_i and phi_j:
+				       // $i$ and $j$:
       for (unsigned int i=0; i<dofs_per_cell; ++i)
 	for (unsigned int j=0; j<dofs_per_cell; ++j)
 	  for (unsigned int q_point=0; q_point<n_q_points; ++q_point)
@@ -652,7 +657,7 @@ void LaplaceProblem::assemble_system ()
 				   // rather than projecting it onto the
 				   // boundary. There is a function in the
 				   // library which does exactly this:
-				   // <code>VectorTools::interpolate_boundary_values</code>. Its
+				   // VectorTools::interpolate_boundary_values(). Its
 				   // parameters are (omitting parameters for
 				   // which default values exist and that we
 				   // don't care about): the DoFHandler object
@@ -690,13 +695,13 @@ void LaplaceProblem::assemble_system ()
 				   // the boundary.
 				   //
 				   // The function describing the boundary
-				   // values is an object of type <code>Function</code>
+				   // values is an object of type Function
 				   // or of a derived class. One of the
-				   // derived classes is <code>ZeroFunction</code>,
+				   // derived classes is ZeroFunction,
 				   // which describes (not unexpectedly) a
 				   // function which is zero everywhere. We
 				   // create such an object in-place and pass
-				   // it to the interpolate_boundary_values
+				   // it to the VectorTools::interpolate_boundary_values()
 				   // function.
 				   //
 				   // Finally, the output object is a
@@ -728,7 +733,7 @@ void LaplaceProblem::assemble_system ()
 }
 
 
-                                 // @sect4{LaplaceProblem::solve}
+                                 // @sect4{Step3::solve}
 
                                  // The following function simply
 				 // solves the discretized
@@ -746,23 +751,23 @@ void LaplaceProblem::assemble_system ()
 				 // direct methods are no longer
 				 // usable and you are forced to use
 				 // methods like CG.
-void LaplaceProblem::solve ()
+void Step3::solve ()
 {
 				   // First, we need to have an object that
 				   // knows how to tell the CG algorithm when
 				   // to stop. This is done by using a
-				   // <code>SolverControl</code> object, and as
-				   // stopping criterion we say: stop after a
-				   // maximum of 1000 iterations (which is far
-				   // more than is needed for 1089 variables;
-				   // see the results section to find out how
-				   // many were really used), and stop if the
-				   // norm of the residual is below 1e-12. In
+				   // SolverControl object, and as stopping
+				   // criterion we say: stop after a maximum
+				   // of 1000 iterations (which is far more
+				   // than is needed for 1089 variables; see
+				   // the results section to find out how many
+				   // were really used), and stop if the norm
+				   // of the residual is below $10^{-12}$. In
 				   // practice, the latter criterion will be
 				   // the one which stops the iteration:
   SolverControl           solver_control (1000, 1e-12);
 				   // Then we need the solver itself. The
-				   // template parameters to the <code>SolverCG</code>
+				   // template parameters to the SolverCG
 				   // class are the matrix type and the type
 				   // of the vectors, but the empty angle
 				   // brackets indicate that we simply take
@@ -786,7 +791,7 @@ void LaplaceProblem::solve ()
 }
 
 
-                                 // @sect4{LaplaceProblem::output_results}
+                                 // @sect4{Step3::output_results}
 
 				 // The last part of a typical finite
 				 // element program is to output the
@@ -798,18 +803,18 @@ void LaplaceProblem::solve ()
 				 // have no such postprocessing here,
 				 // but we would like to write the
 				 // solution to a file.
-void LaplaceProblem::output_results () const
+void Step3::output_results () const
 {
 				   // To write the output to a file,
 				   // we need an object which knows
 				   // about output formats and the
-				   // like. This is the <code>DataOut</code> class,
+				   // like. This is the DataOut class,
 				   // and we need an object of that
 				   // type:
   DataOut<2> data_out;
 				   // Now we have to tell it where to take the
 				   // values from which it shall write. We
-				   // tell it which <code>DoFHandler</code> object to
+				   // tell it which DoFHandler object to
 				   // use, and the solution vector (and
 				   // the name by which the solution variable
 				   // shall appear in the output file). If
@@ -826,7 +831,7 @@ void LaplaceProblem::output_results () const
 				   // handle. The reason is that we
 				   // have separated the frontend
 				   // (which knows about how to treat
-				   // <code>DoFHandler</code> objects and data
+				   // DoFHandler objects and data
 				   // vectors) from the back end (which
 				   // knows many different output formats)
 				   // and use an intermediate data
@@ -849,19 +854,20 @@ void LaplaceProblem::output_results () const
 }
 
 
-                                 // @sect4{LaplaceProblem::run}
+                                 // @sect4{Step3::run}
 
 				 // Finally, the last function of this class
 				 // is the main function which calls all the
-				 // other functions of the <code>LaplaceProblem</code>
+				 // other functions of the <code>Step3</code>
 				 // class. The order in which this is done
 				 // resembles the order in which most finite
 				 // element programs work. Since the names are
 				 // mostly self-explanatory, there is not much
 				 // to comment about:
-void LaplaceProblem::run ()
+void Step3::run ()
 {
-  make_grid_and_dofs ();
+  make_grid ();
+  setup_system();
   assemble_system ();
   solve ();
   output_results ();
@@ -881,7 +887,7 @@ void LaplaceProblem::run ()
 				 // what is done here as well:
 int main ()
 {
-  LaplaceProblem laplace_problem;
+  Step3 laplace_problem;
   laplace_problem.run ();
 
   return 0;
