@@ -1,9 +1,9 @@
-/* $Id: step-5.cc 22321 2010-10-12 21:53:08Z kanschat $ */
+/* $Id: step-5.cc 24100 2011-08-16 16:25:07Z bangerth $ */
 /* Author: Wolfgang Bangerth, University of Heidelberg, 1999 */
 
-/*    $Id: step-5.cc 22321 2010-10-12 21:53:08Z kanschat $       */
+/*    $Id: step-5.cc 24100 2011-08-16 16:25:07Z bangerth $       */
 /*                                                                */
-/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2010 by the deal.II authors */
+/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2010, 2011 by the deal.II authors */
 /*                                                                */
 /*    This file is subject to QPL and may not be  distributed     */
 /*    without copyright and license information. Please refer     */
@@ -15,37 +15,37 @@
 				 // Again, the first few include files
 				 // are already known, so we won't
 				 // comment on them:
-#include <base/quadrature_lib.h>
-#include <base/function.h>
-#include <base/logstream.h>
-#include <lac/vector.h>
-#include <lac/full_matrix.h>
-#include <lac/sparse_matrix.h>
-#include <lac/compressed_sparsity_pattern.h>
-#include <lac/solver_cg.h>
-#include <lac/precondition.h>
-#include <grid/tria.h>
-#include <dofs/dof_handler.h>
-#include <grid/tria_accessor.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_accessor.h>
-#include <dofs/dof_tools.h>
-#include <fe/fe_q.h>
-#include <fe/fe_values.h>
-#include <numerics/vectors.h>
-#include <numerics/matrices.h>
-#include <numerics/data_out.h>
+#include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/function.h>
+#include <deal.II/base/logstream.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/compressed_sparsity_pattern.h>
+#include <deal.II/lac/solver_cg.h>
+#include <deal.II/lac/precondition.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/dofs/dof_tools.h>
+#include <deal.II/fe/fe_q.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/numerics/vectors.h>
+#include <deal.II/numerics/matrices.h>
+#include <deal.II/numerics/data_out.h>
 
 				 // This one is new. We want to read a
 				 // triangulation from disk, and the
 				 // class which does this is declared
 				 // in the following file:
-#include <grid/grid_in.h>
+#include <deal.II/grid/grid_in.h>
 
 				 // We will use a circular domain, and
 				 // the object describing the boundary
 				 // of it comes from this file:
-#include <grid/tria_boundary_lib.h>
+#include <deal.II/grid/tria_boundary_lib.h>
 
 				 // This is C++ ...
 #include <fstream>
@@ -61,7 +61,7 @@
 using namespace dealii;
 
 
-                                 // @sect3{The <code>LaplaceProblem</code> class template}
+                                 // @sect3{The <code>Step5</code> class template}
 
 				 // The main class is mostly as in the
 				 // previous example. The most visible
@@ -74,10 +74,10 @@ using namespace dealii;
 				 // <code>setup_system</code>. Apart from this,
 				 // everything is as before.
 template <int dim>
-class LaplaceProblem
+class Step5
 {
   public:
-    LaplaceProblem ();
+    Step5 ();
     void run ();
 
   private:
@@ -354,20 +354,20 @@ void Coefficient<dim>::value_list (const std::vector<Point<dim> > &points,
 }
 
 
-                                 // @sect3{The <code>LaplaceProblem</code> class implementation}
+                                 // @sect3{The <code>Step5</code> class implementation}
 
-                                 // @sect4{LaplaceProblem::LaplaceProblem}
+                                 // @sect4{Step5::Step5}
 
 				 // This function is as before.
 template <int dim>
-LaplaceProblem<dim>::LaplaceProblem () :
+Step5<dim>::Step5 () :
                 fe (1),
 		dof_handler (triangulation)
 {}
 
 
 
-                                 // @sect4{LaplaceProblem::setup_system}
+                                 // @sect4{Step5::setup_system}
 
 				 // This is the function
 				 // <code>make_grid_and_dofs</code> from the
@@ -375,7 +375,7 @@ LaplaceProblem<dim>::LaplaceProblem () :
 				 // generation of the grid. Everything
 				 // else is unchanged:
 template <int dim>
-void LaplaceProblem<dim>::setup_system ()
+void Step5<dim>::setup_system ()
 {
   dof_handler.distribute_dofs (fe);
 
@@ -395,7 +395,7 @@ void LaplaceProblem<dim>::setup_system ()
 
 
 
-                                 // @sect4{LaplaceProblem::assemble_system}
+                                 // @sect4{Step5::assemble_system}
 
 				 // As in the previous examples, this
 				 // function is not changed much with
@@ -429,7 +429,7 @@ void LaplaceProblem<dim>::setup_system ()
 				 // are completely unchanged from
 				 // before:
 template <int dim>
-void LaplaceProblem<dim>::assemble_system ()
+void Step5<dim>::assemble_system ()
 {
   QGauss<dim>  quadrature_formula(2);
 
@@ -581,7 +581,7 @@ void LaplaceProblem<dim>::assemble_system ()
 }
 
 
-                                 // @sect4{LaplaceProblem::solve}
+                                 // @sect4{Step5::solve}
 
 				 // The solution process again looks
 				 // mostly like in the previous
@@ -633,7 +633,7 @@ void LaplaceProblem<dim>::assemble_system ()
 				 // declared, and the CG solver will
 				 // do the rest for us:
 template <int dim>
-void LaplaceProblem<dim>::solve ()
+void Step5<dim>::solve ()
 {
   SolverControl           solver_control (1000, 1e-12);
   SolverCG<>              solver (solver_control);
@@ -650,7 +650,7 @@ void LaplaceProblem<dim>::solve ()
 }
 
 
-                                 // @sect4{LaplaceProblem::output_results and setting output flags}
+                                 // @sect4{Step5::output_results and setting output flags}
 
 				 // Writing output to a file is mostly
 				 // the same as for the previous
@@ -660,7 +660,7 @@ void LaplaceProblem<dim>::solve ()
 				 // filename for each refinement
 				 // cycle.
 template <int dim>
-void LaplaceProblem<dim>::output_results (const unsigned int cycle) const
+void Step5<dim>::output_results (const unsigned int cycle) const
 {
   DataOut<dim> data_out;
 
@@ -796,7 +796,7 @@ void LaplaceProblem<dim>::output_results (const unsigned int cycle) const
 
 
 
-                                 // @sect4{LaplaceProblem::run}
+                                 // @sect4{Step5::run}
 
 				 // The second to last thing in this
 				 // program is the definition of the
@@ -810,151 +810,117 @@ void LaplaceProblem<dim>::output_results (const unsigned int cycle) const
 				 // print the cycle number, and then
 				 // have to decide what to do with the
 				 // mesh. If this is not the first
-				 // cycle, we can simply refine the
-				 // existing mesh once globally. If
-				 // this is the first cycle, however,
-				 // we first have to generate a mesh:
+				 // cycle, we simply refine the
+				 // existing mesh once
+				 // globally. Before running through
+				 // these cycles, however,
+				 // we have to generate a mesh:
+
+				 // In previous examples, we have
+				 // already used some of the functions
+				 // from the
+				 // <code>GridGenerator</code>
+				 // class. Here we would like to read
+				 // a grid from a file where the cells
+				 // are stored and which may originate
+				 // from someone else, or may be the
+				 // product of a mesh generator tool.
+				 //
+				 // In order to read a grid from a
+				 // file, we generate an object of
+				 // data type GridIn and associate the
+				 // triangulation to it (i.e. we tell
+				 // it to fill our triangulation
+				 // object when we ask it to read the
+				 // file). Then we open the respective
+				 // file and initialize the
+				 // triangulation with the data in the
+				 // file:
 template <int dim>
-void LaplaceProblem<dim>::run ()
+void Step5<dim>::run ()
 {
+  GridIn<dim> grid_in;
+  grid_in.attach_triangulation (triangulation);
+  std::ifstream input_file("circle-grid.inp");
+				   // We would now like to read the
+				   // file. However, the input file is
+				   // only for a two-dimensional
+				   // triangulation, while this
+				   // function is a template for
+				   // arbitrary dimension. Since this
+				   // is only a demonstration program,
+				   // we will not use different input
+				   // files for the different
+				   // dimensions, but rather kill the
+				   // whole program if we are not in
+				   // 2D:
+  Assert (dim==2, ExcInternalError());
+				   // ExcInternalError is a globally
+				   // defined exception, which may be
+				   // thrown whenever something is
+				   // terribly wrong. Usually, one
+				   // would like to use more specific
+				   // exceptions, and particular in
+				   // this case one would of course
+				   // try to do something else if
+				   // <code>dim</code> is not equal to
+				   // two, e.g. create a grid using
+				   // library functions. Aborting a
+				   // program is usually not a good
+				   // idea and assertions should
+				   // really only be used for
+				   // exceptional cases which should
+				   // not occur, but might due to
+				   // stupidity of the programmer,
+				   // user, or someone else. The
+				   // situation above is not a very
+				   // clever use of Assert, but again:
+				   // this is a tutorial and it might
+				   // be worth to show what not to do,
+				   // after all.
+
+				   // So if we got past the assertion,
+				   // we know that dim==2, and we can
+				   // now actually read the grid. It
+				   // is in UCD (unstructured cell
+				   // data) format (but the ending of
+				   // the <code>UCD</code>-file is
+				   // <code>inp</code>), as supported
+				   // as input format by the AVS
+				   // Explorer (a visualization
+				   // program), for example:
+  grid_in.read_ucd (input_file);
+				   // If you like to use another input
+				   // format, you have to use an other
+				   // <code>grid_in.read_xxx</code>
+				   // function. (See the documentation
+				   // of the <code>GridIn</code> class
+				   // to find out what input formats
+				   // are presently supported.)
+
+				   // The grid in the file describes a
+				   // circle. Therefore we have to use
+				   // a boundary object which tells
+				   // the triangulation where to put
+				   // new points on the boundary when
+				   // the grid is refined. This works
+				   // in the same way as in the first
+				   // example. Note that the
+				   // HyperBallBoundary constructor
+				   // takes two parameters, the center
+				   // of the ball and the radius, but
+				   // that their default (the origin
+				   // and 1.0) are the ones which we
+				   // would like to use here.
+  static const HyperBallBoundary<dim> boundary;
+  triangulation.set_boundary (0, boundary);
+  
   for (unsigned int cycle=0; cycle<6; ++cycle)
     {
       std::cout << "Cycle " << cycle << ':' << std::endl;
 
       if (cycle != 0)
 	triangulation.refine_global (1);
-      else
-	{
-					   // If this is the first
-					   // round, then we have no
-					   // grid yet, and we will
-					   // create it here. In
-					   // previous examples, we
-					   // have already used some
-					   // of the functions from
-					   // the <code>GridGenerator</code>
-					   // class. Here we would
-					   // like to read a grid from
-					   // a file where the cells
-					   // are stored and which may
-					   // originate from someone
-					   // else, or may be the
-					   // product of a mesh
-					   // generator tool.
-					   //
-					   // In order to read a grid
-					   // from a file, we generate
-					   // an object of data type
-					   // GridIn and associate the
-					   // triangulation to it
-					   // (i.e. we tell it to fill
-					   // our triangulation object
-					   // when we ask it to read
-					   // the file). Then we open
-					   // the respective file and
-					   // initialize the
-					   // triangulation with the
-					   // data in the file:
-	  GridIn<dim> grid_in;
-	  grid_in.attach_triangulation (triangulation);
-	  std::ifstream input_file("circle-grid.inp");
-					   // We would now like to
-					   // read the file. However,
-					   // the input file is only
-					   // for a two-dimensional
-					   // triangulation, while
-					   // this function is a
-					   // template for arbitrary
-					   // dimension. Since this is
-					   // only a demonstration
-					   // program, we will not use
-					   // different input files
-					   // for the different
-					   // dimensions, but rather
-					   // kill the whole program
-					   // if we are not in 2D:
-	  Assert (dim==2, ExcInternalError());
-					   // ExcInternalError is a
-					   // globally defined
-					   // exception, which may be
-					   // thrown whenever
-					   // something is terribly
-					   // wrong. Usually, one
-					   // would like to use more
-					   // specific exceptions, and
-					   // particular in this case
-					   // one would of course try
-					   // to do something else if
-					   // <code>dim</code> is not equal to
-					   // two, e.g. create a grid
-					   // using library
-					   // functions. Aborting a
-					   // program is usually not a
-					   // good idea and assertions
-					   // should really only be
-					   // used for exceptional
-					   // cases which should not
-					   // occur, but might due to
-					   // stupidity of the
-					   // programmer, user, or
-					   // someone else. The
-					   // situation above is not a
-					   // very clever use of
-					   // Assert, but again: this
-					   // is a tutorial and it
-					   // might be worth to show
-					   // what not to do, after
-					   // all.
-
-					   // So if we got past the
-					   // assertion, we know that
-					   // dim==2, and we can now
-					   // actually read the
-					   // grid. It is in UCD
-					   // (unstructured cell data)
-					   // format (but the ending
-					   // of the <code>UCD</code>-file is
-					   // <code>inp</code>), as supported
-					   // as input format by the
-					   // AVS Explorer (a
-					   // visualization program),
-					   // for example:
-	  grid_in.read_ucd (input_file);
-                                           // If you like to use
-                                           // another input format,
-                                           // you have to use an other
-                                           // <code>grid_in.read_xxx</code>
-                                           // function. (See the
-                                           // documentation of the
-                                           // <code>GridIn</code> class to find
-                                           // out what input formats
-                                           // are presently
-                                           // supported.)
-
-					   // The grid in the file
-					   // describes a
-					   // circle. Therefore we
-					   // have to use a boundary
-					   // object which tells the
-					   // triangulation where to
-					   // put new points on the
-					   // boundary when the grid
-					   // is refined. This works
-					   // in the same way as in
-					   // the first example. Note
-					   // that the
-					   // HyperBallBoundary
-					   // constructor takes two
-					   // parameters, the center
-					   // of the ball and the
-					   // radius, but that their
-					   // default (the origin and
-					   // 1.0) are the ones which
-					   // we would like to use
-					   // here.
-	  static const HyperBallBoundary<dim> boundary;
-	  triangulation.set_boundary (0, boundary);
-	}
 
 				       // Now that we have a mesh for
 				       // sure, we write some output
@@ -986,7 +952,7 @@ int main ()
 {
   deallog.depth_console (0);
 
-  LaplaceProblem<2> laplace_problem_2d;
+  Step5<2> laplace_problem_2d;
   laplace_problem_2d.run ();
 
 				   // Finally, we have promised to
