@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: function.h 22155 2010-09-24 21:23:31Z kanschat $
-//    Version: $Name$
+//    $Id: function.h 23989 2011-08-02 09:47:19Z kronbichler $
 //
 //    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009, 2010 by the deal.II authors
 //
@@ -14,16 +13,15 @@
 #define __deal2__function_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/function_time.h>
-#include <base/subscriptor.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/function_time.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/tensor.h>
+#include <deal.II/base/point.h>
 #include <vector>
 
 DEAL_II_NAMESPACE_OPEN
-template <int dim> class Point;
-template <int rank_, int dim> class Tensor;
-template <int dim> class Tensor<1,dim>;
 template <typename number> class Vector;
 
 /**
@@ -110,7 +108,7 @@ class Function : public FunctionTime,
 				      * template programming.
 				      */
     static const unsigned int dimension = dim;
-    
+
     				     /**
 				      * Number of vector components.
 				      */
@@ -127,7 +125,7 @@ class Function : public FunctionTime,
 				      */
     Function (const unsigned int n_components = 1,
 	      const double       initial_time = 0.0);
-    
+
 				     /**
 				      * Virtual destructor; absolutely
 				      * necessary in this case.
@@ -183,7 +181,7 @@ class Function : public FunctionTime,
                                       * to.
                                       */
     Function & operator= (const Function &f);
-    
+
 				     /**
 				      * Return the value of the
 				      * function at the given
@@ -209,7 +207,7 @@ class Function : public FunctionTime,
 				      */
     virtual void vector_value (const Point<dim>   &p,
                                Vector<double>     &values) const;
-    
+
 				     /**
 				      * Set <tt>values</tt> to the point
 				      * values of the specified
@@ -246,7 +244,7 @@ class Function : public FunctionTime,
 				      */
     virtual void vector_value_list (const std::vector<Point<dim> > &points,
 				    std::vector<Vector<double> >   &values) const;
-    
+
 				     /**
 				      * For each component of the
 				      * function, fill a vector of
@@ -278,7 +276,7 @@ class Function : public FunctionTime,
 				      */
     virtual void vector_gradient (const Point<dim>            &p,
                                   std::vector<Tensor<1,dim> > &gradients) const;
-    
+
 				     /**
 				      * Set <tt>gradients</tt> to the
 				      * gradients of the specified
@@ -291,7 +289,7 @@ class Function : public FunctionTime,
     virtual void gradient_list (const std::vector<Point<dim> > &points,
 				std::vector<Tensor<1,dim> >    &gradients,
 				const unsigned int              component = 0) const;
-    
+
 				     /**
 				      * For each component of the
 				      * function, fill a vector of
@@ -307,12 +305,12 @@ class Function : public FunctionTime,
 				      */
     virtual void vector_gradients (const std::vector<Point<dim> >            &points,
 				   std::vector<std::vector<Tensor<1,dim> > > &gradients) const;
-      
+
 				     /**
 				      * Set <tt>gradients</tt> to the gradients of
 				      * the function at the <tt>points</tt>,
 				      * for all components.
-				      * It is assumed that <tt>gradients</tt> 
+				      * It is assumed that <tt>gradients</tt>
 				      * already has the right size, i.e.
 				      * the same size as the <tt>points</tt> array.
 				      *
@@ -339,7 +337,7 @@ class Function : public FunctionTime,
 				      */
     virtual void vector_laplacian (const Point<dim>   &p,
 				   Vector<double>     &values) const;
-    
+
 				     /**
 				      * Compute the Laplacian of one
 				      * component at a set of points.
@@ -371,7 +369,7 @@ class Function : public FunctionTime,
 				      * quite close to the true
 				      * value.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 };
 
 
@@ -397,13 +395,13 @@ class ZeroFunction : public Function<dim>
 				      * components is preset to one.
 				      */
     ZeroFunction (const unsigned int n_components = 1);
-    
+
 				     /**
 				      * Virtual destructor; absolutely
 				      * necessary in this case.
 				      */
     virtual ~ZeroFunction ();
-    
+
 				     /**
 				      * Return the value of the function
 				      * at the given point for one
@@ -444,7 +442,7 @@ class ZeroFunction : public Function<dim>
 				      */
     virtual void vector_value_list (const std::vector<Point<dim> > &points,
 				    std::vector<Vector<double> >   &values) const;
-    
+
 				     /**
 				      * Return the gradient of the function
 				      * at the given point, for the
@@ -461,7 +459,7 @@ class ZeroFunction : public Function<dim>
 				      */
     virtual void vector_gradient (const Point<dim>            &p,
                                   std::vector<Tensor<1,dim> > &gradients) const;
-    
+
 				     /**
 				      * Set <tt>gradients</tt> to the gradients of
 				      * the function at the <tt>points</tt>,
@@ -473,12 +471,12 @@ class ZeroFunction : public Function<dim>
     virtual void gradient_list (const std::vector<Point<dim> > &points,
 				std::vector<Tensor<1,dim> >    &gradients,
 				const unsigned int              component = 0) const;
-    
+
 				     /**
 				      * Set <tt>gradients</tt> to the gradients of
 				      * the function at the <tt>points</tt>,
 				      * for all components.
-				      * It is assumed that <tt>gradients</tt> 
+				      * It is assumed that <tt>gradients</tt>
 				      * already has the right size, i.e.
 				      * the same size as the <tt>points</tt> array.
 				      *
@@ -528,7 +526,7 @@ class ConstantFunction : public ZeroFunction<dim>
 				      */
     ConstantFunction (const double       value,
 		      const unsigned int n_components = 1);
-    
+
 				     /**
 				      * Virtual destructor; absolutely
 				      * necessary in this case.
@@ -592,7 +590,7 @@ class ConstantFunction : public ZeroFunction<dim>
 				      * quite close to the true
 				      * value.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
   protected:
 				     /**
@@ -657,8 +655,8 @@ class ComponentSelectFunction : public ConstantFunction<dim>
 				      * number of vector components.
 				      */
     ComponentSelectFunction (const std::pair<unsigned int, unsigned int> &selected,
-			     const unsigned int n_components);    
-    
+			     const unsigned int n_components);
+
     				     /**
 				      * Return the value of the function
 				      * at the given point for all
@@ -695,7 +693,7 @@ class ComponentSelectFunction : public ConstantFunction<dim>
 				      * quite close to the true
 				      * value.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
   protected:
 				     /**
