@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: function_lib.h 22155 2010-09-24 21:23:31Z kanschat $
+//    $Id: function_lib.h 23709 2011-05-17 04:34:08Z bangerth $
 //
 //    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
@@ -13,9 +13,9 @@
 #define __deal2__function_lib_h
 
 
-#include <base/config.h>
-#include <base/function.h>
-#include <base/point.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/function.h>
+#include <deal.II/base/point.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -599,7 +599,7 @@ namespace Functions
 					* quite close to the true
 					* value.
 					*/
-      unsigned int memory_consumption () const;
+      std::size_t memory_consumption () const;
       
     protected:
 				       /**
