@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: index_set.h 23073 2010-12-27 19:28:39Z heister $
-//    Version: $Name$
+//    $Id: index_set.h 23944 2011-07-12 12:23:28Z bangerth $
 //
-//    Copyright (C) 2009, 2010 by the deal.II authors
+//    Copyright (C) 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,8 +12,9 @@
 #ifndef __deal2__index_set_h
 #define __deal2__index_set_h
 
-#include <base/config.h>
-#include <base/exceptions.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/base/exceptions.h>
 
 #include <vector>
 #include <algorithm>
@@ -289,7 +289,6 @@ class IndexSet
 				      */
     void block_read(std::istream & in);
 
-
 #ifdef DEAL_II_USE_TRILINOS
 				     /**
 				      * Given an MPI communicator,
@@ -358,16 +357,16 @@ class IndexSet
                                       * consumption (in bytes) of this
                                       * object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
     DeclException1 (ExcIndexNotPresent, int,
 		    << "The global index " << arg1
 		    << " is not an element of this set.");
 		    
-                         /**
-                      * Write or read the data of this object to or 
-                      * from a stream for the purpose of serialization
-                      */ 
+    /**
+     * Write or read the data of this object to or 
+     * from a stream for the purpose of serialization
+     */ 
     template <class Archive>
     void serialize (Archive & ar, const unsigned int version);
 
@@ -394,6 +393,23 @@ class IndexSet
 
 	unsigned int nth_index_in_set;
 
+	/**
+	 * Default constructor. Since there is no useful choice for
+	 * a default constructed interval, this constructor simply
+	 * creates something that resembles an invalid range. We
+	 * need this constructor for serialization purposes, but the
+	 * invalid range should be filled with something read from
+	 * the archive before it is used, so we should hopefully
+	 * never get to see an invalid range in the wild.
+	 **/
+	Range ();
+	
+	/**
+	 * Constructor. Create a half-open interval with the given indices.
+	 *
+	 * @param i1 Left end point of the interval.
+	 * @param i2 First index greater than the last index of the indicated range.
+	 **/
 	Range (const unsigned int i1,
 	       const unsigned int i2);
 
@@ -429,18 +445,17 @@ class IndexSet
 		    (range_1.begin == range_2.begin));
 	  }
 
-	unsigned int memory_consumption () const
-	  {
-	    return sizeof(Range);
-	  }
+	std::size_t memory_consumption () const
+	{
+	  return sizeof(Range);
+	}
 	  
-                         /**
-                      * Write or read the data of this object to or 
-                      * from a stream for the purpose of serialization
-                      */ 
-    template <class Archive>
-    void serialize (Archive & ar, const unsigned int version);
-
+	  /**
+	   * Write or read the data of this object to or 
+	   * from a stream for the purpose of serialization
+	   */ 
+	  template <class Archive>
+	  void serialize (Archive & ar, const unsigned int version);
     };
 
 				     /**
@@ -509,6 +524,14 @@ class IndexSet
 
 /* ------------------ inline functions ------------------ */
 
+inline
+IndexSet::Range::Range ()
+		:
+		begin(numbers::invalid_unsigned_int),
+		end(numbers::invalid_unsigned_int)
+{}
+
+
 inline
 IndexSet::Range::Range (const unsigned int i1,
 			const unsigned int i2)
@@ -518,7 +541,6 @@ IndexSet::Range::Range (const unsigned int i1,
 {}
 
 
-
 inline
 IndexSet::IndexSet ()
 		:
@@ -597,7 +619,7 @@ IndexSet::add_range (const unsigned int begin,
       if (ranges.size() == 0 || begin > ranges.back().end)
 	ranges.push_back(new_range);
       else
-	ranges.insert (std::lower_bound (ranges.begin(),
+	ranges.insert (Utilities::lower_bound (ranges.begin(),
 					 ranges.end(),
 					 new_range),
 		       new_range);
@@ -620,7 +642,7 @@ IndexSet::add_index (const unsigned int index)
   else if (index == ranges.back().end)
     ranges.back().end++;
   else
-    ranges.insert (std::lower_bound (ranges.begin(),
+    ranges.insert (Utilities::lower_bound (ranges.begin(),
 				     ranges.end(),
 				     new_range),
 		   new_range);
@@ -803,7 +825,7 @@ IndexSet::nth_index_in_set (const unsigned int n) const
     }
 
   std::vector<Range>::const_iterator 
-    p = std::lower_bound(range_begin, range_end, r,
+    p = Utilities::lower_bound(range_begin, range_end, r,
 			 Range::nth_index_compare);
 
   if (p != ranges.end())
@@ -849,7 +871,7 @@ IndexSet::index_within_set (const unsigned int n) const
     }
 
   std::vector<Range>::const_iterator 
-    p = std::lower_bound(range_begin, range_end, r,
+    p = Utilities::lower_bound(range_begin, range_end, r,
 			 Range::end_compare);
 
   Assert(p!=ranges.end(), ExcInternalError());
@@ -909,6 +931,8 @@ IndexSet::print (STREAM &out) const
   out << "}" << std::endl;
 }
 
+
+
 template <class Archive>
 inline
 void
@@ -916,7 +940,9 @@ IndexSet::Range::serialize (Archive & ar, const unsigned int)
 {
   ar & begin & end & nth_index_in_set;
 }
-   
+
+
+
 template <class Archive>
 inline
 void
