@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: parameter_handler.h 23081 2010-12-28 17:56:06Z bangerth $
-//    Version: $Name$
+//    $Id: parameter_handler.h 23876 2011-06-28 18:21:51Z kanschat $
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,10 +13,10 @@
 #define __deal2__parameter_handler_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/subscriptor.h>
-#include <base/std_cxx1x/shared_ptr.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/std_cxx1x/shared_ptr.h>
 
 #include <boost/property_tree/ptree_fwd.hpp>
 #include <boost/serialization/split_member.hpp>
@@ -113,7 +112,7 @@ namespace Patterns
 					* bytes) of this object. To
 					* avoid unnecessary
 					* overhead, we do not force
-					* derivd classes to provide
+					* derived classes to provide
 					* this function as a virtual
 					* overloaded one, but rather
 					* try to cast the present
@@ -144,7 +143,7 @@ namespace Patterns
 					* you can still overload
 					* this function.
 					*/
-      virtual unsigned int memory_consumption () const;
+      virtual std::size_t memory_consumption () const;
   };
 
                        /**
@@ -507,7 +506,7 @@ namespace Patterns
 					* the memory consumption (in
 					* bytes) of this object.
 					*/
-      unsigned int memory_consumption () const;
+      std::size_t memory_consumption () const;
 
 				       /**
 					* Creates new object if the start of
@@ -623,7 +622,7 @@ namespace Patterns
 					* the memory consumption (in
 					* bytes) of this object.
 					*/
-      unsigned int memory_consumption () const;
+      std::size_t memory_consumption () const;
 
 				       /** @addtogroup Exceptions
 					* @{ */
@@ -738,7 +737,7 @@ namespace Patterns
 					* the memory consumption (in
 					* bytes) of this object.
 					*/
-      unsigned int memory_consumption () const;
+      std::size_t memory_consumption () const;
 
 				       /** @addtogroup Exceptions
 					* @{ */
@@ -778,7 +777,7 @@ namespace Patterns
   {
     public:
 				       /**
-					* Constrcuctor.
+					* Constructor.
 					*/
       Bool ();
 
@@ -1044,10 +1043,10 @@ namespace Patterns
  *   which provides at run-time for program parameters such as time step
  *   sizes, geometries, right hand sides etc. The input for the program is
  *   given in files, streams or strings in memory using text like
- *   @verbatim
+ *   @code
  *     set Time step size = 0.3
  *     set Geometry       = [0,1]x[0,3]
- *   @endverbatim
+ *   @endcode
  *   Input may be sorted into subsection trees in order to give the input a
  *   logical structure.
  *
@@ -1165,21 +1164,22 @@ namespace Patterns
  *   <h3>Input files and special characters</h3>
  *
  *   For the first example above the input file would look like the following:
- *   @verbatim
+ *   @code
  *     ...
  *     subsection Nonlinear solver
  *       set Nonlinear method = Gradient
+ *       # this is a comment
  *       subsection Linear solver
  *         set Solver                        = CG
  *         set Maxmimum number of iterations = 30
  *       end
  *     end
  *     ...                       # other stuff
- *   @endverbatim
+ *   @endcode
  *   The words <tt>subsection</tt>, <tt>set</tt> and <tt>end</tt> may be either written in lowercase or uppercase
  *   letters. Leading and trailing whitespace is removed, multiple whitespace is condensed into
  *   only one. Since the latter applies also to the name of an entry, an entry name will not
- *   be recognised if in the declaration multiple whitespace is used.
+ *   be recognized if in the declaration multiple whitespace is used.
  *
  *   In entry names and values the following characters are not allowed: <tt>\#</tt>, <tt>{</tt>,
  *   <tt>}</tt>, <tt>|</tt>. Their use is reserved for the MultipleParameterLoop class.
@@ -1197,7 +1197,7 @@ namespace Patterns
  *   In order to read input you can use three possibilities: reading from an
  *   <tt>std::istream</tt> object, reading from a file of which the name is
  *   given and reading from a string in memory in which the lines are
- *   separated by <tt>@\n</tt> characters. These possibilites are used as
+ *   separated by <tt>@\n</tt> characters. These possibilities are used as
  *   follows:
  *   @code
  *     ParameterHandler prm;
@@ -1466,7 +1466,7 @@ namespace Patterns
  *
  *
  *   This is the input file (named "prmtest.prm"):
- *   @verbatim
+ *   @code
  *                                 # first declare the types of equations
  *     set Equation 1 = Poisson
  *     set Equation 2 = Navier-Stokes
@@ -1486,10 +1486,10 @@ namespace Patterns
  *         set Maximum number of iterations = 100
  *       end
  *     end
- *   @endverbatim
+ *   @endcode
  *
- *   And here is the ouput of the program:
- *   @verbatim
+ *   And here is the output of the program:
+ *   @code
  *     Line 8:
  *         The entry value
  *             Gauss-Seidel
@@ -1526,7 +1526,7 @@ namespace Patterns
  *       Problem: outfile=out
  *                eq1=Poisson, eq2=Navier-Stokes
  *                Matrix1=Sparse, Matrix2=Full
- *   @endverbatim
+ *   @endcode
  *
  *
  *
@@ -1644,7 +1644,7 @@ namespace Patterns
  *
  *
  *   @ingroup input
- *   @author Wolfgang Bangerth, October 1997, revised February 1998, 2010
+ *   @author Wolfgang Bangerth, October 1997, revised February 1998, 2010, 2011
  */
 class ParameterHandler : public Subscriptor
 {
@@ -1695,7 +1695,7 @@ class ParameterHandler : public Subscriptor
 
 					   /**
 					    * Write out everything as
-					    * an <a href="http://en.wikipedia.org/wiki/XML>XML</a>
+					    * an <a href="http://en.wikipedia.org/wiki/XML">XML</a>
 					    * file.
 					    *
 					    * See the general
@@ -1867,7 +1867,7 @@ class ParameterHandler : public Subscriptor
 				      * <tt>entry_string</tt> as <tt>long
 				      * int</tt>. (A long int is chosen so
 				      * that even very large unsigned values
-				      * can be returned by this function.)
+				      * can be returned by this function).
 				      */
     long int       get_integer (const std::string &entry_string) const;
 
@@ -2080,19 +2080,21 @@ class ParameterHandler : public Subscriptor
 				      * bytes) of this
 				      * object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
-                         /**
-                      * Write the data of this object to
-                      * a stream for the purpose of serialization.
-                      */
+				     /**
+				      * Write the data of this object to a
+				      * stream for the purpose of
+				      * serialization.
+				      */
    template <class Archive>
    void save (Archive & ar, const unsigned int version) const;
 
-                         /**
-                      * Read the data of this object
-                      * from a stream for the purpose of serialization.
-                      */
+				     /**
+				      * Read the data of this object from a
+				      * stream for the purpose of
+				      * serialization.
+				      */
    template <class Archive>
    void load (Archive & ar, const unsigned int version);
 
@@ -2530,7 +2532,7 @@ class MultipleParameterLoop : public ParameterHandler
 				      * bytes) of this
 				      * object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
   private:
 
@@ -2608,7 +2610,7 @@ class MultipleParameterLoop : public ParameterHandler
 					  * bytes) of this
 					  * object.
 					  */
-	unsigned int memory_consumption () const;
+	std::size_t memory_consumption () const;
     };
 
 				     /**
