@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: polynomial.h 22928 2010-12-06 18:44:56Z rao $
-//    Version: $Name$
+//    $Id: polynomial.h 23974 2011-07-27 14:10:41Z kronbichler $
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,15 +14,15 @@
 
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/subscriptor.h>
-#include <base/std_cxx1x/shared_ptr.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/point.h>
+#include <deal.II/base/std_cxx1x/shared_ptr.h>
 
 #include <vector>
 
 DEAL_II_NAMESPACE_OPEN
-template <int dim> class Point;
 
 /**
  * @addtogroup Polynomials
@@ -75,6 +74,19 @@ namespace Polynomials
                                         */
       Polynomial (const unsigned int n);
 
+				/**
+				 * Constructor for Lagrange polynomial and its
+				 * point of evaluation. The idea is to
+				 * construct $\prod_{i\neq j}
+				 * \frac{x-x_i}{x_j-x_i}$, where j is the
+				 * evaluation point specified as argument and
+				 * the support points contain all points
+				 * (including x_j, which will internally not
+				 * be stored).
+				 */
+    Polynomial (const std::vector<Point<1> > &lagrange_support_points,
+		const unsigned int            evaluation_point);
+
                                        /**
                                         * Default constructor creating
                                         * an illegal object.
@@ -204,16 +216,16 @@ namespace Polynomials
 				                        *  Test for equality of two polynomials.
 				                        */
       bool operator == (const Polynomial<number> & p)  const;
-    
+
                                        /**
                                         * Print coefficients.
                                         */
       void print(std::ostream& out) const;
-      
+
                                        /**
-                                        * Write or read the data of this object to or 
+                                        * Write or read the data of this object to or
                                         * from a stream for the purpose of serialization.
-                                        */ 
+                                        */
       template <class Archive>
       void serialize (Archive & ar, const unsigned int version);
 
@@ -240,6 +252,15 @@ namespace Polynomials
       static void multiply (std::vector<number>& coefficients,
                             const number factor);
 
+				       /**
+					* Transforms polynomial form of
+					* product of linear factors into
+					* standard form, $\sum_i a_i
+					* x^i$. Deletes all data structures
+					* related to the product form.
+					*/
+      void transform_into_standard_form ();
+
                                        /**
                                         * Coefficients of the polynomial
                                         * $\sum_i a_i x^i$. This vector
@@ -253,6 +274,30 @@ namespace Polynomials
                                         * of polynomials.
                                         */
       std::vector<number> coefficients;
+
+				       /**
+					* Stores whether the polynomial is in
+					* Lagrange product form, i.e.,
+					* constructed as a product $(x-x_0)
+					* (x-x_1) \ldots (x-x_n)/c$, or not.
+					*/
+      bool in_lagrange_product_form;
+
+				       /**
+					* If the polynomial is in Lagrange
+					* product form, i.e., constructed as a
+					* product $(x-x_0) (x-x_1) \ldots
+					* (x-x_n)/c$, store the shifts $x_i$.
+					*/
+      std::vector<number> lagrange_support_points;
+
+				       /**
+					* If the polynomial is in Lagrange
+					* product form, i.e., constructed as a
+					* product $(x-x_0) (x-x_1) \ldots
+					* (x-x_n)/c$, store the weight c.
+					*/
+      number lagrange_weight;
   };
 
 
@@ -387,14 +432,16 @@ namespace Polynomials
 
 
 /**
- * Legendre polynomials of arbitrary degree on <tt>[0,1]</tt>.
- *
+ * Legendre polynomials of arbitrary degree.
  * Constructing a Legendre polynomial of degree <tt>p</tt>, the coefficients
- * will be computed by the three-term recursion formula.  The
- * coefficients are stored in a static data vector to be available
- * when needed next time. Since the recursion is performed for the
- * interval $[-1,1]$, the polynomials are shifted to $[0,1]$ by the
- * <tt>scale</tt> and <tt>shift</tt> functions of <tt>Polynomial</tt>, afterwards.
+ * will be computed by the three-term recursion formula.
+ *
+ * @note The polynomials defined by this class differ in two aspects by what
+ * is usually referred to as Legendre polynomials: (i) This classes defines
+ * them on the reference interval $[0,1]$, rather than the commonly used
+ * interval $[-1,1]$. (ii) The polynomials have been scaled in such a way that
+ * they are orthonormal, not just orthogonal; consequently, the polynomials do
+ * not necessarily have boundary values equal to one.
  *
  * @author Guido Kanschat, 2000
  */
@@ -446,6 +493,15 @@ namespace Polynomials
 
                                        /**
                                         * Compute coefficients recursively.
+					* The coefficients are stored in a
+					* static data vector to be available
+					* when needed next time. Since the
+					* recursion is performed for the
+					* interval $[-1,1]$, the polynomials
+					* are shifted to $[0,1]$ by the
+					* <tt>scale</tt> and <tt>shift</tt>
+					* functions of <tt>Polynomial</tt>,
+					* afterwards.
                                         */
       static void compute_coefficients (const unsigned int p);
 
@@ -510,6 +566,7 @@ namespace Polynomials
   };
 
 
+
 /**
  * Hierarchical polynomials of arbitrary degree on <tt>[0,1]</tt>.
  *
@@ -600,10 +657,25 @@ namespace Polynomials
      static const std::vector<double> &
      get_coefficients (const unsigned int p);
 
-     static std::vector<const std::vector<double> *> recursive_coefficients;
+                                       /**
+                                        * Vector with already computed
+                                        * coefficients. For each degree of the
+                                        * polynomial, we keep one pointer to
+                                        * the list of coefficients; we do so
+                                        * rather than keeping a vector of
+                                        * vectors in order to simplify
+                                        * programming multithread-safe. In
+                                        * order to avoid memory leak, we use a
+                                        * shared_ptr in order to correctly
+                                        * free the memory of the vectors when
+                                        * the global destructor is called.
+                                        */
+    static std::vector<std_cxx1x::shared_ptr<const std::vector<double> > > recursive_coefficients;
    };
 }
 
+
+
 /** @} */
 
 /* -------------------------- inline functions --------------------- */
@@ -613,15 +685,27 @@ namespace Polynomials
   template <typename number>
   inline
   Polynomial<number>::Polynomial ()
+    :
+    in_lagrange_product_form (false),
+    lagrange_weight          (1.)
   {}
 
+
+
   template <typename number>
   inline
   unsigned int
   Polynomial<number>::degree () const
   {
-    Assert (coefficients.size()>0, ExcEmptyObject());
-    return coefficients.size() - 1;
+    if (in_lagrange_product_form == true)
+      {
+	return lagrange_support_points.size();
+      }
+    else
+      {
+	Assert (coefficients.size()>0, ExcEmptyObject());
+	return coefficients.size() - 1;
+      }
   }
 
 
@@ -631,17 +715,29 @@ namespace Polynomials
   number
   Polynomial<number>::value (const number x) const
   {
-    Assert (coefficients.size() > 0, ExcEmptyObject());
-    const unsigned int m=coefficients.size();
+    if (in_lagrange_product_form == false)
+      {
+	Assert (coefficients.size() > 0, ExcEmptyObject());
 
                                      // Horner scheme
-    number value = coefficients.back();
-    for (int k=m-2; k>=0; --k)
-      value = value*x + coefficients[k];
-
-    return value;
+	const unsigned int m=coefficients.size();
+	number value = coefficients.back();
+	for (int k=m-2; k>=0; --k)
+	  value = value*x + coefficients[k];
+	return value;
+      }
+    else
+      {
+				// direct evaluation of Lagrange polynomial
+	const unsigned int m = lagrange_support_points.size();
+	number value = 1.;
+	for (unsigned int j=0; j<m; ++j)
+	  value *= x-lagrange_support_points[j];
+	value *= lagrange_weight;
+	return value;
+      }
   }
-  
+
 
 
   template <typename number>
@@ -652,8 +748,11 @@ namespace Polynomials
   {
                                      // forward to serialization
                                      // function in the base class.
-    ar &  static_cast<Subscriptor &>(*this);
+    ar & static_cast<Subscriptor &>(*this);
     ar & coefficients;
+    ar & in_lagrange_product_form;
+    ar & lagrange_support_points;
+    ar & lagrange_weight;
   }
 
 }
