@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: table_indices.h 22928 2010-12-06 18:44:56Z rao $
-//    Version: $Name$
+//    $Id: table_indices.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2005, 2006, 2009 by the deal.II authors
 //
@@ -14,8 +13,8 @@
 #define __deal2__table_indices_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
 
 // we only need output streams, but older compilers did not provide
 // them in a separate include file
