@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: tensor_product_polynomials.h 22678 2010-11-10 15:35:22Z kronbichler $
-//    Version: $Name$
+//    $Id: tensor_product_polynomials.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009 by the deal.II authors
 //
@@ -14,11 +13,11 @@
 #define __deal2__tensor_product_polynomials_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/tensor.h>
-#include <base/point.h>
-#include <base/polynomial.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/tensor.h>
+#include <deal.II/base/point.h>
+#include <deal.II/base/polynomial.h>
 
 #include <vector>
 
