@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: utilities.h 22678 2010-11-10 15:35:22Z kronbichler $
-//    Version: $Name$
+//    $Id: utilities.h 24435 2011-09-26 18:50:03Z bangerth $
 //
-//    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,21 +12,21 @@
 #ifndef __deal2__utilities_h
 #define __deal2__utilities_h
 
-#include <base/config.h>
-#include <base/exceptions.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
 
 #include <vector>
 #include <utility>
+#include <functional>
 #include <string>
 
 #if defined(DEAL_II_COMPILER_SUPPORTS_MPI) || defined(DEAL_II_USE_PETSC)
 #include <mpi.h>
 #else
-typedef int MPI_Comm;
+  typedef int MPI_Comm;
 #endif
 
 #ifdef DEAL_II_USE_TRILINOS
-#  include <Teuchos_RCP.hpp>
 #  include <Epetra_Comm.h>
 #  include <Epetra_Map.h>
 #  ifdef DEAL_II_COMPILER_SUPPORTS_MPI
@@ -91,6 +90,23 @@ namespace Utilities
   std::vector<int>
   string_to_int (const std::vector<std::string> &s);
 
+                                   /**
+                                    * Given a string, convert it to an
+                                    * double. Throw an assertion if that is
+                                    * not possible.
+                                    */
+  double
+  string_to_double (const std::string &s);
+
+
+                                   /**
+                                    * Given a list of strings, convert it to a
+                                    * list of doubles. Throw an assertion if
+                                    * that is not possible.
+                                    */
+  std::vector<double>
+  string_to_double (const std::vector<std::string> &s);
+
                                    /**
                                     * Given a string that contains text
                                     * separated by a @p delimiter, split it into
@@ -174,6 +190,68 @@ namespace Utilities
   T
   fixed_power (const T t);
 
+				   /**
+				    * Optimized replacement for
+				    * <tt>std::lower_bound</tt> for
+				    * searching within the range of
+				    * column indices. Slashes
+				    * execution time by
+				    * approximately one half for the
+				    * present application, partly
+				    * because because the
+				    * binary search is replaced by a
+				    * linear search for small loop
+				    * lengths.
+				    *
+				    * Another reason for this function is
+				    * rather obscure:
+				    * when using the GCC libstdc++
+				    * function std::lower_bound,
+				    * complexity is O(log(N)) as
+				    * required. However, when using the
+				    * debug version of the GCC libstdc++
+				    * as we do when running the testsuite,
+				    * then std::lower_bound tests whether
+				    * the sequence is in fact partitioned
+				    * with respect to the pivot 'value'
+				    * (i.e. in essence that the sequence
+				    * is sorted as required for binary
+				    * search to work). However, verifying
+				    * this means that the complexity of
+				    * std::lower_bound jumps to O(N); we
+				    * call this function O(N) times below,
+				    * making the overall complexity
+				    * O(N**2). The consequence is that a
+				    * few tests with big meshes completely
+				    * run off the wall time limit for
+				    * tests and fail with the libstdc++
+				    * debug mode
+				    *
+				    * This function simply makes the
+				    * assumption that the sequence is
+				    * sorted, and we simply don't do the
+				    * additional check.
+				    */
+  template<typename Iterator, typename T>
+  Iterator
+  lower_bound (Iterator  first,
+	       Iterator  last,
+	       const T  &val);
+
+
+				   /**
+				    * The same function as above, but taking
+				    * an argument that is used to compare
+				    * individual elements of the sequence of
+				    * objects pointed to by the iterators.
+				    */
+  template<typename Iterator, typename T, typename Comp>
+  Iterator
+  lower_bound (Iterator   first,
+	       Iterator   last,
+	       const T   &val,
+	       const Comp comp);
+
 				   /**
 				    * Given a permutation vector (i.e. a
 				    * vector $p_0\ldots p_{N-1}$ where each
@@ -197,85 +275,23 @@ namespace Utilities
 
                                    /**
                                     * A namespace for utility functions that
-                                    * probe system properties.
+                                    * abstract certain operations using the
+				    * Message Passing Interface (MPI) or
+				    * provide fallback operations in
+				    * case deal.II is configured not to use
+				    * MPI at all.
                                     *
                                     * @ingroup utilities
                                     */
-  namespace System
+  namespace MPI
   {
-
-                                     /**
-                                      * Return the CPU load as returned by
-                                      * "uptime". Note that the interpretation
-                                      * of this number depends on the actual
-                                      * number of processors in the
-                                      * machine. This is presently only
-                                      * implemented on Linux, using the
-                                      * /proc/loadavg pseudo-file, on other
-                                      * systems we simply return zero.
-                                      */
-    double get_cpu_load ();
-
-				     /**
-				      * Structure that hold information about
-				      * memory usage in kB. Used by
-				      * get_memory_stats(). See man 5 proc
-				      * entry /status for details.
-				      */
-    struct MemoryStats
-    {
-	unsigned long int VmPeak; /** peak virtual memory size in kB */
-	unsigned long int VmSize; /** current virtual memory size in kB */
-	unsigned long int VmHWM; /** peak resident memory size in kB */
-	unsigned long int VmRSS; /** current resident memory size in kB */
-    };
-    
-    
-				     /**
-				      * Fills the @param stats structure with
-				      * information about the memory
-				      * consumption of this process. This is
-				      * only implemented on Linux.
-				      */
-    void get_memory_stats (MemoryStats & stats);
-    
-
-                                     /**
-                                      * Return the name of the host this
-                                      * process runs on.
-                                      */
-    std::string get_hostname ();
-
-
-                                     /**
-                                      * Return the present time as HH:MM:SS.
-                                      */
-    std::string get_time ();
-
-				     /**
-				      * Return whether (i) deal.II has
-				      * been compiled to support MPI
-				      * (for example by compiling with
-				      * <code>CXX=mpiCC</code>) and if
-				      * so whether (ii)
-				      * <code>MPI_Init()</code> has
-				      * been called (for example using
-				      * the
-				      * Utilities::System::MPI_InitFinalize
-				      * class). In other words, the
-				      * result indicates whether the
-				      * current job is running under
-				      * MPI.
-				      */
-    bool job_supports_mpi ();
-
 				     /**
 				      * Return the number of MPI processes
 				      * there exist in the given communicator
 				      * object. If this is a sequential job,
 				      * it returns 1.
 				      */
-    unsigned int get_n_mpi_processes (const MPI_Comm &mpi_communicator);
+    unsigned int n_mpi_processes (const MPI_Comm &mpi_communicator);
 
 				     /**
 				      * Return the number of the present MPI
@@ -287,8 +303,7 @@ namespace Utilities
 				      * processes (given by
 				      * get_n_mpi_processes()).
 				      */
-    unsigned int get_this_mpi_process (const MPI_Comm &mpi_communicator);
-
+    unsigned int this_mpi_process (const MPI_Comm &mpi_communicator);
 
 				     /**
 				      * Consider an unstructured
@@ -332,8 +347,6 @@ namespace Utilities
     compute_point_to_point_communication_pattern (const MPI_Comm & mpi_comm,
 						  const std::vector<unsigned int> & destinations);
 
-
-
 				     /**
 				      * Given a communicator, generate a new
 				      * communicator that contains the same
@@ -354,10 +367,73 @@ namespace Utilities
 				      */
     MPI_Comm duplicate_communicator (const MPI_Comm &mpi_communicator);
 
+    /**
+     * Return the sum over all processors of the value @p t. This function
+     * is collective over all processors given in the communicator. If
+     * deal.II is not configured for use of MPI, this function simply
+     * returns the value of @p t. This function corresponds to the
+     * <code>MPI_Allreduce</code> function, i.e. all processors receive
+     * the result of this operation.
+     *
+     * @note This function is only implemented for certain template
+     * arguments <code>T</code>, namely <code>float, double, int,
+     * unsigned int</code>.
+     */
+    template <typename T>
+    T sum (const T &t,
+	   const MPI_Comm &mpi_communicator);
+
+				     /**
+				      * Like the previous function,
+				      * but take the sums over the
+				      * elements of an array
+				      * of length N. In other words,
+				      * the i-th element of the
+				      * results array is the sum over
+				      * the i-th entries of the input
+				      * arrays from each processor.
+				      */
+    template <typename T, unsigned int N>
+    inline
+    void sum (const T (&values)[N],
+	      const MPI_Comm &mpi_communicator,
+	      T (&sums)[N]);
+
+    /**
+     * Return the maximum over all processors of the value @p t. This function
+     * is collective over all processors given in the communicator. If
+     * deal.II is not configured for use of MPI, this function simply
+     * returns the value of @p t. This function corresponds to the
+     * <code>MPI_Allreduce</code> function, i.e. all processors receive
+     * the result of this operation.
+     *
+     * @note This function is only implemented for certain template
+     * arguments <code>T</code>, namely <code>float, double, int,
+     * unsigned int</code>.
+     */
+    template <typename T>
+    T max (const T &t,
+	   const MPI_Comm &mpi_communicator);
+
+				     /**
+				      * Like the previous function,
+				      * but take the maxima over the
+				      * elements of an array
+				      * of length N. In other words,
+				      * the i-th element of the
+				      * results array is the maximum of
+				      * the i-th entries of the input
+				      * arrays from each processor.
+				      */
+    template <typename T, unsigned int N>
+    inline
+    void max (const T (&values)[N],
+	      const MPI_Comm &mpi_communicator,
+	      T (&maxima)[N]);
 
 				     /**
 				      * Data structure to store the result of
-				      * calculate_collective_mpi_min_max_avg.
+				      * min_max_avg().
 				      */
     struct MinMaxAvg
     {
@@ -376,16 +452,16 @@ namespace Utilities
 				      * on the given MPI communicator @param
 				      * mpi_communicator . Each processor's
 				      * value is given in @param my_value and
-				      * the result will be returned in @param
+				      * the result will be returned in @p
 				      * result . The result is available on all
 				      * machines.
 				      */
-    void calculate_collective_mpi_min_max_avg(const MPI_Comm &mpi_communicator,
-					      const double my_value,
-					      MinMaxAvg & result);
-    
+    MinMaxAvg
+    min_max_avg (const double my_value,
+		 const MPI_Comm &mpi_communicator);
+
+
 
-    
 				     /**
 				      * A class that is used to initialize the
 				      * MPI system at the beginning of a
@@ -404,8 +480,7 @@ namespace Utilities
 				      * turns calls <code>MPI_Finalize</code>
 				      * to shut down the MPI system.
 				      *
-				      * This class is used in @ref step_32
-				      * "step-32", for example.
+				      * This class is used in step-32, for example.
 				      */
     class MPI_InitFinalize
     {
@@ -446,30 +521,174 @@ namespace Utilities
 					  */
 	const bool owns_mpi;
     };
+  }
+                                   /**
+                                    * A namespace for utility functions that
+                                    * probe system properties.
+                                    *
+                                    * @ingroup utilities
+                                    */
+  namespace System
+  {
+
+                                     /**
+                                      * Return the CPU load as returned by
+                                      * "uptime". Note that the interpretation
+                                      * of this number depends on the actual
+                                      * number of processors in the
+                                      * machine. This is presently only
+                                      * implemented on Linux, using the
+                                      * /proc/loadavg pseudo-file, on other
+                                      * systems we simply return zero.
+                                      */
+    double get_cpu_load ();
 
 				     /**
-				      * Returns whether deal.II has been
-				      * configured to use MPI and if so
-				      * whether MPI has already been
-				      * initialized using MPI_Init().
+				      * Structure that hold information about
+				      * memory usage in kB. Used by
+				      * get_memory_stats(). See man 5 proc
+				      * entry /status for details.
 				      */
-    bool program_uses_mpi();
+    struct MemoryStats
+    {
+	unsigned long int VmPeak; /** peak virtual memory size in kB */
+	unsigned long int VmSize; /** current virtual memory size in kB */
+	unsigned long int VmHWM; /** peak resident memory size in kB */
+	unsigned long int VmRSS; /** current resident memory size in kB */
+    };
+
+
+				     /**
+				      * Fills the @param stats structure with
+				      * information about the memory
+				      * consumption of this process. This is
+				      * only implemented on Linux.
+				      */
+    void get_memory_stats (MemoryStats & stats);
+
+
+                                     /**
+                                      * Return the name of the host this
+                                      * process runs on.
+                                      */
+    std::string get_hostname ();
+
+
+                                     /**
+                                      * Return the present time as HH:MM:SS.
+                                      */
+    std::string get_time ();
+
+				     /**
+				      * Return whether (i) deal.II has
+				      * been compiled to support MPI
+				      * (for example by compiling with
+				      * <code>CXX=mpiCC</code>) and if
+				      * so whether (ii)
+				      * <code>MPI_Init()</code> has
+				      * been called (for example using
+				      * the
+				      * Utilities::System::MPI_InitFinalize
+				      * class). In other words, the
+				      * result indicates whether the
+				      * current job is running under
+				      * MPI.
+				      *
+				      * @note The function does not
+				      * take into account whether an
+				      * MPI job actually runs on more
+				      * than one processor or is, in
+				      * fact, a single-node job that
+				      * happens to run under MPI.
+				      */
+    bool job_supports_mpi ();
+
+				     /**
+				      * Alias for job_supports_mpi().
+				      *
+				      * @deprecated
+				      */
+    bool program_uses_mpi ();
+
+				     /**
+				      * @name Functions that work
+				      * in parallel via MPI. The
+				      * functions following here
+				      * are all deprecated and have
+				      * been moved to namespace
+				      * Utilities::MPI.
+				      */
+				     /** @{ */
+
+				     /**
+				      * This function is an alias for
+				      * Utilities::MPI::n_mpi_processes.
+				      *
+				      * @deprecated
+				      */
+    unsigned int get_n_mpi_processes (const MPI_Comm &mpi_communicator);
+
+				     /**
+				      * This function is an alias for
+				      * Utilities::MPI::this_mpi_process.
+				      *
+				      * @deprecated
+				      */
+    unsigned int get_this_mpi_process (const MPI_Comm &mpi_communicator);
+
+
+				     /**
+				      * This function is an alias for
+				      * Utilities::MPI::compute_point_to_point_communication_pattern.
+				      *
+				      * @deprecated
+				      */
+    using
+    Utilities::MPI::compute_point_to_point_communication_pattern;
+
+
+				     /**
+				      * This function is an alias for
+				      * Utilities::MPI::duplicate_communicator.
+				      *
+				      * @deprecated
+				      */
+    using Utilities::MPI::duplicate_communicator;
+
+				     /**
+				      * An alias for Utilities::MPI::MinMaxAvg.
+				      *
+				      * @deprecated
+				      */
+    using Utilities::MPI::MinMaxAvg;
+
+				     /**
+				      * An alias for Utilities::MPI::min_max_avg.
+				      *
+				      * @deprecated
+				      */
+    void
+    calculate_collective_mpi_min_max_avg (const MPI_Comm &mpi_communicator,
+					  const double my_value,
+					  MinMaxAvg & result);
+
+				     /**
+				      * An alias for Utilities::MPI::MPI_InitFinalize.
+				      *
+				      * @deprecated
+				      */
+    using Utilities::MPI::MPI_InitFinalize;
+
+				     /** @} */
   }
 
 
-  /**
-   * This class provides the basic structures for the use of the Trilinos
-   * classes such as matrices, vectors, and preconditioners. The most
-   * important function in this class is <tt>comm()</tt>, which is needed
-   * for the initialization of Trilinos Epetra_Maps, which design the
-   * %parallel distribution of vectors and matrices. Moreover, this class
-   * provides a unified interface to both serial and %parallel
-   * implementations of Trilinos, sets up the MPI communicator in case the
-   * programs are run in %parallel, and correctly terminates all processes
-   * when the destructor is called. An example usage of this class is shown
-   * in the tutorial program step-32.
-   */
 #ifdef DEAL_II_USE_TRILINOS
+  /**
+   * This namespace provides some of the basic structures used in the
+   * initialization of the Trilinos objects (e.g., matrices, vectors, and
+   * preconditioners).
+   */
   namespace Trilinos
   {
 				     /**
@@ -677,6 +896,225 @@ namespace Utilities
       }
   }
 
+
+
+  template<typename Iterator, typename T>
+  inline
+  Iterator
+  lower_bound (Iterator  first,
+	       Iterator  last,
+	       const T  &val)
+  {
+    return Utilities::lower_bound (first, last, val,
+				   std::less<T>());
+  }
+
+
+
+  template<typename Iterator, typename T, typename Comp>
+  inline
+  Iterator
+  lower_bound (Iterator    first,
+	       Iterator    last,
+	       const T    &val,
+	       const Comp  comp)
+  {
+    unsigned int len = last-first;
+
+    if (len==0)
+      return first;
+
+    while (true)
+      {
+					 // if length equals 8 or less,
+					 // then do a rolled out
+					 // search. use a switch without
+					 // breaks for that and roll-out
+					 // the loop somehow
+	if (len < 8)
+	  {
+	    switch (len)
+	      {
+		case 7:
+		      if (!comp(*first, val))
+			return first;
+		      ++first;
+		case 6:
+		      if (!comp(*first, val))
+			return first;
+		      ++first;
+		case 5:
+		      if (!comp(*first, val))
+			return first;
+		      ++first;
+		case 4:
+		      if (!comp(*first, val))
+			return first;
+		      ++first;
+		case 3:
+		      if (!comp(*first, val))
+			return first;
+		      ++first;
+		case 2:
+		      if (!comp(*first, val))
+			return first;
+		      ++first;
+		case 1:
+		      if (!comp(*first, val))
+			return first;
+		      return first+1;
+		default:
+						       // indices seem
+						       // to not be
+						       // sorted
+						       // correctly!? or
+						       // did len
+						       // become==0
+						       // somehow? that
+						       // shouln't have
+						       // happened
+		      Assert (false, ExcInternalError());
+	      }
+	  }
+
+
+
+	const unsigned int half   = len >> 1;
+	const Iterator     middle = first + half;
+
+					 // if the value is larger than
+					 // that pointed to by the
+					 // middle pointer, then the
+					 // insertion point must be
+					 // right of it
+	if (comp(*middle, val))
+	  {
+	    first = middle + 1;
+	    len  -= half + 1;
+	  }
+	else
+	  len = half;
+      }
+  }
+
+
+  namespace MPI
+  {
+    namespace internal
+    {
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+      /**
+       * Return the corresponding MPI data type id for the argument given.
+       */
+      inline MPI_Datatype mpi_type_id (const int *)
+      {
+	return MPI_INT;
+      }
+
+
+      inline MPI_Datatype mpi_type_id (const long int *)
+      {
+	return MPI_LONG;
+      }
+
+
+      inline MPI_Datatype mpi_type_id (const unsigned int *)
+      {
+	return MPI_UNSIGNED;
+      }
+
+
+      inline MPI_Datatype mpi_type_id (const unsigned long int *)
+      {
+	return MPI_UNSIGNED_LONG;
+      }
+
+
+      inline MPI_Datatype mpi_type_id (const float *)
+      {
+	return MPI_FLOAT;
+      }
+
+
+      inline MPI_Datatype mpi_type_id (const double *)
+      {
+	return MPI_DOUBLE;
+      }
+#endif
+    }
+
+
+    template <typename T>
+    inline
+    T sum (const T &t,
+	   const MPI_Comm &mpi_communicator)
+    {
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+      T sum;
+      MPI_Allreduce (const_cast<void*>(static_cast<const void*>(&t)),
+		     &sum, 1, internal::mpi_type_id(&t), MPI_SUM,
+		     mpi_communicator);
+      return sum;
+#else
+      (void)mpi_communicator;
+      return t;
+#endif
+    }
+
+
+    template <typename T, unsigned int N>
+    inline
+    void sum (const T (&values)[N],
+	      const MPI_Comm &mpi_communicator,
+	      T (&sums)[N])
+    {
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+      MPI_Allreduce (const_cast<void*>(static_cast<const void*>(&values[0])),
+		     &sums[0], N, internal::mpi_type_id(values), MPI_SUM,
+		     mpi_communicator);
+#else
+      (void)mpi_communicator;
+      for (unsigned int i=0; i<N; ++i)
+	sums[i] = values[i];
+#endif
+    }
+
+
+    template <typename T>
+    inline
+    T max (const T &t,
+	   const MPI_Comm &mpi_communicator)
+    {
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+      T sum;
+      MPI_Allreduce (const_cast<void*>(static_cast<const void*>(&t)),
+		     &sum, 1, internal::mpi_type_id(&t), MPI_MAX,
+		     mpi_communicator);
+      return sum;
+#else
+      (void)mpi_communicator;
+      return t;
+#endif
+    }
+
+
+    template <typename T, unsigned int N>
+    inline
+    void max (const T (&values)[N],
+	      const MPI_Comm &mpi_communicator,
+	      T (&maxima)[N])
+    {
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+      MPI_Allreduce (const_cast<void*>(static_cast<const void*>(&values[0])),
+		     &maxima[0], N, internal::mpi_type_id(values), MPI_MAX,
+		     mpi_communicator);
+#else
+      (void)mpi_communicator;
+      for (unsigned int i=0; i<N; ++i)
+	maxima[i] = values[i];
+#endif
+    }
+  }
 }
 
 
