@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: work_stream.h 22368 2010-10-18 16:01:03Z kanschat $
-//    Version: $Name$
+//    $Id: work_stream.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2008, 2009, 2010 by the deal.II authors
 //
@@ -14,14 +13,14 @@
 #define __deal2__work_stream_h
 
 
-#include <base/config.h>
-#include <base/multithread_info.h>
-#include <base/template_constraints.h>
-#include <base/std_cxx1x/function.h>
-#include <base/std_cxx1x/bind.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/multithread_info.h>
+#include <deal.II/base/template_constraints.h>
+#include <deal.II/base/std_cxx1x/function.h>
+#include <deal.II/base/std_cxx1x/bind.h>
 
 #if DEAL_II_USE_MT == 1
-#  include <base/thread_management.h>
+#  include <deal.II/base/thread_management.h>
 #  include <tbb/pipeline.h>
 #endif
 
@@ -629,10 +628,10 @@ namespace WorkStream
     run (begin, end,
 	 std_cxx1x::bind (worker,
 			  std_cxx1x::ref (main_object),
-			  _1, _2, _3),
+			  std_cxx1x::_1, std_cxx1x::_2, std_cxx1x::_3),
 	 std_cxx1x::bind (copier,
 			  std_cxx1x::ref (main_object),
-			  _1),
+			  std_cxx1x::_1),
 	 sample_scratch_data,
 	 sample_copy_data,
 	 queue_length,
