@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: tria.h 22851 2010-11-24 13:02:14Z bangerth $
-//    Version: $Name$
+//    $Id: tria.h 24400 2011-09-23 18:25:59Z bangerth $
 //
-//    Copyright (C) 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,18 +13,22 @@
 #define __deal2__distributed_tria_h
 
 
-#include <base/config.h>
-#include <base/subscriptor.h>
-#include <base/smartpointer.h>
-#include <base/template_constraints.h>
-#include <grid/tria.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/base/template_constraints.h>
+#include <deal.II/grid/tria.h>
 
-#include <base/std_cxx1x/function.h>
+#include <deal.II/base/std_cxx1x/function.h>
 
 #include <vector>
 #include <list>
 #include <utility>
 
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+#  include <mpi.h>
+#endif
+
 #ifdef DEAL_II_USE_P4EST
 #include <p4est_connectivity.h>
 #include <p4est.h>
@@ -209,6 +212,29 @@ namespace parallel
 					  * <code>check_for_distorted_cells</code>
 					  * argument provided by the
 					  * base class.
+					  *
+					  * @note While it is possible to pass
+					  * all of the mesh smoothing flags
+					  * listed in the base class to
+					  * objects of this type, it is not
+					  * always possible to honor all of
+					  * these smoothing options if they
+					  * would require knowledge of
+					  * refinement/coarsening flags on
+					  * cells not locally owned by this
+					  * processor. As a consequence, for
+					  * some of these flags, the ultimate
+					  * number of cells of the parallel
+					  * triangulation may depend on the
+					  * number of processors into which it
+					  * is partitioned. On the other hand,
+					  * if no smoothing flags are passed,
+					  * if you always mark the same cells
+					  * of the mesh, you will always get
+					  * the exact same refined mesh
+					  * independent of the number of
+					  * processors into which the
+					  * triangulation is partitioned.
 					  */
 	Triangulation (MPI_Comm mpi_communicator,
 		       const typename dealii::Triangulation<dim,spacedim>::MeshSmoothing
@@ -328,17 +354,15 @@ namespace parallel
 					  * the 2:1 size ratio of
 					  * adjacent cells.
 					  *
-					  * As a consequence of the
-					  * remark above, the result
-					  * of this function is always
-					  * smaller or equal to the
-					  * result of
-					  * dealii::Triangulation::n_active_cells()
-					  * which includes the active
-					  * ghost and artificial cells
-					  * (see also @ref
-					  * GlossArtificialCell and
-					  * @ref GlossGhostCell).
+					  * As a consequence of the remark
+					  * above, the result of this function
+					  * is always smaller or equal to the
+					  * result of the function with the
+					  * same name in the ::Triangulation
+					  * base class, which includes the
+					  * active ghost and artificial cells
+					  * (see also @ref GlossArtificialCell
+					  * and @ref GlossGhostCell).
 					  */
 	unsigned int n_locally_owned_active_cells () const;
 
@@ -378,7 +402,7 @@ namespace parallel
 					  * Return the local memory
 					  * consumption in bytes.
 					  */
-	virtual unsigned int memory_consumption () const;
+	virtual std::size_t memory_consumption () const;
 
 					 /**
 					  * Return the local memory
@@ -389,7 +413,7 @@ namespace parallel
 					  * available separately for debugging
 					  * purposes.
 					  */
-	virtual unsigned int memory_consumption_p4est () const;
+	virtual std::size_t memory_consumption_p4est () const;
 
 					 /**
 					  * A collective operation that produces
@@ -458,7 +482,7 @@ namespace parallel
 					  * notify_ready_to_unpack().
 					  */
 	unsigned int
-	register_data_attach (const size_t size,
+	register_data_attach (const std::size_t size,
 			      const std_cxx1x::function<void (const cell_iterator &,
 							      const CellStatus,
 							      void*)> & pack_callback);
@@ -475,6 +499,16 @@ namespace parallel
 								const CellStatus,
 								const void*)> & unpack_callback);
 
+					 /**
+					  * Returns a permutation vector for the order the coarse
+					  * cells are handed of to p4est. For example the first
+					  * element i in this vector denotes that the first cell
+					  * in hierarchical ordering is the ith deal cell starting
+					  * from begin(0).
+					  */
+	const std::vector<unsigned int> &
+	get_p4est_tree_to_coarse_cell_permutation() const;
+
   private:
 					 /**
 					  * MPI communicator to be
@@ -534,17 +568,11 @@ namespace parallel
 					  * A flag that indicates
 					  * whether refinement of a
 					  * triangulation is currently
-					  * in progress. We need this
-					  * flag because the
-					  * refinement_notification()
-					  * function is called from
-					  * Triangulation::execute_coarsening_and_refinement(),
-					  * but
-					  * refinement_notification()
-					  * itself also calls
-					  * Triangulation::execute_coarsening_and_refinement()
-					  * and therefore gets called
-					  * recursively again. While the
+					  * in progress. This flag is
+					  * used to disambiguate whether
+					  * a call to execute_coarsening_and_triangulation
+					  * came from the outside or
+					  * through a recursive call. While the
 					  * first time we want to take
 					  * over work to copy things
 					  * from a refined p4est, the
@@ -685,6 +713,27 @@ namespace parallel
 					  */
 	MPI_Comm get_communicator () const;
 
+					 /**
+					  * Return the sum over all
+					  * processors of the number
+					  * of active cells owned by
+					  * each processor. This
+					  * equals the overall number
+					  * of active cells in the
+					  * distributed triangulation.
+					  */
+	unsigned int n_global_active_cells () const;
+
+					 /**
+					  * Returns a permutation vector for the order the coarse
+					  * cells are handed of to p4est. For example the first
+					  * element i in this vector denotes that the first cell
+					  * in hierarchical ordering is the ith deal cell starting
+					  * from begin(0).
+					  */
+	const std::vector<unsigned int> &
+	get_p4est_tree_to_coarse_cell_permutation() const;
+
 					 /**
 					  * Return the subdomain id of
 					  * those cells that are owned
@@ -763,6 +812,15 @@ namespace parallel
 					  * on other processors.
 					  */
 	types::subdomain_id_t locally_owned_subdomain () const;
+
+					 /**
+					  * Return the MPI
+					  * communicator used by this
+					  * triangulation.
+					  */
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+	MPI_Comm get_communicator () const;
+#endif
     };
   }
 }
