@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_accessor.h 23000 2010-12-18 05:00:38Z bangerth $
-//    Version: $Name$
+//    $Id: dof_accessor.h 24171 2011-08-23 09:42:08Z kronbichler $
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,10 +13,10 @@
 #define __deal2__dof_accessor_h
 
 
-#include <base/config.h>
-#include <grid/tria_accessor.h>
-#include <dofs/dof_handler.h>
-#include <hp/dof_handler.h>
+#include <deal.II/base/config.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/hp/dof_handler.h>
 
 #include <vector>
 
@@ -749,10 +748,10 @@ class DoFAccessor : public internal::DoFAccessor::Inheritance<structdim, DH::dim
     template <int dim, int spacedim> friend class DoFHandler;
     template <int dim, int spacedim> friend class hp::DoFHandler;
 
-    friend class internal::DoFHandler::Policy::Implementation;
-    friend class internal::DoFHandler::Implementation;
-    friend class internal::hp::DoFHandler::Implementation;
-    friend class internal::DoFCellAccessor::Implementation;
+    friend struct internal::DoFHandler::Policy::Implementation;
+    friend struct internal::DoFHandler::Implementation;
+    friend struct internal::hp::DoFHandler::Implementation;
+    friend struct internal::DoFCellAccessor::Implementation;
 };
 
 
@@ -1373,10 +1372,10 @@ class DoFAccessor<0,DH<1,spacedim> > : public TriaAccessor<0,1,spacedim>
     template <int, int> friend class DoFHandler;
     template <int, int> friend class hp::DoFHandler;
 
-    friend class internal::DoFHandler::Policy::Implementation;
-    friend class internal::DoFHandler::Implementation;
-    friend class internal::hp::DoFHandler::Implementation;
-    friend class internal::DoFCellAccessor::Implementation;
+    friend struct internal::DoFHandler::Policy::Implementation;
+    friend struct internal::DoFHandler::Implementation;
+    friend struct internal::hp::DoFHandler::Implementation;
+    friend struct internal::DoFCellAccessor::Implementation;
 };
 
 
@@ -1715,7 +1714,7 @@ class DoFCellAccessor :  public DoFAccessor<DH::dimension,DH>
 				      * cell. You could then as well
 				      * get the desired values through
 				      * the @p get_dof_values
-				      * function In the other case,
+				      * function. In the other case,
 				      * when the cell has children, we
 				      * use the restriction matrices
 				      * provided by the finite element
@@ -1786,7 +1785,7 @@ class DoFCellAccessor :  public DoFAccessor<DH::dimension,DH>
 				      * cell. These requirements are
 				      * not taken care of and must be
 				      * enforced by the user
-				      * afterwards.
+				      * afterward.
 				      *
 				      * It is assumed that both
 				      * vectors already have the right
@@ -2069,7 +2068,7 @@ class DoFCellAccessor :  public DoFAccessor<DH::dimension,DH>
 				      * function
 				      */
     template <int dim, int spacedim> friend class DoFHandler;
-    friend class internal::DoFCellAccessor::Implementation;
+    friend struct internal::DoFCellAccessor::Implementation;
 };
 
 
