@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_handler.h 22673 2010-11-10 14:44:22Z kronbichler $
-//    Version: $Name$
+//    $Id: dof_handler.h 24348 2011-09-20 19:07:29Z kanschat $
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,15 +14,17 @@
 
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/smartpointer.h>
-#include <base/index_set.h>
-#include <dofs/block_info.h>
-#include <dofs/dof_iterator_selector.h>
-#include <dofs/number_cache.h>
-#include <dofs/function_map.h>
-#include <dofs/dof_handler_policy.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/base/index_set.h>
+#include <deal.II/dofs/block_info.h>
+#include <deal.II/dofs/dof_iterator_selector.h>
+#include <deal.II/dofs/number_cache.h>
+#include <deal.II/dofs/function_map.h>
+#include <deal.II/dofs/dof_handler_policy.h>
+
+#include <boost/serialization/split_member.hpp>
 
 #include <vector>
 #include <map>
@@ -137,8 +138,34 @@ namespace internal
  * <tt>renumber_dofs(vector<unsigned int>)</tt> function with the array, which
  * converts old into new degree of freedom indices.
  *
+ *
+ * <h3>Serializing (loading or storing) DoFHandler objects</h3>
+ *
+ * Like many other classes in deal.II, the DoFHandler class can stream
+ * its contents to an archive using BOOST's serialization facilities. The
+ * data so stored can later be retrieved again from the archive to restore
+ * the contents of this object. This facility is frequently used to save the
+ * state of a program to disk for possible later resurrection, often in the
+ * context of checkpoint/restart strategies for long running computations or
+ * on computers that aren't very reliable (e.g. on very large clusters where
+ * individual nodes occasionally fail and then bring down an entire MPI
+ * job).
+ *
+ * The model for doing so is similar for the DoFHandler class as it is for
+ * the Triangulation class (see the section in the general documentation of
+ * that class). In particular, the load() function does not exactly restore
+ * the same state as was stored previously using the save() function. Rather,
+ * the function assumes that you load data into a DoFHandler object that is
+ * already associated with a triangulation that has a content that matches
+ * the one that was used when the data was saved. Likewise, the load() function
+ * assumes that the current object is already associated with a finite element
+ * object that matches the one that was associated with it when data was
+ * saved; the latter can be achieved by calling DoFHandler::distribute_dofs()
+ * using the same kind of finite element before re-loading data from the
+ * serialization archive.
+ *
  * @ingroup dofs
- * @author Wolfgang Bangerth, 1998
+ * @author Wolfgang Bangerth
  */
 template <int dim, int spacedim=dim>
 class DoFHandler  :  public Subscriptor
@@ -1191,28 +1218,48 @@ class DoFHandler  :  public Subscriptor
 				      * although the actual object
 				      * might be a derived class.
 				      */
-    virtual unsigned int memory_consumption () const;
+    virtual std::size_t memory_consumption () const;
 
-    				     /**
-				      * @todo Replace by ExcInternalError.
+				     /**
+				      * Write the data of this object to a
+				      * stream for the purpose of
+				      * serialization.
+				      */
+   template <class Archive>
+   void save (Archive & ar, const unsigned int version) const;
+
+				     /**
+				      * Read the data of this object from a
+				      * stream for the purpose of
+				      * serialization.
+				      */
+   template <class Archive>
+   void load (Archive & ar, const unsigned int version);
+
+   BOOST_SERIALIZATION_SPLIT_MEMBER()
+
+				     /**
+				      * We are trying to renumber the
+				      * degrees of freedom, but
+				      * somehow did not count
+				      * correctly.
+				      *
+				      * @ingroup Exceptions
 				      */
     DeclException0 (ExcRenumberingIncomplete);
 				     /**
 				      * Exception
+				      * @ingroup Exceptions
 				      */
     DeclException0 (ExcGridsDoNotMatch);
 				     /**
 				      * Exception
+				      * @ingroup Exceptions
 				      */
     DeclException0 (ExcInvalidBoundaryIndicator);
 				     /**
 				      * Exception
-				      */
-    DeclException1 (ExcMatrixHasWrongSize,
-		    int,
-		    << "The matrix has the wrong dimension " << arg1);
-				     /**
-				      * Exception
+				      * @ingroup Exceptions
 				      */
     DeclException1 (ExcNewNumbersNotConsecutive,
 		    int,
@@ -1220,6 +1267,7 @@ class DoFHandler  :  public Subscriptor
 		    << "the index " << arg1 << " does not exist.");
 				     /**
 				      *  Exception
+				      * @ingroup Exceptions
 				      */
     DeclException1 (ExcInvalidLevel,
 		    int,
@@ -1227,11 +1275,13 @@ class DoFHandler  :  public Subscriptor
 		    << " is not in the valid range!");
 				     /**
 				      * Exception
+				      * @ingroup Exceptions
 				      */
     DeclException0 (ExcFacesHaveNoLevel);
 				     /**
 				      * The triangulation level you
 				      * accessed is empty.
+				      * @ingroup Exceptions
 				      */
     DeclException1 (ExcEmptyLevel,
 		    int,
@@ -1350,11 +1400,11 @@ class DoFHandler  :  public Subscriptor
 				      */
     template <int, class> friend class DoFAccessor;
     template <class> friend class DoFCellAccessor;
-    friend class internal::DoFAccessor::Implementation;
-    friend class internal::DoFCellAccessor::Implementation;
+    friend struct internal::DoFAccessor::Implementation;
+    friend struct internal::DoFCellAccessor::Implementation;
 
-    friend class internal::DoFHandler::Implementation;
-    friend class internal::DoFHandler::Policy::Implementation;
+    friend struct internal::DoFHandler::Implementation;
+    friend struct internal::DoFHandler::Policy::Implementation;
 };
 
 
@@ -1443,6 +1493,67 @@ DoFHandler<dim,spacedim>::block_info () const
 }
 
 
+template <int dim, int spacedim>
+template <class Archive>
+void DoFHandler<dim,spacedim>::save (Archive & ar,
+				     const unsigned int) const
+{
+  ar & block_info_object;
+  ar & vertex_dofs;
+  ar & number_cache;
+  ar & levels;
+  ar & faces;
+
+  // write out the number of triangulation cells and later check
+  // during loading that this number is indeed correct; same with something that
+  // identifies the FE and the policy
+  unsigned int n_cells = tria->n_cells();
+  std::string  fe_name = selected_fe->get_name();
+  std::string  policy_name = typeid(*policy).name();
+
+  ar & n_cells & fe_name & policy_name;
+}
+
+
+template <int dim, int spacedim>
+template <class Archive>
+void DoFHandler<dim,spacedim>::load (Archive & ar,
+				     const unsigned int)
+{
+  ar & block_info_object;
+  ar & vertex_dofs;
+  ar & number_cache;
+
+  // boost::serialization can restore pointers just fine, but if the
+  // pointer object still points to something useful, that object is
+  // not destroyed and we end up with a memory leak. consequently,
+  // first delete previous content before re-loading stuff
+  for (unsigned int i=0; i<levels.size(); ++i)
+    delete levels[i];
+  levels.resize (0);
+  delete faces;
+  faces = 0;
+
+  ar & levels;
+  ar & faces;
+
+  // these are the checks that correspond to the last block in the save() function
+  unsigned int n_cells;
+  std::string  fe_name;
+  std::string  policy_name;
+
+  ar & n_cells & fe_name & policy_name;
+
+  AssertThrow (n_cells == tria->n_cells(),
+	       ExcMessage ("The object being loaded into does not match the triangulation "
+	                   "that has been stored previously."));
+  AssertThrow (fe_name == selected_fe->get_name(),
+	       ExcMessage ("The finite element associated with this DoFHandler does not match "
+	                   "the one that was associated with the DoFHandler previously stored."));
+  AssertThrow (policy_name == typeid(*policy).name(),
+	       ExcMessage ("The policy associated with this DoFHandler does not match "
+	                   "the one that was associated with the DoFHandler previously stored."));
+}
 
 
 #endif // DOXYGEN
