@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_renumbering.h 22920 2010-12-05 22:50:08Z bangerth $
-//    Version: $Name$
+//    $Id: dof_renumbering.h 24178 2011-08-24 16:36:32Z heister $
 //
 //    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
@@ -14,12 +13,12 @@
 #define __deal2__dof_renumbering_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/point.h>
-#include <dofs/dof_handler.h>
-#include <hp/dof_handler.h>
-#include <multigrid/mg_dof_handler.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/point.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/hp/dof_handler.h>
+#include <deal.II/multigrid/mg_dof_handler.h>
 
 #include <vector>
 
@@ -791,6 +790,17 @@ namespace DoFRenumbering
 			  const ENDITERATOR& end,
 			  const std::vector<unsigned int> &target_component);
 
+				   /**
+					* Renumber the degrees cell by cell in hierarchical order
+					* (also known as z-order). The main usage is that this
+					* guarantees the same ordering independent of the
+					* number of processors involved in a parallel
+					* distributed computation.
+					*/
+  template <int dim>
+  void
+  hierarchical (DoFHandler<dim> &dof_handler);
+  
 				   /**
 				    * Cell-wise renumbering for DG
 				    * elements.  This function takes
