@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_base.h 22017 2010-09-17 16:21:31Z kanschat $
+//    $Id: fe_base.h 24072 2011-08-15 12:19:02Z kanschat $
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,17 +12,17 @@
 #ifndef __deal2__fe_base_h
 #define __deal2__fe_base_h
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/subscriptor.h>
-#include <base/point.h>
-#include <base/tensor.h>
-#include <base/table.h>
-#include <base/vector_slice.h>
-#include <base/geometry_info.h>
-#include <lac/full_matrix.h>
-#include <fe/fe_update_flags.h>
-#include <fe/mapping.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/point.h>
+#include <deal.II/base/tensor.h>
+#include <deal.II/base/table.h>
+#include <deal.II/base/vector_slice.h>
+#include <deal.II/base/geometry_info.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/fe/fe_update_flags.h>
+#include <deal.II/fe/mapping.h>
 
 #include <string>
 #include <vector>
@@ -111,6 +111,17 @@ namespace FiniteElementDomination
 				    * options open until we get to the
 				    * second base element.
 				    *
+				    * Finally, the code
+				    * no_requirements exists for cases
+				    * where elements impose no
+				    * continuity requirements. The
+				    * case is primarily meant for
+				    * FE_Nothing which is an element
+				    * that has no degrees of freedom
+				    * in a subdomain. It could also be
+				    * used by discontinuous elements,
+				    * for example.
+				    *
 				    * More details on domination can be found
 				    * in the @ref hp_paper "hp paper".
 				    */
@@ -119,7 +130,8 @@ namespace FiniteElementDomination
 	this_element_dominates,
 	other_element_dominates,
 	neither_element_dominates,
-	either_element_can_dominate
+	either_element_can_dominate,
+        no_requirements
   };
 
 
@@ -231,13 +243,13 @@ class FiniteElementData
 					    * system.
 					    */
 	  unknown = 0x00,
-	  
+
 					   /**
 					    * Discontinuous
 					    * elements. See above!
 					    */
 	  L2 = 0x01,
-	  
+
 					   /**
 					    * Conformity with the
 					    * space
@@ -247,7 +259,7 @@ class FiniteElementData
 					    * field)
 					    */
 	  Hcurl = 0x02,
-	  
+
 					   /**
 					    * Conformity with the
 					    * space
@@ -257,7 +269,7 @@ class FiniteElementData
 					    * field)
 					    */
 	  Hdiv = 0x04,
-	  
+
 					   /**
 					    * Conformity with the
 					    * space
@@ -265,7 +277,7 @@ class FiniteElementData
 					    * (continuous)
 					    */
 	  H1 = Hcurl | Hdiv,
-	  
+
 					   /**
 					    * Conformity with the
 					    * space
@@ -282,7 +294,7 @@ class FiniteElementData
 				      * parameter <tt>dim</tt>
 				      */
     static const unsigned int dimension = dim;
-    
+
 				     /**
 				      * Number of degrees of freedom on
 				      * a vertex.
@@ -318,22 +330,22 @@ class FiniteElementData
 				      * First index of dof on a line.
 				      */
     const unsigned int first_line_index;
-    
+
 				     /**
 				      * First index of dof on a quad.
 				      */
     const unsigned int first_quad_index;
-    
+
 				     /**
 				      * First index of dof on a hexahedron.
 				      */
     const unsigned int first_hex_index;
-    
+
 				     /**
 				      * First index of dof on a line for face data.
 				      */
     const unsigned int first_face_line_index;
-    
+
 				     /**
 				      * First index of dof on a quad for face data.
 				      */
@@ -349,7 +361,7 @@ class FiniteElementData
 				      * face.
 				      */
     const unsigned int dofs_per_face;
-    
+
 				     /**
 				      * Total number of degrees of freedom
 				      * on a cell. This is the
@@ -380,18 +392,6 @@ class FiniteElementData
 				      * base element.
 				      */
     const unsigned int components;
-				     /**
-				      * The number of vector blocks a
-				      * BlockVector for this element
-				      * should contain. For primitive
-				      * elements, this is equal to
-				      * #components, but for vector
-				      * valued base elements it may
-				      * differ. Actually, in systems
-				      * this is the sum of the base
-				      * element multiplicities.
-				      */
-    const unsigned int blocks;
     
 				     /**
 				      * Maximal polynomial degree of a
@@ -404,6 +404,17 @@ class FiniteElementData
 				      * Indicate the space this element conforms to.
 				      */
     const Conformity conforming_space;
+
+				     /**
+				      * Storage for an object
+				      * describing the sizes of each
+				      * block of a compound
+				      * element. For an element which
+				      * is not an FESystem, this
+				      * contains only a single block
+				      * with length #dofs_per_cell.
+				      */
+    BlockIndices block_indices_data;
     
     				     /**
 				      * Default
@@ -437,8 +448,7 @@ class FiniteElementData
 				      * @param conformity The finite
 				      * element space has continuity
 				      * of this Sobolev space.
-				      * @param n_blocks Number of
-				      * vector blocks.
+				      * @param n_blocks obsolete and ignored.
 				      */
     FiniteElementData (const std::vector<unsigned int> &dofs_per_object,
 		       const unsigned int n_components,
@@ -506,7 +516,7 @@ class FiniteElementData
 				      */
     template <int structdim>
     unsigned int n_dofs_per_object () const;
-    
+
     				     /**
 				      * Number of components.
 				      */
@@ -517,6 +527,11 @@ class FiniteElementData
 				      */
     unsigned int n_blocks () const;
 
+				     /**
+				      * Detailed infromation on block sizes.
+				      */
+    const BlockIndices& block_indices() const;
+    
 				     /**
 				      * Return whether the entire
 				      * finite element is primitive,
@@ -534,7 +549,7 @@ class FiniteElementData
 				      * the constructor.
 				      */
     bool is_primitive () const;
-    
+
 				     /**
 				      * Maximal polynomial degree of a
 				      * shape function in a single
@@ -564,14 +579,14 @@ class FiniteElementData
 				      * ordering of indices on a face,
 				      * return the index of the same
 				      * degree of freedom on the cell.
-				      * 
+				      *
 				      */
     unsigned int face_to_cell_index (const unsigned int face_index,
 				     const unsigned int face,
 				     const bool face_orientation = true,
 				     const bool face_flip        = false,
 				     const bool face_rotation    = false) const;
-    
+
 				     /**
 				      * @deprecated This function is
 				      * just a special version of
@@ -611,7 +626,7 @@ class FiniteElementData
 				      *
 				      */
     unsigned int face_to_equivalent_cell_index (const unsigned int index) const;
-    
+
 				     /**
 				      * Comparison operator.
 				      */
@@ -628,7 +643,7 @@ class FiniteElementData
 				      * for details.
 				      */
     void set_primitivity(const bool value);
-    
+
   private:
 				     /**
 				      * Store whether all shape
@@ -670,14 +685,16 @@ namespace FiniteElementDomination
       {
 	case this_element_dominates:
 	      if ((d2 == this_element_dominates) ||
-		  (d2 == either_element_can_dominate))
+		  (d2 == either_element_can_dominate) ||
+		  (d2 == no_requirements))
 		return this_element_dominates;
 	      else
 		return neither_element_dominates;
-	      
+
 	case other_element_dominates:
 	      if ((d2 == other_element_dominates) ||
-		  (d2 == either_element_can_dominate))
+		  (d2 == either_element_can_dominate) ||
+		  (d2 == no_requirements))
 		return other_element_dominates;
 	      else
 		return neither_element_dominates;
@@ -686,6 +703,12 @@ namespace FiniteElementDomination
 	      return neither_element_dominates;
 
 	case either_element_can_dominate:
+	      if (d2 == no_requirements)
+		return either_element_can_dominate;
+	      else
+		return d2;
+
+        case no_requirements:
 	      return d2;
 
 	default:
@@ -700,7 +723,7 @@ namespace FiniteElementDomination
 
 template <int dim>
 inline
-unsigned int 
+unsigned int
 FiniteElementData<dim>::n_dofs_per_vertex () const
 {
   return dofs_per_vertex;
@@ -710,7 +733,7 @@ FiniteElementData<dim>::n_dofs_per_vertex () const
 
 template <int dim>
 inline
-unsigned int 
+unsigned int
 FiniteElementData<dim>::n_dofs_per_line () const
 {
   return dofs_per_line;
@@ -720,7 +743,7 @@ FiniteElementData<dim>::n_dofs_per_line () const
 
 template <int dim>
 inline
-unsigned int 
+unsigned int
 FiniteElementData<dim>::n_dofs_per_quad () const
 {
   return dofs_per_quad;
@@ -730,7 +753,7 @@ FiniteElementData<dim>::n_dofs_per_quad () const
 
 template <int dim>
 inline
-unsigned int 
+unsigned int
 FiniteElementData<dim>::n_dofs_per_hex () const
 {
   return dofs_per_hex;
@@ -740,7 +763,7 @@ FiniteElementData<dim>::n_dofs_per_hex () const
 
 template <int dim>
 inline
-unsigned int 
+unsigned int
 FiniteElementData<dim>::n_dofs_per_face () const
 {
   return dofs_per_face;
@@ -750,7 +773,7 @@ FiniteElementData<dim>::n_dofs_per_face () const
 
 template <int dim>
 inline
-unsigned int 
+unsigned int
 FiniteElementData<dim>::n_dofs_per_cell () const
 {
   return dofs_per_cell;
@@ -784,7 +807,7 @@ FiniteElementData<dim>::n_dofs_per_object () const
 
 template <int dim>
 inline
-unsigned int 
+unsigned int
 FiniteElementData<dim>::n_components () const
 {
   return components;
@@ -792,7 +815,7 @@ FiniteElementData<dim>::n_components () const
 
 
 
-template <int dim>  
+template <int dim>
 inline
 bool
 FiniteElementData<dim>::is_primitive () const
@@ -801,7 +824,7 @@ FiniteElementData<dim>::is_primitive () const
 }
 
 
-template <int dim>  
+template <int dim>
 inline
 void
 FiniteElementData<dim>::set_primitivity (const bool value)
@@ -812,17 +835,27 @@ FiniteElementData<dim>::set_primitivity (const bool value)
 
 template <int dim>
 inline
-unsigned int 
-FiniteElementData<dim>::n_blocks () const
+const BlockIndices&
+FiniteElementData<dim>::block_indices () const
 {
-  return blocks;
+  return block_indices_data;
 }
 
 
 
 template <int dim>
 inline
-unsigned int 
+unsigned int
+FiniteElementData<dim>::n_blocks () const
+{
+  return block_indices_data.size();
+}
+
+
+
+template <int dim>
+inline
+unsigned int
 FiniteElementData<dim>::tensor_degree () const
 {
   return degree;
