@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_dgp_nonparametric.h 21954 2010-09-13 20:49:06Z kanschat $
+//    $Id: fe_dgp_nonparametric.h 24072 2011-08-15 12:19:02Z kanschat $
 //
-//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2010 by the deal.II authors
+//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,10 +12,10 @@
 #ifndef __deal2__fe_dgp_nonparametric_h
 #define __deal2__fe_dgp_nonparametric_h
 
-#include <base/config.h>
-#include <base/polynomial.h>
-#include <base/polynomial_space.h>
-#include <fe/fe.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/polynomial.h>
+#include <deal.II/base/polynomial_space.h>
+#include <deal.II/fe/fe.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -180,34 +180,6 @@ class FE_DGPNonparametric : public FiniteElement<dim,spacedim>
 				      * constructor.
 				      */
     unsigned int get_degree () const;
-
-				     /**
-				      * Number of base elements in a
-				      * mixed discretization. Since
-				      * this is a scalar element,
-				      * return one.
-				      */
-    virtual unsigned int n_base_elements () const;
-    
-				     /**
-				      * Access to base element
-				      * objects. Since this element is
-				      * scalar, <tt>base_element(0)</tt> is
-				      * @p this, and all other
-				      * indices throw an error.
-				      */
-    virtual const FiniteElement<dim,spacedim> &
-    base_element (const unsigned int index) const;
-
-                                     /**
-                                      * Multiplicity of base element
-                                      * @p index. Since this is a
-                                      * scalar element,
-                                      * <tt>element_multiplicity(0)</tt>
-                                      * returns one, and all other
-                                      * indices will throw an error.
-                                      */
-    virtual unsigned int element_multiplicity (const unsigned int index) const;
     
 				     /**
 				      * Return the matrix
@@ -399,7 +371,7 @@ class FE_DGPNonparametric : public FiniteElement<dim,spacedim>
 				      * pointers to their base class,
 				      * rather than the class itself.
 				      */
-    virtual unsigned int memory_consumption () const;
+    virtual std::size_t memory_consumption () const;
 
 
   private:
