@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_poly.templates.h 22829 2010-11-20 22:34:28Z bangerth $
+//    $Id: fe_poly.templates.h 24072 2011-08-15 12:19:02Z kanschat $
 //
-//    Copyright (C) 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -10,11 +10,11 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/qprojector.h>
-#include <base/polynomial_space.h>
-#include <base/tensor_product_polynomials.h>
-#include <fe/fe_values.h>
-#include <fe/fe_poly.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/base/polynomial_space.h>
+#include <deal.II/base/tensor_product_polynomials.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/fe/fe_poly.h>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -508,68 +508,68 @@ fill_fe_face_values (const Mapping<dim,spacedim>                   &mapping,
 
 
 //codimension 1
-template <>
-inline
-void
-FE_Poly<TensorProductPolynomials<1>,1,2>::fill_fe_subface_values (const Mapping<1,2> &,
-							       const Triangulation<1,2>::cell_iterator &,
-							       const unsigned int,
-							       const unsigned int,
-							       const Quadrature<0> &,
-							       Mapping<1,2>::InternalDataBase &,
-							       Mapping<1,2>::InternalDataBase &,
-							       FEValuesData<1,2> &) const
-{
-  AssertThrow(false, ExcNotImplemented());
-}
+// template <>
+// inline
+// void
+// FE_Poly<TensorProductPolynomials<1>,1,2>::fill_fe_subface_values (const Mapping<1,2> &,
+// 							       const Triangulation<1,2>::cell_iterator &,
+// 							       const unsigned int,
+// 							       const unsigned int,
+// 							       const Quadrature<0> &,
+// 							       Mapping<1,2>::InternalDataBase &,
+// 							       Mapping<1,2>::InternalDataBase &,
+// 							       FEValuesData<1,2> &) const
+// {
+//   AssertThrow(false, ExcNotImplemented());
+// }
 
 
-template <>
-inline
-void
-FE_Poly<TensorProductPolynomials<2>,2,3>::fill_fe_subface_values (const Mapping<2,3> &,
-							       const Triangulation<2,3>::cell_iterator &,
-							       const unsigned int,
-							       const unsigned int,
-							       const Quadrature<1> &,
-							       Mapping<2,3>::InternalDataBase &,
-							       Mapping<2,3>::InternalDataBase &,
-							       FEValuesData<2,3> &) const
-{
-  AssertThrow(false, ExcNotImplemented());
-}
+// template <>
+// inline
+// void
+// FE_Poly<TensorProductPolynomials<2>,2,3>::fill_fe_subface_values (const Mapping<2,3> &,
+// 							       const Triangulation<2,3>::cell_iterator &,
+// 							       const unsigned int,
+// 							       const unsigned int,
+// 							       const Quadrature<1> &,
+// 							       Mapping<2,3>::InternalDataBase &,
+// 							       Mapping<2,3>::InternalDataBase &,
+// 							       FEValuesData<2,3> &) const
+// {
+//   AssertThrow(false, ExcNotImplemented());
+// }
 
 
-template <>
-inline
-void
-FE_Poly<PolynomialSpace<1>,1,2>::fill_fe_subface_values (const Mapping<1,2> &,
-							       const Triangulation<1,2>::cell_iterator &,
-							       const unsigned int,
-							       const unsigned int,
-							       const Quadrature<0> &,
-							       Mapping<1,2>::InternalDataBase &,
-							       Mapping<1,2>::InternalDataBase &,
-							       FEValuesData<1,2> &) const
-{
-  AssertThrow(false, ExcNotImplemented());
-}
+// template <>
+// inline
+// void
+// FE_Poly<PolynomialSpace<1>,1,2>::fill_fe_subface_values (const Mapping<1,2> &,
+// 							       const Triangulation<1,2>::cell_iterator &,
+// 							       const unsigned int,
+// 							       const unsigned int,
+// 							       const Quadrature<0> &,
+// 							       Mapping<1,2>::InternalDataBase &,
+// 							       Mapping<1,2>::InternalDataBase &,
+// 							       FEValuesData<1,2> &) const
+// {
+//   AssertThrow(false, ExcNotImplemented());
+// }
 
 
-template <>
-inline
-void
-FE_Poly<PolynomialSpace<2>,2,3>::fill_fe_subface_values (const Mapping<2,3> &,
-							       const Triangulation<2,3>::cell_iterator &,
-							       const unsigned int,
-							       const unsigned int,
-							       const Quadrature<1> &,
-							       Mapping<2,3>::InternalDataBase &,
-							       Mapping<2,3>::InternalDataBase &,
-							       FEValuesData<2,3> &) const
-{
-  AssertThrow(false, ExcNotImplemented());
-}
+// template <>
+// inline
+// void
+// FE_Poly<PolynomialSpace<2>,2,3>::fill_fe_subface_values (const Mapping<2,3> &,
+// 							       const Triangulation<2,3>::cell_iterator &,
+// 							       const unsigned int,
+// 							       const unsigned int,
+// 							       const Quadrature<1> &,
+// 							       Mapping<2,3>::InternalDataBase &,
+// 							       Mapping<2,3>::InternalDataBase &,
+// 							       FEValuesData<2,3> &) const
+// {
+//   AssertThrow(false, ExcNotImplemented());
+// }
 
 
 
@@ -622,34 +622,4 @@ FE_Poly<POLY,dim,spacedim>::fill_fe_subface_values (const Mapping<dim,spacedim>
     this->compute_2nd (mapping, cell, offset, mapping_data, fe_data, data);
 }
 
-
-
-template <class POLY, int dim, int spacedim>
-unsigned int
-FE_Poly<POLY,dim,spacedim>::n_base_elements () const
-{
-  return 1;
-}
-
-
-
-template <class POLY, int dim, int spacedim>
-const FiniteElement<dim,spacedim> &
-FE_Poly<POLY,dim,spacedim>::base_element (const unsigned int index) const
-{
-  Assert (index==0, ExcIndexRange(index, 0, 1));
-  return *this;
-}
-
-
-
-template <class POLY, int dim, int spacedim>
-unsigned int
-FE_Poly<POLY,dim,spacedim>::element_multiplicity (const unsigned int index) const
-{
-  Assert (index==0, ExcIndexRange(index, 0, 1));
-  return 1;
-}
-
-
 DEAL_II_NAMESPACE_CLOSE
