@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_q.h 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: fe_q.h 24130 2011-08-21 00:14:40Z bangerth $
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,9 +12,9 @@
 #ifndef __deal2__fe_q_h
 #define __deal2__fe_q_h
 
-#include <base/config.h>
-#include <base/tensor_product_polynomials.h>
-#include <fe/fe_poly.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/tensor_product_polynomials.h>
+#include <deal.II/fe/fe_poly.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -448,7 +448,7 @@ class FE_Q : public FE_Poly<TensorProductPolynomials<dim>,dim,spacedim>
 				      * pointers to their base class,
 				      * rather than the class itself.
 				      */
-    virtual unsigned int memory_consumption () const;
+    virtual std::size_t memory_consumption () const;
 
   protected:
 				     /**
@@ -582,7 +582,7 @@ class FE_Q : public FE_Poly<TensorProductPolynomials<dim>,dim,spacedim>
 				      */
     template <int, int> friend class FE_Q;
 
-    friend class FE_Q<dim,spacedim>::Implementation;
+    friend struct FE_Q<dim,spacedim>::Implementation;
 };
 
 
