@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_q_hierarchical.h 21954 2010-09-13 20:49:06Z kanschat $
+//    $Id: fe_q_hierarchical.h 23709 2011-05-17 04:34:08Z bangerth $
 //
-//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2010 by the deal.II authors
+//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,9 +12,9 @@
 #ifndef __deal2__fe_q_hierarchical_h
 #define __deal2__fe_q_hierarchical_h
 
-#include <base/config.h>
-#include <base/tensor_product_polynomials.h>
-#include <fe/fe_poly.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/tensor_product_polynomials.h>
+#include <deal.II/fe/fe_poly.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -352,6 +352,22 @@ class FE_Q_Hierarchical : public FE_Poly<TensorProductPolynomials<dim>,dim>
 				      */
     virtual void get_subface_interpolation_matrix (const FiniteElement<dim>& source, const unsigned int subface, FullMatrix<double>& matrix) const;
 
+				     /**
+				      * Return whether this element dominates
+				      * the one given as argument when they
+				      * meet at a common face,
+				      * whether it is the other way around,
+				      * whether neither dominates, or if
+				      * either could dominate.
+				      *
+				      * For a definition of domination, see
+				      * FiniteElementBase::Domination and in
+				      * particular the @ref hp_paper "hp paper".
+				      */
+    virtual
+    FiniteElementDomination::Domination
+    compare_for_face_domination (const FiniteElement<dim> &fe_other) const;
+
 				     /**
 				      * Determine an estimate for the
 				      * memory consumption (in bytes)
@@ -363,7 +379,7 @@ class FE_Q_Hierarchical : public FE_Poly<TensorProductPolynomials<dim>,dim>
 				      * pointers to their base class,
 				      * rather than the class itself.
 				      */
-    virtual unsigned int memory_consumption () const;
+    virtual std::size_t memory_consumption () const;
 
                                      /**
 				      * For a finite element of degree
