@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_system.h 22676 2010-11-10 15:21:40Z kronbichler $
+//    $Id: fe_system.h 24072 2011-08-15 12:19:02Z kanschat $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -16,8 +16,8 @@
 /*----------------------------   fe_system.h     ---------------------------*/
 
 
-#include <base/config.h>
-#include <fe/fe.h>
+#include <deal.II/base/config.h>
+#include <deal.II/fe/fe.h>
 #include <vector>
 #include <utility>
 
@@ -424,26 +424,7 @@ class FESystem : public FiniteElement<dim,spacedim>
     virtual void
     get_interpolation_matrix (const FiniteElement<dim,spacedim> &source,
 			      FullMatrix<double>           &matrix) const;
-
-                                     /**
-				      * Number of different base
-				      * elements of this object.
-				      *
-				      * Since these objects can have
-				      * multiplicity and subobjects
-				      * themselves, this may be
-				      * smaller than the total number
-				      * of finite elements composed
-				      * into this structure.
-				      */
-    virtual unsigned int n_base_elements () const;
-
-				     /**
-				      * How often is a composing
-				      * element used.
-				      */
-    virtual unsigned int element_multiplicity (const unsigned int index) const;
-
+    
 				     /**
 				      * Access to a composing
 				      * element. The index needs to be
@@ -654,7 +635,7 @@ class FESystem : public FiniteElement<dim,spacedim>
 				      * pointers to their base class,
 				      * rather than the class itself.
 				      */
-    virtual unsigned int memory_consumption () const;
+    virtual std::size_t memory_consumption () const;
 
   protected:
 				     /**
@@ -678,16 +659,21 @@ class FESystem : public FiniteElement<dim,spacedim>
 				      */
     virtual FiniteElement<dim,spacedim> * clone() const;
 
-				     /**
-				      * Prepare internal data
-				      * structures and fill in values
-				      * independent of the cell.
-				      */
     virtual typename Mapping<dim,spacedim>::InternalDataBase*
     get_data (const UpdateFlags      update_flags,
 	      const Mapping<dim,spacedim>    &mapping,
 	      const Quadrature<dim> &quadrature) const ;
 
+    virtual typename Mapping<dim,spacedim>::InternalDataBase*
+    get_face_data (const UpdateFlags      update_flags,
+		   const Mapping<dim,spacedim>    &mapping,
+		   const Quadrature<dim-1> &quadrature) const ;
+
+    virtual typename Mapping<dim,spacedim>::InternalDataBase*
+    get_subface_data (const UpdateFlags      update_flags,
+		      const Mapping<dim,spacedim>    &mapping,
+		      const Quadrature<dim-1> &quadrature) const ;
+
 				     /**
 				      * Implementation of the same
 				      * function in
@@ -1263,7 +1249,6 @@ class FESystem : public FiniteElement<dim,spacedim>
 };
 
 
-
 DEAL_II_NAMESPACE_CLOSE
 
 /*----------------------------  fe_system.h  ---------------------------*/
