@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_tools.h 22162 2010-09-26 02:57:21Z bangerth $
+//    $Id: fe_tools.h 23989 2011-08-02 09:47:19Z kronbichler $
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,9 +14,11 @@
 
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/geometry_info.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/geometry_info.h>
+#include <deal.II/base/tensor.h>
+#include <deal.II/base/symmetric_tensor.h>
 
 #include <vector>
 
@@ -24,8 +26,6 @@ DEAL_II_NAMESPACE_OPEN
 
 template <typename number> class FullMatrix;
 template <typename number> class Vector;
-template <int rank, int dim> class SymmetricTensor;
-template <int rank, int dim> class Tensor;
 template <int dim> class Quadrature;
 template <int dim, int spacedim> class FiniteElement;
 template <int dim, int spacedim> class DoFHandler;
@@ -35,10 +35,17 @@ namespace hp
 }
 template <int dim> class FiniteElementData;
 class ConstraintMatrix;
+namespace PETScWrappers
+{
+  namespace MPI
+  {
+    class Vector;
+  }
+}
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
 #include <vector>
 #include <string>
 
@@ -93,6 +100,15 @@ namespace FETools
       virtual FiniteElement<dim,spacedim>*
       get (const unsigned int degree) const = 0;
 
+				       /**
+					* Create a FiniteElement from a
+					* quadrature formula (currently only
+					* implemented for FE_Q) and return a
+					* pointer to it.
+					*/
+
+      virtual FiniteElement<dim,spacedim>*
+      get (const Quadrature<1> &quad) const = 0;
 				       /**
 					* Virtual destructor doing
 					* nothing but making the
@@ -127,6 +143,15 @@ namespace FETools
 					*/
       virtual FiniteElement<FE::dimension,FE::dimension>*
       get (const unsigned int degree) const;
+
+				       /**
+					* Create a FiniteElement from a
+					* quadrature formula (currently only
+					* implemented for FE_Q) and return a
+					* pointer to it.
+					*/
+      virtual FiniteElement<FE::dimension,FE::dimension>*
+      get (const Quadrature<1> &quad) const;
   };
 
 				   /**
@@ -658,8 +683,8 @@ namespace FETools
 				    * vector of values of this function at
 				    * quadrature points as given by the
 				    * second argument. In a sense, this
-				    * function does the opposite of the @p
-				    * compute_projection_from_quadrature_points_matrix
+				    * function does the opposite of the
+				    * FETools::compute_projection_from_quadrature_points_matrix
 				    * function.
 				    */
   template <int dim, int spacedim>
@@ -982,6 +1007,8 @@ namespace FETools
 				const ConstraintMatrix& constraints2,
 				OutVector&              z1_difference);
 
+
+
 				   /**
 				    * $L^2$ projection for
 				    * discontinuous
