@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: mapping.h 23035 2010-12-20 22:37:57Z bangerth $
+//    $Id: mapping.h 23715 2011-05-17 17:26:05Z heltai $
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,12 +13,12 @@
 #define __deal2__mapping_h
 
 
-#include <base/config.h>
-#include <base/tensor.h>
-#include <base/vector_slice.h>
-#include <grid/tria.h>
-#include <dofs/dof_handler.h>
-#include <fe/fe_update_flags.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/tensor.h>
+#include <deal.II/base/vector_slice.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/fe/fe_update_flags.h>
 
 #include <cmath>
 
@@ -245,6 +245,13 @@ class Mapping : public Subscriptor
 				      * the real cell to the point
 				      * @p p_unit on the unit cell
 				      * @p cell and returns @p p_unit.
+				      *
+				      * In the codimension one case,
+				      * this function returns the
+				      * normal projection of the real
+				      * point @p p on the curve or
+				      * surface identified by the @p
+				      * cell.
 				      */
     virtual Point<dim>
     transform_real_to_unit_cell (
@@ -378,7 +385,7 @@ class Mapping : public Subscriptor
 					  * consumption of this
 					  * object.
 					  */
-	virtual unsigned int memory_consumption () const;
+	virtual std::size_t memory_consumption () const;
 
 					 /**
 					  * The determinant of the
