@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: mapping_q.h 22592 2010-11-03 03:34:27Z bangerth $
+//    $Id: mapping_q.h 24167 2011-08-22 16:42:42Z bangerth $
 //
-//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,10 +13,10 @@
 #define __deal2__mapping_q_h
 
 
-#include <base/config.h>
-#include <base/table.h>
-#include <fe/mapping_q1.h>
-#include <fe/fe_q.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/table.h>
+#include <deal.II/fe/mapping_q1.h>
+#include <deal.II/fe/fe_q.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -109,6 +109,13 @@ class MappingQ : public MappingQ1<dim,spacedim>
 				      * Uses Newton iteration and the
 				      * @p transform_unit_to_real_cell
 				      * function.
+				      *
+				      * In the codimension one case,
+				      * this function returns the
+				      * normal projection of the real
+				      * point @p p on the curve or
+				      * surface identified by the @p
+				      * cell.
 				      */
     virtual Point<dim>
     transform_real_to_unit_cell (
@@ -161,7 +168,7 @@ class MappingQ : public MappingQ1<dim,spacedim>
 					  * consumption of this
 					  * object.
 					  */
-	virtual unsigned int memory_consumption () const;
+	virtual std::size_t memory_consumption () const;
 
 					 /**
 					  * Unit normal vectors. Used
