@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: mapping_q1.h 23011 2010-12-19 20:24:14Z bangerth $
+//    $Id: mapping_q1.h 23715 2011-05-17 17:26:05Z heltai $
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,12 +13,12 @@
 #define __deal2__mapping_q1_h
 
 
-#include <base/config.h>
-#include <base/table.h>
-#include <base/qprojector.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_accessor.h>
-#include <fe/mapping.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/table.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/fe/mapping.h>
 
 #include <cmath>
 
@@ -70,6 +70,13 @@ class MappingQ1 : public Mapping<dim,spacedim>
 				      * Uses Newton iteration and the
 				      * @p transform_unit_to_real_cell
 				      * function.
+				      *
+				      * In the codimension one case,
+				      * this function returns the
+				      * normal projection of the real
+				      * point @p p on the curve or
+				      * surface identified by the @p
+				      * cell.
 				      */
     virtual Point<dim>
     transform_real_to_unit_cell (
@@ -164,7 +171,7 @@ class MappingQ1 : public Mapping<dim,spacedim>
 					  * consumption of this
 					  * object.
 					  */
-	virtual unsigned int memory_consumption () const;
+	virtual std::size_t memory_consumption () const;
 
 					 /**
 					  * Values of shape
