@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: filtered_iterator.h 22430 2010-10-23 13:41:09Z heister $
-//    Version: $Name$
+//    $Id: filtered_iterator.h 24518 2011-10-04 00:34:29Z bangerth $
 //
-//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2009 by the deal.II authors
+//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,9 +13,9 @@
 #define __deal2__filtered_iterator_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <grid/tria_iterator_base.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/grid/tria_iterator_base.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -190,6 +189,31 @@ namespace IteratorFilters
 					*/
       const types::subdomain_id_t subdomain_id;
   };
+
+
+
+				   /**
+				    * Filter for iterators that evaluates to
+				    * true if a cell is owned by the current
+				    * processor, i.e., if it is a
+				    * @ref GlossLocallyOwnedCell "locally owned cell".
+				    *
+				    * This class is used in step-32, in
+				    * connection with the methods of the @ref
+				    * distributed module.
+				    *
+				    * @ingroup Iterators
+				    */
+  class LocallyOwnedCell
+  {
+    public:
+				       /**
+					* Evaluation operator. Returns true if
+					* the cell is locally owned.
+					*/
+      template <class Iterator>
+      bool operator () (const Iterator &i) const;
+  };
 }
 
 
@@ -1114,6 +1138,18 @@ namespace IteratorFilters
   {
     return (i->subdomain_id() == subdomain_id);
   }
+
+
+
+// ---------------- IteratorFilters::LocallyOwnedCell ---------
+
+  template <class Iterator>
+  inline
+  bool
+  LocallyOwnedCell::operator () (const Iterator &i) const
+  {
+    return (i->is_locally_owned());
+  }
 }
 
 
