@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: grid_generator.h 22692 2010-11-11 18:35:08Z bangerth $
-//    Version: $Name$
+//    $Id: grid_generator.h 24394 2011-09-23 14:05:12Z bangerth $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,11 +13,11 @@
 #define __deal2__grid_generator_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/point.h>
-#include <base/table.h>
-#include <grid/tria.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/point.h>
+#include <deal.II/base/table.h>
+#include <deal.II/grid/tria.h>
 #include <map>
 
 DEAL_II_NAMESPACE_OPEN
@@ -48,9 +47,6 @@ template <typename number> class SparseMatrix;
  *
  *
  * @ingroup grid
- * @author Wolfgang Bangerth, Ralf Hartmann, Guido Kanschat, Stefan
- * Nauber, Joerg Weimar, Yaqi Wang, Luca Heltai, 1998, 1999, 2000, 2001, 2002,
- * 2003, 2006, 2007, 2008.
  */
 class GridGenerator
 {
@@ -455,7 +451,7 @@ class GridGenerator
                                       * a trapezoid from
                                       * <tt>x=-half_length</tt> to
                                       * <tt>x=+half_length</tt> and
-                                      * from <tt>y=-radius</tt> to
+                                      * from <tt>y=-radius_0</tt> to
                                       * <tt>y=radius_0</tt> at
                                       * <tt>x=-half_length</tt> and
                                       * from <tt>y=-radius_1</tt> to
@@ -584,16 +580,49 @@ class GridGenerator
 				      * that the resulting elements
 				      * have the least aspect ratio.
 				      *
-				      * In 3D, only two different
-				      * numbers are meaningful, 6 for
-				      * a surface based on a
-				      * hexahedron and 12 for the
-				      * rhombic dodecahedron.
+				      * In 3D, only two different numbers are
+				      * meaningful, 6 for a surface based on a
+				      * hexahedron (i.e. 6 panels on the inner
+				      * sphere extruded in radial direction to
+				      * form 6 cells) and 12 for the rhombic
+				      * dodecahedron. These give rise to the
+				      * following meshes upon one refinement:
 				      *
 				      * @image html hypershell3d-6.png
 				      * @image html hypershell3d-12.png
 				      *
-				      * This function is declared to
+				      * Neither of these meshes is
+				      * particularly good since one ends up
+				      * with poorly shaped cells at the inner
+				      * edge upon refinement. For example,
+				      * this is the middle plane of the mesh
+				      * for the <code>n_cells=6</code>:
+				      *
+				      * @image html hyper_shell_6_cross_plane.png
+				      *
+				      * The mesh generated with
+				      * <code>n_cells=6</code> is better but
+				      * still not good. As a consequence, you
+				      * may also specify
+				      * <code>n_cells=96</code> as a third
+				      * option. The mesh generated in this way
+				      * is based on a once refined version of
+				      * the one with <code>n_cells=12</code>,
+				      * where all internal nodes are re-placed
+				      * along a shell somewhere between the
+				      * inner and outer boundary of the
+				      * domain. The following two images
+				      * compare half of the hyper shell for
+				      * <code>n_cells=12</code> and
+				      * <code>n_cells=96</code> (note that the
+				      * doubled radial lines on the cross
+				      * section are artifacts of the
+				      * visualization):
+				      *
+				      * @image html hyper_shell_12_cut.png
+				      * @image html hyper_shell_96_cut.png
+				      *
+				      * @note This function is declared to
 				      * exist for triangulations of
 				      * all space dimensions, but
 				      * throws an error if called in
@@ -642,6 +671,12 @@ class GridGenerator
 				      * the resulting elements have
 				      * the least aspect ratio.
 				      *
+				      * If colorize is set to true, the
+				      * inner, outer, left, and right
+				      * boundary get indicator 0, 1, 2,
+				      * and 3, respectively. Otherwise
+				      * all indicators are set to 0.
+				      *
 				      * @note The triangulation needs to be
 				      * void upon calling this
 				      * function.
@@ -651,7 +686,47 @@ class GridGenerator
 				  const Point<dim>     &center,
 				  const double        inner_radius,
 				  const double        outer_radius,
-				  const unsigned int  n_cells = 0);
+				  const unsigned int  n_cells = 0,
+				  const bool colorize = false);
+
+
+				     /**
+				      * Produce a quarter hyper-shell,
+				      * i.e. the space between two
+				      * circles in two space
+				      * dimensions and the region
+				      * between two spheres in 3d,
+				      * with given inner and outer
+				      * radius and a given number of
+				      * elements for this initial
+				      * triangulation. All components are
+				      * restricted to be positive, so the
+				      * opening angle is 90 degrees.
+				      *
+				      * If the number of
+				      * initial cells is zero (as is
+				      * the default), then it is
+				      * computed adaptively such that
+				      * the resulting elements have
+				      * the least aspect ratio.
+				      *
+				      * If colorize is set to true, the
+				      * inner, outer, left, and right
+				      * boundary get indicator 0, 1, 2,
+				      * and 3, respectively. Otherwise
+				      * all indicators are set to 0.
+				      *
+					  * @note The triangulation needs to be
+				      * void upon calling this
+				      * function. Only implemented in 2d so far.
+				      */
+    template <int dim>
+    static void quarter_hyper_shell (Triangulation<dim>   &tria,
+				  const Point<dim>     &center,
+				  const double        inner_radius,
+				  const double        outer_radius,
+				  const unsigned int  n_cells = 0,
+				  const bool colorize = false);
 
 				     /**
 				      * Produce a domain that is the space
@@ -683,6 +758,33 @@ class GridGenerator
                                 const unsigned int  n_radial_cells = 0,
                                 const unsigned int  n_axial_cells = 0);
 
+
+
+                                     /**
+				      * Produce the surface meshing of the
+				      * torus. The axis of the torus is the
+				      * $y$-axis while the plane of the torus
+				      * is the $x$-$z$ plane. The boundary of
+				      * this object can be described by the
+				      * TorusBoundary class.
+				      *
+				      * @param tria The triangulation to be
+				      * filled.
+				      *
+				      * @param R The radius of the circle,
+				      * which forms the middle line of the
+				      * torus containing the loop of
+				      * cells. Must be greater than @p r.
+				      *
+				      * @param r The inner radius of the
+				      * torus.
+				      */
+
+    static void torus (Triangulation<2,3>&  tria,
+		       const double         R,
+		       const double         r);
+
+
 				     /**
 				      * This class produces a square
 				      * on the <i>xy</i>-plane with a
@@ -739,6 +841,67 @@ class GridGenerator
 			 const double         r);
 
 				     /**
+				      * Given the two triangulations
+				      * specified as the first two
+				      * arguments, create the
+				      * triangulation that contains
+				      * the cells of both
+				      * triangulation and store it in
+				      * the third parameter. Previous
+				      * content of @p result will be
+				      * deleted.
+				      *
+				      * This function is most often used
+				      * to compose meshes for more
+				      * complicated geometries if the
+				      * geometry can be composed of
+				      * simpler parts for which functions
+				      * exist to generate coarse meshes.
+				      * For example, the channel mesh used
+				      * in step-35 could in principle be
+				      * created using a mesh created by the
+				      * GridGenerator::hyper_cube_with_cylindrical_hole
+				      * function and several rectangles,
+				      * and merging them using the current
+				      * function. The rectangles will
+				      * have to be translated to the
+				      * right for this, a task that can
+				      * be done using the GridTools::shift
+				      * function (other tools to transform
+				      * individual mesh building blocks are
+				      * GridTools::transform, GridTools::rotate,
+				      * and GridTools::scale).
+				      *
+				      * @note The two input triangulations
+				      * must be coarse meshes that have
+				      * no refined cells.
+				      *
+				      * @note The function copies the material ids
+				      * of the cells of the two input
+				      * triangulations into the output
+				      * triangulation but it currently makes
+				      * no attempt to do the same for boundary
+				      * ids. In other words, if the two
+				      * coarse meshes have anything but
+				      * the default boundary indicators,
+				      * then you will currently have to set
+				      * boundary indicators again by hand
+				      * in the output triangulation.
+				      *
+				      * @note For a related operation
+				      * on refined meshes when both
+				      * meshes are derived from the
+				      * same coarse mesh, see
+				      * GridTools::create_union_triangulation .
+				      */
+    template <int dim, int spacedim>
+    static
+    void
+    merge_triangulations (const Triangulation<dim, spacedim> &triangulation_1,
+			  const Triangulation<dim, spacedim> &triangulation_2,
+			  Triangulation<dim, spacedim>       &result);
+
+                                     /**
 				      * This function transformes the
 				      * @p Triangulation @p tria
 				      * smoothly to a domain that is
