@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: grid_out.h 22595 2010-11-03 11:30:11Z bangerth $
-//    Version: $Name$
+//    $Id: grid_out.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010 by the deal.II authors
 //
@@ -15,11 +14,11 @@
 
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/point.h>
-#include <grid/tria.h>
-#include <fe/mapping.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/point.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/fe/mapping.h>
 
 #include <string>
 
@@ -1225,7 +1224,7 @@ class GridOut
 				      * memory consumption (in bytes)
 				      * of this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
 				     /**
 				      * Exception
