@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: grid_reordering.h 21377 2010-06-25 19:52:09Z bangerth $
-//    Version: $Name$
+//    $Id: grid_reordering.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
@@ -14,8 +13,8 @@
 #define __deal2__grid_reordering_h
 
 
-#include <base/config.h>
-#include <grid/tria.h>
+#include <deal.II/base/config.h>
+#include <deal.II/grid/tria.h>
 
 #include <vector>
 
