@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: tria_accessor.h 23002 2010-12-18 13:29:36Z bangerth $
-//    Version: $Name$
+//    $Id: tria_accessor.h 24303 2011-09-11 05:10:03Z bangerth $
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,11 +13,12 @@
 #define __deal2__tria_accessor_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/geometry_info.h>
-#include <grid/tria_iterator_base.h>
-#include <grid/tria_iterator_selector.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/geometry_info.h>
+#include <deal.II/base/point.h>
+#include <deal.II/grid/tria_iterator_base.h>
+#include <deal.II/grid/tria_iterator_selector.h>
 
 
 namespace std
@@ -29,8 +29,6 @@ namespace std
 
 DEAL_II_NAMESPACE_OPEN
 
-template <int dim> class Point;
-
 template <int dim, int spacedim> class Triangulation;
 template <typename Accessor> class TriaRawIterator;
 template <typename Accessor> class TriaIterator;
@@ -450,6 +448,24 @@ class TriaAccessorBase
 				      *  Return the index of the
 				      *  element presently pointed to
 				      *  on the present level.
+				      *
+				      * Note that these indices are not
+				      * globally unique for cells (though they
+				      * may be for faces or edges). Rather, a
+				      * cell is identified by its refinement
+				      * level and index within this refinement
+				      * level, the latter piece of information
+				      * being what this function
+				      * returns. Consequently, there may be
+				      * multiple cells on different refinement
+				      * levels but with the same index within
+				      * their level.
+				      *
+				      * Similarly, the index returned by this
+				      * function is not a contiguous set of
+				      * numbers of each level: going from cell
+				      * to cell, some of the indices in a
+				      * level may be unused.
 				      */
     int index () const;
 
@@ -1563,8 +1579,8 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 
     template <int, int> friend class Triangulation;
 
-    friend class internal::Triangulation::Implementation;
-    friend class internal::TriaAccessor::Implementation;
+    friend struct internal::Triangulation::Implementation;
+    friend struct internal::TriaAccessor::Implementation;
 };
 
 
@@ -1832,6 +1848,14 @@ class TriaAccessor<0, 1, spacedim>
 				      */
     Point<spacedim> & vertex (const unsigned int i = 0) const;
 
+				     /**
+				      * Return the center of this object,
+				      * which of course co-incides with the
+				      * location of the vertex this object
+				      * refers to.
+				      */
+    Point<spacedim> center () const;
+
 				     /**
 				      * Pointer to the @p ith line
 				      * bounding this object. Will
@@ -1996,17 +2020,40 @@ class TriaAccessor<0, 1, spacedim>
     int isotropic_child_index (const unsigned int i);
 
 				     /**
-				      * @brief Do nothing and throw and error
+				      * Set the boundary indicator.
+				      * The same applies as for the
+				      * <tt>boundary_indicator()</tt>
+				      * function.
+				      *
+				      * @warning You should never set the
+				      * boundary indicator of an interior face
+				      * (a face not at the boundary of the
+				      * domain), or set set the boundary
+				      * indicator of an exterior face to 255
+				      * (this value is reserved for another
+				      * purpose). Algorithms may not work or
+				      * produce very confusing results if
+				      * boundary cells have a boundary
+				      * indicator of 255 or if interior cells
+				      * have boundary indicators other than
+				      * 255. Unfortunately, the current object
+				      * has no means of finding out whether it
+				      * really is at the boundary of the
+				      * domain and so cannot determine whether
+				      * the value you are trying to set makes
+				      * sense under the current circumstances.
 				      */
-    static
     void
     set_boundary_indicator (const unsigned char);
 
 				     /**
-				      * @brief Do nothing and throw and error
+				      * Since this object only represents a
+				      * single vertex, call
+				      * set_boundary_indicator with the same
+				      * argument.
 				      */
-    static
-    void set_all_boundary_indicators (const unsigned char);
+    void
+    set_all_boundary_indicators (const unsigned char);
 
 				     /**
 				      * Return whether the vertex
@@ -2536,6 +2583,10 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * For a typical use of this
 				      * function, see the @ref step_28
 				      * "step-28" tutorial program.
+				      *
+				      * See the @ref GlossMaterialId
+				      * "glossary" for more
+				      * information.
 				      */
     unsigned char material_id () const;
 
@@ -2546,6 +2597,10 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * For a typical use of this
 				      * function, see the @ref step_28
 				      * "step-28" tutorial program.
+				      *
+				      * See the @ref GlossMaterialId
+				      * "glossary" for more
+				      * information.
 				      */
     void set_material_id (const unsigned char new_material_id) const;
 
@@ -2554,6 +2609,10 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * cell and all its children (and
 				      * grand-children, and so on) to
 				      * the given value.
+				      *
+				      * See the @ref GlossMaterialId
+				      * "glossary" for more
+				      * information.
 				      */
     void recursively_set_material_id (const unsigned char new_material_id) const;
 				     /**
@@ -2660,6 +2719,23 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      */
     bool active () const;
 
+    /**
+     * Return whether this cell is owned by the current processor
+     * or is owned by another processor. The function always returns
+     * true if applied to an object of type dealii::Triangulation,
+     * but may yield false if the triangulation is of type
+     * parallel::distributed::Triangulation.
+     *
+     * See the @ref GlossGhostCell
+     * "glossary" and the @ref
+     * distributed module for more
+     * information.
+     * 
+     * @post The returned value is equal to <code>!is_ghost() &&
+     * !is_artificial()</code>.
+     **/
+    bool is_locally_owned () const;
+    
 				     /**
 				      * Return whether this cell
 				      * exists in the global mesh but
@@ -2682,6 +2758,10 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * "glossary" and the @ref
 				      * distributed module for more
 				      * information.
+				      *
+				      * @post The returned value is equal to
+				      * <code>!is_locally_owned() &&
+				      * !is_artificial()</code>.
 				      */
     bool is_ghost () const;
 
@@ -2716,6 +2796,10 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * GlossArtificialCell "glossary"
 				      * and the @ref distributed
 				      * module for more information.
+				      *
+				      * @post The returned value is equal to
+				      * <code>!is_ghost() &&
+				      * !is_artificial()</code>.
 				      */
     bool is_artificial () const;
 
@@ -2843,8 +2927,7 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 
     template <int, int> friend class Triangulation;
 
-    friend class internal::Triangulation::Implementation;
-
+    friend struct internal::Triangulation::Implementation;
 };
 
 
