@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: tria_accessor.templates.h 23054 2010-12-21 08:48:10Z heister $
-//    Version: $Name$
+//    $Id: tria_accessor.templates.h 24303 2011-09-11 05:10:03Z bangerth $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,16 +13,16 @@
 #define __deal2__tria_accessor_templates_h
 
 
-#include <base/config.h>
-#include <base/geometry_info.h>
-#include <base/template_constraints.h>
-#include <grid/tria.h>
-#include <grid/tria_levels.h>
-#include <grid/tria_faces.h>
-#include <grid/tria_iterator.h>
-#include <grid/tria_accessor.h>
-#include <grid/tria_iterator.templates.h>
-#include <distributed/tria.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/geometry_info.h>
+#include <deal.II/base/template_constraints.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_levels.h>
+#include <deal.II/grid/tria_faces.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/grid/tria_iterator.templates.h>
+#include <deal.II/distributed/tria.h>
 
 #include <cmath>
 
@@ -2143,6 +2142,17 @@ TriaAccessor<0, 1, spacedim>::vertex (const unsigned int i) const
 }
 
 
+
+template <int spacedim>
+inline
+Point<spacedim>
+TriaAccessor<0, 1, spacedim>::center () const
+{
+  return this->tria->vertices[global_vertex_index];
+}
+
+
+
 template <int spacedim>
 inline
 typename internal::Triangulation::Iterators<1,spacedim>::line_iterator
@@ -2199,9 +2209,15 @@ TriaAccessor<0, 1, spacedim>::boundary_indicator () const
   switch (vertex_kind)
     {
       case left_vertex:
-	    return 0;
       case right_vertex:
-	    return 1;
+      {
+	Assert (tria->vertex_to_boundary_id_map_1d->find (this->vertex_index())
+		!= tria->vertex_to_boundary_id_map_1d->end(),
+		ExcInternalError());
+
+	return (*tria->vertex_to_boundary_id_map_1d)[this->vertex_index()];
+      }
+
       default:
 	    return 255;
     }
@@ -2324,22 +2340,22 @@ int TriaAccessor<0, 1, spacedim>::isotropic_child_index (const unsigned int)
 template <int spacedim>
 inline
 void
-TriaAccessor<0, 1, spacedim>::set_boundary_indicator (const unsigned char)
+TriaAccessor<0, 1, spacedim>::set_boundary_indicator (const unsigned char b)
 {
-  Assert(false,
-	 ExcMessage ("The boundary indicator of vertices is determined by their "
-		     "location within a triangulation and can not be set explicitly"));
+  Assert (tria->vertex_to_boundary_id_map_1d->find (this->vertex_index())
+	  != tria->vertex_to_boundary_id_map_1d->end(),
+	  ExcInternalError());
+
+  (*tria->vertex_to_boundary_id_map_1d)[this->vertex_index()] = b;
 }
 
 
 
 template <int spacedim>
 inline
-void TriaAccessor<0, 1, spacedim>::set_all_boundary_indicators (const unsigned char)
+void TriaAccessor<0, 1, spacedim>::set_all_boundary_indicators (const unsigned char b)
 {
-  Assert(false,
-	 ExcMessage ("The boundary indicator of vertices is determined by their "
-		     "location within a triangulation and can not be set explicitly"));
+  set_boundary_indicator (b);
 }
 
 
@@ -2590,7 +2606,6 @@ inline
 internal::SubfaceCase<1>
 CellAccessor<1>::subface_case(const unsigned int) const
 {
-  Assert(false, ExcImpossibleInDim(1));
   return internal::SubfaceCase<1>::case_none;
 }
 
@@ -2599,7 +2614,6 @@ inline
 internal::SubfaceCase<1>
 CellAccessor<1,2>::subface_case(const unsigned int) const
 {
-  Assert(false, ExcImpossibleInDim(1));
   return internal::SubfaceCase<1>::case_none;
 }
 
@@ -2612,7 +2626,9 @@ CellAccessor<2>::subface_case(const unsigned int face_no) const
   Assert(active(), TriaAccessorExceptions::ExcCellNotActive());
   Assert(face_no<GeometryInfo<2>::faces_per_cell,
 	 ExcIndexRange(face_no,0,GeometryInfo<2>::faces_per_cell));
-  return (face(face_no)->has_children()) ? internal::SubfaceCase<2>::case_x : internal::SubfaceCase<2>::case_none;
+  return ((face(face_no)->has_children()) ?
+	  internal::SubfaceCase<2>::case_x :
+	  internal::SubfaceCase<2>::case_none);
 }
 
 template <>
@@ -2623,7 +2639,9 @@ CellAccessor<2,3>::subface_case(const unsigned int face_no) const
   Assert(active(), TriaAccessorExceptions::ExcCellNotActive());
   Assert(face_no<GeometryInfo<2>::faces_per_cell,
 	 ExcIndexRange(face_no,0,GeometryInfo<2>::faces_per_cell));
-  return (face(face_no)->has_children()) ? internal::SubfaceCase<2>::case_x : internal::SubfaceCase<2>::case_none;
+  return ((face(face_no)->has_children()) ?
+	  internal::SubfaceCase<2>::case_x :
+	  internal::SubfaceCase<2>::case_none);
 }
 
 
@@ -2788,6 +2806,29 @@ CellAccessor<dim,spacedim>::active () const
 
 
 
+template <int dim, int spacedim>
+inline
+bool
+CellAccessor<dim,spacedim>::is_locally_owned () const
+{
+#ifndef DEAL_II_USE_P4EST
+  return true;
+#else
+  const types::subdomain_id_t subdomain = this->subdomain_id();
+  if (subdomain == types::artificial_subdomain_id)
+    return false;
+
+  const parallel::distributed::Triangulation<dim,spacedim> *pdt
+    = dynamic_cast<const parallel::distributed::Triangulation<dim,spacedim> *>(this->tria);
+
+  if (pdt == 0)
+    return true;
+  else
+    return (subdomain == pdt->locally_owned_subdomain());
+#endif
+}
+
+
 template <int dim, int spacedim>
 inline
 bool
