@@ -1,8 +1,7 @@
 //----------------------------------------------------------------------
-//    $Id: dof_handler.h 22749 2010-11-16 13:50:47Z kronbichler $
-//    Version: $Name$
+//    $Id: dof_handler.h 24128 2011-08-21 00:00:54Z bangerth $
 //
-//    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,14 +14,14 @@
 
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/template_constraints.h>
-#include <base/smartpointer.h>
-#include <dofs/function_map.h>
-#include <dofs/dof_iterator_selector.h>
-#include <dofs/number_cache.h>
-#include <hp/fe_collection.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/template_constraints.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/dofs/function_map.h>
+#include <deal.II/dofs/dof_iterator_selector.h>
+#include <deal.II/dofs/number_cache.h>
+#include <deal.II/hp/fe_collection.h>
 
 #include <vector>
 #include <map>
@@ -74,8 +73,7 @@ namespace hp
  * @ingroup hp
  */
   template <int dim, int spacedim=dim>
-  class DoFHandler : public Subscriptor,
-                     protected Triangulation<dim,spacedim>::RefinementListener
+  class DoFHandler : public Subscriptor
   {
       typedef internal::DoFHandler::Iterators<DoFHandler<dim,spacedim> > IteratorSelector;
     public:
@@ -984,7 +982,7 @@ namespace hp
                                         * although the actual object
                                         * might be a derived class.
                                         */
-      virtual unsigned int memory_consumption () const;
+      virtual std::size_t memory_consumption () const;
 
                                        /**
                                         * Exception
@@ -1125,16 +1123,18 @@ namespace hp
       void create_active_fe_table ();
 
                                        /**
-                                        *  Methods of the
-                                        *  RefinementListener coming
-                                        *  from the Triangulation.
-                                        *  Here they are used to
+                                        *  Functions that will be triggered
+					*  through signals whenever the
+					*  triangulation is modified.
+					*
+					*  Here they are used to
                                         *  administrate the the
                                         *  active_fe_fields during the
                                         *  spatial refinement.
                                         */
-      virtual void pre_refinement_notification (const Triangulation<dim,spacedim> &tria);
-      virtual void post_refinement_notification (const Triangulation<dim,spacedim> &tria);
+      void pre_refinement_action ();
+      void post_refinement_action ();
+
 
 				       /**
 					* Compute identities between
@@ -1289,13 +1289,20 @@ namespace hp
                                         */
       std::vector<std::vector<bool> *> has_children;
 
+      /**
+       * A list of connections with which this object connects
+       * to the triangulation to get information about when the
+       * triangulation changes.
+       */
+      std::vector<boost::signals2::connection> tria_listeners;
+
                                        /**
                                         * Make accessor objects friends.
                                         */
       template <int, class> friend class dealii::DoFAccessor;
       template <class> friend class dealii::DoFCellAccessor;
-      friend class internal::DoFAccessor::Implementation;
-      friend class internal::DoFCellAccessor::Implementation;
+      friend struct internal::DoFAccessor::Implementation;
+      friend struct internal::DoFCellAccessor::Implementation;
 
                                        /**
                                         * Likewise for DoFLevel
@@ -1306,7 +1313,7 @@ namespace hp
                                         */
       template <int> friend class internal::hp::DoFLevel;
       template <int> friend class internal::hp::DoFObjects;
-      friend class internal::hp::DoFHandler::Implementation;
+      friend struct internal::hp::DoFHandler::Implementation;
   };
 
 
