@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_objects.h 22189 2010-09-29 13:05:10Z heister $
-//    Version: $Name$
+//    $Id: dof_objects.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2006, 2008 by the deal.II authors
 //
@@ -13,9 +12,9 @@
 #ifndef __deal2__hp_dof_objects_h
 #define __deal2__hp_dof_objects_h
 
-#include <base/config.h>
-#include <hp/fe_collection.h>
-#include <hp/dof_handler.h>
+#include <deal.II/base/config.h>
+#include <deal.II/hp/fe_collection.h>
+#include <deal.II/hp/dof_handler.h>
 
 #include <vector>
 
@@ -235,7 +234,7 @@ namespace internal
                                           * memory consumption (in bytes)
                                           * of this object.
                                           */
-        unsigned int memory_consumption () const;
+        std::size_t memory_consumption () const;
     };
 
 
