@@ -1,6 +1,5 @@
 //----------------------------  hp_fe_values.h  ---------------------------
-//    $Id: fe_values.h 22592 2010-11-03 03:34:27Z bangerth $
-//    Version: $Name$
+//    $Id: fe_values.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2003, 2004, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
@@ -13,15 +12,15 @@
 #ifndef __deal2__hp_fe_values_h
 #define __deal2__hp_fe_values_h
 
-#include <base/config.h>
-#include <fe/fe.h>
-#include <hp/fe_collection.h>
-#include <hp/q_collection.h>
-#include <hp/mapping_collection.h>
-#include <fe/fe_values.h>
+#include <deal.II/base/config.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/hp/fe_collection.h>
+#include <deal.II/hp/q_collection.h>
+#include <deal.II/hp/mapping_collection.h>
+#include <deal.II/fe/fe_values.h>
 
 #include <map>
-#include <base/std_cxx1x/shared_ptr.h>
+#include <deal.II/base/std_cxx1x/shared_ptr.h>
 
 DEAL_II_NAMESPACE_OPEN
 
