@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: elasticity.h 22880 2010-11-29 17:17:26Z kanschat $
+//    $Id: elasticity.h 24271 2011-09-06 15:56:41Z kanschat $
 //
-//    Copyright (C) 2010 by the deal.II authors
+//    Copyright (C) 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,13 +13,13 @@
 #define __deal2__integrators_elasticity_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/quadrature.h>
-#include <lac/full_matrix.h>
-#include <fe/mapping.h>
-#include <fe/fe_values.h>
-#include <numerics/mesh_worker_info.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/fe/mapping.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/meshworker/dof_info.h>
 
 DEAL_II_NAMESPACE_OPEN
 
