@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: block_list.h 23142 2011-01-07 00:48:27Z kanschat $
+//    $Id: block_list.h 24458 2011-09-28 23:05:21Z bangerth $
 //
 //    Copyright (C) 2010, 2011 by the deal.II authors
 //
@@ -13,17 +13,18 @@
 #define __deal2__block_list_h
 
 
-#include <base/subscriptor.h>
-#include <base/template_constraints.h>
-#include <fe/fe.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/template_constraints.h>
+#include <deal.II/lac/sparsity_pattern.h>
+#include <deal.II/fe/fe.h>
 
 #include <vector>
 
 DEAL_II_NAMESPACE_OPEN
 
 /**
- * @warning This class is still experimental and will most likely be
- * changed in a future release.
+ * @deprecated This class is experimental and will be
+ * removed in a future release.
  *
  * A vector of index sets listing the indices of small blocks of a
  * linear system. For each block, the indices in that block are
@@ -52,6 +53,20 @@ class BlockList :
 				     /// The iterator for individual indices
     typedef block_container::const_iterator const_iterator;
 
+				     /**
+				      * Since SparsityPattern can
+				      * handle the tasks of BlockList,
+				      * this function allows us to
+				      * create one from an already
+				      * filled BlockList. A first step
+				      * to make BlockList obsolete.
+				      *
+				      * The additional integer
+				      * argument is the dimension of
+				      * the vector space.
+				      */
+    void create_sparsity_pattern(SparsityPattern& sparsity, unsigned int n) const;
+
 				     /**
 				      * Add the indices in
 				      * <tt>indices</tt> to block
@@ -59,7 +74,7 @@ class BlockList :
 				      * repeated indices.
 				      */
     void add(unsigned int block, const std::vector<unsigned int>& indices);
-    
+
 				     /**
 				      * Add the indices in
 				      * <tt>indices</tt> to block
@@ -71,7 +86,13 @@ class BlockList :
     void add(unsigned int block,
 	     const std::vector<unsigned int>& indices,
 	     const std::vector<bool>& selected_indices);
-    
+
+				     /**
+				      * Just set up the correct size
+				      * and assign indices to blocks later.
+				      */
+    void initialize(unsigned int n_blocks);
+
 				     /**
 				      * Set up all index sets using an
 				      * DoF iterator range. This
@@ -91,7 +112,7 @@ class BlockList :
     void initialize(unsigned int n_blocks,
 		    const ITERATOR begin,
 		    const typename identity<ITERATOR>::type end);
-    
+
 				     /**
 				      * @deprecated This function will
 				      * move to DoFTools.
@@ -114,7 +135,7 @@ class BlockList :
     void initialize_mg(unsigned int n_blocks,
 		       const ITERATOR begin,
 		       const typename identity<ITERATOR>::type end);
-    
+
 				     /**
 				      * @deprecated This function will
 				      * move to DoFTools.
@@ -229,12 +250,12 @@ class BlockList :
     template <int dim, typename ITERATOR>
     bool cell_generates_vertex_patch(const ITERATOR cell,
 				     bool same_level_only) const;
-    
+
 				     /**
 				      * The number of blocks.
 				      */
     unsigned int size() const;
-    
+
 				     /**
 				      * The size of a single block.
 				      */
@@ -256,7 +277,7 @@ class BlockList :
 				      * if the index is not in the block.
 				      */
     unsigned int local_index(unsigned int block, unsigned int index) const;
-    
+
   private:
 				     /**
 				      * The container for t he index sets.
@@ -265,12 +286,30 @@ class BlockList :
 };
 
 
+inline
+void
+BlockList::create_sparsity_pattern(SparsityPattern& sparsity, unsigned int n) const
+{
+  std::vector<unsigned int> sizes(size());
+  for (unsigned int b=0;b<size();++b)
+    sizes[b] = block_size(b);
+
+  sparsity.reinit(size(), n, sizes);
+  for (unsigned int b=0;b<size();++b)
+    {
+      for (const_iterator i = begin(b); i != end(b); ++i)
+	sparsity.add(b,*i);
+    }
+  sparsity.compress();
+}
+
+
 inline
 void
 BlockList::add(const unsigned int block, const std::vector<unsigned int>& indices)
 {
   AssertIndexRange(block, index_sets.size());
-  
+
   for (unsigned int i=0;i<indices.size();++i)
     {
       const unsigned int k = indices[i];
@@ -292,7 +331,7 @@ BlockList::add(
 {
   AssertIndexRange(block, index_sets.size());
   AssertDimension(indices.size(), selected.size());
-  
+
   for (unsigned int i=0;i<indices.size();++i)
     {
       const unsigned int k = indices[i];
@@ -305,6 +344,14 @@ BlockList::add(
 }
 
 
+inline
+void
+BlockList::initialize(unsigned int n_blocks)
+{
+  index_sets.resize(n_blocks);
+}
+
+
 template <typename ITERATOR>
 inline
 void
@@ -356,7 +403,7 @@ BlockList::initialize(
     {
       indices.resize(cell->get_fe().dofs_per_cell);
       AssertDimension(selected_dofs.size(), cell->get_fe().dofs_per_cell);
-      
+
       cell->get_dof_indices(indices);
       add(k, indices, selected_dofs);
     }
@@ -380,7 +427,7 @@ BlockList::initialize_mg(
     {
       indices.resize(cell->get_fe().dofs_per_cell);
       AssertDimension(selected_dofs.size(), cell->get_fe().dofs_per_cell);
-      
+
       cell->get_mg_dof_indices(indices);
       add(k, indices, selected_dofs);
     }
@@ -455,7 +502,7 @@ BlockList::initialize_vertex_patches_mg(
   Assert(offset==0, ExcNotImplemented());
   const FiniteElement<dim>& fe = begin->get_fe();
   Assert(fe.dofs_per_vertex == 0, ExcNotImplemented());
-  
+
   index_sets.resize(n_blocks);
   std::vector<unsigned int> indices;
   unsigned int k = 0;
@@ -472,32 +519,32 @@ BlockList::initialize_vertex_patches_mg(
 		    for (unsigned int i=0;i<fe.dofs_per_face;++i)
 		      {
 			indices[fe.face_to_cell_index(i,1)] = numbers::invalid_unsigned_int;
-			indices[fe.face_to_cell_index(i,3)] = numbers::invalid_unsigned_int;			
-			indices[fe.face_to_cell_index(i,4)] = numbers::invalid_unsigned_int;			
-		      }		    
+			indices[fe.face_to_cell_index(i,3)] = numbers::invalid_unsigned_int;
+			indices[fe.face_to_cell_index(i,4)] = numbers::invalid_unsigned_int;
+		      }
 		    add(k, indices);
 		    cell->neighbor(4)->neighbor(0)->get_mg_dof_indices(indices);
 		    for (unsigned int i=0;i<fe.dofs_per_face;++i)
 		      {
 			indices[fe.face_to_cell_index(i,0)] = numbers::invalid_unsigned_int;
-			indices[fe.face_to_cell_index(i,3)] = numbers::invalid_unsigned_int;			
-			indices[fe.face_to_cell_index(i,4)] = numbers::invalid_unsigned_int;			
+			indices[fe.face_to_cell_index(i,3)] = numbers::invalid_unsigned_int;
+			indices[fe.face_to_cell_index(i,4)] = numbers::invalid_unsigned_int;
 		      }
 		    add(k, indices);
 		    cell->neighbor(4)->neighbor(2)->get_mg_dof_indices(indices);
 		    for (unsigned int i=0;i<fe.dofs_per_face;++i)
 		      {
 			indices[fe.face_to_cell_index(i,1)] = numbers::invalid_unsigned_int;
-			indices[fe.face_to_cell_index(i,2)] = numbers::invalid_unsigned_int;			
-			indices[fe.face_to_cell_index(i,4)] = numbers::invalid_unsigned_int;			
+			indices[fe.face_to_cell_index(i,2)] = numbers::invalid_unsigned_int;
+			indices[fe.face_to_cell_index(i,4)] = numbers::invalid_unsigned_int;
 		      }
 		    add(k, indices);
 		    cell->neighbor(4)->neighbor(2)->neighbor(0)->get_mg_dof_indices(indices);
 		    for (unsigned int i=0;i<fe.dofs_per_face;++i)
 		      {
 			indices[fe.face_to_cell_index(i,0)] = numbers::invalid_unsigned_int;
-			indices[fe.face_to_cell_index(i,2)] = numbers::invalid_unsigned_int;			
-			indices[fe.face_to_cell_index(i,4)] = numbers::invalid_unsigned_int;			
+			indices[fe.face_to_cell_index(i,2)] = numbers::invalid_unsigned_int;
+			indices[fe.face_to_cell_index(i,4)] = numbers::invalid_unsigned_int;
 		      }
 		    add(k, indices);
 	      case 2:
@@ -505,18 +552,18 @@ BlockList::initialize_vertex_patches_mg(
 		    for (unsigned int i=0;i<fe.dofs_per_face;++i)
 		      {
 			indices[fe.face_to_cell_index(i,1)] = numbers::invalid_unsigned_int;
-			indices[fe.face_to_cell_index(i,2)] = numbers::invalid_unsigned_int;			
+			indices[fe.face_to_cell_index(i,2)] = numbers::invalid_unsigned_int;
 			if (dim>2)
-			  indices[fe.face_to_cell_index(i,5)] = numbers::invalid_unsigned_int;			
+			  indices[fe.face_to_cell_index(i,5)] = numbers::invalid_unsigned_int;
 		      }
 		    add(k, indices);
 		    cell->neighbor(2)->neighbor(0)->get_mg_dof_indices(indices);
 		    for (unsigned int i=0;i<fe.dofs_per_face;++i)
 		      {
 			indices[fe.face_to_cell_index(i,0)] = numbers::invalid_unsigned_int;
-			indices[fe.face_to_cell_index(i,2)] = numbers::invalid_unsigned_int;			
+			indices[fe.face_to_cell_index(i,2)] = numbers::invalid_unsigned_int;
 			if (dim>2)
-			  indices[fe.face_to_cell_index(i,5)] = numbers::invalid_unsigned_int;			
+			  indices[fe.face_to_cell_index(i,5)] = numbers::invalid_unsigned_int;
 		      }
 		    add(k, indices);
 						     // no break here
@@ -526,9 +573,9 @@ BlockList::initialize_vertex_patches_mg(
 		      {
 			indices[fe.face_to_cell_index(i,1)] = numbers::invalid_unsigned_int;
 			if (dim>1)
-			  indices[fe.face_to_cell_index(i,3)] = numbers::invalid_unsigned_int;			
+			  indices[fe.face_to_cell_index(i,3)] = numbers::invalid_unsigned_int;
 			if (dim>2)
-			  indices[fe.face_to_cell_index(i,5)] = numbers::invalid_unsigned_int;			
+			  indices[fe.face_to_cell_index(i,5)] = numbers::invalid_unsigned_int;
 		      }
 		    add(k, indices);
 		    cell->neighbor(0)->get_mg_dof_indices(indices);
@@ -536,9 +583,9 @@ BlockList::initialize_vertex_patches_mg(
 		      {
 			indices[fe.face_to_cell_index(i,0)] = numbers::invalid_unsigned_int;
 			if (dim>1)
-			  indices[fe.face_to_cell_index(i,3)] = numbers::invalid_unsigned_int;			
+			  indices[fe.face_to_cell_index(i,3)] = numbers::invalid_unsigned_int;
 			if (dim>2)
-			  indices[fe.face_to_cell_index(i,5)] = numbers::invalid_unsigned_int;			
+			  indices[fe.face_to_cell_index(i,5)] = numbers::invalid_unsigned_int;
 		      }
 		    add(k, indices);
 		    break;
