@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: block_matrix_array.h 22200 2010-09-30 00:29:05Z kanschat $
+//    $Id: block_matrix_array.h 24150 2011-08-21 04:01:52Z heister $
 //
 //    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
@@ -12,12 +12,12 @@
 #ifndef __deal2__block_matrix_array_h
 #define __deal2__block_matrix_array_h
 
-#include <base/config.h>
-#include <base/subscriptor.h>
-#include <base/table.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/table.h>
 
-#include <lac/pointer_matrix.h>
-#include <lac/vector_memory.h>
+#include <deal.II/lac/pointer_matrix.h>
+#include <deal.II/lac/vector_memory.h>
 
 #include <vector>
 #include <map>
@@ -585,12 +585,12 @@ class BlockTrianglePrecondition
 				     /**
 				      * Make function of base class available.
 				      */
-    BlockMatrixArray<number>::print_latex;
+    using BlockMatrixArray<number>::print_latex;
 
 				     /**
 				      * Make function of base class available.
 				      */
-    BlockMatrixArray<number>::n_block_rows;
+    using BlockMatrixArray<number>::n_block_rows;
 
 				     /**
 				      * Make function of base class available.
