@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: block_matrix_base.h 22430 2010-10-23 13:41:09Z heister $
-//    Version: $Name$
+//    $Id: block_matrix_base.h 23944 2011-07-12 12:23:28Z bangerth $
 //
-//    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,14 +13,15 @@
 #define __deal2__block_matrix_base_h
 
 
-#include <base/config.h>
-#include <base/table.h>
-#include <base/smartpointer.h>
-#include <base/memory_consumption.h>
-#include <lac/block_indices.h>
-#include <lac/exceptions.h>
-#include <lac/full_matrix.h>
-#include <lac/matrix_iterator.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/table.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/lac/block_indices.h>
+#include <deal.II/lac/exceptions.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/matrix_iterator.h>
 
 #include <cmath>
 
@@ -371,7 +371,7 @@ class BlockMatrixBase : public Subscriptor
     typedef const value_type       *const_pointer;
     typedef value_type             &reference;
     typedef const value_type       &const_reference;
-    typedef size_t                  size_type;
+    typedef std::size_t             size_type;
 
     typedef
     MatrixIterator<BlockMatrixIterators::Accessor<BlockMatrixBase, false> >
@@ -907,7 +907,7 @@ class BlockMatrixBase : public Subscriptor
 					* returned in case this is called in
 					* an MPI-based program.
 					*/
-      unsigned int memory_consumption () const;
+      std::size_t memory_consumption () const;
     
       				     /** @addtogroup Exceptions
 				      * @{ */
@@ -1722,10 +1722,10 @@ copy_from (const BlockMatrixType &source)
 
 
 template <class MatrixType>
-unsigned int
+std::size_t
 BlockMatrixBase<MatrixType>::memory_consumption () const
 {
-  unsigned int mem =
+  std::size_t mem =
     MemoryConsumption::memory_consumption(row_block_indices)+
     MemoryConsumption::memory_consumption(column_block_indices)+
     MemoryConsumption::memory_consumption(sub_objects)+
@@ -2148,7 +2148,7 @@ BlockMatrixBase<MatrixType>::add (const unsigned int   row,
 
       if (this->n_block_cols() > 1)
 	{
-	  const unsigned int * first_block = std::lower_bound (col_indices,
+	  const unsigned int * first_block = Utilities::lower_bound (col_indices,
 							       col_indices+n_cols,
 							       this->column_block_indices.block_start(1));
 
