@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: block_sparsity_pattern.h 22430 2010-10-23 13:41:09Z heister $
-//    Version: $Name$
+//    $Id: block_sparsity_pattern.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
@@ -14,17 +13,17 @@
 #define __deal2__block_sparsity_pattern_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/table.h>
-#include <base/subscriptor.h>
-#include <base/smartpointer.h>
-#include <lac/sparsity_pattern.h>
-#include <lac/trilinos_sparsity_pattern.h>
-#include <lac/compressed_sparsity_pattern.h>
-#include <lac/compressed_set_sparsity_pattern.h>
-#include <lac/compressed_simple_sparsity_pattern.h>
-#include <lac/block_indices.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/table.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/lac/sparsity_pattern.h>
+#include <deal.II/lac/trilinos_sparsity_pattern.h>
+#include <deal.II/lac/compressed_sparsity_pattern.h>
+#include <deal.II/lac/compressed_set_sparsity_pattern.h>
+#include <deal.II/lac/compressed_simple_sparsity_pattern.h>
+#include <deal.II/lac/block_indices.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -552,7 +551,7 @@ class BlockSparsityPattern : public BlockSparsityPatternBase<SparsityPattern>
 				      * memory consumption (in bytes)
 				      * of this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
 				     /**
 				      * Copy data from an object of
