@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: block_vector.h 22344 2010-10-14 15:59:09Z kanschat $
-//    Version: $Name$
+//    $Id: block_vector.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
@@ -14,10 +13,10 @@
 #define __deal2__block_vector_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <lac/block_indices.h>
-#include <lac/block_vector_base.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/lac/block_indices.h>
+#include <deal.II/lac/block_vector_base.h>
 
 #include <cstdio>
 #include <vector>
