@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: block_vector_base.h 22344 2010-10-14 15:59:09Z kanschat $
-//    Version: $Name$
+//    $Id: block_vector_base.h 23876 2011-06-28 18:21:51Z kanschat $
 //
-//    Copyright (C) 2004, 2005, 2006, 2007, 2010 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,13 +13,13 @@
 #define __deal2__block_vector_base_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/subscriptor.h>
-#include <base/memory_consumption.h>
-#include <lac/exceptions.h>
-#include <lac/block_indices.h>
-#include <lac/vector.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/lac/exceptions.h>
+#include <deal.II/lac/block_indices.h>
+#include <deal.II/lac/vector.h>
 #include <vector>
 #include <iterator>
 #include <cmath>
@@ -1104,12 +1103,29 @@ class BlockVectorBase : public Subscriptor
     void equ (const value_type a, const BlockVectorBase& V,
 	      const value_type b, const BlockVectorBase& W);
 
+                                     /**
+                                      * This function does nothing but is
+                                      * there for compatibility with the
+                                      * @p PETScWrappers::Vector class.
+                                      *
+                                      * For the PETSc vector wrapper class,
+                                      * this function updates the ghost
+                                      * values of the PETSc vector. This
+                                      * is necessary after any modification
+                                      * before reading ghost values.
+                                      *
+                                      * However, for the implementation of
+                                      * this class, it is immaterial and thus
+                                      * an empty function.
+                                      */
+    void update_ghost_values () const;
+
 				     /**
 				      * Determine an estimate for the
 				      * memory consumption (in bytes)
 				      * of this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
   protected:
 				     /**
@@ -2212,9 +2228,10 @@ void BlockVectorBase<VectorType>::equ (const value_type a,
 
 
 template <class VectorType>
-unsigned int BlockVectorBase<VectorType>::memory_consumption () const
+std::size_t
+BlockVectorBase<VectorType>::memory_consumption () const
 {
-  unsigned int mem = sizeof(this->n_blocks());
+  std::size_t mem = sizeof(this->n_blocks());
   for (unsigned int i=0; i<this->components.size(); ++i)
     mem += MemoryConsumption::memory_consumption (this->components[i]);
   mem += MemoryConsumption::memory_consumption (this->block_indices);
@@ -2240,6 +2257,15 @@ void BlockVectorBase<VectorType>::equ (const value_type    a,
 
 
 
+template <class VectorType>
+void BlockVectorBase<VectorType>::update_ghost_values () const
+{
+  for (unsigned int i=0; i<n_blocks(); ++i)
+    block(i).update_ghost_values ();
+}
+
+
+
 template <class VectorType>
 BlockVectorBase<VectorType>&
 BlockVectorBase<VectorType>::operator = (const value_type s)
