@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: chunk_sparse_matrix.h 22344 2010-10-14 15:59:09Z kanschat $
-//    Version: $Name:  $
+//    $Id: chunk_sparse_matrix.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2008, 2009, 2010 by the deal.II authors
 //
@@ -14,12 +13,12 @@
 #define __deal2__chunk_sparse_matrix_h
 
 
-#include <base/config.h>
-#include <base/subscriptor.h>
-#include <base/smartpointer.h>
-#include <lac/chunk_sparsity_pattern.h>
-#include <lac/identity_matrix.h>
-#include <lac/exceptions.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/lac/chunk_sparsity_pattern.h>
+#include <deal.II/lac/identity_matrix.h>
+#include <deal.II/lac/exceptions.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -346,7 +345,7 @@ class ChunkSparseMatrix : public virtual Subscriptor
 				      * of this object. See
 				      * MemoryConsumption.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
     
 //@}
 /**
