@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: constraint_matrix.h 23063 2010-12-22 15:20:02Z kronbichler $
-//    Version: $Name$
+//    $Id: constraint_matrix.h 24181 2011-08-24 18:30:31Z heister $
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,13 +13,13 @@
 #ifndef __deal2__constraint_matrix_h
 #define __deal2__constraint_matrix_h
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/index_set.h>
-#include <base/subscriptor.h>
-#include <base/template_constraints.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/index_set.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/template_constraints.h>
 
-#include <lac/vector.h>
+#include <deal.II/lac/vector.h>
 
 #include <vector>
 #include <map>
@@ -50,7 +49,7 @@ template <typename MatrixType> struct IsBlockMatrix;
 
 namespace internals
 {
-  struct GlobalRowsFromLocal;
+  class GlobalRowsFromLocal;
 }
 
 
@@ -108,6 +107,15 @@ namespace internals
  * been added, you need to call close(), which compresses the storage format
  * and sorts the entries.
  *
+ * @note Many of the algorithms this class implements are discussed in
+ * the @ref hp_paper . The algorithms are also related to those shown
+ * in <i>M. S. Shephard: Linear multipoint constraints applied via
+ * transformation as part of a direct stiffness assembly process. Int. J.
+ * Numer. Meth. Engrg., vol. 20 (1984), pp. 2107-2112.</i>, with the
+ * difference that the algorithms shown there completely eliminated
+ * constrained degrees of freedom, whereas we usually keep them as part
+ * of the linear system.
+ *
  * @ingroup dofs
  * @ingroup constraints
  * @author Wolfgang Bangerth, Martin Kronbichler, 1998, 2004, 2008, 2009
@@ -155,7 +163,7 @@ class ConstraintMatrix : public Subscriptor
 					    */
 	  right_object_wins
     };
-    
+
 				     /**
 				      * Constructor. The supplied IndexSet
 				      * defines which indices might be
@@ -357,6 +365,8 @@ class ConstraintMatrix : public Subscriptor
 				      * constraint line <i>i</i>, according
 				      * to the discussion in the general
 				      * class description.
+				      * Note: the line needs to be added with
+				      * one of the add_line() calls first.
 				      */
     void set_inhomogeneity (const unsigned int line,
 			    const double       value);
@@ -562,7 +572,7 @@ class ConstraintMatrix : public Subscriptor
                                       * line. Unconstrained dofs also return a
                                       * zero value.
                                       */
-    double get_inhomogeneity (unsigned int line) const;
+    double get_inhomogeneity (const unsigned int line) const;
 
 				     /**
 				      * Print the constraint lines. Mainly
@@ -604,7 +614,7 @@ class ConstraintMatrix : public Subscriptor
 				      * consumption (in bytes) of this
 				      * object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
 				     /**
 				      * @}
@@ -1525,7 +1535,7 @@ class ConstraintMatrix : public Subscriptor
 					  * memory consumption (in bytes) of
 					  * this object.
 					  */
-	unsigned int memory_consumption () const;
+	std::size_t memory_consumption () const;
     };
 
 				     /**
@@ -1849,7 +1859,12 @@ void
 ConstraintMatrix::set_inhomogeneity (const unsigned int line,
 				     const double       value)
 {
-  ConstraintLine* line_ptr = &lines[lines_cache[calculate_line_index(line)]];
+  const unsigned int line_index = calculate_line_index(line);
+  Assert( line_index < lines_cache.size() &&
+      lines_cache[line_index] != numbers::invalid_unsigned_int, 
+      ExcMessage("call add_line() before calling set_inhomogeneity()"));
+  Assert(lines_cache[line_index] < lines.size(), ExcInternalError());
+  ConstraintLine* line_ptr = &lines[lines_cache[line_index]];
   line_ptr->inhomogeneity = value;
 }
 
@@ -1875,12 +1890,14 @@ ConstraintMatrix::is_inhomogeneously_constrained (const unsigned int index) cons
 				// constrained. could use is_constrained, but
 				// that means computing the line index twice
   const unsigned int line_index = calculate_line_index(index);
-  if (line_index > lines_cache.size() ||
-      lines_cache[line_index] == numbers::invalid_unsigned_int ||
-      lines[lines_cache[line_index]].inhomogeneity == 0)
+  if (line_index >= lines_cache.size() ||
+      lines_cache[line_index] == numbers::invalid_unsigned_int)
     return false;
   else
-    return true;
+    {
+      Assert(lines_cache[line_index] < lines.size(), ExcInternalError());
+      return !(lines[lines_cache[line_index]].inhomogeneity == 0);
+    }
 }
 
 
@@ -1893,7 +1910,7 @@ ConstraintMatrix::get_constraint_entries (unsigned int line) const
 				// constrained. could use is_constrained, but
 				// that means computing the line index twice
   const unsigned int line_index = calculate_line_index(line);
-  if (line_index > lines_cache.size() ||
+  if (line_index >= lines_cache.size() ||
       lines_cache[line_index] == numbers::invalid_unsigned_int)
     return 0;
   else
@@ -1904,13 +1921,13 @@ ConstraintMatrix::get_constraint_entries (unsigned int line) const
 
 inline
 double
-ConstraintMatrix::get_inhomogeneity (unsigned int line) const
+ConstraintMatrix::get_inhomogeneity (const unsigned int line) const
 {
 				// check whether the entry is
 				// constrained. could use is_constrained, but
 				// that means computing the line index twice
   const unsigned int line_index = calculate_line_index(line);
-  if (line_index > lines_cache.size() ||
+  if (line_index >= lines_cache.size() ||
       lines_cache[line_index] == numbers::invalid_unsigned_int)
     return 0;
   else
