@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: constraint_matrix.templates.h 22893 2010-12-01 20:21:18Z bangerth $
-//    Version: $Name$
+//    $Id: constraint_matrix.templates.h 23944 2011-07-12 12:23:28Z bangerth $
 //
 //    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
@@ -15,14 +14,14 @@
 #define __deal2__constraint_matrix_templates_h
 
 
-#include <lac/constraint_matrix.h>
+#include <deal.II/lac/constraint_matrix.h>
 
-#include <base/table.h>
-#include <lac/full_matrix.h>
-#include <lac/sparsity_pattern.h>
-#include <lac/sparse_matrix.h>
-#include <lac/block_sparsity_pattern.h>
-#include <lac/block_sparse_matrix.h>
+#include <deal.II/base/table.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/sparsity_pattern.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/block_sparsity_pattern.h>
+#include <deal.II/lac/block_sparse_matrix.h>
 
 #include <iomanip>
 
@@ -1326,7 +1325,7 @@ namespace internals
     for (unsigned int i=1; i<n_active_rows; ++i)
       Assert (total_row_indices[i-1] < total_row_indices[i], ExcInternalError());
 
-    pos = std::lower_bound (total_row_indices.begin(),
+    pos = Utilities::lower_bound (total_row_indices.begin(),
 			    total_row_indices.begin()+n_active_rows,
 			    row_value);
     if (pos->global_row == global_row)
@@ -1430,7 +1429,7 @@ namespace internals
     for (unsigned int i=1;i<num_blocks;++i)
       {
 	row_iterator first_block =
-	  std::lower_bound (block_indices,
+	  Utilities::lower_bound (block_indices,
 			    global_rows.total_row_indices.begin()+n_active_rows,
 			    Distributing(block_object.get_row_indices().block_start(i)));
 	block_starts[i] = first_block - global_rows.total_row_indices.begin();
@@ -1469,7 +1468,7 @@ namespace internals
     for (unsigned int i=1;i<num_blocks;++i)
       {
 	row_iterator first_block =
-	  std::lower_bound (col_indices,
+	  Utilities::lower_bound (col_indices,
 			    row_indices.end(),
 			    block_object.get_row_indices().block_start(i));
 	block_starts[i] = first_block - row_indices.begin();
@@ -2123,7 +2122,7 @@ make_sorted_row_list (const std::vector<unsigned int> &local_dof_indices,
 	  else
 	    {
 	      std::vector<unsigned int>::iterator it =
-		std::lower_bound(active_dofs.begin(),
+		Utilities::lower_bound(active_dofs.begin(),
 				 active_dofs.end()-i+1,
 				 new_index);
 	      if (*it != new_index)
