@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: filtered_matrix.h 22349 2010-10-15 14:09:00Z kanschat $
-//    Version: $Name$
+//    $Id: filtered_matrix.h 23876 2011-06-28 18:21:51Z kanschat $
 //
-//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,12 +14,12 @@
 
 
 
-#include <base/config.h>
-#include <base/smartpointer.h>
-#include <base/thread_management.h>
-#include <base/memory_consumption.h>
-#include <lac/pointer_matrix.h>
-#include <lac/vector_memory.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/base/thread_management.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/lac/pointer_matrix.h>
+#include <deal.II/lac/vector_memory.h>
 #include <vector>
 #include <algorithm>
 
@@ -58,13 +57,13 @@ template <class VECTOR> class FilteredMatrixBlock;
  * hand side vector (through apply_constraints()) and use this
  * object as matrix object in a linear solver. As linear solvers
  * should only use vmult() and residual() functions of a
- * matrix class, this class should be as a good a matrix as any other
+ * matrix class, this class should be as good a matrix as any other
  * for that purpose.
  *
  * Furthermore, also the precondition_Jacobi() function is
  * provided (since the computation of diagonal elements of the
  * filtered matrix $A_X$ is simple), so you can use this as a
- * preconditioner. Some other function useful for matrices are also
+ * preconditioner. Some other functions useful for matrices are also
  * available.
  *
  * A typical code snippet showing the above steps is as follows:
@@ -94,12 +93,13 @@ template <class VECTOR> class FilteredMatrixBlock;
  *   solver.solve (filtered_A, x, b, filtered_prec);
  * @endverbatim
  *
+ *
  * <h3>Connection to other classes</h3>
  *
  * The function MatrixTools::apply_boundary_values() does exactly
  * the same that this class does, except for the fact that that
- * function actually modifies the matrix. Due to this, it is only
- * possible to solve with a matrix onto which
+ * function actually modifies the matrix. Consequently, it is only
+ * possible to solve with a matrix to which
  * MatrixTools::apply_boundary_values() was applied for one right
  * hand side and one set of boundary values since the modification of
  * the right hand side depends on the original matrix.
@@ -107,7 +107,7 @@ template <class VECTOR> class FilteredMatrixBlock;
  * While this is a feasible method in cases where only
  * one solution of the linear system is required, for example in
  * solving linear stationary systems, one would often like to have the
- * ability to solve multiply with the same matrix in nonlinear
+ * ability to solve multiple times with the same matrix in nonlinear
  * problems (where one often does not want to update the Hessian
  * between Newton steps, despite having different right hand sides in
  * subsequent steps) or time dependent problems, without having to
@@ -194,10 +194,9 @@ template <class VECTOR> class FilteredMatrixBlock;
  *
  * <h3>Thread-safety</h3>
  *
- * The functions that operate as a matrix and do not change the
- * internal state of this object are synchronised and thus
- * threadsafe. You need not serialize calls to @p vmult or
- * @p residual therefore.
+ * The functions that operate as a matrix and do not change the internal state
+ * of this object are synchronised and thus threadsafe. Consequently, you do
+ * not need to serialize calls to @p vmult or @p residual .
  *
  * @author Wolfgang Bangerth 2001, Luca Heltai 2006, Guido Kanschat 2007, 2008
  */
@@ -552,7 +551,7 @@ class FilteredMatrix : public Subscriptor
 				      * referenced, its memory
 				      * consumption is not included.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
     
   private:
 				     /**
@@ -1099,11 +1098,11 @@ FilteredMatrix<VECTOR>::Tvmult_add (VECTOR& dst, const VECTOR& src) const
 
 template <class VECTOR>
 inline
-unsigned int
+std::size_t
 FilteredMatrix<VECTOR>::memory_consumption () const
 {
-    return (MemoryConsumption::memory_consumption (matrix) +
-	    MemoryConsumption::memory_consumption (constraints));
+  return (MemoryConsumption::memory_consumption (matrix) +
+	  MemoryConsumption::memory_consumption (constraints));
 }
 
 
