@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: full_matrix.h 22628 2010-11-07 23:11:12Z kanschat $
-//    Version: $Name$
+//    $Id: full_matrix.h 24098 2011-08-16 16:01:11Z bangerth $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,11 +13,12 @@
 #define __deal2__full_matrix_h
 
 
-#include <base/config.h>
-#include <base/numbers.h>
-#include <base/table.h>
-#include <lac/exceptions.h>
-#include <lac/identity_matrix.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/numbers.h>
+#include <deal.II/base/table.h>
+#include <deal.II/lac/exceptions.h>
+#include <deal.II/lac/identity_matrix.h>
+#include <deal.II/base/tensor.h>
 
 #include <vector>
 #include <iomanip>
@@ -30,7 +30,6 @@ DEAL_II_NAMESPACE_OPEN
 // forward declarations
 template <typename number> class Vector;
 template <typename number> class LAPACKFullMatrix;
-template <int rank, int dim> class Tensor;
 
 
 /*! @addtogroup Matrix1
@@ -634,7 +633,7 @@ class FullMatrix : public Table<2,number>
 				      * memory consumption (in bytes)
 				      * of this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
 //@}
 ///@name Iterator functions
@@ -1157,7 +1156,35 @@ class FullMatrix : public Table<2,number>
     void TmTmult (FullMatrix<number2>       &C,
 		  const FullMatrix<number2> &B,
 		  const bool                 adding=false) const;
-
+    
+				     /**
+				      * Add to the current matrix the
+				      * triple product <b>B
+				      * A<sup>-1</sup>
+				      * D</b>. Optionally, use the
+				      * transposes of the matrices
+				      * <b>B</b> and <b>D</b>. The
+				      * scaling factor scales the
+				      * whole product, which is
+				      * helpful when adding a multiple
+				      * of the triple product to the
+				      * matrix.
+				      *
+				      * This product was written with
+				      * the Schur complement
+				      * <b>B<sup>T</sup>
+				      * A<sup>-1</sup> D</b> in mind.
+				      * Note that in this case the
+				      * argument for <tt>A</tt> must be
+				      * the inverse of the matrix <b>A</b>.
+				      */
+    void triple_product(const FullMatrix<number>& A,
+			const FullMatrix<number>& B,
+			const FullMatrix<number>& D,
+			const bool transpose_B = false,
+			const bool transpose_D = false,
+			const number scaling = number(1.));
+    
 				     /**
 				      * Matrix-vector-multiplication.
 				      *
