@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: full_matrix.templates.h 22631 2010-11-08 00:19:59Z kanschat $
-//    Version: $Name$
+//    $Id: full_matrix.templates.h 24018 2011-08-05 14:46:28Z kanschat $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -16,12 +15,12 @@
 
 //TODO: this file has a lot of operations between matrices and matrices or matrices and vectors of different precision. we should go through the file and in each case pick the more accurate data type for intermediate results. currently, the choice is pretty much random. this may also allow us some operations where one operand is complex and the other is not
 
-#include <base/config.h>
-#include <base/template_constraints.h>
-#include <lac/vector.h>
-#include <lac/full_matrix.h>
-#include <lac/lapack_full_matrix.h>
-#include <lac/lapack_templates.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/template_constraints.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/lapack_full_matrix.h>
+#include <deal.II/lac/lapack_templates.h>
 
 #include <vector>
 #include <cmath>
@@ -881,6 +880,62 @@ void FullMatrix<number>::TmTmult (FullMatrix<number2>       &dst,
 }
 
 
+template <typename number>
+void
+FullMatrix<number>::triple_product(
+  const FullMatrix<number>& A,
+  const FullMatrix<number>& B,
+  const FullMatrix<number>& D,
+  const bool transpose_B,
+  const bool transpose_D,
+  const number scaling)
+{  
+  if (transpose_B)
+    {
+      AssertDimension(B.m(), A.m());
+      AssertDimension(B.n(), m());
+    }
+  else
+    {
+      AssertDimension(B.n(), A.m());
+      AssertDimension(B.m(), m());
+    }
+  if (transpose_D)
+    {
+      AssertDimension(D.n(), A.n());
+      AssertDimension(D.m(), n());
+    }
+  else
+    {
+      AssertDimension(D.m(), A.n());
+      AssertDimension(D.n(), n());
+    }
+
+				   // For all entries of the product
+				   // AD
+  for (unsigned int i=0; i<A.m();++i)
+    for (unsigned int j=0; j<n();++j)
+      {
+					 // Compute the entry
+	number ADij = 0.;
+	if (transpose_D)
+	  for (unsigned int k=0;k<A.n();++k)
+	    ADij += A(i,k)*D(j,k);
+	else
+	  for (unsigned int k=0;k<A.n();++k)
+	    ADij += A(i,k)*D(k,j);
+				   // And add it to this after
+				   // multiplying with the right
+				   // factor from B
+	if (transpose_B)
+	  for (unsigned int k=0;k<m();++k)
+	    this->operator()(k,j) += scaling * ADij * B(i,k);
+	else
+	  for (unsigned int k=0;k<m();++k)
+	    this->operator()(k,j) += scaling * ADij * B(k,i);
+      }
+}
+    
 
 template <typename number>
 template <typename number2>
@@ -1838,11 +1893,12 @@ FullMatrix<number>::gauss_jordan ()
 
 
 template <typename number>
-unsigned int
+std::size_t
 FullMatrix<number>::memory_consumption () const
 {
-  return sizeof(*this) - sizeof (Table<2,number>)
-    + Table<2,number>::memory_consumption();
+  return (sizeof(*this) - sizeof (Table<2,number>)
+	  +
+	  Table<2,number>::memory_consumption());
 }
 
 
