@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: lapack_full_matrix.h 22662 2010-11-10 12:21:37Z bangerth $
-//    Version: $Name$
+//    $Id: lapack_full_matrix.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2005, 2006, 2008, 2009, 2010 by the deal.II authors
 //
@@ -14,13 +13,13 @@
 #define __deal2__lapack_full_matrix_h
 
 
-#include <base/config.h>
-#include <base/smartpointer.h>
-#include <base/table.h>
-#include <lac/lapack_support.h>
-#include <lac/vector_memory.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/base/table.h>
+#include <deal.II/lac/lapack_support.h>
+#include <deal.II/lac/vector_memory.h>
 
-#include <base/std_cxx1x/shared_ptr.h>
+#include <deal.II/base/std_cxx1x/shared_ptr.h>
 #include <vector>
 #include <complex>
 
