@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: matrix_block.h 22798 2010-11-17 20:00:48Z bangerth $
-//    Version: $Name$
+//    $Id: matrix_block.h 24123 2011-08-20 23:31:16Z heister $
 //
-//    Copyright (C) 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,16 +12,16 @@
 #ifndef __deal2__matrix_block_h
 #define __deal2__matrix_block_h
 
-#include <base/config.h>
-#include <base/named_data.h>
-#include <base/smartpointer.h>
-#include <base/std_cxx1x/shared_ptr.h>
-#include <base/memory_consumption.h>
-#include <base/mg_level_object.h>
-#include <lac/block_indices.h>
-#include <lac/block_sparsity_pattern.h>
-#include <lac/sparse_matrix.h>
-#include <lac/full_matrix.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/named_data.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/base/std_cxx1x/shared_ptr.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/base/mg_level_object.h>
+#include <deal.II/lac/block_indices.h>
+#include <deal.II/lac/block_sparsity_pattern.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/full_matrix.h>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -330,7 +329,7 @@ class MatrixBlock
 				     /**
 				      * The memory used by this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
 				     /**
 				      * The block number computed from
@@ -441,7 +440,7 @@ class MatrixBlockVector
 				     /**
 				      * The memory used by this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
 				     /**
 				      * Access a constant reference to
@@ -462,10 +461,13 @@ class MatrixBlockVector
 				      */
     MATRIX& matrix(unsigned int i);
 
-    NamedData<std_cxx1x::shared_ptr<value_type> >::subscribe;
-    NamedData<std_cxx1x::shared_ptr<value_type> >::unsubscribe;
-    NamedData<std_cxx1x::shared_ptr<value_type> >::size;
-    NamedData<std_cxx1x::shared_ptr<value_type> >::name;
+				     /**
+				      * import functions from private base class
+				      */
+    using NamedData<std_cxx1x::shared_ptr<value_type> >::subscribe;
+    using NamedData<std_cxx1x::shared_ptr<value_type> >::unsubscribe;
+    using NamedData<std_cxx1x::shared_ptr<value_type> >::size;
+    using NamedData<std_cxx1x::shared_ptr<value_type> >::name;
 };
 
 
@@ -639,7 +641,7 @@ class MGMatrixBlockVector
 				     /**
 				      * The memory used by this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
   private:
 				     /// Clear one of the matrix objects
     void clear_object(NamedData<MGLevelObject<MatrixBlock<MATRIX> > >&);
@@ -906,13 +908,12 @@ MatrixBlock<MATRIX>::Tvmult_add (VECTOR& w, const VECTOR& v) const
 
 template <class MATRIX>
 inline
-unsigned int
+std::size_t
 MatrixBlock<MATRIX>::memory_consumption () const
 {
-  unsigned int mem = sizeof(*this)
-		     + MemoryConsumption::memory_consumption(matrix)
-		     - sizeof(matrix);
-  return mem;
+  return (sizeof(*this)
+	  + MemoryConsumption::memory_consumption(matrix)
+	  - sizeof(matrix));
 }
 
 //----------------------------------------------------------------------//
