@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_matrix_base.h 22661 2010-11-10 12:05:43Z young $
-//    Version: $Name$
+//    $Id: petsc_matrix_base.h 24320 2011-09-13 09:41:50Z young $
 //
-//    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,16 +13,16 @@
 #define __deal2__petsc_matrix_base_h
 
 
-#include <base/config.h>
+#include <deal.II/base/config.h>
 
 #ifdef DEAL_II_USE_PETSC
 
-#  include <base/subscriptor.h>
-#  include <lac/full_matrix.h>
-#  include <lac/exceptions.h>
+#  include <deal.II/base/subscriptor.h>
+#  include <deal.II/lac/full_matrix.h>
+#  include <deal.II/lac/exceptions.h>
 
 #  include <petscmat.h>
-#  include <base/std_cxx1x/shared_ptr.h>
+#  include <deal.II/base/std_cxx1x/shared_ptr.h>
 
 #  include <vector>
 #  include <cmath>
@@ -1109,7 +1108,12 @@ namespace PETScWrappers
 					 * $1000\times32$-bit machine
 					 * precision.
 					 */
-      PetscTruth is_symmetric (const double tolerance = 1.e-12);
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
+      PetscTruth 
+#else
+	PetscBool
+#endif
+	is_symmetric (const double tolerance = 1.e-12);
 
 #if DEAL_II_PETSC_VERSION_GTE(2,3,0)
                                         /**
@@ -1121,9 +1125,14 @@ namespace PETScWrappers
 					 * $1000\times32$-bit machine
 					 * precision.
 					 */
-      PetscTruth is_hermitian (const double tolerance = 1.e-12);
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
+      PetscTruth 
+#else
+	PetscBool
 #endif
-
+	is_hermitian (const double tolerance = 1.e-12);
+#endif
+      
                                         /*
 					 * Abstract PETSc object that helps view
 					 * in ASCII other PETSc objects. Currently
@@ -1136,7 +1145,7 @@ namespace PETScWrappers
 					*  Returns the number bytes consumed
 					*  by this matrix on this CPU.
 					*/
-      unsigned int memory_consumption() const;
+      std::size_t memory_consumption() const;
 
                                        /**
                                         * Exception
@@ -1150,6 +1159,23 @@ namespace PETScWrappers
                                         */
       DeclException0 (ExcSourceEqualsDestination);
 
+      /**
+        * Exception.
+	*/
+      DeclException2 (ExcWrongMode,
+		      int, int,
+		      << "You tried to do a "
+		      << (arg1 == 1 ?
+			  "'set'" :
+			  (arg1 == 2 ?
+			   "'add'" : "???"))
+		      << " operation but the matrix is currently in "
+		      << (arg2 == 1 ?
+			  "'set'" :
+			  (arg2 == 2 ?
+			   "'add'" : "???"))
+		      << " mode. You first have to call 'compress()'.");
+
     protected:
                                        /**
                                         * A generic matrix object in
@@ -1191,9 +1217,10 @@ namespace PETScWrappers
       LastAction::Values last_action;
 
 				       /**
-					* Flush buffers on all CPUs when
-					* switching between inserting and
-					* adding to elements, no-op otherwise.
+					* Ensure that the add/set mode that
+					* is required for actions following
+					* this call is compatible with the
+					* current mode.
 					* Should be called from all internal
 					* functions accessing matrix elements.
 					*/
@@ -1434,7 +1461,6 @@ namespace PETScWrappers
 		   const unsigned int j,
 		   const PetscScalar  value)
   {
-
     Assert (numbers::is_finite(value), ExcNumberNotFinite());
 
     set (i, 1, &j, &value, false);
@@ -1794,18 +1820,12 @@ namespace PETScWrappers
   void
   MatrixBase::prepare_action(const LastAction::Values new_action)
   {
-				     // flush PETSc buffers when switching
-				     // actions, otherwise just return.
-    if (last_action == new_action) return;
-
-    int ierr;
-    ierr = MatAssemblyBegin(matrix, MAT_FLUSH_ASSEMBLY);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-    ierr = MatAssemblyEnd(matrix, MAT_FLUSH_ASSEMBLY);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-    last_action = new_action;
+    if (last_action == new_action)
+      ;
+    else if (last_action == LastAction::none)
+      last_action = new_action;
+    else
+      Assert (false, ExcWrongMode (last_action, new_action));
   }
 
 
