@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_parallel_vector.h 22430 2010-10-23 13:41:09Z heister $
-//    Version: $Name$
+//    $Id: petsc_parallel_vector.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
@@ -14,15 +13,15 @@
 #define __deal2__petsc_parallel_vector_h
 
 
-#include <base/config.h>
+#include <deal.II/base/config.h>
 
 #ifdef DEAL_II_USE_PETSC
 
-#  include <base/subscriptor.h>
-#  include <lac/exceptions.h>
-#  include <lac/vector.h>
-#  include <lac/petsc_vector_base.h>
-#  include <base/index_set.h>
+#  include <deal.II/base/subscriptor.h>
+#  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/vector.h>
+#  include <deal.II/lac/petsc_vector_base.h>
+#  include <deal.II/base/index_set.h>
 
 DEAL_II_NAMESPACE_OPEN
 
