@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_precondition.h 22742 2010-11-15 13:01:58Z bangerth $
-//    Version: $Name$
+//    $Id: petsc_precondition.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2004, 2005, 2006, 2007, 2010 by the deal.II authors
 //
@@ -14,11 +13,11 @@
 #define __deal2__petsc_precondition_h
 
 
-#include <base/config.h>
+#include <deal.II/base/config.h>
 
 #ifdef DEAL_II_USE_PETSC
 
-#  include <lac/exceptions.h>
+#  include <deal.II/lac/exceptions.h>
 #  include <petscpc.h>
 
 DEAL_II_NAMESPACE_OPEN
@@ -47,31 +46,54 @@ namespace PETScWrappers
  * for parallel jobs, such as for example the ILU preconditioner.
  *
  * @ingroup PETScWrappers
- * @author Wolfgang Bangerth, 2004
+ * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
  */
   class PreconditionerBase
   {
     public:
                                        /**
-                                        * Constructor. Take a pointer to the
-                                        * matrix from which the preconditioner
-                                        * shall be constructed.
+                                        * Constructor.
                                         */
-      PreconditionerBase (const MatrixBase &matrix);
+      PreconditionerBase ();
 
                                        /**
                                         * Destructor.
                                         */
       virtual ~PreconditionerBase ();
 
+				       /**
+					* Apply the preconditioner once to the
+					* given src vector.
+					*/
+      void vmult (VectorBase       &dst,
+		  const VectorBase &src) const;
 
+
+				       /**
+					* Gives access to the underlying PETSc
+					* object.
+					*/
+      const PC & get_pc () const;
+      
     protected:
+				       /**
+					* the PETSc preconditioner object
+					*/
+      PC pc;
+      
                                        /**
                                         * A pointer to the matrix that acts as
                                         * a preconditioner.
                                         */
-      const Mat matrix;
+      Mat matrix;
 
+				       /**
+					* Internal function to create the
+					* PETSc preconditioner object. Fails
+					* if called twice.
+					*/
+      void create_pc ();
+      
                                        /**
                                         * Conversion operator to get a
                                         * representation of the matrix that
@@ -81,16 +103,7 @@ namespace PETScWrappers
                                         * the PETSc solvers.
                                         */
       operator Mat () const;
-
-                                       /**
-                                        * Function that takes a Krylov
-                                        * Subspace Preconditioner context
-                                        * object, and sets the type of
-                                        * preconditioner that is requested by
-                                        * the derived class.
-                                        */
-      virtual void set_preconditioner_type (PC &pc) const = 0;
-
+      
                                        /**
                                         * Make the solver class a friend,
                                         * since it needs to call the
@@ -109,7 +122,7 @@ namespace PETScWrappers
  * preconditioner may or may not work.
  *
  * @ingroup PETScWrappers
- * @author Wolfgang Bangerth, 2004
+ * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
  */
   class PreconditionJacobi : public PreconditionerBase
   {
@@ -122,6 +135,14 @@ namespace PETScWrappers
       struct AdditionalData
       {};
 
+				       /**
+					* Empty Constructor. You need to call
+					* initialize() before using this
+					* object.
+					*/
+      PreconditionJacobi ();
+      
+      
                                        /**
                                         * Constructor. Take the matrix which
                                         * is used to form the preconditioner,
@@ -131,21 +152,26 @@ namespace PETScWrappers
       PreconditionJacobi (const MatrixBase     &matrix,
                           const AdditionalData &additional_data = AdditionalData());
 
+				       /**
+					* Initializes the preconditioner
+					* object and calculate all data that
+					* is necessary for applying it in a
+					* solver. This function is
+					* automatically called when calling
+					* the constructor with the same
+					* arguments and is only used if you
+					* create the preconditioner without
+					* arguments.
+					*/
+      void initialize (const MatrixBase     &matrix,
+		       const AdditionalData &additional_data = AdditionalData());
+
     protected:
                                        /**
                                         * Store a copy of the flags for this
                                         * particular preconditioner.
                                         */
-      const AdditionalData additional_data;
-
-                                       /**
-                                        * Function that takes a Krylov
-                                        * Subspace Preconditioner context
-                                        * object, and sets the type of
-                                        * preconditioner that is appropriate
-                                        * for the present class.
-                                        */
-      virtual void set_preconditioner_type (PC &pc) const;
+      AdditionalData additional_data;
   };
 
 
@@ -165,7 +191,7 @@ namespace PETScWrappers
  * preconditioner may or may not work.
  *
  * @ingroup PETScWrappers
- * @author Wolfgang Bangerth, 2004
+ * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
  */
   class PreconditionBlockJacobi : public PreconditionerBase
   {
@@ -177,6 +203,13 @@ namespace PETScWrappers
                                         */
       struct AdditionalData
       {};
+      
+				       /**
+					* Empty Constructor. You need to call
+					* initialize() before using this
+					* object.
+					*/
+      PreconditionBlockJacobi ();
 
                                        /**
                                         * Constructor. Take the matrix which
@@ -187,21 +220,26 @@ namespace PETScWrappers
       PreconditionBlockJacobi (const MatrixBase     &matrix,
                                const AdditionalData &additional_data = AdditionalData());
 
+				       /**
+					* Initializes the preconditioner
+					* object and calculate all data that
+					* is necessary for applying it in a
+					* solver. This function is
+					* automatically called when calling
+					* the constructor with the same
+					* arguments and is only used if you
+					* create the preconditioner without
+					* arguments.
+					*/
+      void initialize (const MatrixBase     &matrix,
+		       const AdditionalData &additional_data = AdditionalData());
+
     protected:
                                        /**
                                         * Store a copy of the flags for this
                                         * particular preconditioner.
                                         */
-      const AdditionalData additional_data;
-
-                                       /**
-                                        * Function that takes a Krylov
-                                        * Subspace Preconditioner context
-                                        * object, and sets the type of
-                                        * preconditioner that is appropriate
-                                        * for the present class.
-                                        */
-      virtual void set_preconditioner_type (PC &pc) const;
+      AdditionalData additional_data;
   };
 
 
@@ -214,7 +252,7 @@ namespace PETScWrappers
  * preconditioner may or may not work.
  *
  * @ingroup PETScWrappers
- * @author Wolfgang Bangerth, 2004
+ * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
  */
   class PreconditionSOR : public PreconditionerBase
   {
@@ -239,6 +277,13 @@ namespace PETScWrappers
           double omega;
       };
 
+				       /**
+					* Empty Constructor. You need to call
+					* initialize() before using this
+					* object.
+					*/
+      PreconditionSOR ();
+
                                        /**
                                         * Constructor. Take the matrix which
                                         * is used to form the preconditioner,
@@ -248,21 +293,26 @@ namespace PETScWrappers
       PreconditionSOR (const MatrixBase     &matrix,
                        const AdditionalData &additional_data = AdditionalData());
 
+				       /**
+					* Initializes the preconditioner
+					* object and calculate all data that
+					* is necessary for applying it in a
+					* solver. This function is
+					* automatically called when calling
+					* the constructor with the same
+					* arguments and is only used if you
+					* create the preconditioner without
+					* arguments.
+					*/
+      void initialize (const MatrixBase     &matrix,
+                       const AdditionalData &additional_data = AdditionalData());
+
     protected:
                                        /**
                                         * Store a copy of the flags for this
                                         * particular preconditioner.
                                         */
-      const AdditionalData additional_data;
-
-                                       /**
-                                        * Function that takes a Krylov
-                                        * Subspace Preconditioner context
-                                        * object, and sets the type of
-                                        * preconditioner that is appropriate
-                                        * for the present class.
-                                        */
-      virtual void set_preconditioner_type (PC &pc) const;
+      AdditionalData additional_data;
   };
 
 
@@ -275,7 +325,7 @@ namespace PETScWrappers
  * preconditioner may or may not work.
  *
  * @ingroup PETScWrappers
- * @author Wolfgang Bangerth, 2004
+ * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
  */
   class PreconditionSSOR : public PreconditionerBase
   {
@@ -300,6 +350,13 @@ namespace PETScWrappers
           double omega;
       };
 
+				       /**
+					* Empty Constructor. You need to call
+					* initialize() before using this
+					* object.
+					*/
+      PreconditionSSOR ();
+
                                        /**
                                         * Constructor. Take the matrix which
                                         * is used to form the preconditioner,
@@ -309,21 +366,26 @@ namespace PETScWrappers
       PreconditionSSOR (const MatrixBase     &matrix,
                         const AdditionalData &additional_data = AdditionalData());
 
+				       /**
+					* Initializes the preconditioner
+					* object and calculate all data that
+					* is necessary for applying it in a
+					* solver. This function is
+					* automatically called when calling
+					* the constructor with the same
+					* arguments and is only used if you
+					* create the preconditioner without
+					* arguments.
+					*/
+      void initialize (const MatrixBase     &matrix,
+                       const AdditionalData &additional_data = AdditionalData());
+
     protected:
                                        /**
                                         * Store a copy of the flags for this
                                         * particular preconditioner.
                                         */
-      const AdditionalData additional_data;
-
-                                       /**
-                                        * Function that takes a Krylov
-                                        * Subspace Preconditioner context
-                                        * object, and sets the type of
-                                        * preconditioner that is appropriate
-                                        * for the present class.
-                                        */
-      virtual void set_preconditioner_type (PC &pc) const;
+      AdditionalData additional_data;
   };
 
 
@@ -336,7 +398,7 @@ namespace PETScWrappers
  * preconditioner may or may not work.
  *
  * @ingroup PETScWrappers
- * @author Wolfgang Bangerth, 2004
+ * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
  */
   class PreconditionEisenstat : public PreconditionerBase
   {
@@ -361,6 +423,13 @@ namespace PETScWrappers
           double omega;
       };
 
+				       /**
+					* Empty Constructor. You need to call
+					* initialize() before using this
+					* object.
+					*/
+      PreconditionEisenstat ();
+
                                        /**
                                         * Constructor. Take the matrix which
                                         * is used to form the preconditioner,
@@ -370,21 +439,26 @@ namespace PETScWrappers
       PreconditionEisenstat (const MatrixBase     &matrix,
                              const AdditionalData &additional_data = AdditionalData());
 
+				       /**
+					* Initializes the preconditioner
+					* object and calculate all data that
+					* is necessary for applying it in a
+					* solver. This function is
+					* automatically called when calling
+					* the constructor with the same
+					* arguments and is only used if you
+					* create the preconditioner without
+					* arguments.
+					*/
+      void initialize (const MatrixBase     &matrix,
+                       const AdditionalData &additional_data = AdditionalData());
+
     protected:
                                        /**
                                         * Store a copy of the flags for this
                                         * particular preconditioner.
                                         */
-      const AdditionalData additional_data;
-
-                                       /**
-                                        * Function that takes a Krylov
-                                        * Subspace Preconditioner context
-                                        * object, and sets the type of
-                                        * preconditioner that is appropriate
-                                        * for the present class.
-                                        */
-      virtual void set_preconditioner_type (PC &pc) const;
+      AdditionalData additional_data;
   };
 
 
@@ -397,7 +471,7 @@ namespace PETScWrappers
  * preconditioner may or may not work.
  *
  * @ingroup PETScWrappers
- * @author Wolfgang Bangerth, 2004
+ * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
  */
   class PreconditionICC : public PreconditionerBase
   {
@@ -422,6 +496,13 @@ namespace PETScWrappers
           unsigned int levels;
       };
 
+				       /**
+					* Empty Constructor. You need to call
+					* initialize() before using this
+					* object.
+					*/
+      PreconditionICC ();
+
                                        /**
                                         * Constructor. Take the matrix which
                                         * is used to form the preconditioner,
@@ -431,21 +512,26 @@ namespace PETScWrappers
       PreconditionICC (const MatrixBase     &matrix,
                        const AdditionalData &additional_data = AdditionalData());
 
+				       /**
+					* Initializes the preconditioner
+					* object and calculate all data that
+					* is necessary for applying it in a
+					* solver. This function is
+					* automatically called when calling
+					* the constructor with the same
+					* arguments and is only used if you
+					* create the preconditioner without
+					* arguments.
+					*/
+      void initialize (const MatrixBase     &matrix,
+                       const AdditionalData &additional_data = AdditionalData());
+
     protected:
                                        /**
                                         * Store a copy of the flags for this
                                         * particular preconditioner.
                                         */
-      const AdditionalData additional_data;
-
-                                       /**
-                                        * Function that takes a Krylov
-                                        * Subspace Preconditioner context
-                                        * object, and sets the type of
-                                        * preconditioner that is appropriate
-                                        * for the present class.
-                                        */
-      virtual void set_preconditioner_type (PC &pc) const;
+      AdditionalData additional_data;
   };
 
 
@@ -458,7 +544,7 @@ namespace PETScWrappers
  * preconditioner may or may not work.
  *
  * @ingroup PETScWrappers
- * @author Wolfgang Bangerth, 2004
+ * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
  */
   class PreconditionILU : public PreconditionerBase
   {
@@ -483,6 +569,13 @@ namespace PETScWrappers
           unsigned int levels;
       };
 
+				       /**
+					* Empty Constructor. You need to call
+					* initialize() before using this
+					* object.
+					*/
+      PreconditionILU ();
+
                                        /**
                                         * Constructor. Take the matrix which
                                         * is used to form the preconditioner,
@@ -492,21 +585,26 @@ namespace PETScWrappers
       PreconditionILU (const MatrixBase     &matrix,
                        const AdditionalData &additional_data = AdditionalData());
 
+				       /**
+					* Initializes the preconditioner
+					* object and calculate all data that
+					* is necessary for applying it in a
+					* solver. This function is
+					* automatically called when calling
+					* the constructor with the same
+					* arguments and is only used if you
+					* create the preconditioner without
+					* arguments.
+					*/
+      void initialize (const MatrixBase     &matrix,
+                       const AdditionalData &additional_data = AdditionalData());
+
     protected:
                                        /**
                                         * Store a copy of the flags for this
                                         * particular preconditioner.
                                         */
-      const AdditionalData additional_data;
-
-                                       /**
-                                        * Function that takes a Krylov
-                                        * Subspace Preconditioner context
-                                        * object, and sets the type of
-                                        * preconditioner that is appropriate
-                                        * for the present class.
-                                        */
-      virtual void set_preconditioner_type (PC &pc) const;
+      AdditionalData additional_data;
   };
 
 
@@ -568,6 +666,13 @@ namespace PETScWrappers
 	  double damping;
       };
 
+				       /**
+					* Empty Constructor. You need to call
+					* initialize() before using this
+					* object.
+					*/
+      PreconditionLU ();
+
                                        /**
                                         * Constructor. Take the matrix which
                                         * is used to form the preconditioner,
@@ -577,21 +682,26 @@ namespace PETScWrappers
       PreconditionLU (const MatrixBase     &matrix,
 		      const AdditionalData &additional_data = AdditionalData());
 
+				       /**
+					* Initializes the preconditioner
+					* object and calculate all data that
+					* is necessary for applying it in a
+					* solver. This function is
+					* automatically called when calling
+					* the constructor with the same
+					* arguments and is only used if you
+					* create the preconditioner without
+					* arguments.
+					*/
+      void initialize (const MatrixBase     &matrix,
+                       const AdditionalData &additional_data = AdditionalData());
+
     protected:
                                        /**
                                         * Store a copy of the flags for this
                                         * particular preconditioner.
                                         */
-      const AdditionalData additional_data;
-
-                                       /**
-                                        * Function that takes a Krylov
-                                        * Subspace Preconditioner context
-                                        * object, and sets the type of
-                                        * preconditioner that is appropriate
-                                        * for the present class.
-                                        */
-      virtual void set_preconditioner_type (PC &pc) const;
+      AdditionalData additional_data;
   };
 
 
@@ -696,6 +806,13 @@ namespace PETScWrappers
 
 
 
+				       /**
+					* Empty Constructor. You need to call
+					* initialize() before using this
+					* object.
+					*/
+      PreconditionBoomerAMG ();
+
                                        /**
                                         * Constructor. Take the matrix which
                                         * is used to form the preconditioner,
@@ -705,21 +822,26 @@ namespace PETScWrappers
       PreconditionBoomerAMG (const MatrixBase     &matrix,
 			     const AdditionalData &additional_data = AdditionalData());
 
+				       /**
+					* Initializes the preconditioner
+					* object and calculate all data that
+					* is necessary for applying it in a
+					* solver. This function is
+					* automatically called when calling
+					* the constructor with the same
+					* arguments and is only used if you
+					* create the preconditioner without
+					* arguments.
+					*/
+      void initialize (const MatrixBase     &matrix,
+                       const AdditionalData &additional_data = AdditionalData());
+
     protected:
                                        /**
                                         * Store a copy of the flags for this
                                         * particular preconditioner.
                                         */
-      const AdditionalData additional_data;
-
-                                       /**
-                                        * Function that takes a Krylov
-                                        * Subspace Preconditioner context
-                                        * object, and sets the type of
-                                        * preconditioner that is appropriate
-                                        * for the present class.
-                                        */
-      virtual void set_preconditioner_type (PC &pc) const;
+      AdditionalData additional_data;
   };
 }
 
