@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_solver.h 20481 2010-02-01 12:41:43Z kronbichler $
-//    Version: $Name$
+//    $Id: petsc_solver.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
@@ -14,13 +13,13 @@
 #define __deal2__petsc_solver_h
 
 
-#include <base/config.h>
+#include <deal.II/base/config.h>
 
 #ifdef DEAL_II_USE_PETSC
 
-#  include <lac/exceptions.h>
-#  include <lac/solver_control.h>
-#  include <base/std_cxx1x/shared_ptr.h>
+#  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/solver_control.h>
+#  include <deal.II/base/std_cxx1x/shared_ptr.h>
 
 #  include <petscksp.h>
 
@@ -1040,11 +1039,12 @@ namespace PETScWrappers
 
 /**
  * An implementation of the solver interface using the PETSc PREONLY
- * solver. Actually this is NOT a real solution algorithm. Its only
- * purpose is to provide a solver object, when the preconditioner
- * should be used as real solver. It is very useful in conjunction with
- * the complete LU decomposition preconditioner <tt> PreconditionLU </tt>,
- * which in conjunction with this solver class becomes a direct solver.
+ * solver. Actually this is NOT a real solution algorithm. solve() only
+ * applies the preconditioner once and returns immediately. Its only purpose
+ * is to provide a solver object, when the preconditioner should be used as a
+ * real solver. It is very useful in conjunction with the complete LU
+ * decomposition preconditioner <tt> PreconditionLU </tt>, which in
+ * conjunction with this solver class becomes a direct solver.
  *
  * @ingroup PETScWrappers
  * @author Wolfgang Bangerth, 2004, Oliver Kayser-Herold, 2004
