@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_vector.h 22914 2010-12-03 22:43:02Z heister $
-//    Version: $Name$
+//    $Id: petsc_vector.h 24320 2011-09-13 09:41:50Z young $
 //
 //    Copyright (C) 2004, 2005, 2006, 2007, 2009 by the deal.II authors
 //
@@ -14,15 +13,15 @@
 #define __deal2__petsc_vector_h
 
 
-#include <base/config.h>
+#include <deal.II/base/config.h>
 
 #ifdef DEAL_II_USE_PETSC
 
-#  include <base/subscriptor.h>
-#  include <lac/exceptions.h>
-#  include <lac/petsc_vector_base.h>
-#  include <lac/petsc_parallel_vector.h>
-#  include <lac/vector.h>
+#  include <deal.II/base/subscriptor.h>
+#  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/petsc_vector_base.h>
+#  include <deal.II/lac/petsc_parallel_vector.h>
+#  include <deal.II/lac/vector.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -284,7 +283,11 @@ namespace PETScWrappers
                                      // the petsc function we call wants to
                                      // generate the vector itself, so destroy
                                      // the old one first
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
 	ierr = VecDestroy (vector);
+#else
+	ierr = VecDestroy (&vector);
+#endif
 	AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
 
@@ -297,9 +300,7 @@ namespace PETScWrappers
                                      // are hard to understand. gets me all
                                      // annoyed at their development
                                      // model</rant>
-#if (PETSC_VERSION_MAJOR <= 2) && \
-    ((PETSC_VERSION_MINOR < 2) ||  \
-     ((PETSC_VERSION_MINOR == 2) && (PETSC_VERSION_SUBMINOR == 0)))
+#if DEAL_II_PETSC_VERSION_LT(2,2,0)
     ierr = VecConvertMPIToSeqAll (static_cast<const Vec &>(v),
                                   &vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -334,7 +335,11 @@ namespace PETScWrappers
 #endif        
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
     ierr = VecScatterDestroy (ctx);
+#else
+    ierr = VecScatterDestroy (&ctx);
+#endif
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 #endif
     
