@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_vector_base.h 22914 2010-12-03 22:43:02Z heister $
-//    Version: $Name$
+//    $Id: petsc_vector_base.h 24007 2011-08-03 16:36:41Z bangerth $
 //
-//    Copyright (C) 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,18 +13,18 @@
 #define __deal2__petsc_vector_base_h
 
 
-#include <base/config.h>
+#include <deal.II/base/config.h>
 
 #ifdef DEAL_II_USE_PETSC
 
-#  include <base/subscriptor.h>
-#  include <lac/exceptions.h>
+#  include <deal.II/base/subscriptor.h>
+#  include <deal.II/lac/exceptions.h>
 
 #  include <vector>
 #  include <utility>
 
 #  include <petscvec.h>
-#  include <base/index_set.h>
+#  include <deal.II/base/index_set.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -173,6 +172,22 @@ namespace PETScWrappers
                         << " of a distributed vector, but only elements "
                         << arg2 << " through " << arg3
                         << " are stored locally and can be accessed.");
+	/**
+	 * Exception.
+	 */
+	DeclException2 (ExcWrongMode,
+			int, int,
+		        << "You tried to do a "
+			<< (arg1 == 1 ?
+			    "'set'" :
+			    (arg1 == 2 ?
+			     "'add'" : "???"))
+			<< " operation but the vector is currently in "
+			<< (arg2 == 1 ?
+			    "'set'" :
+			    (arg2 == 2 ?
+			     "'add'" : "???"))
+			<< " mode. You first have to call 'compress()'.");
 
       private:
                                          /**
@@ -236,7 +251,7 @@ namespace PETScWrappers
                                         */
       typedef PetscScalar               value_type;
       typedef PetscReal                 real_type;
-      typedef size_t                    size_type;
+      typedef std::size_t               size_type;
       typedef internal::VectorReference reference;
       typedef const internal::VectorReference const_reference;
 
@@ -372,6 +387,12 @@ namespace PETScWrappers
 					*/
       bool in_local_range (const unsigned int index) const;
 
+				       /**
+					* Return if the vector contains ghost
+					* elements.
+					*/
+      bool has_ghost_elements() const;
+       
                                        /**
                                         * Provide access to a given element,
                                         * both read and write.
@@ -742,7 +763,7 @@ namespace PETScWrappers
 					* consumption (not implemented
 					* for this class).
 					*/
-      unsigned int memory_consumption () const;
+      std::size_t memory_consumption () const;
 
     protected:
                                        /**
@@ -901,15 +922,11 @@ namespace PETScWrappers
     const VectorReference &
     VectorReference::operator = (const PetscScalar &value) const
     {
-      if (vector.last_action != VectorBase::LastAction::insert)
-        {
-          int ierr;
-          ierr = VecAssemblyBegin (vector);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-          ierr = VecAssemblyEnd (vector);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
-        }
+      Assert ((vector.last_action == VectorBase::LastAction::insert)
+              ||
+              (vector.last_action == VectorBase::LastAction::none),
+	      ExcWrongMode (VectorBase::LastAction::insert,
+			    vector.last_action));
 
 #ifdef PETSC_USE_64BIT_INDICES
       const PetscInt petsc_i = index;
@@ -932,17 +949,13 @@ namespace PETScWrappers
     const VectorReference &
     VectorReference::operator += (const PetscScalar &value) const
     {
-      if (vector.last_action != VectorBase::LastAction::add)
-        {
-          int ierr;
-          ierr = VecAssemblyBegin (vector);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
+      Assert ((vector.last_action == VectorBase::LastAction::add)
+              ||
+              (vector.last_action == VectorBase::LastAction::none),
+	      ExcWrongMode (VectorBase::LastAction::add,
+			    vector.last_action));
 
-          ierr = VecAssemblyEnd (vector);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-          vector.last_action = VectorBase::LastAction::add;
-        }
+      vector.last_action = VectorBase::LastAction::add;
 
                                        // we have to do above actions in any
                                        // case to be consistent with the MPI
@@ -974,19 +987,15 @@ namespace PETScWrappers
     const VectorReference &
     VectorReference::operator -= (const PetscScalar &value) const
     {
-      if (vector.last_action != VectorBase::LastAction::add)
-        {
-          int ierr;
-          ierr = VecAssemblyBegin (vector);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
+      Assert ((vector.last_action == VectorBase::LastAction::add)
+              ||
+              (vector.last_action == VectorBase::LastAction::none),
+	      ExcWrongMode (VectorBase::LastAction::add,
+			    vector.last_action));
 
-          ierr = VecAssemblyEnd (vector);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
+      vector.last_action = VectorBase::LastAction::add;
 
-          vector.last_action = VectorBase::LastAction::add;
-        }
-
-                                       // we have to do above actions in any
+	                               // we have to do above actions in any
                                        // case to be consistent with the MPI
                                        // communication model (see the
                                        // comments in the documentation of
@@ -1017,17 +1026,13 @@ namespace PETScWrappers
     const VectorReference &
     VectorReference::operator *= (const PetscScalar &value) const
     {
-      if (vector.last_action != VectorBase::LastAction::insert)
-        {
-          int ierr;
-          ierr = VecAssemblyBegin (vector);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
+      Assert ((vector.last_action == VectorBase::LastAction::insert)
+              ||
+              (vector.last_action == VectorBase::LastAction::none),
+	      ExcWrongMode (VectorBase::LastAction::insert,
+			    vector.last_action));
 
-          ierr = VecAssemblyEnd (vector);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-          vector.last_action = VectorBase::LastAction::insert;
-        }
+      vector.last_action = VectorBase::LastAction::insert;
 
                                        // we have to do above actions in any
                                        // case to be consistent with the MPI
@@ -1061,17 +1066,13 @@ namespace PETScWrappers
     const VectorReference &
     VectorReference::operator /= (const PetscScalar &value) const
     {
-      if (vector.last_action != VectorBase::LastAction::insert)
-        {
-          int ierr;
-          ierr = VecAssemblyBegin (vector);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
+      Assert ((vector.last_action == VectorBase::LastAction::insert)
+              ||
+              (vector.last_action == VectorBase::LastAction::none),
+	      ExcWrongMode (VectorBase::LastAction::insert,
+			    vector.last_action));
 
-          ierr = VecAssemblyEnd (vector);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-          vector.last_action = VectorBase::LastAction::insert;
-        }
+      vector.last_action = VectorBase::LastAction::insert;
 
                                        // we have to do above actions in any
                                        // case to be consistent with the MPI
@@ -1120,6 +1121,13 @@ namespace PETScWrappers
             (index < static_cast<unsigned int>(end)));
   }
 
+  inline
+  bool
+  VectorBase::has_ghost_elements() const
+  {
+    return ghosted;
+  }
+  
 
 
   inline
