@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: pointer_matrix.h 22198 2010-09-29 20:34:34Z bangerth $
-//    Version: $Name$
+//    $Id: pointer_matrix.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
@@ -13,10 +12,10 @@
 #ifndef __deal2__pointer_matrix_h
 #define __deal2__pointer_matrix_h
 
-#include <base/subscriptor.h>
-#include <base/smartpointer.h>
-#include <lac/vector.h>
-#include <lac/vector_memory.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/vector_memory.h>
 
 DEAL_II_NAMESPACE_OPEN
 
