@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: precondition.h 21447 2010-07-02 18:17:17Z kanschat $
-//    Version: $Name$
+//    $Id: precondition.h 24170 2011-08-23 09:41:55Z kronbichler $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,11 +14,12 @@
 
 // This file contains simple preconditioners.
 
-#include <base/config.h>
-#include <base/smartpointer.h>
-#include <base/template_constraints.h>
-#include <lac/tridiagonal_matrix.h>
-#include <lac/vector_memory.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/base/template_constraints.h>
+#include <deal.II/lac/tridiagonal_matrix.h>
+#include <deal.II/lac/vector_memory.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -57,10 +57,10 @@ class PreconditionIdentity : public Subscriptor
   public:
 
 				     /**
-				      * This function is only 
-                                      * present to 
+				      * This function is only
+                                      * present to
                                       * provide the interface of
-                                      * a precondtioner to be 
+                                      * a preconditioner to be
                                       * handed to a smoother.
                                       * This does nothing.
 				      */
@@ -115,10 +115,10 @@ class PreconditionIdentity : public Subscriptor
     void Tvmult_add (VECTOR&, const VECTOR&) const;
 
 				     /**
-				      * This function is only 
-                                      * present to 
+				      * This function is only
+                                      * present to
                                       * provide the interface of
-                                      * a precondtioner to be 
+                                      * a preconditioner to be
                                       * handed to a smoother.
                                       * This does nothing.
 				      */
@@ -219,10 +219,10 @@ class PreconditionRichardson : public Subscriptor
     void Tvmult_add (VECTOR&, const VECTOR&) const;
 
 				     /**
-				      * This function is only 
-				      * present to 
+				      * This function is only
+				      * present to
 				      * provide the interface of
-				      * a precondtioner to be 
+				      * a preconditioner to be
 				      * handed to a smoother.
 				      * This does nothing.
 				      */
@@ -1337,7 +1337,7 @@ PreconditionSSOR<MATRIX>::initialize (const MATRIX &rA,
 				       // line denotes the diagonal element,
 				       // which we need not check.
 	  pos_right_of_diagonal[row] =
-	    std::lower_bound (&colnums[*rowstart_ptr+1],
+	    Utilities::lower_bound (&colnums[*rowstart_ptr+1],
 			      &colnums[*(rowstart_ptr+1)],
 			      row)
 	    - colnums;
@@ -1592,6 +1592,8 @@ PreconditionChebyshev<MATRIX,VECTOR>::initialize (const MATRIX &matrix,
 						  const AdditionalData &additional_data)
 {
   Assert (matrix.m() == matrix.n(), ExcMessage("Matrix not quadratic."));
+  Assert (additional_data.eig_cg_n_iterations > 2,
+	  ExcMessage ("Need to set at least two iterations to find eigenvalues."));
   matrix_ptr = &matrix;
   data = additional_data;
   if (data.matrix_diagonal_inverse.size() != matrix.m())
@@ -1644,7 +1646,10 @@ PreconditionChebyshev<MATRIX,VECTOR>::initialize (const MATRIX &matrix,
 	g.add (alpha, update1);
 	res = g.l2_norm();
 
-	if (it > data.eig_cg_n_iterations || res < data.eig_cg_residual)
+				// need at least two iterations to have
+				// maximum and minimum eigenvalue
+	if (it > data.eig_cg_n_iterations || (it > 2 &&
+					      res < data.eig_cg_residual))
 	  break;
 
 	beta = gh;
