@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: precondition_block.h 22871 2010-11-28 23:21:29Z kanschat $
-//    Version: $Name$
+//    $Id: precondition_block.h 24139 2011-08-21 00:42:52Z bangerth $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,11 +13,11 @@
 #define __deal2__precondition_block_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/subscriptor.h>
-#include <base/smartpointer.h>
-#include <lac/precondition_block_base.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/lac/precondition_block_base.h>
 
 #include <vector>
 
@@ -199,11 +198,50 @@ class PreconditionBlock
 		     const AdditionalData parameters);
 
 				     /**
-				      * Set the permutation and its
-				      * inverse. These vectors are
-				      * copied into private data, so
-				      * they can be reused or deleted
-				      * after a call to this function.
+				      * Set either the permutation of
+				      * rows or the permutation of
+				      * blocks, depending on the size
+				      * of the vector.
+				      *
+				      * If the size of the permutation
+				      * vectors is equal to the
+				      * dimension of the linear
+				      * system, it is assumed that
+				      * rows are permuted
+				      * individually. In this case,
+				      * set_permutation() must be
+				      * called before initialize(),
+				      * since the diagonal blocks are
+				      * built from the permuted
+				      * entries of the matrix.
+				      *
+				      * If the size of the permutation
+				      * vector is not equal to the
+				      * dimension of the system, the
+				      * diagonal blocks are computed
+				      * from the unpermuted
+				      * entries. Instead, the
+				      * relaxation methods step() and
+				      * Tstep() are executed applying
+				      * the blocks in the order given
+				      * by the permutation
+				      * vector. They will throw an
+				      * exception if length of this
+				      * vector is not equal to the
+				      * number of blocks.
+				      *
+				      * @note Permutation of blocks
+				      * can only be applied to the
+				      * relaxation operators step()
+				      * and Tstep(), not to the
+				      * preconditioning operators
+				      * vmult() and Tvmult().
+				      *
+				      * @note It is safe to call
+				      * set_permutation() before
+				      * initialize(), while the other
+				      * order is only admissible for
+				      * block permutation.
 				      */
     void set_permutation(const std::vector<unsigned int>& permutation,
 			 const std::vector<unsigned int>& inverse_permutation);
@@ -338,7 +376,7 @@ class PreconditionBlock
 				      * memory consumption (in bytes)
 				      * of this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
     				     /** @addtogroup Exceptions
 				      * @{ */
@@ -389,7 +427,7 @@ class PreconditionBlock
 				      * used by derived classes.
 				      */
     double relaxation;
-    
+
 				     /**
 				      * The permutation vector
 				      */
@@ -561,22 +599,25 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
         Accessor accessor;
     };
 
-    PreconditionBlock<MATRIX, inverse_type>::initialize;
-    PreconditionBlock<MATRIX, inverse_type>::clear;
-    PreconditionBlock<MATRIX, inverse_type>::empty;
-    PreconditionBlock<MATRIX, inverse_type>::el;
-    PreconditionBlock<MATRIX, inverse_type>::set_same_diagonal;
-    PreconditionBlock<MATRIX, inverse_type>::invert_diagblocks;
-    PreconditionBlock<MATRIX, inverse_type>::block_size;
-    PreconditionBlockBase<inverse_type>::size;
-    PreconditionBlockBase<inverse_type>::inverse;
-    PreconditionBlockBase<inverse_type>::inverse_householder;
-    PreconditionBlockBase<inverse_type>::inverse_svd;
+				     /**
+				      * import functions from private base class
+				      */
+    using PreconditionBlock<MATRIX, inverse_type>::initialize;
+    using PreconditionBlock<MATRIX, inverse_type>::clear;
+    using PreconditionBlock<MATRIX, inverse_type>::empty;
+    using PreconditionBlock<MATRIX, inverse_type>::el;
+    using PreconditionBlock<MATRIX, inverse_type>::set_same_diagonal;
+    using PreconditionBlock<MATRIX, inverse_type>::invert_diagblocks;
+    using PreconditionBlock<MATRIX, inverse_type>::block_size;
+    using PreconditionBlockBase<inverse_type>::size;
+    using PreconditionBlockBase<inverse_type>::inverse;
+    using PreconditionBlockBase<inverse_type>::inverse_householder;
+    using PreconditionBlockBase<inverse_type>::inverse_svd;
 				     /**
 				      * @deprecated Use size() instead
 				      */
-    PreconditionBlock<MATRIX, inverse_type>::n_blocks;
-    PreconditionBlock<MATRIX, inverse_type>::set_permutation;
+    using PreconditionBlock<MATRIX, inverse_type>::n_blocks;
+    using PreconditionBlock<MATRIX, inverse_type>::set_permutation;
 
 				     /**
 				      * Execute block Jacobi
@@ -722,27 +763,27 @@ class PreconditionBlockSOR : public virtual Subscriptor,
 				      * Default constructor.
 				      */
     PreconditionBlockSOR();
-    
+
 				     /**
 				      * Define number type of matrix.
 				      */
     typedef typename MATRIX::value_type number;
 
 				     /**
-				      * Make type publicly available.
+				      * import types and functions from protected base class.
 				      */
-    PreconditionBlock<MATRIX,inverse_type>::AdditionalData;
-    PreconditionBlock<MATRIX, inverse_type>::initialize;
-    PreconditionBlock<MATRIX, inverse_type>::clear;
-    PreconditionBlock<MATRIX, inverse_type>::empty;
-    PreconditionBlockBase<inverse_type>::size;
-    PreconditionBlockBase<inverse_type>::inverse;
-    PreconditionBlockBase<inverse_type>::inverse_householder;
-    PreconditionBlockBase<inverse_type>::inverse_svd;
-    PreconditionBlock<MATRIX, inverse_type>::el;
-    PreconditionBlock<MATRIX, inverse_type>::set_same_diagonal;
-    PreconditionBlock<MATRIX, inverse_type>::invert_diagblocks;
-    PreconditionBlock<MATRIX, inverse_type>::set_permutation;
+    using typename PreconditionBlock<MATRIX,inverse_type>::AdditionalData;
+    using PreconditionBlock<MATRIX, inverse_type>::initialize;
+    using PreconditionBlock<MATRIX, inverse_type>::clear;
+    using PreconditionBlock<MATRIX, inverse_type>::empty;
+    using PreconditionBlockBase<inverse_type>::size;
+    using PreconditionBlockBase<inverse_type>::inverse;
+    using PreconditionBlockBase<inverse_type>::inverse_householder;
+    using PreconditionBlockBase<inverse_type>::inverse_svd;
+    using PreconditionBlock<MATRIX, inverse_type>::el;
+    using PreconditionBlock<MATRIX, inverse_type>::set_same_diagonal;
+    using PreconditionBlock<MATRIX, inverse_type>::invert_diagblocks;
+    using PreconditionBlock<MATRIX, inverse_type>::set_permutation;
 
 				     /**
 				      * Execute block SOR
@@ -840,7 +881,7 @@ class PreconditionBlockSOR : public virtual Subscriptor,
 				      * PreconditionBlockSSOR.
 				      */
     PreconditionBlockSOR(bool store);
-    
+
 				     /**
 				      * Implementation of the forward
 				      * substitution loop called by
@@ -916,7 +957,7 @@ class PreconditionBlockSSOR : public virtual Subscriptor,
     PreconditionBlockSSOR ();
 
 				     // Keep AdditionalData accessible
-    PreconditionBlockSOR<MATRIX,inverse_type>::AdditionalData;
+    using typename PreconditionBlockSOR<MATRIX,inverse_type>::AdditionalData;
 
 				     // The following are the
 				     // functions of the base classes
@@ -926,17 +967,17 @@ class PreconditionBlockSSOR : public virtual Subscriptor,
 				      * Make initialization function
 				      * publicly available.
 				      */
-    PreconditionBlockSOR<MATRIX,inverse_type>::initialize;
-    PreconditionBlockSOR<MATRIX,inverse_type>::clear;
-    PreconditionBlockBase<inverse_type>::size;
-    PreconditionBlockBase<inverse_type>::inverse;
-    PreconditionBlockBase<inverse_type>::inverse_householder;
-    PreconditionBlockBase<inverse_type>::inverse_svd;
-    PreconditionBlockSOR<MATRIX,inverse_type>::set_permutation;
-    PreconditionBlockSOR<MATRIX, inverse_type>::empty;
-    PreconditionBlockSOR<MATRIX, inverse_type>::el;
-    PreconditionBlockSOR<MATRIX,inverse_type>::set_same_diagonal;
-    PreconditionBlockSOR<MATRIX,inverse_type>::invert_diagblocks;
+    using PreconditionBlockSOR<MATRIX,inverse_type>::initialize;
+    using PreconditionBlockSOR<MATRIX,inverse_type>::clear;
+    using PreconditionBlockBase<inverse_type>::size;
+    using PreconditionBlockBase<inverse_type>::inverse;
+    using PreconditionBlockBase<inverse_type>::inverse_householder;
+    using PreconditionBlockBase<inverse_type>::inverse_svd;
+    using PreconditionBlockSOR<MATRIX,inverse_type>::set_permutation;
+    using PreconditionBlockSOR<MATRIX, inverse_type>::empty;
+    using PreconditionBlockSOR<MATRIX, inverse_type>::el;
+    using PreconditionBlockSOR<MATRIX,inverse_type>::set_same_diagonal;
+    using PreconditionBlockSOR<MATRIX,inverse_type>::invert_diagblocks;
 
 				     /**
 				      * Execute block SSOR
