@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: precondition_block.templates.h 22871 2010-11-28 23:21:29Z kanschat $
-//    Version: $Name$
+//    $Id: precondition_block.templates.h 24140 2011-08-21 00:43:11Z bangerth $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,14 +13,14 @@
 #define __deal2__precondition_block_templates_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/logstream.h>
-#include <base/memory_consumption.h>
-#include <lac/householder.h>
-#include <lac/precondition_block.h>
-#include <lac/vector.h>
-#include <lac/full_matrix.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/logstream.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/lac/householder.h>
+#include <deal.II/lac/precondition_block.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/full_matrix.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -75,7 +74,7 @@ void PreconditionBlock<MATRIX,inverse_type>::initialize (
   const AdditionalData parameters)
 {
   const unsigned int bsize = parameters.block_size;
-  
+
   clear();
   Assert (M.m() == M.n(), ExcNotQuadratic());
   A = &M;
@@ -86,9 +85,14 @@ void PreconditionBlock<MATRIX,inverse_type>::initialize (
   const unsigned int nblocks = A->m()/bsize;
   this->reinit(nblocks, blocksize, parameters.same_diagonal,
 	       parameters.inversion);
-  
+
   if (parameters.invert_diagonal)
-    invert_diagblocks();
+    {
+      if (permutation.size() == M.m())
+	invert_permuted_diagblocks(permutation, inverse_permutation);
+      else
+	invert_diagblocks();
+    }
 }
 
 
@@ -99,22 +103,8 @@ void PreconditionBlock<MATRIX,inverse_type>::initialize (
   const std::vector<unsigned int>& inverse_permutation,
   const AdditionalData parameters)
 {
-  
-  const unsigned int bsize = parameters.block_size;
-  
-  clear();
-  Assert (M.m() == M.n(), ExcNotQuadratic());
-  A = &M;
-  Assert (bsize>0, ExcIndexRange(bsize, 1, M.m()));
-  Assert (A->m()%bsize==0, ExcWrongBlockSize(bsize, A->m()));
-  blocksize=bsize;
-  relaxation = parameters.relaxation;
-  const unsigned int nblocks = A->m()/bsize;
-  this->reinit(nblocks, blocksize, parameters.same_diagonal);
-  
-  if (parameters.invert_diagonal)
-    invert_permuted_diagblocks(permutation, inverse_permutation);
-  
+  set_permutation(permutation, inverse_permutation);
+  initialize(M, parameters);
 }
 
 template <class MATRIX, typename inverse_type>
@@ -127,13 +117,15 @@ void PreconditionBlock<MATRIX,inverse_type>::invert_permuted_diagblocks(
 
   const MATRIX &M=*A;
   Assert (this->inverses_ready()==0, ExcInverseMatricesAlreadyExist());
+  AssertDimension (permutation.size(), M.m());
+  AssertDimension (inverse_permutation.size(), M.m());
 
   FullMatrix<inverse_type> M_cell(blocksize);
 
   if (this->same_diagonal())
     {
       deallog << "PreconditionBlock uses only one diagonal block" << std::endl;
-      
+
       for (unsigned int row_cell=0; row_cell<blocksize; ++row_cell)
 	{
 	  typename MATRIX::const_iterator entry = M.begin(row_cell);
@@ -174,7 +166,7 @@ void PreconditionBlock<MATRIX,inverse_type>::invert_permuted_diagblocks(
 				       // row, column are the global numbering
 				       // of the unkowns.
       M_cell = 0;
-      
+
       for (unsigned int cell=0; cell<this->size(); ++cell)
 	{
 	  const unsigned int cell_start = cell*blocksize;
@@ -192,14 +184,14 @@ void PreconditionBlock<MATRIX,inverse_type>::invert_permuted_diagblocks(
 		  //if (entry->column()<cell_start)
 		  if (inverse_permutation[entry->column()]<cell_start)
 		    continue;
-		  
+
 		  const unsigned int column_cell = inverse_permutation[entry->column()]-cell_start;
 		  if (column_cell >= blocksize)
 		    continue;
 		  M_cell(row_cell, column_cell) = entry->value();
 		}
 	    }
-	  
+
  	  if (this->store_diagonals())
  	    this->diagonal(cell) = M_cell;
 	  switch (this->inversion)
@@ -216,7 +208,7 @@ void PreconditionBlock<MATRIX,inverse_type>::invert_permuted_diagblocks(
 		    break;
 	      default:
 		    Assert(false, ExcNotImplemented());
-		    
+
 	    }
 	}
     }
@@ -234,14 +226,21 @@ void PreconditionBlock<MATRIX,inverse_type>::forward_step (
   const bool transpose_diagonal) const
 {
   Assert (this->A!=0, ExcNotInitialized());
-  
+
   const MATRIX &M=*this->A;
-  const bool permuted = (permutation.size() != 0);
-  if (permuted)
-    {
-      Assert (permutation.size() == M.m(), ExcDimensionMismatch(permutation.size(), M.m()));
-    }
-  
+
+  if (permutation.size() != 0)
+    Assert (permutation.size() == M.m() || permutation.size() == this->size(),
+	    ExcMessage("Permutation vector size must be equal to either the number of blocks or the dimension of the system"));
+
+  const bool permuted = (permutation.size() == M.m());
+  const bool cell_permuted = (permutation.size() == this->size());
+
+//   deallog << "Permutation " << permutation.size();
+//   if (permuted) deallog << " point";
+//   if (cell_permuted) deallog << " block";
+//   deallog << std::endl;
+
   Vector<number2> b_cell(this->blocksize), x_cell(this->blocksize);
 
 				       // cell_row, cell_column are the
@@ -251,26 +250,32 @@ void PreconditionBlock<MATRIX,inverse_type>::forward_step (
 				       // blocks.
 				       // row, column are the global numbering
 				       // of the unkowns.
-  unsigned int row, row_cell, block_start=0;
+  unsigned int row, row_cell;
   number2 b_cell_row;
 				   // The diagonal block if the
 				   // inverses were not precomputed
   FullMatrix<number> M_cell(this->blocksize);
 
 				   // Loop over all blocks
-  for (unsigned int cell=0; cell < this->size(); ++cell)
+  for (unsigned int rawcell=0; rawcell < this->size(); ++rawcell)
     {
+      const unsigned int cell = cell_permuted ? permutation[rawcell] : rawcell;
+      const unsigned int block_start = cell*this->blocksize;
       const unsigned int permuted_block_start = permuted
 						? permutation[block_start]
-						:block_start;
-      
+						: block_start;
+
+//       deallog << std::endl << cell << '-' << block_start
+// 	      << '-' << permuted_block_start << (permuted ? 't' : 'f') << '\t';
+
       for (row = permuted_block_start, row_cell = 0;
 	   row_cell < this->blocksize;
 	   ++row_cell, ++row)
 	{
+// 	  deallog << ' ' << row;
 	  const typename MATRIX::const_iterator row_end = M.end(row);
 	  typename MATRIX::const_iterator entry = M.begin(row);
-	  
+
 	  b_cell_row=src(row);
 	  for (; entry != row_end; ++entry)
 	    {
@@ -278,12 +283,11 @@ void PreconditionBlock<MATRIX,inverse_type>::forward_step (
 	      const unsigned int inverse_permuted_column = permuted
 							   ? inverse_permutation[column]
 							   : column;
-	      
 	      b_cell_row -= entry->value() * prev(column);
-//TODO:[GK] Find out if this is really once column and once permuted 
+//TODO:[GK] Find out if this is really once column and once permuted
 	      if (!this->inverses_ready()
 		  && inverse_permuted_column >= block_start
-		  && column < block_start + this->blocksize)
+		  && inverse_permuted_column < block_start + this->blocksize)
 		{
 		  const unsigned int column_cell = column - block_start;
 		  if (transpose_diagonal)
@@ -306,14 +310,12 @@ void PreconditionBlock<MATRIX,inverse_type>::forward_step (
 	  Householder<number> house(M_cell);
 	  house.least_squares(x_cell,b_cell);
 	}
-      
+
 				       // distribute x_cell to dst
       for (row=permuted_block_start, row_cell=0;
 	   row_cell<this->blocksize;
 	   ++row_cell, ++row)
 	dst(row) = prev(row) + this->relaxation*x_cell(row_cell);
-      
-      block_start+=this->blocksize;
     }
 }
 
@@ -327,13 +329,15 @@ void PreconditionBlock<MATRIX,inverse_type>::backward_step (
   const bool transpose_diagonal) const
 {
   Assert (this->A!=0, ExcNotInitialized());
-  
+
   const MATRIX &M=*this->A;
-  const bool permuted = (permutation.size() != 0);
-  if (permuted)
-    {
-      Assert (permutation.size() == M.m(), ExcDimensionMismatch(permutation.size(), M.m()));
-    }
+
+  if (permutation.size() != 0)
+    Assert (permutation.size() == M.m() || permutation.size() == this->size(),
+	    ExcMessage("Permutation vector size must be equal to either the number of blocks or the dimension of the system"));
+
+  const bool permuted = (permutation.size() == M.m());
+  const bool cell_permuted = (permutation.size() == this->size());
 
   Vector<number2> b_cell(this->blocksize), x_cell(this->blocksize);
 
@@ -345,25 +349,25 @@ void PreconditionBlock<MATRIX,inverse_type>::backward_step (
 				       // row, column are the global numbering
 				       // of the unkowns.
   unsigned int row, row_cell;
-  unsigned int block_end=this->blocksize * this->size();
   number2 b_cell_row;
 
   FullMatrix<number> M_cell(this->blocksize);
-  for (unsigned int cell=this->size(); cell!=0 ;)
+  for (unsigned int rawcell=this->size(); rawcell!=0 ;)
     {
-      --cell;
-      const unsigned int block_start = block_end - this->blocksize;
-				       // Collect upper triangle
-      const unsigned int permuted_block_start = (permutation.size() != 0)
+      --rawcell;
+      const unsigned int cell = cell_permuted ? permutation[rawcell] : rawcell;
+      const unsigned int block_start = cell*this->blocksize;
+      const unsigned int block_end = block_start + this->blocksize;
+      const unsigned int permuted_block_start = permuted
 						? permutation[block_start]
-						:block_start;
+						: block_start;
       for (row = permuted_block_start, row_cell = 0;
 	   row_cell<this->blocksize;
 	   ++row_cell, ++row)
 	{
 	  const typename MATRIX::const_iterator row_end = M.end(row);
 	  typename MATRIX::const_iterator entry = M.begin(row);
-	  
+
 	  b_cell_row=src(row);
 	  for (; entry != row_end; ++entry)
 	    {
@@ -403,15 +407,13 @@ void PreconditionBlock<MATRIX,inverse_type>::backward_step (
 	  Householder<number> house(M_cell);
 	  house.least_squares(x_cell,b_cell);
 	}
-      
-      
+
+
 				       // distribute x_cell to dst
       for (row=permuted_block_start, row_cell=0;
 	   row_cell<this->blocksize;
 	   ++row_cell, ++row)
 	dst(row) = prev(row) + this->relaxation*x_cell(row_cell);
-      block_end = block_start;
-      
     }
 }
 
@@ -431,7 +433,7 @@ void PreconditionBlock<MATRIX,inverse_type>::invert_diagblocks()
 
   const MATRIX &M=*A;
   Assert (this->inverses_ready()==0, ExcInverseMatricesAlreadyExist());
-  
+
   FullMatrix<inverse_type> M_cell(blocksize);
 
   if (this->same_diagonal())
@@ -469,7 +471,7 @@ void PreconditionBlock<MATRIX,inverse_type>::invert_diagblocks()
   else
     {
       M_cell = 0;
-      
+
       for (unsigned int cell=0; cell<this->size(); ++cell)
 	{
 	  const unsigned int cell_start = cell*blocksize;
@@ -483,7 +485,7 @@ void PreconditionBlock<MATRIX,inverse_type>::invert_diagblocks()
 		{
 		  if (entry->column()<cell_start)
 		    continue;
-		  
+
 		  const unsigned int column_cell = entry->column()-cell_start;
 		  if (column_cell >= blocksize)
 		    continue;
@@ -521,6 +523,12 @@ void PreconditionBlock<MATRIX,inverse_type>::set_permutation (
   const std::vector<unsigned int>& i)
 {
   Assert (p.size() == i.size(), ExcDimensionMismatch(p.size(), i.size()));
+
+  if (this->inverses_ready())
+    {
+      AssertDimension(p.size(), this->size());
+    }
+
   permutation.resize(p.size());
   inverse_permutation.resize(p.size());
   for (unsigned int k=0;k<p.size();++k)
@@ -532,12 +540,12 @@ void PreconditionBlock<MATRIX,inverse_type>::set_permutation (
 
 
 template <class MATRIX, typename inverse_type>
-unsigned int
+std::size_t
 PreconditionBlock<MATRIX,inverse_type>::memory_consumption () const
 {
-  unsigned int mem = sizeof(*this) - sizeof(PreconditionBlockBase<inverse_type>);
-  mem += PreconditionBlockBase<inverse_type>::memory_consumption();
-  return mem;
+  return (sizeof(*this)
+	  - sizeof(PreconditionBlockBase<inverse_type>)
+	  + PreconditionBlockBase<inverse_type>::memory_consumption());
 }
 
 
@@ -571,7 +579,7 @@ void PreconditionBlockJacobi<MATRIX,inverse_type>
 				   // args...
   typedef PreconditionBlock<MATRIX,inverse_type> BaseClass;
   Assert(this->A!=0, ExcNotInitialized());
-  
+
   const MATRIX &M=*this->A;
 
   Vector<number2> b_cell(this->blocksize), x_cell(this->blocksize);
@@ -609,7 +617,7 @@ void PreconditionBlockJacobi<MATRIX,inverse_type>
 	      dst(row)+=x_cell(row_cell);
 	    else
 	      dst(row)=x_cell(row_cell);
-	  
+
 	  begin_diag_block+=this->blocksize;
 	}
     }
@@ -631,7 +639,7 @@ void PreconditionBlockJacobi<MATRIX,inverse_type>
 	    dst(row)+=x_cell(row_cell);
 	  else
 	    dst(row)=x_cell(row_cell);
-	
+
 	begin_diag_block+=this->blocksize;
       }
   dst.scale(this->relaxation);
@@ -687,8 +695,8 @@ void PreconditionBlockJacobi<MATRIX,inverse_type>
   GrowingVectorMemory<Vector<number2> > mem;
   typename VectorMemory<Vector<number2> >::Pointer aux(mem);
   aux->reinit(dst);
-  
-  forward_step(*aux, dst, src, false);
+
+  this->forward_step(*aux, dst, src, false);
   dst = *aux;
 }
 
@@ -702,8 +710,8 @@ void PreconditionBlockJacobi<MATRIX,inverse_type>
   GrowingVectorMemory<Vector<number2> > mem;
   typename VectorMemory<Vector<number2> >::Pointer aux(mem);
   aux->reinit(dst);
-  
-  backward_step(*aux, dst, src, true);
+
+  this->backward_step(*aux, dst, src, true);
   dst = *aux;
 }
 
@@ -716,13 +724,13 @@ void PreconditionBlockJacobi<MATRIX,inverse_type>
 template <class MATRIX, typename inverse_type>
 PreconditionBlockSOR<MATRIX,inverse_type>::PreconditionBlockSOR ()
 		: PreconditionBlock<MATRIX,inverse_type> (false)
-		
+
 {}
 
 template <class MATRIX, typename inverse_type>
 PreconditionBlockSOR<MATRIX,inverse_type>::PreconditionBlockSOR (bool store)
 		: PreconditionBlock<MATRIX,inverse_type> (store)
-		
+
 {}
 
 template <class MATRIX, typename inverse_type>
@@ -752,14 +760,14 @@ void PreconditionBlockSOR<MATRIX,inverse_type>::forward (
   typedef PreconditionBlock<MATRIX,inverse_type> BaseClass;
 
   Assert (this->A!=0, ExcNotInitialized());
-  
+
   const MATRIX &M=*this->A;
   const bool permuted = (this->permutation.size() != 0);
   if (permuted)
     {
       Assert (this->permutation.size() == M.m(), ExcDimensionMismatch(this->permutation.size(), M.m()));
     }
-  
+
   Vector<number2> b_cell(this->blocksize), x_cell(this->blocksize);
 
 				       // cell_row, cell_column are the
@@ -774,20 +782,20 @@ void PreconditionBlockSOR<MATRIX,inverse_type>::forward (
 				   // The diagonal block if the
 				   // inverses were not precomputed
   FullMatrix<number> M_cell(this->blocksize);
-  
+
   for (unsigned int cell=0; cell < this->size(); ++cell)
     {
       const unsigned int permuted_block_start = permuted
 						? this->permutation[block_start]
 						:block_start;
-      
+
       for (row = permuted_block_start, row_cell = 0;
 	   row_cell < this->blocksize;
 	   ++row_cell, ++row)
 	{
 	  const typename MATRIX::const_iterator row_end = M.end(row);
 	  typename MATRIX::const_iterator entry = M.begin(row);
-	  
+
 	  b_cell_row=src(row);
 	  for (; entry != row_end; ++entry)
 	    {
@@ -795,7 +803,7 @@ void PreconditionBlockSOR<MATRIX,inverse_type>::forward (
 	      const unsigned int inverse_permuted_column = permuted
 							   ? this->inverse_permutation[column]
 							   : column;
-	      
+
 	      if (inverse_permuted_column < block_start)
 		b_cell_row -= entry->value() * dst(column);
 	      else if (!this->inverses_ready() && column < block_start + this->blocksize)
@@ -821,13 +829,13 @@ void PreconditionBlockSOR<MATRIX,inverse_type>::forward (
 	  Householder<number> house(M_cell);
 	  house.least_squares(x_cell,b_cell);
 	}
-      
+
 				       // distribute x_cell to dst
       for (row=permuted_block_start, row_cell=0;
 	   row_cell<this->blocksize;
 	   ++row_cell, ++row)
 	dst(row)=this->relaxation*x_cell(row_cell);
-      
+
       block_start+=this->blocksize;
     }
 }
@@ -860,7 +868,7 @@ void PreconditionBlockSOR<MATRIX,inverse_type>::backward (
   typedef PreconditionBlock<MATRIX,inverse_type> BaseClass;
 
   Assert (this->A!=0, ExcNotInitialized());
-  
+
   const MATRIX &M=*this->A;
   const bool permuted = (this->permutation.size() != 0);
   if (permuted)
@@ -896,7 +904,7 @@ void PreconditionBlockSOR<MATRIX,inverse_type>::backward (
 	{
 	  const typename MATRIX::const_iterator row_end = M.end(row);
 	  typename MATRIX::const_iterator entry = M.begin(row);
-	  
+
 	  b_cell_row=src(row);
 	  for (; entry != row_end; ++entry)
 	    {
@@ -935,15 +943,15 @@ void PreconditionBlockSOR<MATRIX,inverse_type>::backward (
 	  Householder<number> house(M_cell);
 	  house.least_squares(x_cell,b_cell);
 	}
-      
-      
+
+
 				       // distribute x_cell to dst
       for (row=permuted_block_start, row_cell=0;
 	   row_cell<this->blocksize;
 	   ++row_cell, ++row)
 	dst(row)=this->relaxation*x_cell(row_cell);
       block_end = block_start;
-      
+
     }
 }
 
@@ -995,7 +1003,7 @@ void PreconditionBlockSOR<MATRIX,inverse_type>
 ::step (Vector<number2>       &dst,
 	const Vector<number2> &src) const
 {
-  forward_step(dst, dst, src, false);
+  this->forward_step(dst, dst, src, false);
 }
 
 
@@ -1005,7 +1013,7 @@ void PreconditionBlockSOR<MATRIX,inverse_type>
 ::Tstep (Vector<number2>       &dst,
 	const Vector<number2> &src) const
 {
-  backward_step(dst, dst, src, true);
+  this->backward_step(dst, dst, src, true);
 }
 
 
@@ -1017,7 +1025,7 @@ void PreconditionBlockSOR<MATRIX,inverse_type>
 template <class MATRIX, typename inverse_type>
 PreconditionBlockSSOR<MATRIX,inverse_type>::PreconditionBlockSSOR ()
 		: PreconditionBlockSOR<MATRIX,inverse_type> (true)
-		
+
 {}
 
 
@@ -1028,18 +1036,18 @@ void PreconditionBlockSSOR<MATRIX,inverse_type>::vmult (Vector<number2>       &d
 {
   Vector<number2> help;
   help.reinit(dst);
-  
+
   this->forward(help, src, false, false);
 
   Vector<inverse_type> cell_src(this->blocksize);
   Vector<inverse_type> cell_dst(this->blocksize);
   const double scaling = (2.-this->relaxation)/this->relaxation;
-  
+
 				   // Multiply with diagonal blocks
   for (unsigned int cell=0; cell < this->size(); ++cell)
     {
       unsigned int row = cell*this->blocksize;
-      
+
       for (unsigned int row_cell=0; row_cell<this->blocksize; ++row_cell)
 	cell_src(row_cell)=help(row+row_cell);
 
@@ -1048,7 +1056,7 @@ void PreconditionBlockSSOR<MATRIX,inverse_type>::vmult (Vector<number2>       &d
       for (unsigned int row_cell=0; row_cell<this->blocksize; ++row_cell)
 	help(row+row_cell) = scaling * cell_dst(row_cell);
     }
-  
+
   this->backward(dst, help, false, false);
 }
 
@@ -1059,18 +1067,18 @@ void PreconditionBlockSSOR<MATRIX,inverse_type>::Tvmult (Vector<number2>       &
 {
   Vector<number2> help;
   help.reinit(dst);
-  
+
   this->backward(help, src, true, false);
 
   Vector<inverse_type> cell_src(this->blocksize);
   Vector<inverse_type> cell_dst(this->blocksize);
   const double scaling = (2.-this->relaxation)/this->relaxation;
-  
+
 				   // Multiply with diagonal blocks
   for (unsigned int cell=0; cell < this->size(); ++cell)
     {
       unsigned int row = cell*this->blocksize;
-      
+
       for (unsigned int row_cell=0; row_cell<this->blocksize; ++row_cell)
 	cell_src(row_cell)=help(row+row_cell);
 
@@ -1079,7 +1087,7 @@ void PreconditionBlockSSOR<MATRIX,inverse_type>::Tvmult (Vector<number2>       &
       for (unsigned int row_cell=0; row_cell<this->blocksize; ++row_cell)
 	help(row+row_cell) = scaling * cell_dst(row_cell);
     }
-  
+
   this->forward(dst, help, true, false);
 }
 
@@ -1090,8 +1098,8 @@ void PreconditionBlockSSOR<MATRIX,inverse_type>
 ::step (Vector<number2>       &dst,
 	const Vector<number2> &src) const
 {
-  forward_step(dst, dst, src, false);
-  backward_step(dst, dst, src, false);
+  this->forward_step(dst, dst, src, false);
+  this->backward_step(dst, dst, src, false);
 }
 
 
@@ -1101,8 +1109,8 @@ void PreconditionBlockSSOR<MATRIX,inverse_type>
 ::Tstep (Vector<number2>       &dst,
 	const Vector<number2> &src) const
 {
-  backward_step(dst, dst, src, true);
-  forward_step(dst, dst, src, true);
+  this->backward_step(dst, dst, src, true);
+  this->forward_step(dst, dst, src, true);
 }
 
 
