@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: precondition_block_base.h 22826 2010-11-20 00:00:06Z kanschat $
-//    Version: $Name$
+//    $Id: precondition_block_base.h 23876 2011-06-28 18:21:51Z kanschat $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,13 +13,13 @@
 #define __deal2__precondition_block_base_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/subscriptor.h>
-#include <base/smartpointer.h>
-#include <base/memory_consumption.h>
-#include <lac/householder.h>
-#include <lac/lapack_full_matrix.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/lac/householder.h>
+#include <deal.II/lac/lapack_full_matrix.h>
 
 #include <vector>
 
@@ -234,7 +233,7 @@ class PreconditionBlockBase
 				      * memory consumption (in bytes)
 				      * of this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
     
 				     /**
 				      * You are trying to access a
@@ -663,10 +662,10 @@ PreconditionBlockBase<number>::inverses_ready() const
 
 template <typename number>
 inline
-unsigned int
+std::size_t
 PreconditionBlockBase<number>::memory_consumption () const
 {
-  unsigned int mem = sizeof(*this);
+  std::size_t mem = sizeof(*this);
   for (unsigned int i=0; i<var_inverse_full.size(); ++i)
     mem += MemoryConsumption::memory_consumption(var_inverse_full[i]);
   for (unsigned int i=0; i<var_diagonal.size(); ++i)
