@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: slepc_solver.h 21609 2010-08-04 14:06:34Z young $
-//    Version: $Name$
+//    $Id: slepc_solver.h 24301 2011-09-10 18:01:38Z young $
 //    Author: Toby D. Young, Polish Academy of Sciences, 2008, 2009
 //
 //    Copyright (C) 2009, 2010 by the deal.II authors
@@ -16,15 +15,15 @@
 #define __deal2__slepc_solver_h
 
 
-#include <base/config.h>
+#include <deal.II/base/config.h>
 
 #ifdef DEAL_II_USE_SLEPC
 
-#  include <base/std_cxx1x/shared_ptr.h>
-#  include <lac/exceptions.h>
-#  include <lac/solver_control.h>
-#  include <lac/petsc_matrix_base.h>
-#  include <lac/slepc_spectral_transformation.h>
+#  include <deal.II/base/std_cxx1x/shared_ptr.h>
+#  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/solver_control.h>
+#  include <deal.II/lac/petsc_matrix_base.h>
+#  include <deal.II/lac/slepc_spectral_transformation.h>
 
 #  include <petscksp.h>
 #  include <slepceps.h>
@@ -319,21 +318,29 @@ namespace SLEPcWrappers
                                     */
       virtual void set_solver_type (EPS &eps) const = 0;
 
-                                   /*
-                                    * Attributes that store the
-				    * relevant information for the
-				    * eigenproblem solver
-				    * context. These are: which
-				    * portion of the spectrum to solve
-				    * from; and the matrices and
-				    * vectors that discribe the
-				    * problem.
+                                   /**
+                                    * Which portion of the spectrum to
+				    * solve from.
                                     */
-      EPSWhich                           set_which;
+      EPSWhich set_which;
 
-      const PETScWrappers::MatrixBase   *opA;
-      const PETScWrappers::MatrixBase   *opB;
-      const PETScWrappers::VectorBase   *initial_vector;
+                                   /**
+                                    * The matrix $A$ of the
+                                    * generalized eigenvalue problem
+                                    * $Ax=B\lambda x$, or the standard
+                                    * eigenvalue problem $Ax=\lambda
+                                    * x$.
+                                    */
+      const PETScWrappers::MatrixBase *opA;
+
+                                   /**
+                                    * The matrix $B$ of the
+                                    * generalized eigenvalue problem
+                                    * $Ax=B\lambda x$.
+                                    */
+      const PETScWrappers::MatrixBase *opB;
+
+      const PETScWrappers::VectorBase *initial_vector;
 
                                    /**
                                     * Pointer to an an object that
@@ -426,8 +433,8 @@ namespace SLEPcWrappers
                                     * the PETScWrappers, but you can
                                     * change that.
                                     */
-       SolverKrylovSchur (SolverControl        &cn,
-			  const MPI_Comm       &mpi_communicator = PETSC_COMM_WORLD,
+       SolverKrylovSchur (SolverControl        &cn, 
+			  const MPI_Comm       &mpi_communicator = PETSC_COMM_SELF,
 			  const AdditionalData &data = AdditionalData());
 
     protected:
@@ -452,7 +459,7 @@ namespace SLEPcWrappers
  * solver. Usage: All spectrum, all problem types, complex.
  *
  * @ingroup SLEPcWrappers
- * @author Toby D. Young 2008
+ * @author Toby D. Young 2008, 2011
  */
   class SolverArnoldi : public SolverBase
     {
@@ -463,7 +470,21 @@ namespace SLEPcWrappers
                                     * should it be needed.
                                     */
       struct AdditionalData
-      {};
+      {
+                                   /**
+                                    * Constructor. By default, set the
+                                    * option of delayed
+                                    * reorthogonalization to false,
+                                    * i.e. don't do it.
+                                    */
+          AdditionalData (const bool delayed_reorthogonalization = false);
+
+	                           /**
+				    * Flag for delayed
+				    * reorthogonalization.
+				    */
+	  bool delayed_reorthogonalization;
+      };
 
                                    /**
                                     * SLEPc solvers will want to have
@@ -475,7 +496,7 @@ namespace SLEPcWrappers
                                     * change that.
                                     */
       SolverArnoldi (SolverControl        &cn,
-		     const MPI_Comm       &mpi_communicator = PETSC_COMM_WORLD,
+		     const MPI_Comm       &mpi_communicator = PETSC_COMM_SELF,
 		     const AdditionalData &data = AdditionalData());
 
     protected:
@@ -524,7 +545,7 @@ namespace SLEPcWrappers
                                     * change that.
                                     */
       SolverLanczos (SolverControl        &cn,
-		     const MPI_Comm       &mpi_communicator = PETSC_COMM_WORLD,
+		     const MPI_Comm       &mpi_communicator = PETSC_COMM_SELF,
 		     const AdditionalData &data = AdditionalData());
 
     protected:
@@ -574,7 +595,7 @@ namespace SLEPcWrappers
                                     * change that.
                                     */
       SolverPower (SolverControl        &cn,
-		   const MPI_Comm       &mpi_communicator = PETSC_COMM_WORLD,
+		   const MPI_Comm       &mpi_communicator = PETSC_COMM_SELF,
 		   const AdditionalData &data = AdditionalData());
 
     protected:
@@ -624,7 +645,7 @@ namespace SLEPcWrappers
                                     * change that.
                                     */
       SolverDavidson (SolverControl        &cn,
-		      const MPI_Comm       &mpi_communicator = PETSC_COMM_WORLD,
+		      const MPI_Comm       &mpi_communicator = PETSC_COMM_SELF,
 		      const AdditionalData &data = AdditionalData());
 
     protected:
@@ -673,8 +694,7 @@ namespace SLEPcWrappers
       vr.resize (n_converged, vr.front());
       kr.resize (n_converged);
 
-      for (unsigned int index=0; index < n_converged;
-	   +index)
+      for (unsigned int index=0; index < n_converged; ++index)
 	get_eigenpair (index, kr[index], vr[index]);
     }
 
