@@ -1,6 +1,5 @@
 //----------------------------  solver_gmres.h  ---------------------------
-//    $Id: solver_gmres.h 22348 2010-10-15 08:47:45Z heister $
-//    Version: $Name$
+//    $Id: solver_gmres.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
@@ -15,14 +14,14 @@
 
 
 
-#include <base/config.h>
-#include <base/subscriptor.h>
-#include <base/logstream.h>
-#include <lac/householder.h>
-#include <lac/solver.h>
-#include <lac/solver_control.h>
-#include <lac/full_matrix.h>
-#include <lac/vector.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/logstream.h>
+#include <deal.II/lac/householder.h>
+#include <deal.II/lac/solver.h>
+#include <deal.II/lac/solver_control.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/vector.h>
 
 #include <vector>
 #include <cmath>
@@ -121,7 +120,7 @@ namespace internal
  *
  * The AdditionalData structure contains the number of temporary
  * vectors used. The size of the Arnoldi basis is this number minus
- * three. Addinitonally, it allows you to choose bet right or left
+ * three. Additionally, it allows you to choose between right or left
  * preconditioning. The default is left preconditioning. Finally it
  * includes a flag indicating whether or not the default residual is
  * used as stopping criterion.
