@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: sparse_matrix.h 22344 2010-10-14 15:59:09Z kanschat $
+//    $Id: sparse_matrix.h 23709 2011-05-17 04:34:08Z bangerth $
 //
 //    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
@@ -13,12 +13,12 @@
 #define __deal2__sparse_matrix_h
 
 
-#include <base/config.h>
-#include <base/subscriptor.h>
-#include <base/smartpointer.h>
-#include <lac/sparsity_pattern.h>
-#include <lac/identity_matrix.h>
-#include <lac/exceptions.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/lac/sparsity_pattern.h>
+#include <deal.II/lac/identity_matrix.h>
+#include <deal.II/lac/exceptions.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -546,7 +546,7 @@ class SparseMatrix : public virtual Subscriptor
     };
 
 /**
- * @name Constructors and initalization.
+ * @name Constructors and initalization
  */
 //@{
 				     /**
@@ -749,15 +749,21 @@ class SparseMatrix : public virtual Subscriptor
 				      */
     unsigned int n () const;
 
-				     /**
-				      * Return the number of nonzero
-				      * elements of this
-				      * matrix. Actually, it returns
-				      * the number of entries in the
-				      * sparsity pattern; if any of
-				      * the entries should happen to
-				      * be zero, it is counted anyway.
-				      */
+                                     /**
+                                      * Return the number of entries
+                                      * in a specific row.
+                                      */
+    unsigned int get_row_length (const unsigned int row) const;
+
+                                     /**
+                                      * Return the number of nonzero
+                                      * elements of this
+                                      * matrix. Actually, it returns
+                                      * the number of entries in the
+                                      * sparsity pattern; if any of
+                                      * the entries should happen to
+                                      * be zero, it is counted anyway.
+                                      */
     unsigned int n_nonzero_elements () const;
 
 				     /**
