@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: sparse_matrix.templates.h 22628 2010-11-07 23:11:12Z kanschat $
-//    Version: $Name$
+//    $Id: sparse_matrix.templates.h 23987 2011-08-02 08:59:22Z kronbichler $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,16 +14,17 @@
 #define __deal2__sparse_matrix_templates_h
 
 
-#include <base/config.h>
-#include <base/template_constraints.h>
-#include <base/parallel.h>
-#include <base/thread_management.h>
-#include <base/multithread_info.h>
-#include <lac/sparse_matrix.h>
-#include <lac/vector.h>
-#include <lac/full_matrix.h>
-#include <lac/compressed_simple_sparsity_pattern.h>
-#include <lac/vector_memory.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/template_constraints.h>
+#include <deal.II/base/parallel.h>
+#include <deal.II/base/thread_management.h>
+#include <deal.II/base/multithread_info.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/compressed_simple_sparsity_pattern.h>
+#include <deal.II/lac/vector_memory.h>
 
 // we only need output streams, but older compilers did not provide
 // them in a separate include file
@@ -40,7 +40,7 @@
 #include <cmath>
 #include <vector>
 #include <numeric>
-#include <base/std_cxx1x/bind.h>
+#include <deal.II/base/std_cxx1x/bind.h>
 
 
 
@@ -126,6 +126,22 @@ SparseMatrix<number>::~SparseMatrix ()
 
 
 
+namespace internal
+{
+  namespace SparseMatrix
+  {
+    template<typename T>
+    void zero_subrange (const unsigned int begin,
+			const unsigned int end,
+			T *dst)
+    {
+      std::memset (dst+begin,0,(end-begin)*sizeof(T));
+    }
+  }
+}
+
+
+
 template <typename number>
 SparseMatrix<number> &
 SparseMatrix<number>::operator = (const double d)
@@ -135,8 +151,31 @@ SparseMatrix<number>::operator = (const double d)
   Assert (cols != 0, ExcNotInitialized());
   Assert (cols->compressed || cols->empty(), SparsityPattern::ExcNotCompressed());
 
-  if (val != 0)
-    memset (&val[0], 0, cols->n_nonzero_elements()*sizeof(number));
+				// do initial zeroing of elements in
+				// parallel. Try to achieve a similar layout
+				// as when doing matrix-vector products, as on
+				// some NUMA systems, a memory block is
+				// assigned to memory banks where the first
+				// access is generated. For sparse matrices,
+				// the first operations is usually the
+				// operator=. The grain size is chosen to
+				// reflect the number of rows in
+				// minimum_parallel_grain_size, weighted by
+				// the number of nonzero entries per row on
+				// average.
+  const unsigned int matrix_size = cols->n_nonzero_elements();
+  const unsigned int grain_size =
+    internal::SparseMatrix::minimum_parallel_grain_size *
+    (cols->n_nonzero_elements()+m()) / m();
+  if (matrix_size>grain_size)
+    parallel::apply_to_subranges (0U, matrix_size,
+				  std_cxx1x::bind(&internal::SparseMatrix::template
+						  zero_subrange<number>,
+						  std_cxx1x::_1, std_cxx1x::_2,
+						  val),
+				  grain_size);
+  else if (matrix_size > 0)
+    memset (&val[0], 0, matrix_size*sizeof(number));
 
   return *this;
 }
@@ -214,6 +253,16 @@ SparseMatrix<number>::empty () const
 
 
 
+template <typename number>
+unsigned int
+SparseMatrix<number>::get_row_length (const unsigned int row) const
+{
+  Assert (cols != 0, ExcNotInitialized());
+  return cols->row_length(row);
+}
+
+
+
 template <typename number>
 unsigned int
 SparseMatrix<number>::n_nonzero_elements () const
@@ -231,7 +280,8 @@ SparseMatrix<number>::n_actually_nonzero_elements (const double threshold) const
   Assert (cols != 0, ExcNotInitialized());
   Assert (threshold >= 0, ExcMessage ("Negative threshold!"));
   unsigned int nnz = 0;
-  for (unsigned int i=0; i<n_nonzero_elements(); ++i)
+  const unsigned int nnz_alloc = n_nonzero_elements();
+  for (unsigned int i=0; i<nnz_alloc; ++i)
     if (std::fabs(val[i]) > threshold)
       ++nnz;
   return nnz;
@@ -424,9 +474,9 @@ SparseMatrix<number>::add (const unsigned int  row,
 				   // find diagonal and add it if found
 	  Assert (this_cols[0] == row, ExcInternalError());
 	  const unsigned int * diag_pos =
-	    internals::SparsityPatternTools::optimized_lower_bound (col_indices,
-								    col_indices+n_cols,
-								    row);
+	    Utilities::lower_bound (col_indices,
+					      col_indices+n_cols,
+					      row);
 	  const unsigned int diag = diag_pos - col_indices;
 	  unsigned int post_diag = diag;
 	  if (diag != n_cols && *diag_pos == row)
@@ -626,9 +676,9 @@ SparseMatrix<number>::vmult (OutVector& dst,
   Assert (!PointerComparison::equal(&src, &dst), ExcSourceEqualsDestination());
 
   parallel::apply_to_subranges (0U, m(),
-				std_cxx1x::bind (internal::SparseMatrix::vmult_on_subrange
+				std_cxx1x::bind (&internal::SparseMatrix::vmult_on_subrange
 						 <number,InVector,OutVector>,
-						 _1, _2,
+						 std_cxx1x::_1, std_cxx1x::_2,
 						 val,
 						 cols->rowstart,
 						 cols->colnums,
@@ -681,9 +731,9 @@ SparseMatrix<number>::vmult_add (OutVector& dst,
   Assert (!PointerComparison::equal(&src, &dst), ExcSourceEqualsDestination());
 
   parallel::apply_to_subranges (0U, m(),
-				std_cxx1x::bind (internal::SparseMatrix::vmult_on_subrange
+				std_cxx1x::bind (&internal::SparseMatrix::vmult_on_subrange
 						 <number,InVector,OutVector>,
-						 _1, _2,
+						 std_cxx1x::_1, std_cxx1x::_2,
 						 val,
 						 cols->rowstart,
 						 cols->colnums,
@@ -769,9 +819,9 @@ SparseMatrix<number>::matrix_norm_square (const Vector<somenumber>& v) const
 
   return
     parallel::accumulate_from_subranges<number>
-    (std_cxx1x::bind (internal::SparseMatrix::matrix_norm_sqr_on_subrange
+    (std_cxx1x::bind (&internal::SparseMatrix::matrix_norm_sqr_on_subrange
 		      <number,Vector<somenumber> >,
-		      _1, _2,
+		      std_cxx1x::_1, std_cxx1x::_2,
 		      val, cols->rowstart, cols->colnums,
 		      std_cxx1x::cref(v)),
      0, m(),
@@ -834,9 +884,9 @@ SparseMatrix<number>::matrix_scalar_product (const Vector<somenumber>& u,
 
   return
     parallel::accumulate_from_subranges<number>
-    (std_cxx1x::bind (internal::SparseMatrix::matrix_scalar_product_on_subrange
+    (std_cxx1x::bind (&internal::SparseMatrix::matrix_scalar_product_on_subrange
 		      <number,Vector<somenumber> >,
-		      _1, _2,
+		      std_cxx1x::_1, std_cxx1x::_2,
 		      val, cols->rowstart, cols->colnums,
 		      std_cxx1x::cref(u),
 		      std_cxx1x::cref(v)),
@@ -1242,9 +1292,9 @@ SparseMatrix<number>::residual (Vector<somenumber>       &dst,
 
   return
     std::sqrt (parallel::accumulate_from_subranges<number>
-	       (std_cxx1x::bind (internal::SparseMatrix::residual_sqr_on_subrange
+	       (std_cxx1x::bind (&internal::SparseMatrix::residual_sqr_on_subrange
 				 <number,Vector<somenumber>,Vector<somenumber> >,
-				 _1, _2,
+				 std_cxx1x::_1, std_cxx1x::_2,
 				 val, cols->rowstart, cols->colnums,
 				 std_cxx1x::cref(u),
 				 std_cxx1x::cref(b),
@@ -1385,9 +1435,9 @@ SparseMatrix<number>::precondition_SSOR (Vector<somenumber>              &dst,
 				       // line denotes the diagonal element,
 				       // which we need not check.
       const unsigned int first_right_of_diagonal_index
-	= (internals::SparsityPatternTools::optimized_lower_bound (&cols->colnums[*rowstart_ptr+1],
-								   &cols->colnums[*(rowstart_ptr+1)],
-								   row)
+	= (Utilities::lower_bound (&cols->colnums[*rowstart_ptr+1],
+					     &cols->colnums[*(rowstart_ptr+1)],
+					     row)
 	   -
 	   &cols->colnums[0]);
 
@@ -1413,9 +1463,9 @@ SparseMatrix<number>::precondition_SSOR (Vector<somenumber>              &dst,
     {
       const unsigned int end_row = *(rowstart_ptr+1);
       const unsigned int first_right_of_diagonal_index
-	= (internals::SparsityPatternTools::optimized_lower_bound (&cols->colnums[*rowstart_ptr+1],
-								   &cols->colnums[end_row],
-								   static_cast<unsigned int>(row)) -
+	= (Utilities::lower_bound (&cols->colnums[*rowstart_ptr+1],
+					     &cols->colnums[end_row],
+					     static_cast<unsigned int>(row)) -
 	   &cols->colnums[0]);
       number s = 0;
       for (unsigned int j=first_right_of_diagonal_index; j<end_row; ++j)
@@ -1485,7 +1535,7 @@ SparseMatrix<number>::SOR (Vector<somenumber>& dst,
 	  if (col < row)
 	    s -= val[j] * dst(col);
 	}
-      
+
       Assert(val[cols->rowstart[row]]!= 0., ExcDivideByZero());
       dst(row) = s * om / val[cols->rowstart[row]];
     }
@@ -1712,7 +1762,7 @@ SparseMatrix<number>::SSOR (Vector<somenumber>& dst,
 {
 //TODO: Is this called anywhere? If so, multiplication with om(2-om)D is missing
   Assert(false, ExcNotImplemented());
-  
+
   Assert (cols != 0, ExcNotInitialized());
   Assert (val != 0, ExcNotInitialized());
   Assert (cols->optimize_diagonal(),
