@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_precondition.h 21406 2010-06-29 14:59:33Z bangerth $
-//    Version: $Name$
+//    $Id: trilinos_precondition.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2008, 2010 by the deal.II authors
 //
@@ -14,14 +13,14 @@
 #define __deal2__trilinos_precondition_h
 
 
-#include <base/config.h>
+#include <deal.II/base/config.h>
 
 #ifdef DEAL_II_USE_TRILINOS
 
-#  include <base/subscriptor.h>
-#  include <base/std_cxx1x/shared_ptr.h>
+#  include <deal.II/base/subscriptor.h>
+#  include <deal.II/base/std_cxx1x/shared_ptr.h>
 
-#  include <lac/trilinos_vector_base.h>
+#  include <deal.II/lac/trilinos_vector_base.h>
 
 #  ifdef DEAL_II_COMPILER_SUPPORTS_MPI
 #    include <Epetra_MpiComm.h>
@@ -1447,7 +1446,7 @@ namespace TrilinosWrappers
 					* Prints an estimate of the memory
 					* consumption of this class.
 					*/
-      unsigned int memory_consumption () const;
+      std::size_t memory_consumption () const;
 
     private:
 				       /**
