@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_sparsity_pattern.h 22674 2010-11-10 15:07:55Z kronbichler $
-//    Version: $Name$
+//    $Id: trilinos_sparsity_pattern.h 23876 2011-06-28 18:21:51Z kanschat $
 //
-//    Copyright (C) 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,19 +13,19 @@
 #define __deal2__trilinos_sparsity_pattern_h
 
 
-#include <base/config.h>
+#include <deal.II/base/config.h>
 
 #ifdef DEAL_II_USE_TRILINOS
 
-#  include <base/subscriptor.h>
-#  include <base/index_set.h>
-#  include <lac/exceptions.h>
+#  include <deal.II/base/subscriptor.h>
+#  include <deal.II/base/index_set.h>
+#  include <deal.II/lac/exceptions.h>
 
 #  include <vector>
 #  include <cmath>
 #  include <memory>
 
-#  include <base/std_cxx1x/shared_ptr.h>
+#  include <deal.II/base/std_cxx1x/shared_ptr.h>
 
 #  include <Epetra_FECrsGraph.h>
 #  include <Epetra_Map.h>
@@ -1092,7 +1091,7 @@ namespace TrilinosWrappers
 					* of this object. Currently not
 					* implemented for this class.
 					*/
-      unsigned int memory_consumption () const;
+      std::size_t memory_consumption () const;
 
 //@}
 /**
@@ -1128,7 +1127,7 @@ namespace TrilinosWrappers
                                         * Epetra_CrsGraph data that stores
                                         * the sparsity pattern.
                                         */
-      const Epetra_CrsGraph & trilinos_sparsity_pattern () const;
+      const Epetra_FECrsGraph & trilinos_sparsity_pattern () const;
 
                                        /**
                                         * Return a const reference to the
@@ -1640,10 +1639,10 @@ namespace TrilinosWrappers
 
 
   inline
-  const Epetra_CrsGraph &
+  const Epetra_FECrsGraph &
   SparsityPattern::trilinos_sparsity_pattern () const
   {
-    return static_cast<const Epetra_CrsGraph&>(*graph);
+    return *graph;
   }
 
 
