@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_vector.h 22855 2010-11-24 15:01:47Z bangerth $
-//    Version: $Name$
+//    $Id: trilinos_vector.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 2008, 2009, 2010 by the deal.II authors
 //
@@ -14,17 +13,17 @@
 #define __deal2__trilinos_vector_h
 
 
-#include <base/config.h>
+#include <deal.II/base/config.h>
 
 #ifdef DEAL_II_USE_TRILINOS
 
-#  include <base/std_cxx1x/shared_ptr.h>
-#  include <base/subscriptor.h>
-#  include <base/index_set.h>
-#  include <base/utilities.h>
-#  include <lac/exceptions.h>
-#  include <lac/vector.h>
-#  include <lac/trilinos_vector_base.h>
+#  include <deal.II/base/std_cxx1x/shared_ptr.h>
+#  include <deal.II/base/subscriptor.h>
+#  include <deal.II/base/index_set.h>
+#  include <deal.II/base/utilities.h>
+#  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/vector.h>
+#  include <deal.II/lac/trilinos_vector_base.h>
 
 #  include "Epetra_Map.h"
 #  include "Epetra_LocalMap.h"
@@ -723,6 +722,23 @@ namespace TrilinosWrappers
 					*/
       Vector &
 	operator = (const Vector &V);
+
+				       /**
+					* This function does nothing but is
+					* there for compatibility with the
+					* @p PETScWrappers::Vector class.
+					*
+					* For the PETSc vector wrapper class,
+					* this function updates the ghost
+					* values of the PETSc vector. This
+					* is necessary after any modification
+					* before reading ghost values.
+					*
+					* However, for the implementation of
+					* this class, it is immaterial and thus
+					* an empty function.
+					*/
+      void update_ghost_values () const;
   };
 
 
@@ -796,6 +812,14 @@ namespace TrilinosWrappers
     return *this;
   }
 
+
+
+  inline
+  void
+  Vector::update_ghost_values () const
+  {}
+
+
 #endif
 
 
