@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_vector_base.h 22855 2010-11-24 15:01:47Z bangerth $
-//    Version: $Name$
+//    $Id: trilinos_vector_base.h 24420 2011-09-26 13:20:22Z bangerth $
 //
-//    Copyright (C) 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,15 +13,15 @@
 #define __deal2__trilinos_vector_base_h
 
 
-#include <base/config.h>
+#include <deal.II/base/config.h>
 
 #ifdef DEAL_II_USE_TRILINOS
 
-#include <base/utilities.h>
-#  include <base/std_cxx1x/shared_ptr.h>
-#  include <base/subscriptor.h>
-#  include <lac/exceptions.h>
-#  include <lac/vector.h>
+#include <deal.II/base/utilities.h>
+#  include <deal.II/base/std_cxx1x/shared_ptr.h>
+#  include <deal.II/base/subscriptor.h>
+#  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/vector.h>
 
 #  include <vector>
 #  include <utility>
@@ -259,7 +258,7 @@ namespace TrilinosWrappers
                                         */
       typedef TrilinosScalar            value_type;
       typedef TrilinosScalar            real_type;
-      typedef size_t                    size_type;
+      typedef std::size_t               size_type;
       typedef internal::VectorReference reference;
       typedef const internal::VectorReference const_reference;
 
@@ -477,6 +476,12 @@ namespace TrilinosWrappers
 					*/
       bool in_local_range (const unsigned int index) const;
 
+				       /**
+					* Return if the vector contains ghost
+					* elements.
+					*/
+      bool has_ghost_elements() const;
+
                                        /**
                                         * Return the scalar (inner)
                                         * product of two vectors. The
@@ -497,6 +502,12 @@ namespace TrilinosWrappers
                                         */
       TrilinosScalar mean_value () const;
 
+                                       /**
+                                        * Compute the minimal value of
+                                        * the elements of this vector.
+                                        */
+      TrilinosScalar minimal_value () const;
+
                                        /**
                                         * $l_1$-norm of the vector.  The
                                         * sum of the absolute values.
@@ -898,7 +909,7 @@ namespace TrilinosWrappers
 					* Estimate for the memory
 					* consumption in bytes.
 					*/
-      unsigned int memory_consumption () const;
+      std::size_t memory_consumption () const;
 				       //@}
 
 				       /**
@@ -1104,6 +1115,15 @@ namespace TrilinosWrappers
 
 
 
+  inline
+  bool
+  VectorBase::has_ghost_elements() const
+  {
+    return vector->Map().UniqueGIDs()==false;
+  }
+
+
+
   inline
   internal::VectorReference
   VectorBase::operator () (const unsigned int index)
@@ -1121,7 +1141,8 @@ namespace TrilinosWrappers
     Assert (vector.get() != 0,
 	    ExcMessage("Vector has not been constructed properly."));
 
-    if (fast == false || local_range() != v.local_range())
+    if (fast == false ||
+	vector_partitioner().SameAs(v.vector_partitioner())==false)
       vector.reset (new Epetra_FEVector(*v.vector));
   }
 
@@ -1133,24 +1154,26 @@ namespace TrilinosWrappers
   {
     Epetra_CombineMode mode = (last_action != Zero) ?
 			      last_action : given_last_action;
-    #ifdef DEBUG
-    #ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+#ifdef DEBUG
+#  ifdef DEAL_II_COMPILER_SUPPORTS_MPI
 				     // check that every process has decided
 				     // to use the same mode. This will
 				     // otherwise result in undefined
 				     // behaviour in the call to
 				     // GlobalAssemble().
     double double_mode = mode;
-    struct Utilities::System::MinMaxAvg result;
-    Utilities::System::calculate_collective_mpi_min_max_avg(
-      dynamic_cast<const Epetra_MpiComm*>(&vector_partitioner().Comm())->GetMpiComm(),
-      double_mode,
-      result);
-    Assert(result.max-result.min<1e-5,  ExcTrilinosError(0));
-    
-    #endif
-    #endif
-    
+    Utilities::MPI::MinMaxAvg result
+      = Utilities::MPI::min_max_avg (double_mode,
+				     dynamic_cast<const Epetra_MpiComm*>
+				     (&vector_partitioner().Comm())->GetMpiComm());
+    Assert(result.max-result.min<1e-5,
+	   ExcMessage ("Not all processors agree whether the last operation on "
+		       "this vector was an addition or a set operation. This will "
+		       "prevent the compress() operation from succeeding."));
+
+#  endif
+#endif
+
 				 // Now pass over the information about
 				 // what we did last to the vector.
     const int ierr = vector->GlobalAssemble(mode);
@@ -1212,9 +1235,8 @@ namespace TrilinosWrappers
   {
 				     // if we have ghost values, do not allow
 				     // writing to this vector at all.
-    Assert (vector->Map().UniqueGIDs()==true,
-	    ExcGhostsPresent());    
-    
+    Assert (!has_ghost_elements(), ExcGhostsPresent());
+
     if (last_action == Add)
       vector->GlobalAssemble(Add);
 
@@ -1245,6 +1267,9 @@ namespace TrilinosWrappers
   VectorBase::add (const std::vector<unsigned int>    &indices,
 		   const std::vector<TrilinosScalar>  &values)
   {
+				     // if we have ghost values, do not allow
+				     // writing to this vector at all.
+    Assert (!has_ghost_elements(), ExcGhostsPresent());
     Assert (indices.size() == values.size(),
 	    ExcDimensionMismatch(indices.size(),values.size()));
 
@@ -1258,6 +1283,9 @@ namespace TrilinosWrappers
   VectorBase::add (const std::vector<unsigned int>        &indices,
 		   const ::dealii::Vector<TrilinosScalar> &values)
   {
+				     // if we have ghost values, do not allow
+				     // writing to this vector at all.
+    Assert (!has_ghost_elements(), ExcGhostsPresent());
     Assert (indices.size() == values.size(),
 	    ExcDimensionMismatch(indices.size(),values.size()));
 
@@ -1274,8 +1302,7 @@ namespace TrilinosWrappers
   {
 				     // if we have ghost values, do not allow
 				     // writing to this vector at all.
-    Assert (vector->Map().UniqueGIDs()==true,
-	    ExcGhostsPresent());    
+    Assert (!has_ghost_elements(), ExcGhostsPresent());
 
     if (last_action != Add)
       {
@@ -1307,7 +1334,7 @@ namespace TrilinosWrappers
   unsigned int
   VectorBase::size () const
   {
-    return (unsigned int) (vector->Map().MaxAllGID() + 1 - 
+    return (unsigned int) (vector->Map().MaxAllGID() + 1 -
 			   vector->Map().MinAllGID());
   }
 
@@ -1340,8 +1367,7 @@ namespace TrilinosWrappers
   {
     Assert (vector->Map().SameAs(vec.vector->Map()),
 	    ExcDifferentParallelPartitioning());
-    Assert (vector->Map().UniqueGIDs()==true,
-	    ExcGhostsPresent());
+    Assert (!has_ghost_elements(), ExcGhostsPresent());
 
     TrilinosScalar result;
 
@@ -1367,7 +1393,7 @@ namespace TrilinosWrappers
   TrilinosScalar
   VectorBase::mean_value () const
   {
-    Assert (vector->Map().UniqueGIDs()==true, ExcGhostsPresent());
+    Assert (!has_ghost_elements(), ExcGhostsPresent());
 
     TrilinosScalar mean;
     const int ierr = vector->MeanValue (&mean);
@@ -1378,12 +1404,24 @@ namespace TrilinosWrappers
 
 
 
+  inline
+  TrilinosScalar
+  VectorBase::minimal_value () const
+  {
+    TrilinosScalar min_value;
+    const int ierr = vector->MinValue (&min_value);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    return min_value;
+  }
+
+
+
   inline
   VectorBase::real_type
   VectorBase::l1_norm () const
   {
-    Assert (vector->Map().UniqueGIDs()==true,
-	    ExcGhostsPresent());
+    Assert (!has_ghost_elements(), ExcGhostsPresent());
 
     TrilinosScalar d;
     const int ierr = vector->Norm1 (&d);
@@ -1398,8 +1436,7 @@ namespace TrilinosWrappers
   VectorBase::real_type
   VectorBase::l2_norm () const
   {
-    Assert (vector->Map().UniqueGIDs()==true,
-	    ExcGhostsPresent());
+    Assert (!has_ghost_elements(), ExcGhostsPresent());
 
     TrilinosScalar d;
     const int ierr = vector->Norm2 (&d);
@@ -1414,8 +1451,7 @@ namespace TrilinosWrappers
   VectorBase::real_type
   VectorBase::lp_norm (const TrilinosScalar p) const
   {
-    Assert (vector->Map().UniqueGIDs()==true,
-	    ExcGhostsPresent());
+    Assert (!has_ghost_elements(), ExcGhostsPresent());
 
     TrilinosScalar norm = 0;
     TrilinosScalar sum=0;
@@ -1460,7 +1496,6 @@ namespace TrilinosWrappers
   VectorBase &
   VectorBase::operator *= (const TrilinosScalar a)
   {
-
     Assert (numbers::is_finite(a), ExcNumberNotFinite());
 
     const int ierr = vector->Scale(a);
@@ -1475,7 +1510,6 @@ namespace TrilinosWrappers
   VectorBase &
   VectorBase::operator /= (const TrilinosScalar a)
   {
-
     Assert (numbers::is_finite(a), ExcNumberNotFinite());
 
     const TrilinosScalar factor = 1./a;
@@ -1528,7 +1562,6 @@ namespace TrilinosWrappers
   void
   VectorBase::add (const TrilinosScalar s)
   {
-
     Assert (numbers::is_finite(s), ExcNumberNotFinite());
 
     unsigned int n_local = local_size();
@@ -1543,10 +1576,8 @@ namespace TrilinosWrappers
   VectorBase::add (const TrilinosScalar  a,
 		   const VectorBase     &v)
   {
-    Assert (size() == v.size(),
-	    ExcDimensionMismatch(size(), v.size()));
-    Assert (vector->Map().SameAs(v.vector->Map()),
-	    ExcDifferentParallelPartitioning());
+    Assert (local_size() == v.local_size(),
+	    ExcDimensionMismatch(local_size(), v.local_size()));
 
     Assert (numbers::is_finite(a), ExcNumberNotFinite());
 
@@ -1563,14 +1594,10 @@ namespace TrilinosWrappers
 		   const TrilinosScalar  b,
 		   const VectorBase     &w)
   {
-    Assert (size() == v.size(),
-	    ExcDimensionMismatch(size(), v.size()));
-    Assert (size() == w.size(),
-	    ExcDimensionMismatch(size(), w.size()));
-    Assert (vector->Map().SameAs(v.vector->Map()),
-	    ExcDifferentParallelPartitioning());
-    Assert (vector->Map().SameAs(w.vector->Map()),
-	    ExcDifferentParallelPartitioning());
+    Assert (local_size() == v.local_size(),
+	    ExcDimensionMismatch(local_size(), v.local_size()));
+    Assert (local_size() == w.local_size(),
+	    ExcDimensionMismatch(local_size(), w.local_size()));
 
     Assert (numbers::is_finite(a), ExcNumberNotFinite());
     Assert (numbers::is_finite(b), ExcNumberNotFinite());
@@ -1587,10 +1614,8 @@ namespace TrilinosWrappers
   VectorBase::sadd (const TrilinosScalar  s,
 		    const VectorBase     &v)
   {
-    Assert (size() == v.size(),
-	    ExcDimensionMismatch(size(), v.size()));
-    Assert (vector->Map().SameAs(v.vector->Map()),
-	    ExcDifferentParallelPartitioning());
+    Assert (local_size() == v.local_size(),
+	    ExcDimensionMismatch(local_size(), v.local_size()));
 
     Assert (numbers::is_finite(s), ExcNumberNotFinite());
 
@@ -1607,10 +1632,8 @@ namespace TrilinosWrappers
 		    const TrilinosScalar  a,
 		    const VectorBase     &v)
   {
-    Assert (size() == v.size(),
-	    ExcDimensionMismatch(size(), v.size()));
-    Assert (vector->Map().SameAs(v.vector->Map()),
-	    ExcDifferentParallelPartitioning());
+    Assert (local_size() == v.local_size(),
+	    ExcDimensionMismatch(local_size(), v.local_size()));
 
     Assert (numbers::is_finite(s), ExcNumberNotFinite());
     Assert (numbers::is_finite(a), ExcNumberNotFinite());
@@ -1630,14 +1653,10 @@ namespace TrilinosWrappers
 		    const TrilinosScalar  b,
 		    const VectorBase     &w)
   {
-    Assert (size() == v.size(),
-	    ExcDimensionMismatch(size(), v.size()));
-    Assert (size() == w.size(),
-	    ExcDimensionMismatch(size(), w.size()));
-    Assert (vector->Map().SameAs(v.vector->Map()),
-	    ExcDifferentParallelPartitioning());
-    Assert (vector->Map().SameAs(w.vector->Map()),
-	    ExcDifferentParallelPartitioning());
+    Assert (local_size() == v.local_size(),
+	    ExcDimensionMismatch(local_size(), v.local_size()));
+    Assert (local_size() == w.local_size(),
+	    ExcDimensionMismatch(local_size(), w.local_size()));
 
     Assert (numbers::is_finite(s), ExcNumberNotFinite());
     Assert (numbers::is_finite(a), ExcNumberNotFinite());
@@ -1660,18 +1679,12 @@ namespace TrilinosWrappers
 		    const TrilinosScalar  c,
 		    const VectorBase     &x)
   {
-    Assert (size() == v.size(),
-	    ExcDimensionMismatch(size(), v.size()));
-    Assert (size() == w.size(),
-	    ExcDimensionMismatch(size(), w.size()));
-    Assert (size() == x.size(),
-	    ExcDimensionMismatch(size(), x.size()));
-    Assert (vector->Map().SameAs(v.vector->Map()),
-	    ExcDifferentParallelPartitioning());
-    Assert (vector->Map().SameAs(w.vector->Map()),
-	    ExcDifferentParallelPartitioning());
-    Assert (vector->Map().SameAs(x.vector->Map()),
-	    ExcDifferentParallelPartitioning());
+    Assert (local_size() == v.local_size(),
+	    ExcDimensionMismatch(local_size(), v.local_size()));
+    Assert (local_size() == w.local_size(),
+	    ExcDimensionMismatch(local_size(), w.local_size()));
+    Assert (local_size() == x.local_size(),
+	    ExcDimensionMismatch(local_size(), x.local_size()));
 
     Assert (numbers::is_finite(s), ExcNumberNotFinite());
     Assert (numbers::is_finite(a), ExcNumberNotFinite());
@@ -1694,10 +1707,8 @@ namespace TrilinosWrappers
   void
   VectorBase::scale (const VectorBase &factors)
   {
-    Assert (size() == factors.size(),
-	    ExcDimensionMismatch(size(), factors.size()));
-    Assert (vector->Map().SameAs(factors.vector->Map()),
-	    ExcDifferentParallelPartitioning());
+    Assert (local_size() == factors.local_size(),
+	    ExcDimensionMismatch(local_size(), factors.local_size()));
 
     const int ierr = vector->Multiply (1.0, *(factors.vector), *vector, 0.0);
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
@@ -1721,11 +1732,7 @@ namespace TrilinosWrappers
     else
       {
 				   // Otherwise, just update
-	int ierr;
-	ierr = vector->GlobalAssemble(last_action);
-	AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-
-	ierr = vector->Update(a, *v.vector, 0.0);
+	int ierr = vector->Update(a, *v.vector, 0.0);
 	AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
 	last_action = Zero;
@@ -1742,8 +1749,8 @@ namespace TrilinosWrappers
 		   const TrilinosScalar  b,
 		   const VectorBase     &w)
   {
-    Assert (v.size() == w.size(),
-	    ExcDimensionMismatch (v.size(), w.size()));
+    Assert (v.local_size() == w.local_size(),
+	    ExcDimensionMismatch (v.local_size(), w.local_size()));
 
     Assert (numbers::is_finite(a), ExcNumberNotFinite());
     Assert (numbers::is_finite(b), ExcNumberNotFinite());
@@ -1756,14 +1763,14 @@ namespace TrilinosWrappers
       }
     else
       {
-				   // Otherwise, just update
-	int ierr;
+				   // Otherwise, just update. verify
+				   // that *this does not only have
+				   // the same map as v (the
+				   // if-condition above) but also as
+				   // w
 	Assert (vector->Map().SameAs(w.vector->Map()),
 		ExcDifferentParallelPartitioning());
-	ierr = vector->GlobalAssemble(last_action);
-	AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-
-	ierr = vector->Update(a, *v.vector, b, *w.vector, 0.0);
+	int ierr = vector->Update(a, *v.vector, b, *w.vector, 0.0);
 	AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
 	last_action = Zero;
@@ -1777,14 +1784,10 @@ namespace TrilinosWrappers
   VectorBase::ratio (const VectorBase &v,
 		     const VectorBase &w)
   {
-    Assert (v.size() == w.size(),
-	    ExcDimensionMismatch (v.size(), w.size()));
-    Assert (size() == w.size(),
-	    ExcDimensionMismatch (size(), w.size()));
-    Assert (vector->Map().SameAs(v.vector->Map()),
-	    ExcDifferentParallelPartitioning());
-    Assert (vector->Map().SameAs(w.vector->Map()),
-	    ExcDifferentParallelPartitioning());
+    Assert (v.local_size() == w.local_size(),
+	    ExcDimensionMismatch (v.local_size(), w.local_size()));
+    Assert (local_size() == w.local_size(),
+	    ExcDimensionMismatch (local_size(), w.local_size()));
 
     const int ierr = vector->ReciprocalMultiply(1.0, *(w.vector),
 						*(v.vector), 0.0);
