@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: vector.h 22928 2010-12-06 18:44:56Z rao $
-//    Version: $Name$
+//    $Id: vector.h 23876 2011-06-28 18:21:51Z kanschat $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,11 +13,11 @@
 #define __deal2__vector_h
 
 
-#include <base/config.h>
-#include <base/logstream.h>
-#include <base/exceptions.h>
-#include <base/parallel.h>
-#include <base/subscriptor.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/logstream.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/parallel.h>
+#include <deal.II/base/subscriptor.h>
 #include <boost/lambda/lambda.hpp>
 #include <boost/serialization/array.hpp>
 #include <boost/serialization/split_member.hpp>
@@ -103,7 +102,7 @@ class Vector : public Subscriptor
     typedef const value_type                                 *const_iterator;
     typedef value_type                                       &reference;
     typedef const value_type                                 &const_reference;
-    typedef size_t                                            size_type;
+    typedef std::size_t                                       size_type;
 
 				     /**
 				      * Declare a type that has holds
@@ -851,6 +850,23 @@ class Vector : public Subscriptor
 				      */
     void ratio (const Vector<Number> &a,
 		const Vector<Number> &b);
+
+                                     /**
+                                      * This function does nothing but is
+                                      * there for compatibility with the
+                                      * @p PETScWrappers::Vector class.
+                                      *
+                                      * For the PETSc vector wrapper class,
+                                      * this function updates the ghost
+                                      * values of the PETSc vector. This
+                                      * is necessary after any modification
+                                      * before reading ghost values.
+                                      *
+                                      * However, for the implementation of
+                                      * this class, it is immaterial and thus
+                                      * an empty function.
+                                      */
+    void update_ghost_values () const;
 				     //@}
 
 
@@ -939,20 +955,20 @@ class Vector : public Subscriptor
 				      * memory consumption (in bytes)
 				      * of this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 				     //@}
-	
+
                      /**
                       * Write the data of this object to
                       * a stream for the purpose of serialization.
-                      */ 
+                      */
     template <class Archive>
     void save (Archive & ar, const unsigned int version) const;
 
                      /**
-                      * Read the data of this object 
+                      * Read the data of this object
                       * from a stream for the purpose of serialization.
-                      */    
+                      */
     template <class Archive>
     void load (Archive & ar, const unsigned int version);
 
@@ -1146,7 +1162,7 @@ Vector<Number> & Vector<Number>::operator = (const Number s)
     parallel::apply_to_subranges (0U, vec_size,
 				  std_cxx1x::bind(&internal::Vector::template
 						  set_subrange<Number>,
-						  s, _1, _2, std_cxx1x::ref(*this)),
+						  s, std_cxx1x::_1, std_cxx1x::_2, std_cxx1x::ref(*this)),
 				  internal::Vector::minimum_parallel_grain_size);
   else if (vec_size > 0)
     internal::Vector::set_subrange<Number>(s, 0U, vec_size, *this);
@@ -1185,7 +1201,7 @@ Vector<Number>::operator = (const Vector<Number>& v)
     parallel::apply_to_subranges (0U, vec_size,
 				  std_cxx1x::bind(&internal::Vector::template
 						  copy_subrange<Number>,
-						  std_cxx1x::cref(v), _1, _2,
+						  std_cxx1x::cref(v), std_cxx1x::_1, std_cxx1x::_2,
 						  std_cxx1x::ref(*this)),
 				  internal::Vector::minimum_parallel_grain_size);
   else if (vec_size > 0)
@@ -1208,7 +1224,7 @@ Vector<Number>::operator = (const Vector<Number2>& v)
     parallel::apply_to_subranges (0U, vec_size,
 				  std_cxx1x::bind(&internal::Vector::template
 						  copy_subrange_ext<Number2,Number>,
-						  std_cxx1x::cref(v), _1, _2,
+						  std_cxx1x::cref(v), std_cxx1x::_1, std_cxx1x::_2,
 						  std_cxx1x::ref(*this)),
 				  internal::Vector::minimum_parallel_grain_size);
   else if (vec_size > 0)
@@ -1444,6 +1460,15 @@ Vector<Number>::compress () const
 {}
 
 
+
+template <typename Number>
+inline
+void
+Vector<Number>::update_ghost_values () const
+{}
+
+
+
 // Moved from vector.templates.h as an inline function by Luca Heltai
 // on 2009/04/12 to prevent strange compiling errors, after making
 // swap virtual.
@@ -1468,7 +1493,7 @@ Vector<Number>::save (Archive & ar, const unsigned int) const
                                      // forward to serialization
                                      // function in the base class.
   ar &  static_cast<const Subscriptor &>(*this);
-  
+
   ar & vec_size & max_vec_size ;
   ar & boost::serialization::make_array(val, max_vec_size);
 }
@@ -1484,9 +1509,9 @@ Vector<Number>::load (Archive & ar, const unsigned int)
                                      // forward to serialization
                                      // function in the base class.
   ar &  static_cast<Subscriptor &>(*this);
-  
+
   ar & vec_size & max_vec_size ;
-  
+
   val = new Number[max_vec_size];
   ar & boost::serialization::make_array(val, max_vec_size);
 }
@@ -1524,7 +1549,7 @@ inline
 std::ostream&
 operator << (std::ostream& os, const Vector<number>& v)
 {
-  v.print(os);  
+  v.print(os);
   return os;
 }
 
@@ -1537,7 +1562,7 @@ LogStream&
 operator << (LogStream& os, const Vector<number>& v)
 {
   v.print(os);
-  return os;  
+  return os;
 }
 
 
