@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: vector.templates.h 22751 2010-11-16 15:16:53Z kanschat $
-//    Version: $Name$
+//    $Id: vector.templates.h 23876 2011-06-28 18:21:51Z kanschat $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,18 +13,18 @@
 #define __deal2__vector_templates_h
 
 
-#include <base/template_constraints.h>
-#include <base/numbers.h>
-#include <lac/vector.h>
-#include <lac/block_vector.h>
+#include <deal.II/base/template_constraints.h>
+#include <deal.II/base/numbers.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
 
 #ifdef DEAL_II_USE_PETSC
-#  include <lac/petsc_vector.h>
-#  include <lac/petsc_parallel_vector.h>
+#  include <deal.II/lac/petsc_vector.h>
+#  include <deal.II/lac/petsc_parallel_vector.h>
 #endif
 
 #ifdef DEAL_II_USE_TRILINOS
-#  include <lac/trilinos_vector.h>
+#  include <deal.II/lac/trilinos_vector.h>
 #endif
 
 #include <cmath>
@@ -1225,7 +1224,7 @@ void Vector<Number>::block_read (std::istream &in)
 
 
 template <typename Number>
-unsigned int
+std::size_t
 Vector<Number>::memory_consumption () const
 {
   return sizeof(*this) + (max_vec_size * sizeof(Number));
