@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: mg_dof_accessor.h 23000 2010-12-18 05:00:38Z bangerth $
-//    Version: $Name$
+//    $Id: mg_dof_accessor.h 24157 2011-08-21 04:17:31Z bangerth $
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,9 +13,9 @@
 #define __deal2__mg_dof_accessor_h
 
 
-#include <base/config.h>
-#include <dofs/dof_accessor.h>
-#include <multigrid/mg_dof_iterator_selector.h>
+#include <deal.II/base/config.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/multigrid/mg_dof_iterator_selector.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -28,7 +27,7 @@ namespace internal
 {
   namespace MGDoFHandler
   {
-    class Implementation;
+    struct Implementation;
   }
 }
 
@@ -417,7 +416,7 @@ class MGDoFAccessor : public internal::MGDoFAccessor::Inheritance<structdim,dim,
     operator = (const MGDoFAccessor &da);
 
     template <int, int> friend class MGDoFHandler;
-    friend class internal::MGDoFHandler::Implementation;
+    friend struct internal::MGDoFHandler::Implementation;
 };
 
 
