@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: mg_dof_handler.h 22870 2010-11-28 23:20:00Z kanschat $
-//    Version: $Name$
+//    $Id: mg_dof_handler.h 24157 2011-08-21 04:17:31Z bangerth $
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,9 +13,9 @@
 #define __deal2__mg_dof_handler_h
 
 
-#include <base/config.h>
-#include <dofs/dof_handler.h>
-#include <multigrid/mg_dof_iterator_selector.h>
+#include <deal.II/base/config.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/multigrid/mg_dof_iterator_selector.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -25,7 +24,7 @@ namespace internal
 {
   namespace MGDoFHandler
   {
-    class Implementation;
+    struct Implementation;
   }
 }
 
@@ -105,7 +104,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * Destructor
 				      */
     virtual ~MGDoFHandler ();
-    
+
 				     /**
 				      * Go through the triangulation
 				      * and distribute the degrees of
@@ -751,7 +750,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * although the actual object
 				      * might be a derived class.
 				      */
-    virtual unsigned int memory_consumption () const;
+    virtual std::size_t memory_consumption () const;
 
 
 				     /**
@@ -1014,7 +1013,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * Make accessor objects friends.
 				      */
     template <int dim1, int dim2, int dim3> friend class MGDoFAccessor;
-    friend class internal::MGDoFHandler::Implementation;
+    friend struct internal::MGDoFHandler::Implementation;
 };
 
 /*@}*/
