@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: mg_smoother.h 22344 2010-10-14 15:59:09Z kanschat $
-//    Version: $Name$
+//    $Id: mg_smoother.h 24455 2011-09-28 05:10:06Z kanschat $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,12 +13,12 @@
 #define __deal2__mg_smoother_h
 
 
-#include <base/config.h>
-#include <base/smartpointer.h>
-#include <lac/pointer_matrix.h>
-#include <lac/vector_memory.h>
-#include <multigrid/mg_base.h>
-#include <base/mg_level_object.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/lac/pointer_matrix.h>
+#include <deal.II/lac/vector_memory.h>
+#include <deal.II/multigrid/mg_base.h>
+#include <deal.II/base/mg_level_object.h>
 #include <vector>
 
 DEAL_II_NAMESPACE_OPEN
@@ -230,7 +229,7 @@ namespace mg
  * @date 2003, 2009, 2010
  */
 template<class RELAX, class VECTOR>
-class SmootherRelaxation : public MGSmoother<VECTOR>
+class SmootherRelaxation : public MGLevelObject<RELAX>, public MGSmoother<VECTOR>
 {
   public:
 				     /**
@@ -258,7 +257,20 @@ class SmootherRelaxation : public MGSmoother<VECTOR>
     template <class MATRIX2>
     void initialize (const MGLevelObject<MATRIX2>& matrices,
 		     const typename RELAX::AdditionalData & additional_data = typename RELAX::AdditionalData());
-
+    
+				     /**
+				      * Initialize matrices and
+				      * additional data for each
+				      * level.
+				      *
+				      * If minimal or maximal level of
+				      * the two objects differ, the
+				      * greatest common range is
+				      * utilized. This way, smoothing
+				      * can be restricted to certain
+				      * levels even if the matrix was
+				      * generated for all levels.
+				      */
     template <class MATRIX2, class DATA>
     void initialize (const MGLevelObject<MATRIX2>& matrices,
 		     const MGLevelObject<DATA>& additional_data);
@@ -295,13 +307,7 @@ class SmootherRelaxation : public MGSmoother<VECTOR>
     				     /**
 				      * Memory used by this object.
 				      */
-    unsigned int memory_consumption () const;
-  private:
- 				     /**
-				      * Object containing relaxation
-				      * methods.
-				      */
-    MGLevelObject<RELAX> smoothers;
+    std::size_t memory_consumption () const;
 };
 }
 
@@ -471,7 +477,7 @@ class MGSmootherRelaxation : public MGSmoother<VECTOR>
     				     /**
 				      * Memory used by this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
 
  private:
@@ -640,7 +646,7 @@ class MGSmootherPrecondition : public MGSmoother<VECTOR>
     				     /**
 				      * Memory used by this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
 
  private:
@@ -764,7 +770,7 @@ namespace mg
   inline void
   SmootherRelaxation<RELAX, VECTOR>::clear ()
   {
-    smoothers.clear();
+    MGLevelObject<RELAX>::clear();
   }
   
   
@@ -778,10 +784,10 @@ namespace mg
     const unsigned int min = m.get_minlevel();
     const unsigned int max = m.get_maxlevel();
     
-    smoothers.resize(min, max);
+    this->resize(min, max);
     
     for (unsigned int i=min;i<=max;++i)
-      smoothers[i].initialize(m[i], data);
+      (*this)[i].initialize(m[i], data);
   }
 
   
@@ -792,18 +798,13 @@ namespace mg
     const MGLevelObject<MATRIX2>& m,
     const MGLevelObject<DATA>& data)
   {
-    const unsigned int min = m.get_minlevel();
-    const unsigned int max = m.get_maxlevel();
+    const unsigned int min = std::max(m.get_minlevel(), data.get_minlevel());
+    const unsigned int max = std::min(m.get_maxlevel(), data.get_maxlevel());
     
-    Assert (data.get_minlevel() == min,
-	    ExcDimensionMismatch(data.get_minlevel(), min));
-    Assert (data.get_maxlevel() == max,
-	    ExcDimensionMismatch(data.get_maxlevel(), max));
-    
-    smoothers.resize(min, max);
+    this->resize(min, max);
     
     for (unsigned int i=min;i<=max;++i)
-      smoothers[i].initialize(m[i], data[i]);
+      (*this)[i].initialize(m[i], data[i]);
   }
 
   
@@ -814,7 +815,7 @@ namespace mg
     VECTOR& u,
     const VECTOR& rhs) const
   {
-    unsigned int maxlevel = smoothers.get_maxlevel();
+    unsigned int maxlevel = this->get_maxlevel();
     unsigned int steps2 = this->steps;
 
     if (this->variable)
@@ -829,9 +830,9 @@ namespace mg
     for (unsigned int i=0; i<steps2; ++i)
       {
 	if (T)
-	  smoothers[level].Tstep(u, rhs);
+	  (*this)[level].Tstep(u, rhs);
 	else
-	  smoothers[level].step(u, rhs);
+	  (*this)[level].step(u, rhs);
 	if (this->symmetric)
 	  T = !T;
       }
@@ -839,12 +840,14 @@ namespace mg
 
 
   template <class RELAX, class VECTOR>
-  inline unsigned int
+  inline
+  std::size_t
   SmootherRelaxation<RELAX, VECTOR>::
   memory_consumption () const
   {
     return sizeof(*this)
-      + smoothers.memory_consumption()
+      -sizeof(MGLevelObject<RELAX>)
+      + MGLevelObject<RELAX>::memory_consumption()
       + this->mem->memory_consumption();
   }
 }
@@ -1066,7 +1069,8 @@ MGSmootherRelaxation<MATRIX, RELAX, VECTOR>::smooth(
 
 
 template <class MATRIX, class RELAX, class VECTOR>
-inline unsigned int
+inline
+std::size_t
 MGSmootherRelaxation<MATRIX, RELAX, VECTOR>::
 memory_consumption () const
 {
@@ -1269,7 +1273,8 @@ MGSmootherPrecondition<MATRIX, PRECONDITIONER, VECTOR>::smooth(
 
 
 template <class MATRIX, class PRECONDITIONER, class VECTOR>
-inline unsigned int
+inline
+std::size_t
 MGSmootherPrecondition<MATRIX, PRECONDITIONER, VECTOR>::
 memory_consumption () const
 {
