@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: multigrid.h 22084 2010-09-20 16:04:25Z janssen $
-//    Version: $Name$
+//    $Id: multigrid.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
@@ -14,15 +13,15 @@
 #define __deal2__multigrid_h
 
 
-#include <base/config.h>
-#include <base/subscriptor.h>
-#include <base/smartpointer.h>
-#include <numerics/data_out.h>
-#include <lac/sparse_matrix.h>
-#include <lac/vector.h>
-#include <multigrid/mg_base.h>
-#include <base/mg_level_object.h>
-#include <multigrid/mg_dof_handler.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/numerics/data_out.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/multigrid/mg_base.h>
+#include <deal.II/base/mg_level_object.h>
+#include <deal.II/multigrid/mg_dof_handler.h>
 
 #include <vector>
 
