@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: data_out.h 22544 2010-10-28 23:53:11Z bangerth $
-//    Version: $Name$
+//    $Id: data_out.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
@@ -15,19 +14,19 @@
 
 
 
-#include <base/config.h>
-#include <base/smartpointer.h>
-#include <base/data_out_base.h>
-#include <dofs/dof_handler.h>
-#include <fe/mapping.h>
-#include <hp/q_collection.h>
-#include <hp/fe_collection.h>
-#include <hp/mapping_collection.h>
-#include <hp/fe_values.h>
-#include <numerics/data_postprocessor.h>
-#include <numerics/data_component_interpretation.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/smartpointer.h>
+#include <deal.II/base/data_out_base.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/fe/mapping.h>
+#include <deal.II/hp/q_collection.h>
+#include <deal.II/hp/fe_collection.h>
+#include <deal.II/hp/mapping_collection.h>
+#include <deal.II/hp/fe_values.h>
+#include <deal.II/numerics/data_postprocessor.h>
+#include <deal.II/numerics/data_component_interpretation.h>
 
-#include <base/std_cxx1x/shared_ptr.h>
+#include <deal.II/base/std_cxx1x/shared_ptr.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -654,7 +653,7 @@ class DataOut_DoFData : public DataOutInterface<patch_dim,patch_space_dim>
 				      * memory consumption (in bytes)
 				      * of this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
 				     /**
 				      * Exception
@@ -864,7 +863,7 @@ class DataOut_DoFData : public DataOutInterface<patch_dim,patch_space_dim>
 					  * the memory consumption (in
 					  * bytes) of this object.
 					  */
-	virtual unsigned int memory_consumption () const = 0;
+	virtual std::size_t memory_consumption () const = 0;
 
 					 /**
 					  * Names of the components of this
@@ -1031,7 +1030,7 @@ class DataOut_DoFData : public DataOutInterface<patch_dim,patch_space_dim>
 					  * the memory consumption (in
 					  * bytes) of this object.
 					  */
-	virtual unsigned int memory_consumption () const;
+	virtual std::size_t memory_consumption () const;
 
       private:
                                          /**
