@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_output_operator.h 21513 2010-07-18 04:00:43Z kanschat $
+//    $Id: dof_output_operator.h 23709 2011-05-17 04:34:08Z bangerth $
 //
 //    Copyright (C) 2010 by the deal.II authors
 //
@@ -13,11 +13,11 @@
 #ifndef __deal2__dof_output_operator_h
 #define __deal2__dof_output_operator_h
 
-#include <base/config.h>
-#include <base/named_data.h>
-#include <base/event.h>
-#include <algorithms/operator.h>
-#include <dofs/dof_handler.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/named_data.h>
+#include <deal.II/base/event.h>
+#include <deal.II/algorithms/operator.h>
+#include <deal.II/dofs/dof_handler.h>
 
 #include <fstream>
 
