@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_output_operator.templates.h 21513 2010-07-18 04:00:43Z kanschat $
+//    $Id: dof_output_operator.templates.h 23709 2011-05-17 04:34:08Z bangerth $
 //
 //    Copyright (C) 2006, 2007, 2008, 2009, 2010 by Guido Kanschat
 //
@@ -10,8 +10,8 @@
 //
 //---------------------------------------------------------------------------
 
-#include <numerics/dof_output_operator.h>
-#include <numerics/data_out.h>
+#include <deal.II/numerics/dof_output_operator.h>
+#include <deal.II/numerics/data_out.h>
 
 DEAL_II_NAMESPACE_OPEN
 
