@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: error_estimator.h 22430 2010-10-23 13:41:09Z heister $
-//    Version: $Name$
+//    $Id: error_estimator.h 24061 2011-08-13 20:34:23Z kanschat $
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,11 +13,11 @@
 #define __deal2__error_estimator_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/function.h>
-#include <base/multithread_info.h>
-#include <dofs/function_map.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/function.h>
+#include <deal.II/base/multithread_info.h>
+#include <deal.II/dofs/function_map.h>
 #include <map>
 
 DEAL_II_NAMESPACE_OPEN
@@ -37,18 +36,19 @@ namespace hp
 
 
 /**
- *  Implementation of the error estimator by Kelly, Gago, Zienkiewicz
- *  and Babuska.  This error estimator tries to approximate the error
+ *  Implementation of the error indicator by Kelly, Gago, Zienkiewicz
+ *  and Babuska.  This error indicator tries to approximate the error
  *  per cell by integration of the jump of the gradient of the
  *  solution along the faces of each cell.  It can be understood as a
  *  gradient recovery estimator; see the survey of Ainsworth for a
  *  complete discussion.
  *
- *  It seem as if this error estimator should only be valid for linear trial
- *  spaces, and there are indications that for higher order trial spaces the
- *  integrals computed here show superconvergence properties, i.e. they tend
- *  to zero faster than the error itself, thus ruling out the values as error
- *  indicators.
+ *  @note In spite of the name, this is not truly an a posteriori
+ *  error estimator, even if applied to the Poisson problem only. It
+ *  gives good hints for mesh refinement, but the estimate is not to
+ *  be trusted. For higher order trial spaces the integrals computed
+ *  here tend to zero faster than the error itself, thus ruling out
+ *  the values as error estimators.
  *
  *  The error estimator really only estimates the error for the generalized
  *  Poisson equation $-\nabla\cdot a(x) \nabla u = f$ with either Dirichlet
@@ -329,11 +329,11 @@ class KellyErrorEstimator
     static void estimate (const Mapping<dim, spacedim>      &mapping,
 			  const DH                &dof,
 			  const Quadrature<dim-1> &quadrature,
-			  const typename FunctionMap<dim>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const InputVector       &solution,
 			  Vector<float>           &error,
 			  const std::vector<bool> &component_mask = std::vector<bool>(),
-			  const Function<dim>     *coefficients   = 0,
+			  const Function<spacedim>     *coefficients   = 0,
 			  const unsigned int       n_threads = numbers::invalid_unsigned_int,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int       material_id = numbers::invalid_unsigned_int);
@@ -346,11 +346,11 @@ class KellyErrorEstimator
     template <typename InputVector, class DH>
     static void estimate (const DH                &dof,
 			  const Quadrature<dim-1> &quadrature,
-			  const typename FunctionMap<dim>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const InputVector       &solution,
 			  Vector<float>           &error,
 			  const std::vector<bool> &component_mask = std::vector<bool>(),
-			  const Function<dim>     *coefficients   = 0,
+			  const Function<spacedim>     *coefficients   = 0,
 			  const unsigned int       n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int       material_id = numbers::invalid_unsigned_int);
@@ -386,11 +386,11 @@ class KellyErrorEstimator
     static void estimate (const Mapping<dim, spacedim>          &mapping,
 			  const DH                    &dof,
 			  const Quadrature<dim-1>     &quadrature,
-			  const typename FunctionMap<dim>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const std::vector<const InputVector *> &solutions,
 			  std::vector<Vector<float>*> &errors,
 			  const std::vector<bool>     &component_mask = std::vector<bool>(),
-			  const Function<dim>         *coefficients   = 0,
+			  const Function<spacedim>         *coefficients   = 0,
 			  const unsigned int           n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int           material_id = numbers::invalid_unsigned_int);
@@ -403,11 +403,11 @@ class KellyErrorEstimator
     template <typename InputVector, class DH>
     static void estimate (const DH                    &dof,
 			  const Quadrature<dim-1>     &quadrature,
-			  const typename FunctionMap<dim>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const std::vector<const InputVector *> &solutions,
 			  std::vector<Vector<float>*> &errors,
 			  const std::vector<bool>     &component_mask = std::vector<bool>(),
-			  const Function<dim>         *coefficients   = 0,
+			  const Function<spacedim>         *coefficients   = 0,
 			  const unsigned int           n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int           material_id = numbers::invalid_unsigned_int);
@@ -423,11 +423,11 @@ class KellyErrorEstimator
     static void estimate (const Mapping<dim, spacedim>      &mapping,
 			  const DH                &dof,
 			  const hp::QCollection<dim-1> &quadrature,
-			  const typename FunctionMap<dim>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const InputVector       &solution,
 			  Vector<float>           &error,
 			  const std::vector<bool> &component_mask = std::vector<bool>(),
-			  const Function<dim>     *coefficients   = 0,
+			  const Function<spacedim>     *coefficients   = 0,
 			  const unsigned int       n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int       material_id = numbers::invalid_unsigned_int);
@@ -442,11 +442,11 @@ class KellyErrorEstimator
     template <typename InputVector, class DH>
     static void estimate (const DH                &dof,
 			  const hp::QCollection<dim-1> &quadrature,
-			  const typename FunctionMap<dim>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const InputVector       &solution,
 			  Vector<float>           &error,
 			  const std::vector<bool> &component_mask = std::vector<bool>(),
-			  const Function<dim>     *coefficients   = 0,
+			  const Function<spacedim>     *coefficients   = 0,
 			  const unsigned int       n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int       material_id = numbers::invalid_unsigned_int);
@@ -462,11 +462,11 @@ class KellyErrorEstimator
     static void estimate (const Mapping<dim, spacedim>          &mapping,
 			  const DH                    &dof,
 			  const hp::QCollection<dim-1> &quadrature,
-			  const typename FunctionMap<dim>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const std::vector<const InputVector *> &solutions,
 			  std::vector<Vector<float>*> &errors,
 			  const std::vector<bool>     &component_mask = std::vector<bool>(),
-			  const Function<dim>         *coefficients   = 0,
+			  const Function<spacedim>         *coefficients   = 0,
 			  const unsigned int           n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int           material_id = numbers::invalid_unsigned_int);
@@ -481,11 +481,11 @@ class KellyErrorEstimator
     template <typename InputVector, class DH>
     static void estimate (const DH                    &dof,
 			  const hp::QCollection<dim-1> &quadrature,
-			  const typename FunctionMap<dim>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const std::vector<const InputVector *> &solutions,
 			  std::vector<Vector<float>*> &errors,
 			  const std::vector<bool>     &component_mask = std::vector<bool>(),
-			  const Function<dim>         *coefficients   = 0,
+			  const Function<spacedim>*    coefficients   = 0,
 			  const unsigned int           n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int           material_id = numbers::invalid_unsigned_int);
@@ -592,11 +592,11 @@ class KellyErrorEstimator<1,spacedim>
     static void estimate (const Mapping<1,spacedim>  &mapping,
 			  const DH   &dof,
 			  const Quadrature<0> &quadrature,
-			  const FunctionMap<1>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const InputVector       &solution,
 			  Vector<float>           &error,
 			  const std::vector<bool> &component_mask = std::vector<bool>(),
-			  const Function<1>     *coefficients   = 0,
+			  const Function<spacedim>     *coefficients   = 0,
 			  const unsigned int       n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int       material_id = numbers::invalid_unsigned_int);
@@ -609,11 +609,11 @@ class KellyErrorEstimator<1,spacedim>
     template <typename InputVector, class DH>
     static void estimate (const DH   &dof,
 			  const Quadrature<0> &quadrature,
-			  const FunctionMap<1>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const InputVector       &solution,
 			  Vector<float>           &error,
 			  const std::vector<bool> &component_mask = std::vector<bool>(),
-			  const Function<1>     *coefficients   = 0,
+			  const Function<spacedim>     *coefficients   = 0,
 			  const unsigned int       n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int       material_id = numbers::invalid_unsigned_int);
@@ -649,11 +649,11 @@ class KellyErrorEstimator<1,spacedim>
     static void estimate (const Mapping<1,spacedim>          &mapping,
 			  const DH       &dof,
 			  const Quadrature<0>     &quadrature,
-			  const FunctionMap<1>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const std::vector<const InputVector *> &solutions,
 			  std::vector<Vector<float>*> &errors,
 			  const std::vector<bool>     &component_mask = std::vector<bool>(),
-			  const Function<1>         *coefficients   = 0,
+			  const Function<spacedim>         *coefficients   = 0,
 			  const unsigned int           n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int           material_id = numbers::invalid_unsigned_int);
@@ -666,11 +666,11 @@ class KellyErrorEstimator<1,spacedim>
     template <typename InputVector, class DH>
     static void estimate (const DH       &dof,
 			  const Quadrature<0>     &quadrature,
-			  const FunctionMap<1>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const std::vector<const InputVector *> &solutions,
 			  std::vector<Vector<float>*> &errors,
 			  const std::vector<bool>     &component_mask = std::vector<bool>(),
-			  const Function<1>         *coefficients   = 0,
+			  const Function<spacedim>         *coefficients   = 0,
 			  const unsigned int           n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int           material_id = numbers::invalid_unsigned_int);
@@ -686,11 +686,11 @@ class KellyErrorEstimator<1,spacedim>
     static void estimate (const Mapping<1,spacedim>      &mapping,
 			  const DH                &dof,
 			  const hp::QCollection<0> &quadrature,
-			  const typename FunctionMap<1>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const InputVector       &solution,
 			  Vector<float>           &error,
 			  const std::vector<bool> &component_mask = std::vector<bool>(),
-			  const Function<1>     *coefficients   = 0,
+			  const Function<spacedim>     *coefficients   = 0,
 			  const unsigned int       n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int       material_id = numbers::invalid_unsigned_int);
@@ -705,11 +705,11 @@ class KellyErrorEstimator<1,spacedim>
     template <typename InputVector, class DH>
     static void estimate (const DH                &dof,
 			  const hp::QCollection<0> &quadrature,
-			  const typename FunctionMap<1>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const InputVector       &solution,
 			  Vector<float>           &error,
 			  const std::vector<bool> &component_mask = std::vector<bool>(),
-			  const Function<1>     *coefficients   = 0,
+			  const Function<spacedim>     *coefficients   = 0,
 			  const unsigned int       n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int       material_id = numbers::invalid_unsigned_int);
@@ -725,11 +725,11 @@ class KellyErrorEstimator<1,spacedim>
     static void estimate (const Mapping<1,spacedim>          &mapping,
 			  const DH                    &dof,
 			  const hp::QCollection<0> &quadrature,
-			  const typename FunctionMap<1>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const std::vector<const InputVector *> &solutions,
 			  std::vector<Vector<float>*> &errors,
 			  const std::vector<bool>     &component_mask = std::vector<bool>(),
-			  const Function<1>         *coefficients   = 0,
+			  const Function<spacedim>         *coefficients   = 0,
 			  const unsigned int           n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int           material_id = numbers::invalid_unsigned_int);
@@ -744,11 +744,11 @@ class KellyErrorEstimator<1,spacedim>
     template <typename InputVector, class DH>
     static void estimate (const DH                    &dof,
 			  const hp::QCollection<0> &quadrature,
-			  const typename FunctionMap<1>::type &neumann_bc,
+			  const typename FunctionMap<spacedim>::type &neumann_bc,
 			  const std::vector<const InputVector *> &solutions,
 			  std::vector<Vector<float>*> &errors,
 			  const std::vector<bool>     &component_mask = std::vector<bool>(),
-			  const Function<1>         *coefficients   = 0,
+			  const Function<spacedim>         *coefficients   = 0,
 			  const unsigned int           n_threads = multithread_info.n_default_threads,
                           const types::subdomain_id_t subdomain_id = types::invalid_subdomain_id,
                           const unsigned int           material_id = numbers::invalid_unsigned_int);
