@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: matrices.h 21377 2010-06-25 19:52:09Z bangerth $
-//    Version: $Name$
+//    $Id: matrices.h 24326 2011-09-15 05:43:25Z bangerth $
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,10 +13,10 @@
 #define __deal2__matrices_h
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/thread_management.h>
-#include <dofs/function_map.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/thread_management.h>
+#include <deal.II/dofs/function_map.h>
 
 #include <map>
 
@@ -79,10 +78,9 @@ namespace TrilinosWrappers
 
 
 /**
- * This class provides functions that assemble certain standard matrices for a
+ * This namespace provides functions that assemble certain standard matrices for a
  * given triangulation, using a given finite element, a given mapping and a
- * quadrature formula.  All functions are static, so it is not necessary to
- * create an object of this type.
+ * quadrature formula.
  *
  *
  * <h3>Conventions for all functions</h3>
@@ -209,9 +207,8 @@ namespace TrilinosWrappers
  * @ingroup numerics
  * @author Wolfgang Bangerth, 1998, Ralf Hartmann, 2001
  */
-class MatrixCreator
+namespace MatrixCreator
 {
-  public:
 				     /**
 				      * Assemble the mass matrix. If no
 				      * coefficient is given, it is assumed
@@ -225,7 +222,7 @@ class MatrixCreator
 				      * for more information.
 				      */
     template <int dim, typename number, int spacedim>
-    static void create_mass_matrix (const Mapping<dim, spacedim>       &mapping,
+     void create_mass_matrix (const Mapping<dim, spacedim>       &mapping,
 				    const DoFHandler<dim,spacedim>    &dof,
 				    const Quadrature<dim>    &q,
 				    SparseMatrix<number>     &matrix,
@@ -237,7 +234,7 @@ class MatrixCreator
 				      * <tt>mapping=MappingQ1@<dim@>()</tt>.
 				      */
     template <int dim, typename number, int spacedim>
-    static void create_mass_matrix (const DoFHandler<dim,spacedim>    &dof,
+     void create_mass_matrix (const DoFHandler<dim,spacedim>    &dof,
 				    const Quadrature<dim>    &q,
 				    SparseMatrix<number>     &matrix,
 				    const Function<spacedim> * const a = 0);
@@ -256,7 +253,7 @@ class MatrixCreator
 				      * class for more information.
 				      */
     template <int dim, typename number, int spacedim>
-    static void create_mass_matrix (const Mapping<dim, spacedim>       &mapping,
+     void create_mass_matrix (const Mapping<dim, spacedim>       &mapping,
 				    const DoFHandler<dim,spacedim>    &dof,
 				    const Quadrature<dim>    &q,
 				    SparseMatrix<number>     &matrix,
@@ -270,7 +267,7 @@ class MatrixCreator
 				      * <tt>mapping=MappingQ1@<dim@>()</tt>.
 				      */
     template <int dim, typename number, int spacedim>
-    static void create_mass_matrix (const DoFHandler<dim,spacedim>    &dof,
+     void create_mass_matrix (const DoFHandler<dim,spacedim>    &dof,
 				    const Quadrature<dim>    &q,
 				    SparseMatrix<number>     &matrix,
 				    const Function<spacedim>      &rhs,
@@ -281,7 +278,7 @@ class MatrixCreator
 				      * objects.
 				      */
     template <int dim, typename number, int spacedim>
-    static void create_mass_matrix (const hp::MappingCollection<dim,spacedim>       &mapping,
+     void create_mass_matrix (const hp::MappingCollection<dim,spacedim>       &mapping,
 				    const hp::DoFHandler<dim,spacedim>    &dof,
 				    const hp::QCollection<dim>    &q,
 				    SparseMatrix<number>     &matrix,
@@ -292,7 +289,7 @@ class MatrixCreator
 				      * objects.
 				      */
     template <int dim, typename number, int spacedim>
-    static void create_mass_matrix (const hp::DoFHandler<dim,spacedim>    &dof,
+     void create_mass_matrix (const hp::DoFHandler<dim,spacedim>    &dof,
 				    const hp::QCollection<dim>    &q,
 				    SparseMatrix<number>     &matrix,
 				    const Function<spacedim> * const a = 0);
@@ -302,7 +299,7 @@ class MatrixCreator
 				      * objects.
 				      */
     template <int dim, typename number, int spacedim>
-    static void create_mass_matrix (const hp::MappingCollection<dim,spacedim>       &mapping,
+     void create_mass_matrix (const hp::MappingCollection<dim,spacedim>       &mapping,
 				    const hp::DoFHandler<dim,spacedim>    &dof,
 				    const hp::QCollection<dim>    &q,
 				    SparseMatrix<number>     &matrix,
@@ -315,7 +312,7 @@ class MatrixCreator
 				      * objects.
 				      */
     template <int dim, typename number, int spacedim>
-    static void create_mass_matrix (const hp::DoFHandler<dim,spacedim>    &dof,
+     void create_mass_matrix (const hp::DoFHandler<dim,spacedim>    &dof,
 				    const hp::QCollection<dim>    &q,
 				    SparseMatrix<number>     &matrix,
 				    const Function<spacedim>      &rhs,
@@ -363,7 +360,7 @@ class MatrixCreator
 				      * cell-dependent shape functions.
 				      */
     template <int dim, int spacedim>
-    static
+
     void create_boundary_mass_matrix (const Mapping<dim, spacedim>       &mapping,
 				      const DoFHandler<dim,spacedim>    &dof,
 				      const Quadrature<dim-1>  &q,
@@ -376,7 +373,7 @@ class MatrixCreator
 
 // 				      * Same function, but for 1d.
 // 				      */
-//     static
+//
 //     void create_boundary_mass_matrix (const Mapping<1,1>       &mapping,
 // 				      const DoFHandler<1,1>    &dof,
 // 				      const Quadrature<0>    &q,
@@ -386,7 +383,7 @@ class MatrixCreator
 // 				      std::vector<unsigned int>&dof_to_boundary_mapping,
 // 				      const Function<1> * const a = 0);
 // //codimension 1
-//     static
+//
 //     void create_boundary_mass_matrix (const Mapping<1,2>       &mapping,
 // 				      const DoFHandler<1,2>    &dof,
 // 				      const Quadrature<0>    &q,
@@ -405,7 +402,7 @@ class MatrixCreator
 				      * <tt>mapping=MappingQ1@<dim@>()</tt>.
 				      */
     template <int dim, int spacedim>
-    static
+
     void create_boundary_mass_matrix (const DoFHandler<dim,spacedim>    &dof,
 				      const Quadrature<dim-1>  &q,
 				      SparseMatrix<double>     &matrix,
@@ -420,7 +417,7 @@ class MatrixCreator
 				      * objects.
 				      */
     template <int dim, int spacedim>
-    static
+
     void create_boundary_mass_matrix (const hp::MappingCollection<dim,spacedim>       &mapping,
 				      const hp::DoFHandler<dim,spacedim>    &dof,
 				      const hp::QCollection<dim-1>  &q,
@@ -435,7 +432,7 @@ class MatrixCreator
 				      * Same function as above, but for hp
 				      * objects.
 				      */
-//     static
+//
 //     void create_boundary_mass_matrix (const hp::MappingCollection<1,1>       &mapping,
 // 				      const hp::DoFHandler<1,1>    &dof,
 // 				      const hp::QCollection<0>    &q,
@@ -450,7 +447,7 @@ class MatrixCreator
 				      * objects.
 				      */
     template <int dim, int spacedim>
-    static
+
     void create_boundary_mass_matrix (const hp::DoFHandler<dim,spacedim>    &dof,
 				      const hp::QCollection<dim-1>  &q,
 				      SparseMatrix<double>     &matrix,
@@ -474,7 +471,7 @@ class MatrixCreator
 				      * class for more information.
 				      */
     template <int dim, int spacedim>
-    static void create_laplace_matrix (const Mapping<dim, spacedim>       &mapping,
+     void create_laplace_matrix (const Mapping<dim, spacedim>       &mapping,
 				       const DoFHandler<dim,spacedim>    &dof,
 				       const Quadrature<dim>    &q,
 				       SparseMatrix<double>     &matrix,
@@ -487,7 +484,7 @@ class MatrixCreator
 				      * <tt>mapping=MappingQ1@<dim@>()</tt>.
 				      */
     template <int dim, int spacedim>
-    static void create_laplace_matrix (const DoFHandler<dim,spacedim>    &dof,
+     void create_laplace_matrix (const DoFHandler<dim,spacedim>    &dof,
 				       const Quadrature<dim>    &q,
 				       SparseMatrix<double>     &matrix,
 				       const Function<spacedim> * const a = 0);
@@ -507,7 +504,7 @@ class MatrixCreator
 				      * class for more information.
 				      */
     template <int dim, int spacedim>
-    static void create_laplace_matrix (const Mapping<dim, spacedim>       &mapping,
+     void create_laplace_matrix (const Mapping<dim, spacedim>       &mapping,
 				       const DoFHandler<dim,spacedim>    &dof,
 				       const Quadrature<dim>    &q,
 				       SparseMatrix<double>     &matrix,
@@ -522,7 +519,7 @@ class MatrixCreator
 				      * <tt>mapping=MappingQ1@<dim@>()</tt>.
 				      */
     template <int dim, int spacedim>
-    static void create_laplace_matrix (const DoFHandler<dim,spacedim>    &dof,
+     void create_laplace_matrix (const DoFHandler<dim,spacedim>    &dof,
 				       const Quadrature<dim>    &q,
 				       SparseMatrix<double>     &matrix,
 				       const Function<spacedim>      &rhs,
@@ -535,7 +532,7 @@ class MatrixCreator
 				      * collections.
 				      */
     template <int dim, int spacedim>
-    static void create_laplace_matrix (const hp::MappingCollection<dim,spacedim>       &mapping,
+     void create_laplace_matrix (const hp::MappingCollection<dim,spacedim>       &mapping,
 				       const hp::DoFHandler<dim,spacedim>    &dof,
 				       const hp::QCollection<dim>    &q,
 				       SparseMatrix<double>     &matrix,
@@ -547,7 +544,7 @@ class MatrixCreator
 				      * collections.
 				      */
     template <int dim, int spacedim>
-    static void create_laplace_matrix (const hp::DoFHandler<dim,spacedim>    &dof,
+     void create_laplace_matrix (const hp::DoFHandler<dim,spacedim>    &dof,
 				       const hp::QCollection<dim>    &q,
 				       SparseMatrix<double>     &matrix,
 				       const Function<spacedim> * const a = 0);
@@ -558,7 +555,7 @@ class MatrixCreator
 				      * collections.
 				      */
     template <int dim, int spacedim>
-    static void create_laplace_matrix (const hp::MappingCollection<dim,spacedim>       &mapping,
+     void create_laplace_matrix (const hp::MappingCollection<dim,spacedim>       &mapping,
 				       const hp::DoFHandler<dim,spacedim>    &dof,
 				       const hp::QCollection<dim>    &q,
 				       SparseMatrix<double>     &matrix,
@@ -572,7 +569,7 @@ class MatrixCreator
 				      * collections.
 				      */
     template <int dim, int spacedim>
-    static void create_laplace_matrix (const hp::DoFHandler<dim,spacedim>    &dof,
+     void create_laplace_matrix (const hp::DoFHandler<dim,spacedim>    &dof,
 				       const hp::QCollection<dim>    &q,
 				       SparseMatrix<double>     &matrix,
 				       const Function<spacedim>      &rhs,
@@ -583,93 +580,7 @@ class MatrixCreator
 				      * Exception
 				      */
     DeclException0 (ExcComponentMismatch);
-  private:
-				     /**
-				      * Convenience abbreviation for
-				      * pairs of DoF handler cell
-				      * iterators. This type works
-				      * just like a
-				      * <tt>std::pair<iterator,iterator></tt>
-				      * but is templatized on the
-				      * dof handler that shouls be used.
-				      */
-    template <typename DH>
-    struct IteratorRange
-    {
-					 /**
-					  * Typedef for the iterator type.
-					  */
-	typedef typename DH::active_cell_iterator active_cell_iterator;
-
-					 /**
-					  * Abbreviation for a pair of
-					  * iterators.
-					  */
-	typedef std::pair<active_cell_iterator,active_cell_iterator> iterator_pair;
-
-					 /**
-					  * Constructor. Initialize
-					  * the two values by the
-					  * given values.
-					  */
-	IteratorRange (const active_cell_iterator &first,
-		       const active_cell_iterator &second);
-
-					 /**
-					  * Constructor taking a pair
-					  * of values for
-					  * initialization.
-					  */
-	IteratorRange (const iterator_pair &ip);
-
-					 /**
-					  * Pair of iterators denoting
-					  * a half-open range.
-					  */
-	active_cell_iterator first, second;
-    };
-
-				     /**
-				      * Version of the same function
-				      * (without suffix @p _1) with
-				      * the same argument list that
-				      * operates only on an interval
-				      * of iterators. Used for
-				      * parallelization. The mutex is
-				      * used to synchronise access to
-				      * the matrix.
-				      */
-    template <int dim, int spacedim>
-    static
-    void create_boundary_mass_matrix_1 (std_cxx1x::tuple<const Mapping<dim,spacedim> &,
-					const DoFHandler<dim,spacedim> &,
-					const Quadrature<dim-1> & > commons,
-					SparseMatrix<double>     &matrix,
-					const typename FunctionMap<spacedim>::type        &boundary_functions,
-					Vector<double>           &rhs_vector,
-					std::vector<unsigned int>&dof_to_boundary_mapping,
-					const Function<spacedim> * const coefficient,
-					const std::vector<unsigned int>& component_mapping,
-					const IteratorRange<DoFHandler<dim,spacedim> >  range,
-					Threads::ThreadMutex     &mutex);
-
-				     /**
-				      * Same as above, but for hp.
-				      */
-    template <int dim, int spacedim>
-    static
-    void create_boundary_mass_matrix_1 (std_cxx1x::tuple<const hp::MappingCollection<dim,spacedim> &,
-					const hp::DoFHandler<dim,spacedim>&,
-					const hp::QCollection<dim-1> & > commons,
-					SparseMatrix<double>     &matrix,
-					const typename FunctionMap<spacedim>::type        &boundary_functions,
-					Vector<double>           &rhs_vector,
-					std::vector<unsigned int>&dof_to_boundary_mapping,
-					const Function<spacedim> * const coefficient,
-					const std::vector<unsigned int>& component_mapping,
-					const IteratorRange<hp::DoFHandler<dim,spacedim> >  range,
-					Threads::ThreadMutex     &mutex);
-};
+}
 
 
 
@@ -874,302 +785,311 @@ class MatrixCreator
  * @ingroup numerics
  * @author Wolfgang Bangerth, 1998, 2000, 2004, 2005
  */
-class MatrixTools : public MatrixCreator
+namespace MatrixTools
 {
-  public:
-				     /**
-				      * Apply dirichlet boundary conditions
-				      * to the system matrix and vectors
-				      * as described in the general
-				      * documentation.
-				      */
-    template <typename number>
-    static void
-    apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
-			   SparseMatrix<number>  &matrix,
-			   Vector<number>        &solution,
-			   Vector<number>        &right_hand_side,
-			   const bool             eliminate_columns = true);
+				   /**
+				    * Import namespace MatrixCreator for
+				    * backward compatibility with older
+				    * versions of deal.II in which these
+				    * namespaces were classes and class
+				    * MatrixTools was publicly derived from
+				    * class MatrixCreator.
+				    */
+  using namespace MatrixCreator;
 
-				     /**
-				      * Apply dirichlet boundary
-				      * conditions to the system
-				      * matrix and vectors as
-				      * described in the general
-				      * documentation. This function
-				      * works for block sparse
-				      * matrices and block vectors
-				      */
-    template <typename number>
-    static void
-    apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
-			   BlockSparseMatrix<number>           &matrix,
-			   BlockVector<number>                 &solution,
-			   BlockVector<number>                 &right_hand_side,
-			   const bool           eliminate_columns = true);
+				   /**
+				    * Apply dirichlet boundary conditions
+				    * to the system matrix and vectors
+				    * as described in the general
+				    * documentation.
+				    */
+  template <typename number>
+  void
+  apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
+			 SparseMatrix<number>  &matrix,
+			 Vector<number>        &solution,
+			 Vector<number>        &right_hand_side,
+			 const bool             eliminate_columns = true);
+
+				   /**
+				    * Apply dirichlet boundary
+				    * conditions to the system
+				    * matrix and vectors as
+				    * described in the general
+				    * documentation. This function
+				    * works for block sparse
+				    * matrices and block vectors
+				    */
+  template <typename number>
+  void
+  apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
+			 BlockSparseMatrix<number>           &matrix,
+			 BlockVector<number>                 &solution,
+			 BlockVector<number>                 &right_hand_side,
+			 const bool           eliminate_columns = true);
 
 #ifdef DEAL_II_USE_PETSC
-				     /**
-				      * Apply dirichlet boundary conditions to
-				      * the system matrix and vectors as
-				      * described in the general
-				      * documentation. This function works on
-				      * the classes that are used to wrap
-				      * PETSc objects.
-				      *
- 				      * Note that this function is not very
- 				      * efficient: it needs to alternatingly
- 				      * read and write into the matrix, a
- 				      * situation that PETSc does not handle
- 				      * too well. In addition, we only get rid
- 				      * of rows corresponding to boundary
- 				      * nodes, but the corresponding case of
- 				      * deleting the respective columns
- 				      * (i.e. if @p eliminate_columns is @p
- 				      * true) is not presently implemented,
- 				      * and probably will never because it is
- 				      * too expensive without direct access to
- 				      * the PETSc data structures. (This leads
- 				      * to the situation where the action
- 				      * indicates by the default value of the
- 				      * last argument is actually not
- 				      * implemented; that argument has
- 				      * <code>true</code> as its default value
- 				      * to stay consistent with the other
- 				      * functions of same name in this class.)
- 				      * A third reason against this function
- 				      * is that it doesn't handle the case
- 				      * where the matrix is distributed across
- 				      * an MPI system.
- 				      *
- 				      * This function is used in
-				      * step-17 and
-				      * step-18.
- 				      */
-    static void
-    apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
-			   PETScWrappers::SparseMatrix  &matrix,
-			   PETScWrappers::Vector  &solution,
-			   PETScWrappers::Vector  &right_hand_side,
-			   const bool             eliminate_columns = true);
+				   /**
+				    * Apply dirichlet boundary conditions to
+				    * the system matrix and vectors as
+				    * described in the general
+				    * documentation. This function works on
+				    * the classes that are used to wrap
+				    * PETSc objects.
+				    *
+				    * Note that this function is not very
+				    * efficient: it needs to alternatingly
+				    * read and write into the matrix, a
+				    * situation that PETSc does not handle
+				    * too well. In addition, we only get rid
+				    * of rows corresponding to boundary
+				    * nodes, but the corresponding case of
+				    * deleting the respective columns
+				    * (i.e. if @p eliminate_columns is @p
+				    * true) is not presently implemented,
+				    * and probably will never because it is
+				    * too expensive without direct access to
+				    * the PETSc data structures. (This leads
+				    * to the situation where the action
+				    * indicates by the default value of the
+				    * last argument is actually not
+				    * implemented; that argument has
+				    * <code>true</code> as its default value
+				    * to stay consistent with the other
+				    * functions of same name in this class.)
+				    * A third reason against this function
+				    * is that it doesn't handle the case
+				    * where the matrix is distributed across
+				    * an MPI system.
+				    *
+				    * This function is used in
+				    * step-17 and
+				    * step-18.
+				    */
+  void
+  apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
+			 PETScWrappers::SparseMatrix  &matrix,
+			 PETScWrappers::Vector  &solution,
+			 PETScWrappers::Vector  &right_hand_side,
+			 const bool             eliminate_columns = true);
 
-                                     /**
-                                      * Same function, but for parallel PETSc
-                                      * matrices.
-                                      */
-    static void
-    apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
-			   PETScWrappers::MPI::SparseMatrix  &matrix,
-			   PETScWrappers::MPI::Vector  &solution,
-			   PETScWrappers::MPI::Vector  &right_hand_side,
-			   const bool             eliminate_columns = true);
+				   /**
+				    * Same function, but for parallel PETSc
+				    * matrices.
+				    */
+  void
+  apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
+			 PETScWrappers::MPI::SparseMatrix  &matrix,
+			 PETScWrappers::MPI::Vector  &solution,
+			 PETScWrappers::MPI::Vector  &right_hand_side,
+			 const bool             eliminate_columns = true);
 
-                                     /**
-                                      * Same function, but for
-                                      * parallel PETSc matrices. Note
-                                      * that this function only
-                                      * operates on the local range of
-                                      * the parallel matrix, i.e. it
-                                      * only eliminates rows
-                                      * corresponding to degrees of
-                                      * freedom for which the row is
-                                      * stored on the present
-                                      * processor. All other boundary
-                                      * nodes are ignored, and it
-                                      * doesn't matter whether they
-                                      * are present in the first
-                                      * argument to this function or
-                                      * not. A consequence of this,
-                                      * however, is that this function
-                                      * has to be called from all
-                                      * processors that participate in
-                                      * sharing the contents of the
-                                      * given matrices and vectors. It
-                                      * is also implied that the local
-                                      * range for all objects passed
-                                      * to this function is the same.
-                                      */
-    static void
-    apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
-			   PETScWrappers::MPI::SparseMatrix  &matrix,
-			   PETScWrappers::Vector       &solution,
-			   PETScWrappers::MPI::Vector  &right_hand_side,
-			   const bool             eliminate_columns = true);
+				   /**
+				    * Same function, but for
+				    * parallel PETSc matrices. Note
+				    * that this function only
+				    * operates on the local range of
+				    * the parallel matrix, i.e. it
+				    * only eliminates rows
+				    * corresponding to degrees of
+				    * freedom for which the row is
+				    * stored on the present
+				    * processor. All other boundary
+				    * nodes are ignored, and it
+				    * doesn't matter whether they
+				    * are present in the first
+				    * argument to this function or
+				    * not. A consequence of this,
+				    * however, is that this function
+				    * has to be called from all
+				    * processors that participate in
+				    * sharing the contents of the
+				    * given matrices and vectors. It
+				    * is also implied that the local
+				    * range for all objects passed
+				    * to this function is the same.
+				    */
+  void
+  apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
+			 PETScWrappers::MPI::SparseMatrix  &matrix,
+			 PETScWrappers::Vector       &solution,
+			 PETScWrappers::MPI::Vector  &right_hand_side,
+			 const bool             eliminate_columns = true);
 
-                                    /**
-                                      * Same as above but for BlockSparseMatrix.
-				      */
-    static void
-    apply_boundary_values (const std::map<unsigned int,double>  &boundary_values,
-                       PETScWrappers::MPI::BlockSparseMatrix  &matrix,
-                       PETScWrappers::MPI::BlockVector        &solution,
-                       PETScWrappers::MPI::BlockVector        &right_hand_side,
-                       const bool       eliminate_columns = true);
+				   /**
+				    * Same as above but for BlockSparseMatrix.
+				    */
+  void
+  apply_boundary_values (const std::map<unsigned int,double>  &boundary_values,
+			 PETScWrappers::MPI::BlockSparseMatrix  &matrix,
+			 PETScWrappers::MPI::BlockVector        &solution,
+			 PETScWrappers::MPI::BlockVector        &right_hand_side,
+			 const bool       eliminate_columns = true);
 
 #endif
 
 #ifdef DEAL_II_USE_TRILINOS
-				     /**
-				      * Apply dirichlet boundary
-				      * conditions to the system matrix
-				      * and vectors as described in the
-				      * general documentation. This
-				      * function works on the classes
-				      * that are used to wrap Trilinos
-				      * objects.
-				      *
- 				      * Note that this function is not
- 				      * very efficient: it needs to
- 				      * alternatingly read and write
- 				      * into the matrix, a situation
- 				      * that Trilinos does not handle
- 				      * too well. In addition, we only
- 				      * get rid of rows corresponding to
- 				      * boundary nodes, but the
- 				      * corresponding case of deleting
- 				      * the respective columns (i.e. if
- 				      * @p eliminate_columns is @p true)
- 				      * is not presently implemented,
- 				      * and probably will never because
- 				      * it is too expensive without
- 				      * direct access to the Trilinos
- 				      * data structures. (This leads to
- 				      * the situation where the action
- 				      * indicates by the default value
- 				      * of the last argument is actually
- 				      * not implemented; that argument
- 				      * has <code>true</code> as its
- 				      * default value to stay consistent
- 				      * with the other functions of same
- 				      * name in this class.)  A third
- 				      * reason against this function is
- 				      * that it doesn't handle the case
- 				      * where the matrix is distributed
- 				      * across an MPI system.
- 				      */
-    static void
-    apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
-			   TrilinosWrappers::SparseMatrix  &matrix,
-			   TrilinosWrappers::Vector        &solution,
-			   TrilinosWrappers::Vector        &right_hand_side,
-			   const bool             eliminate_columns = true);
+				   /**
+				    * Apply dirichlet boundary
+				    * conditions to the system matrix
+				    * and vectors as described in the
+				    * general documentation. This
+				    * function works on the classes
+				    * that are used to wrap Trilinos
+				    * objects.
+				    *
+				    * Note that this function is not
+				    * very efficient: it needs to
+				    * alternatingly read and write
+				    * into the matrix, a situation
+				    * that Trilinos does not handle
+				    * too well. In addition, we only
+				    * get rid of rows corresponding to
+				    * boundary nodes, but the
+				    * corresponding case of deleting
+				    * the respective columns (i.e. if
+				    * @p eliminate_columns is @p true)
+				    * is not presently implemented,
+				    * and probably will never because
+				    * it is too expensive without
+				    * direct access to the Trilinos
+				    * data structures. (This leads to
+				    * the situation where the action
+				    * indicates by the default value
+				    * of the last argument is actually
+				    * not implemented; that argument
+				    * has <code>true</code> as its
+				    * default value to stay consistent
+				    * with the other functions of same
+				    * name in this class.)  A third
+				    * reason against this function is
+				    * that it doesn't handle the case
+				    * where the matrix is distributed
+				    * across an MPI system.
+				    */
+  void
+  apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
+			 TrilinosWrappers::SparseMatrix  &matrix,
+			 TrilinosWrappers::Vector        &solution,
+			 TrilinosWrappers::Vector        &right_hand_side,
+			 const bool             eliminate_columns = true);
 
-				     /**
-				      * This function does the same as
-				      * the one above, except now
-				      * working on block structures.
-				      */
-    static void
-    apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
-			   TrilinosWrappers::BlockSparseMatrix  &matrix,
-			   TrilinosWrappers::BlockVector        &solution,
-			   TrilinosWrappers::BlockVector        &right_hand_side,
-			   const bool                eliminate_columns = true);
+				   /**
+				    * This function does the same as
+				    * the one above, except now
+				    * working on block structures.
+				    */
+  void
+  apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
+			 TrilinosWrappers::BlockSparseMatrix  &matrix,
+			 TrilinosWrappers::BlockVector        &solution,
+			 TrilinosWrappers::BlockVector        &right_hand_side,
+			 const bool                eliminate_columns = true);
 
-				     /**
-				      * Apply dirichlet boundary
-				      * conditions to the system matrix
-				      * and vectors as described in the
-				      * general documentation. This
-				      * function works on the classes
-				      * that are used to wrap Trilinos
-				      * objects.
-				      *
- 				      * Note that this function is not
- 				      * very efficient: it needs to
- 				      * alternatingly read and write
- 				      * into the matrix, a situation
- 				      * that Trilinos does not handle
- 				      * too well. In addition, we only
- 				      * get rid of rows corresponding to
- 				      * boundary nodes, but the
- 				      * corresponding case of deleting
- 				      * the respective columns (i.e. if
- 				      * @p eliminate_columns is @p true)
- 				      * is not presently implemented,
- 				      * and probably will never because
- 				      * it is too expensive without
- 				      * direct access to the Trilinos
- 				      * data structures. (This leads to
- 				      * the situation where the action
- 				      * indicates by the default value
- 				      * of the last argument is actually
- 				      * not implemented; that argument
- 				      * has <code>true</code> as its
- 				      * default value to stay consistent
- 				      * with the other functions of same
- 				      * name in this class.) This
- 				      * function does work on MPI vector
- 				      * types.
- 				      */
-    static void
-    apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
-			   TrilinosWrappers::SparseMatrix  &matrix,
-			   TrilinosWrappers::MPI::Vector   &solution,
-			   TrilinosWrappers::MPI::Vector   &right_hand_side,
-			   const bool             eliminate_columns = true);
+				   /**
+				    * Apply dirichlet boundary
+				    * conditions to the system matrix
+				    * and vectors as described in the
+				    * general documentation. This
+				    * function works on the classes
+				    * that are used to wrap Trilinos
+				    * objects.
+				    *
+				    * Note that this function is not
+				    * very efficient: it needs to
+				    * alternatingly read and write
+				    * into the matrix, a situation
+				    * that Trilinos does not handle
+				    * too well. In addition, we only
+				    * get rid of rows corresponding to
+				    * boundary nodes, but the
+				    * corresponding case of deleting
+				    * the respective columns (i.e. if
+				    * @p eliminate_columns is @p true)
+				    * is not presently implemented,
+				    * and probably will never because
+				    * it is too expensive without
+				    * direct access to the Trilinos
+				    * data structures. (This leads to
+				    * the situation where the action
+				    * indicates by the default value
+				    * of the last argument is actually
+				    * not implemented; that argument
+				    * has <code>true</code> as its
+				    * default value to stay consistent
+				    * with the other functions of same
+				    * name in this class.) This
+				    * function does work on MPI vector
+				    * types.
+				    */
+  void
+  apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
+			 TrilinosWrappers::SparseMatrix  &matrix,
+			 TrilinosWrappers::MPI::Vector   &solution,
+			 TrilinosWrappers::MPI::Vector   &right_hand_side,
+			 const bool             eliminate_columns = true);
 
-				     /**
-				      * This function does the same as
-				      * the one above, except now working
-				      * on block structures.
-				      */
-    static void
-    apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
-			   TrilinosWrappers::BlockSparseMatrix  &matrix,
-			   TrilinosWrappers::MPI::BlockVector   &solution,
-			   TrilinosWrappers::MPI::BlockVector   &right_hand_side,
-			   const bool                eliminate_columns = true);
+				   /**
+				    * This function does the same as
+				    * the one above, except now working
+				    * on block structures.
+				    */
+  void
+  apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
+			 TrilinosWrappers::BlockSparseMatrix  &matrix,
+			 TrilinosWrappers::MPI::BlockVector   &solution,
+			 TrilinosWrappers::MPI::BlockVector   &right_hand_side,
+			 const bool                eliminate_columns = true);
 #endif
 
-                                     /**
-                                      * Rather than applying boundary
-                                      * values to the global matrix
-                                      * and vector after creating the
-                                      * global matrix, this function
-                                      * does so during assembly, by
-                                      * modifying the local matrix and
-                                      * vector contributions. If you
-                                      * call this function on all
-                                      * local contributions, the
-                                      * resulting matrix will have the
-                                      * same entries, and the final
-                                      * call to
-                                      * apply_boundary_values() on the
-                                      * global system will not be
-                                      * necessary.
-                                      *
-                                      * Since this function does not
-                                      * have to work on the
-                                      * complicated data structures of
-                                      * sparse matrices, it is
-                                      * relatively cheap. It may
-                                      * therefore be a win if you have
-                                      * many fixed degrees of freedom
-                                      * (e.g. boundary nodes), or if
-                                      * access to the sparse matrix is
-                                      * expensive (e.g. for block
-                                      * sparse matrices, or for PETSc
-                                      * or trilinos
-                                      * matrices). However, it doesn't
-                                      * work as expected if there are
-                                      * also hanging nodes to be
-                                      * considered. More caveats are
-                                      * listed in the general
-                                      * documentation of this class.
-                                      */
-    static void
-    local_apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
-                                 const std::vector<unsigned int> &local_dof_indices,
-                                 FullMatrix<double> &local_matrix,
-                                 Vector<double>     &local_rhs,
-                                 const bool          eliminate_columns);
+				   /**
+				    * Rather than applying boundary
+				    * values to the global matrix
+				    * and vector after creating the
+				    * global matrix, this function
+				    * does so during assembly, by
+				    * modifying the local matrix and
+				    * vector contributions. If you
+				    * call this function on all
+				    * local contributions, the
+				    * resulting matrix will have the
+				    * same entries, and the final
+				    * call to
+				    * apply_boundary_values() on the
+				    * global system will not be
+				    * necessary.
+				    *
+				    * Since this function does not
+				    * have to work on the
+				    * complicated data structures of
+				    * sparse matrices, it is
+				    * relatively cheap. It may
+				    * therefore be a win if you have
+				    * many fixed degrees of freedom
+				    * (e.g. boundary nodes), or if
+				    * access to the sparse matrix is
+				    * expensive (e.g. for block
+				    * sparse matrices, or for PETSc
+				    * or trilinos
+				    * matrices). However, it doesn't
+				    * work as expected if there are
+				    * also hanging nodes to be
+				    * considered. More caveats are
+				    * listed in the general
+				    * documentation of this class.
+				    */
+  void
+  local_apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
+			       const std::vector<unsigned int> &local_dof_indices,
+			       FullMatrix<double> &local_matrix,
+			       Vector<double>     &local_rhs,
+			       const bool          eliminate_columns);
 
-				     /**
-				      * Exception
-				      */
-    DeclException0 (ExcBlocksDontMatch);
-};
+				   /**
+				    * Exception
+				    */
+  DeclException0 (ExcBlocksDontMatch);
+}
 
 
 
