@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: operator.h 21516 2010-07-18 05:11:48Z kanschat $
+//    $Id: operator.h 23709 2011-05-17 04:34:08Z bangerth $
 //
 //    Copyright (C) 2010 by the deal.II authors
 //
@@ -15,5 +15,5 @@
  * @deprecated This file has moved to <tt>lac/algorithms</tt>
  */
 
-#include <algorithms/operator.h>
-#include <numerics/dof_output_operator.h>
+#include <deal.II/algorithms/operator.h>
+#include <deal.II/numerics/dof_output_operator.h>
