@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: theta_timestepping.h 21514 2010-07-18 04:08:12Z kanschat $
+//    $Id: theta_timestepping.h 23709 2011-05-17 04:34:08Z bangerth $
 //
 //    Copyright (C) 2010 by the deal.II authors
 //
@@ -15,4 +15,4 @@
  * @deprecated This file has moved to <tt>lac/algorithms</tt>
  */
 
-#include <algorithms/theta_timestepping.h>
+#include <deal.II/algorithms/theta_timestepping.h>
