@@ -1,6 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: time_dependent.h 21377 2010-06-25 19:52:09Z bangerth $
-//    Version: $Name$
+//    $Id: time_dependent.h 23876 2011-06-28 18:21:51Z kanschat $
 //
 //    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009, 2010 by the deal.II authors
 //
@@ -17,10 +16,10 @@
 /*----------------------------   time-dependent.h     ---------------------------*/
 
 
-#include <base/config.h>
-#include <base/exceptions.h>
-#include <base/subscriptor.h>
-#include <base/smartpointer.h>
+#include <deal.II/base/config.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/base/smartpointer.h>
 
 #include <vector>
 #include <utility>
@@ -683,7 +682,7 @@ class TimeDependent
 				      * memory consumption (in bytes)
 				      * of this object.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
 				     /**
 				      * Exception.
@@ -1001,7 +1000,7 @@ class TimeStepBase : public Subscriptor
 				      * result of this function to
 				      * your result.
 				      */
-    virtual unsigned int memory_consumption () const;
+    virtual std::size_t memory_consumption () const;
 
 				     /**
 				      * Exception
@@ -1772,7 +1771,7 @@ class TimeStepBase_Tria : public TimeStepBase
 				      * result of this function to
 				      * your result.
 				      */
-    virtual unsigned int memory_consumption () const;
+    virtual std::size_t memory_consumption () const;
 
 				     /**
 				      * Exception
