@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: flow_function.cc 22349 2010-10-15 14:09:00Z kanschat $
+//    $Id: flow_function.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2007, 2008, 2010 by the deal.II authors
+//    Copyright (C) 2007, 2008, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,10 +12,10 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/tensor.h>
-#include <base/point.h>
-#include <base/flow_function.h>
-#include <lac/vector.h>
+#include <deal.II/base/tensor.h>
+#include <deal.II/base/point.h>
+#include <deal.II/base/flow_function.h>
+#include <deal.II/lac/vector.h>
 
 #include <cmath>
 
@@ -171,7 +171,8 @@ namespace Functions
 
 
   template<int dim>
-  unsigned int FlowFunction<dim>::memory_consumption () const
+  std::size_t
+  FlowFunction<dim>::memory_consumption () const
   {
     Assert(false, ExcNotImplemented());
     return 0;
