@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: job_identifier.cc 22087 2010-09-20 16:52:02Z kanschat $
+//    $Id: job_identifier.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2009, 2010 by the deal.II authors
@@ -12,7 +12,7 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/job_identifier.h>
+#include <deal.II/base/job_identifier.h>
 #include <ctime>
 
 #if HAVE_GETHOSTNAME
