@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: quadrature_lib.cc 22041 2010-09-19 00:36:17Z bangerth $
+//    $Id: quadrature_lib.cc 24054 2011-08-12 22:37:55Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,8 +12,8 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/quadrature_lib.h>
-#include <base/geometry_info.h>
+#include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/geometry_info.h>
 #include <cmath>
 
 #ifdef HAVE_STD_NUMERIC_LIMITS
@@ -41,12 +41,13 @@ namespace
 template <>
 QGauss<0>::QGauss (const unsigned int)
                 :
-                Quadrature<0> (0)
+						 // there are n_q^dim == 1
+						 // points
+                Quadrature<0> (1)
 {
-                                   // this function has to be provided to
-                                   // avoid certain linker failures, but it
-                                   // should never be called
-  Assert (false, ExcInternalError());
+				   // the single quadrature point gets unit
+				   // weight
+  this->weights[0] = 1;
 }
 
 
@@ -54,12 +55,13 @@ QGauss<0>::QGauss (const unsigned int)
 template <>
 QGaussLobatto<0>::QGaussLobatto (const unsigned int)
                 :
-                Quadrature<0> (0)
+						 // there are n_q^dim == 1
+						 // points
+                Quadrature<0> (1)
 {
-                                   // this function has to be provided to
-                                   // avoid certain linker failures, but it
-                                   // should never be called
-  Assert (false, ExcInternalError());
+				   // the single quadrature point gets unit
+				   // weight
+  this->weights[0] = 1;
 }
 
 
