@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//      $Id: utilities.cc 22487 2010-10-25 23:51:42Z bangerth $
+//      $Id: utilities.cc 24435 2011-09-26 18:50:03Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2005, 2006, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2005, 2006, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,8 +11,8 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/utilities.h>
-#include <base/exceptions.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/base/exceptions.h>
 
 #include <fstream>
 #include <iomanip>
@@ -25,6 +25,7 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sstream>
+#include <iostream>
 
 #ifdef HAVE_STD_NUMERIC_LIMITS
 #  include <limits>
@@ -35,10 +36,11 @@
 #ifdef DEAL_II_USE_TRILINOS
 #  ifdef DEAL_II_COMPILER_SUPPORTS_MPI
 #    include <Epetra_MpiComm.h>
-#    include <lac/vector_memory.h>
-#    include <lac/trilinos_vector.h>
-#    include <lac/trilinos_block_vector.h>
+#    include <deal.II/lac/vector_memory.h>
+#    include <deal.II/lac/trilinos_vector.h>
+#    include <deal.II/lac/trilinos_block_vector.h>
 #  endif
+#  include "Teuchos_RCP.hpp"
 #  include "Epetra_SerialComm.h"
 #endif
 
@@ -140,10 +142,14 @@ namespace Utilities
 
     int i = max_int;
     ss >> i;
-
                                      // check for errors
     AssertThrow (i != max_int, ExcCantConvertString (s));
 
+//TODO: The test for errors above doesn't work, as can easily be
+//verified. furthermore, it doesn't catch cases like when calling
+//string_to_int("1.23.4") since it just reads in however much it can, without
+//realizing that there is more
+
     return i;
   }
 
@@ -160,6 +166,44 @@ namespace Utilities
 
 
 
+  double
+  string_to_double (const std::string &s)
+  {
+    std::istringstream ss(s);
+
+#ifdef HAVE_STD_NUMERIC_LIMITS
+    static const double max_double = std::numeric_limits<double>::max();
+#else
+    static const double max_double = DBL_MAX;
+#endif
+
+    double i = max_double;
+    ss >> i;
+
+                                     // check for errors
+    AssertThrow (i != max_double, ExcCantConvertString (s));
+
+//TODO: The test for errors above doesn't work, as can easily be
+//verified. furthermore, it doesn't catch cases like when calling
+//string_to_int("1.23.4") since it just reads in however much it can, without
+//realizing that there is more
+
+    return i;
+  }
+
+
+
+  std::vector<double>
+  string_to_double (const std::vector<std::string> &s)
+  {
+    std::vector<double> tmp (s.size());
+    for (unsigned int i=0; i<s.size(); ++i)
+      tmp[i] = string_to_double (s[i]);
+    return tmp;
+  }
+
+
+
   std::vector<std::string>
   split_string_list (const std::string &s,
                      const char         delimiter)
@@ -394,103 +438,9 @@ namespace Utilities
   }
 
 
-
-
-  namespace System
+  namespace MPI
   {
-#if defined(__linux__)
-
-    double get_cpu_load ()
-    {
-      std::ifstream cpuinfo;
-      cpuinfo.open("/proc/loadavg");
-
-      AssertThrow(cpuinfo, ExcIO());
-
-      double load;
-      cpuinfo >> load;
-
-      return load;
-    }
-
-#else
-
-    double get_cpu_load ()
-    {
-      return 0.;
-    }
-
-#endif
-
-
-    
-    void get_memory_stats (MemoryStats & stats)
-    {
-      stats.VmPeak = stats.VmSize = stats.VmHWM = stats.VmRSS = 0;
-
-				       // parsing /proc/self/stat would be a
-				       // lot easier, but it does not contain
-				       // VmHWM, so we use /status instead.
-#if defined(__linux__)  
-      std::ifstream file("/proc/self/status");
-      std::string line;
-      std::string name;
-      while (!file.eof())
-	{
-	  file >> name;
-	  if (name == "VmPeak:")
-	    file >> stats.VmPeak;
-	  else if (name == "VmSize:")
-	    file >> stats.VmSize;
-	  else if (name == "VmHWM:")
-	    file >> stats.VmHWM;
-	  else if (name == "VmRSS:")
-	    {
-	      file >> stats.VmRSS;
-	      break; //this is always the last entry
-	    }
-	  
-	  getline(file, line);
-	}
-#endif
-    }
-    
-    
-
-    std::string get_hostname ()
-    {
-      const unsigned int N=1024;
-      char hostname[N];
-      gethostname (&(hostname[0]), N-1);
-      return hostname;
-    }
-
-
-
-    std::string get_time ()
-    {
-      std::time_t  time1= std::time (0);
-      std::tm     *time = std::localtime(&time1);
-
-      std::ostringstream o;
-      o << time->tm_hour << ":"
-        << (time->tm_min < 10 ? "0" : "") << time->tm_min << ":"
-        << (time->tm_sec < 10 ? "0" : "") << time->tm_sec;
-
-      return o.str();
-    }
-
-
 #ifdef DEAL_II_COMPILER_SUPPORTS_MPI
-
-    bool job_supports_mpi ()
-    {
-      int MPI_has_been_started = 0;
-      MPI_Initialized(&MPI_has_been_started);
-
-      return (MPI_has_been_started > 0);
-    }
-
                                 // Unfortunately, we have to work
                                 // around an oddity in the way PETSc
                                 // and some gcc versions interact. If
@@ -524,7 +474,7 @@ namespace Utilities
                                 // ``void'' -- not beautiful, but
                                 // helps calming down unwarranted
                                 // compiler warnings...
-    unsigned int get_n_mpi_processes (const MPI_Comm &mpi_communicator)
+    unsigned int n_mpi_processes (const MPI_Comm &mpi_communicator)
     {
       int n_jobs=1;
       (void) MPI_Comm_size (mpi_communicator, &n_jobs);
@@ -533,7 +483,7 @@ namespace Utilities
     }
 
 
-    unsigned int get_this_mpi_process (const MPI_Comm &mpi_communicator)
+    unsigned int this_mpi_process (const MPI_Comm &mpi_communicator)
     {
       int rank=0;
       (void) MPI_Comm_rank (mpi_communicator, &rank);
@@ -550,92 +500,12 @@ namespace Utilities
     }
 
 
-    namespace 
-    {
-				       // custom MIP_Op for
-				       // calculate_collective_mpi_min_max_avg
-      void max_reduce ( const void * in_lhs_,
-			void * inout_rhs_,
-			int * len,
-			MPI_Datatype * )
-      {
-	const MinMaxAvg * in_lhs = static_cast<const MinMaxAvg*>(in_lhs_);
-	MinMaxAvg * inout_rhs = static_cast<MinMaxAvg*>(inout_rhs_);
-	
-	Assert(*len==1, ExcInternalError());
-	
-	inout_rhs->sum += in_lhs->sum;
-	if (inout_rhs->min>in_lhs->min)
-	  {
-	    inout_rhs->min = in_lhs->min;
-	    inout_rhs->min_index = in_lhs->min_index;
-	  }
-	else if (inout_rhs->min == in_lhs->min)
-	  { // choose lower cpu index when tied to make operator cumutative
-	    if (inout_rhs->min_index > in_lhs->min_index)
-	      inout_rhs->min_index = in_lhs->min_index;
-	  }
-	
-	if (inout_rhs->max < in_lhs->max)
-	  {
-	  inout_rhs->max = in_lhs->max;
-	  inout_rhs->max_index = in_lhs->max_index;
-	  }
-	else if (inout_rhs->max == in_lhs->max)
-	  { // choose lower cpu index when tied to make operator cumutative
-	    if (inout_rhs->max_index > in_lhs->max_index)
-	      inout_rhs->max_index = in_lhs->max_index;
-	  }
-      }
-    }
-
-
-    
-    void calculate_collective_mpi_min_max_avg(const MPI_Comm &mpi_communicator,
-					      double my_value,
-					      MinMaxAvg & result)
-    {
-      const unsigned int my_id
-	= dealii::Utilities::System::get_this_mpi_process(mpi_communicator);
-      const unsigned int numproc
-	= dealii::Utilities::System::get_n_mpi_processes(mpi_communicator);
-      
-      MPI_Op op;
-      int ierr = MPI_Op_create((MPI_User_function *)&max_reduce, true, &op);
-      AssertThrow(ierr == MPI_SUCCESS, ExcInternalError());
-      
-      MinMaxAvg in;
-      in.sum = in.min = in.max = my_value;
-      in.min_index = in.max_index = my_id;
-      
-      MPI_Datatype type;
-      int lengths[]={3,2};
-      MPI_Aint displacements[]={0,offsetof(MinMaxAvg, min_index)};
-      MPI_Datatype types[]={MPI_DOUBLE, MPI_INT};
-      
-      ierr = MPI_Type_struct(2, lengths, displacements, types, &type);
-      AssertThrow(ierr == MPI_SUCCESS, ExcInternalError());
-      
-      ierr = MPI_Type_commit(&type);
-      ierr = MPI_Allreduce ( &in, &result, 1, type, op, mpi_communicator );
-      AssertThrow(ierr == MPI_SUCCESS, ExcInternalError());
-	  
-      ierr = MPI_Type_free (&type);
-      AssertThrow(ierr == MPI_SUCCESS, ExcInternalError());
-	  
-      ierr = MPI_Op_free(&op);
-      AssertThrow(ierr == MPI_SUCCESS, ExcInternalError());
-
-      result.avg = result.sum / numproc;
-    }
-
-    
     std::vector<unsigned int>
     compute_point_to_point_communication_pattern (const MPI_Comm & mpi_comm,
 						  const std::vector<unsigned int> & destinations)
     {
-      unsigned int myid = Utilities::System::get_this_mpi_process(mpi_comm);
-      unsigned int n_procs = Utilities::System::get_n_mpi_processes(mpi_comm);
+      unsigned int myid = Utilities::MPI::this_mpi_process(mpi_comm);
+      unsigned int n_procs = Utilities::MPI::n_mpi_processes(mpi_comm);
 
       for (unsigned int i=0; i<destinations.size(); ++i)
 	{
@@ -650,11 +520,8 @@ namespace Utilities
 				       // communicate the maximal
 				       // number of destinations they
 				       // have
-      unsigned int my_n_destinations = destinations.size();
-      unsigned int max_n_destinations = 0;
-
-      MPI_Allreduce (&my_n_destinations, &max_n_destinations, 1, MPI_UNSIGNED,
-		    MPI_MAX, mpi_comm);
+      const unsigned int max_n_destinations
+	= Utilities::MPI::max (destinations.size(), mpi_comm);
 
 				       // now that we know the number
 				       // of data packets every
@@ -698,49 +565,134 @@ namespace Utilities
       return origins;
     }
 
-#else
 
-    bool job_supports_mpi ()
+    namespace
     {
-      return false;
+				       // custom MIP_Op for
+				       // calculate_collective_mpi_min_max_avg
+      void max_reduce ( const void * in_lhs_,
+			void * inout_rhs_,
+			int * len,
+			MPI_Datatype * )
+      {
+	const MinMaxAvg * in_lhs = static_cast<const MinMaxAvg*>(in_lhs_);
+	MinMaxAvg * inout_rhs = static_cast<MinMaxAvg*>(inout_rhs_);
+
+	Assert(*len==1, ExcInternalError());
+
+	inout_rhs->sum += in_lhs->sum;
+	if (inout_rhs->min>in_lhs->min)
+	  {
+	    inout_rhs->min = in_lhs->min;
+	    inout_rhs->min_index = in_lhs->min_index;
+	  }
+	else if (inout_rhs->min == in_lhs->min)
+	  { // choose lower cpu index when tied to make operator cumutative
+	    if (inout_rhs->min_index > in_lhs->min_index)
+	      inout_rhs->min_index = in_lhs->min_index;
+	  }
+
+	if (inout_rhs->max < in_lhs->max)
+	  {
+	  inout_rhs->max = in_lhs->max;
+	  inout_rhs->max_index = in_lhs->max_index;
+	  }
+	else if (inout_rhs->max == in_lhs->max)
+	  { // choose lower cpu index when tied to make operator cumutative
+	    if (inout_rhs->max_index > in_lhs->max_index)
+	      inout_rhs->max_index = in_lhs->max_index;
+	  }
+      }
     }
 
 
 
-    unsigned int get_n_mpi_processes (const MPI_Comm &)
+    MinMaxAvg
+    min_max_avg(const double my_value,
+		const MPI_Comm &mpi_communicator)
+    {
+      MinMaxAvg result;
+
+      const unsigned int my_id
+	= dealii::Utilities::MPI::this_mpi_process(mpi_communicator);
+      const unsigned int numproc
+	= dealii::Utilities::MPI::n_mpi_processes(mpi_communicator);
+
+      MPI_Op op;
+      int ierr = MPI_Op_create((MPI_User_function *)&max_reduce, true, &op);
+      AssertThrow(ierr == MPI_SUCCESS, ExcInternalError());
+
+      MinMaxAvg in;
+      in.sum = in.min = in.max = my_value;
+      in.min_index = in.max_index = my_id;
+
+      MPI_Datatype type;
+      int lengths[]={3,2};
+      MPI_Aint displacements[]={0,offsetof(MinMaxAvg, min_index)};
+      MPI_Datatype types[]={MPI_DOUBLE, MPI_INT};
+
+      ierr = MPI_Type_struct(2, lengths, displacements, types, &type);
+      AssertThrow(ierr == MPI_SUCCESS, ExcInternalError());
+
+      ierr = MPI_Type_commit(&type);
+      ierr = MPI_Allreduce (&in, &result, 1, type, op, mpi_communicator);
+      AssertThrow(ierr == MPI_SUCCESS, ExcInternalError());
+
+      ierr = MPI_Type_free (&type);
+      AssertThrow(ierr == MPI_SUCCESS, ExcInternalError());
+
+      ierr = MPI_Op_free(&op);
+      AssertThrow(ierr == MPI_SUCCESS, ExcInternalError());
+
+      result.avg = result.sum / numproc;
+
+      return result;
+    }
+
+#else
+
+    unsigned int n_mpi_processes (const MPI_Comm &)
     {
       return 1;
     }
 
 
 
-    unsigned int get_this_mpi_process (const MPI_Comm &)
+    unsigned int this_mpi_process (const MPI_Comm &)
     {
       return 0;
     }
 
-    
-    void calculate_collective_mpi_min_max_avg(const MPI_Comm &,
-					      double my_value,
-					      MinMaxAvg & result)
-    {
-      result.sum = my_value;
-      result.avg = my_value;
-      result.min = my_value;
-      result.max = my_value;
-      result.min_index = 0;
-      result.max_index = 0;      
-    }
-    
 
     MPI_Comm duplicate_communicator (const MPI_Comm &mpi_communicator)
     {
       return mpi_communicator;
     }
+
+
+
+
+    MinMaxAvg
+    min_max_avg(const double my_value,
+		const MPI_Comm &)
+    {
+      MinMaxAvg result;
+
+      result.sum = my_value;
+      result.avg = my_value;
+      result.min = my_value;
+      result.max = my_value;
+      result.min_index = 0;
+      result.max_index = 0;
+
+      return result;
+    }
+
 #endif
 
 
 
+
     MPI_InitFinalize::MPI_InitFinalize (int    &argc,
 					char** &argv)
 		    :
@@ -807,9 +759,20 @@ namespace Utilities
 
       int MPI_has_been_started = 0;
       MPI_Initialized(&MPI_has_been_started);
-      if (program_uses_mpi() == true && owns_mpi == true &&
+      if (Utilities::System::program_uses_mpi() == true && owns_mpi == true &&
 	  MPI_has_been_started != 0)
-	mpi_err = MPI_Finalize();
+	{
+	  if (std::uncaught_exception())
+	    {
+	      std::cerr << "ERROR: Uncaught exception in MPI_InitFinalize on proc "
+			<< this_mpi_process(MPI_COMM_WORLD)
+			<< ". Skipping MPI_Finalize() to avoid a deadlock."
+			<< std::endl;
+	    }
+	  else
+	    mpi_err = MPI_Finalize();
+	}
+
 
       AssertThrow (mpi_err == 0,
 		   ExcMessage ("An error occurred while calling MPI_Finalize()"));
@@ -817,6 +780,99 @@ namespace Utilities
     }
 
 
+  }
+
+
+  namespace System
+  {
+#if defined(__linux__)
+
+    double get_cpu_load ()
+    {
+      std::ifstream cpuinfo;
+      cpuinfo.open("/proc/loadavg");
+
+      AssertThrow(cpuinfo, ExcIO());
+
+      double load;
+      cpuinfo >> load;
+
+      return load;
+    }
+
+#else
+
+    double get_cpu_load ()
+    {
+      return 0.;
+    }
+
+#endif
+
+
+
+    void get_memory_stats (MemoryStats & stats)
+    {
+      stats.VmPeak = stats.VmSize = stats.VmHWM = stats.VmRSS = 0;
+
+				       // parsing /proc/self/stat would be a
+				       // lot easier, but it does not contain
+				       // VmHWM, so we use /status instead.
+#if defined(__linux__)
+      std::ifstream file("/proc/self/status");
+      std::string line;
+      std::string name;
+      while (!file.eof())
+	{
+	  file >> name;
+	  if (name == "VmPeak:")
+	    file >> stats.VmPeak;
+	  else if (name == "VmSize:")
+	    file >> stats.VmSize;
+	  else if (name == "VmHWM:")
+	    file >> stats.VmHWM;
+	  else if (name == "VmRSS:")
+	    {
+	      file >> stats.VmRSS;
+	      break; //this is always the last entry
+	    }
+
+	  getline(file, line);
+	}
+#endif
+    }
+
+
+
+    std::string get_hostname ()
+    {
+      const unsigned int N=1024;
+      char hostname[N];
+      gethostname (&(hostname[0]), N-1);
+      return hostname;
+    }
+
+
+
+    std::string get_time ()
+    {
+      std::time_t  time1= std::time (0);
+      std::tm     *time = std::localtime(&time1);
+
+      std::ostringstream o;
+      o << time->tm_hour << ":"
+        << (time->tm_min < 10 ? "0" : "") << time->tm_min << ":"
+        << (time->tm_sec < 10 ? "0" : "") << time->tm_sec;
+
+      return o.str();
+    }
+
+
+    bool job_supports_mpi ()
+    {
+      return program_uses_mpi();
+    }
+
 
     bool
     program_uses_mpi ()
@@ -830,6 +886,29 @@ namespace Utilities
       return false;
 #endif
     }
+
+
+    unsigned int get_n_mpi_processes (const MPI_Comm &mpi_communicator)
+    {
+      return MPI::n_mpi_processes (mpi_communicator);
+    }
+
+    unsigned int get_this_mpi_process (const MPI_Comm &mpi_communicator)
+    {
+      return MPI::this_mpi_process (mpi_communicator);
+    }
+
+
+
+    void calculate_collective_mpi_min_max_avg(const MPI_Comm &mpi_communicator,
+					      double my_value,
+					      MinMaxAvg & result)
+    {
+      result = Utilities::MPI::min_max_avg (my_value,
+					    mpi_communicator);
+    }
+
+
   }
 
 
