@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: grid_refinement.cc 22676 2010-11-10 15:21:40Z kronbichler $
+//    $Id: grid_refinement.cc 24418 2011-09-26 12:52:19Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
@@ -13,25 +13,26 @@
 
 
 
-#include <base/utilities.h>
-#include <lac/vector.h>
-#include <lac/block_vector.h>
-#include <lac/petsc_vector.h>
-#include <lac/petsc_block_vector.h>
-#include <lac/trilinos_vector.h>
-#include <lac/trilinos_block_vector.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/petsc_vector.h>
+#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/trilinos_vector.h>
+#include <deal.II/lac/trilinos_block_vector.h>
 
 #ifdef DEAL_II_USE_P4EST
 
-#include <grid/grid_refinement.h>
-#include <grid/tria_accessor.h>
-#include <grid/tria_iterator.h>
-#include <grid/tria.h>
+#include <deal.II/grid/grid_refinement.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/grid/tria.h>
 
-#include <distributed/grid_refinement.h>
+#include <deal.II/distributed/grid_refinement.h>
 
 #include <numeric>
 #include <algorithm>
+#include <limits>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -44,7 +45,11 @@ namespace
   number
   max_element (const Vector<number> &criteria)
   {
-    return *std::max_element(criteria.begin(), criteria.end());
+    return (criteria.size()>0)
+	?
+	(*std::max_element(criteria.begin(), criteria.end()))
+	:
+	std::numeric_limits<number>::min();
   }
 
 
@@ -54,7 +59,11 @@ namespace
   number
   min_element (const Vector<number> &criteria)
   {
-    return *std::min_element(criteria.begin(), criteria.end());
+    return (criteria.size()>0)
+	?
+	(*std::min_element(criteria.begin(), criteria.end()))
+	:
+	std::numeric_limits<number>::max();
   }
 
 
@@ -91,7 +100,7 @@ namespace
 
 				     // make sure only processor zero
 				     // got something
-    if (Utilities::System::get_this_mpi_process (mpi_communicator) != 0)
+    if (Utilities::MPI::this_mpi_process (mpi_communicator) != 0)
       Assert ((result[0] == 0) && (result[1] == 0),
 	      ExcInternalError());
 
@@ -125,7 +134,7 @@ namespace
 
 				     // make sure only processor zero
 				     // got something
-    if (Utilities::System::get_this_mpi_process (mpi_communicator) != 0)
+    if (Utilities::MPI::this_mpi_process (mpi_communicator) != 0)
       Assert (result == 0, ExcInternalError());
 
     return result;
@@ -578,7 +587,7 @@ namespace parallel
 					 // from here on designate a
 					 // master and slaves
 	double top_threshold, bottom_threshold;
-	if (Utilities::System::get_this_mpi_process (mpi_communicator) == 0)
+	if (Utilities::MPI::this_mpi_process (mpi_communicator) == 0)
 	  {
 					     // this is the master
 					     // processor
@@ -692,7 +701,7 @@ namespace parallel
 					 // from here on designate a
 					 // master and slaves
 	double top_threshold, bottom_threshold;
-	if (Utilities::System::get_this_mpi_process (mpi_communicator) == 0)
+	if (Utilities::MPI::this_mpi_process (mpi_communicator) == 0)
 	  {
 					     // this is the master
 					     // processor
