@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: tria.cc 22727 2010-11-14 23:03:25Z bangerth $
+//    $Id: tria.cc 24418 2011-09-26 12:52:19Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,16 +11,16 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/utilities.h>
-#include <base/memory_consumption.h>
-#include <base/logstream.h>
-#include <lac/sparsity_tools.h>
-#include <lac/sparsity_pattern.h>
-#include <grid/tria.h>
-#include <grid/tria_accessor.h>
-#include <grid/tria_iterator.h>
-#include <distributed/tria.h>
-#include <grid/grid_tools.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/base/logstream.h>
+#include <deal.II/lac/sparsity_tools.h>
+#include <deal.II/lac/sparsity_pattern.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/distributed/tria.h>
+#include <deal.II/grid/grid_tools.h>
 
 #ifdef DEAL_II_USE_P4EST
 #  include <p4est_bits.h>
@@ -149,7 +149,7 @@ namespace internal
 	static
 	types<2>::forest * (&load) (const char *filename,
 				    MPI_Comm mpicomm,
-				    std::size_t data_size,
+				    size_t data_size,
 				    int load_data,
 				    void *user_pointer,
 				    types<2>::connectivity ** p4est);
@@ -1374,7 +1374,7 @@ namespace
 	      break;
 	    }
 	    default:
-		  throw ExcInternalError();
+		  AssertThrow (false, ExcInternalError());
 	  }
       }
   }
@@ -1803,9 +1803,9 @@ namespace parallel
 		    dealii::Triangulation<dim,spacedim>
 		    (smooth_grid,
 		     false),
-		    mpi_communicator (Utilities::System::
+		    mpi_communicator (Utilities::MPI::
 				      duplicate_communicator(mpi_communicator)),
-		    my_subdomain (Utilities::System::get_this_mpi_process (this->mpi_communicator)),
+		    my_subdomain (Utilities::MPI::this_mpi_process (this->mpi_communicator)),
 		    triangulation_has_content (false),
 		    connectivity (0),
 		    parallel_forest (0),
@@ -1816,10 +1816,10 @@ namespace parallel
       // initialize p4est. do this in a separate function since it has
       // to happen only once, even if we have triangulation objects
       // for several different space dimensions
-      internal::p4est::InitFinalize::do_initialize ();
+      dealii::internal::p4est::InitFinalize::do_initialize ();
 
       number_cache.n_locally_owned_active_cells
-	.resize (Utilities::System::get_n_mpi_processes (mpi_communicator));
+	.resize (Utilities::MPI::n_mpi_processes (mpi_communicator));
     }
 
 
@@ -1873,7 +1873,7 @@ namespace parallel
 
       setup_coarse_cell_to_p4est_tree_permutation ();
 
-      copy_new_triangulation_to_p4est (internal::int2type<dim>());
+      copy_new_triangulation_to_p4est (dealii::internal::int2type<dim>());
 
       try
 	{
@@ -1901,13 +1901,13 @@ namespace parallel
 
       if (parallel_forest != 0)
 	{
-	  internal::p4est::functions<dim>::destroy (parallel_forest);
+	  dealii::internal::p4est::functions<dim>::destroy (parallel_forest);
 	  parallel_forest = 0;
 	}
 
       if (connectivity != 0)
 	{
-	  internal::p4est::functions<dim>::connectivity_destroy (connectivity);
+	  dealii::internal::p4est::functions<dim>::connectivity_destroy (connectivity);
 	  connectivity = 0;
 	}
 
@@ -1942,7 +1942,7 @@ namespace parallel
     {
       Assert (parallel_forest != 0,
 	      ExcMessage ("Can't produce output when no forest is created yet."));
-      internal::p4est::functions<dim>::
+      dealii::internal::p4est::functions<dim>::
 	vtk_write_file (parallel_forest, 0, file_basename);
     }
 
@@ -1954,7 +1954,7 @@ namespace parallel
     {
       Assert (parallel_forest != 0,
 	      ExcMessage ("Can't produce a check sum when no forest is created yet."));
-      return internal::p4est::functions<dim>::checksum (parallel_forest);
+      return dealii::internal::p4est::functions<dim>::checksum (parallel_forest);
     }
 
 
@@ -1977,7 +1977,7 @@ namespace parallel
 
     template <>
     void
-    Triangulation<2,2>::copy_new_triangulation_to_p4est (internal::int2type<2>)
+    Triangulation<2,2>::copy_new_triangulation_to_p4est (dealii::internal::int2type<2>)
     {
       const unsigned int dim = 2, spacedim = 2;
       Assert (this->n_cells(0) > 0, ExcInternalError());
@@ -1998,7 +1998,7 @@ namespace parallel
       get_vertex_to_cell_mappings (*this,
 				   vertex_touch_count,
 				   vertex_to_cell);
-      const internal::p4est::types<2>::locidx
+      const dealii::internal::p4est::types<2>::locidx
 	num_vtt = std::accumulate (vertex_touch_count.begin(),
 				   vertex_touch_count.end(),
 				   0);
@@ -2018,7 +2018,7 @@ namespace parallel
 	;
 
       connectivity
-	= internal::p4est::functions<2>::
+	= dealii::internal::p4est::functions<2>::
 	connectivity_new ((set_vertex_info == true ? this->n_vertices() : 0),
 			  this->n_cells(0),
 			  this->n_vertices(),
@@ -2037,7 +2037,7 @@ namespace parallel
 				       // now create a forest out of the
 				       // connectivity data structure
       parallel_forest
-	= internal::p4est::functions<2>::
+	= dealii::internal::p4est::functions<2>::
 	new_forest (mpi_communicator,
 		    connectivity,
 		    /* minimum initial number of quadrants per tree */ 0,
@@ -2055,7 +2055,7 @@ namespace parallel
 				// dim template argument, but let spacedim open
     template <>
     void
-    Triangulation<2,3>::copy_new_triangulation_to_p4est (internal::int2type<2>)
+    Triangulation<2,3>::copy_new_triangulation_to_p4est (dealii::internal::int2type<2>)
     {
       const unsigned int dim = 2, spacedim = 3;
       Assert (this->n_cells(0) > 0, ExcInternalError());
@@ -2076,7 +2076,7 @@ namespace parallel
       get_vertex_to_cell_mappings (*this,
 				   vertex_touch_count,
 				   vertex_to_cell);
-      const internal::p4est::types<2>::locidx
+      const dealii::internal::p4est::types<2>::locidx
 	num_vtt = std::accumulate (vertex_touch_count.begin(),
 				   vertex_touch_count.end(),
 				   0);
@@ -2096,7 +2096,7 @@ namespace parallel
 	;
 
       connectivity
-	= internal::p4est::functions<2>::
+	= dealii::internal::p4est::functions<2>::
 	connectivity_new ((set_vertex_info == true ? this->n_vertices() : 0),
 			  this->n_cells(0),
 			  this->n_vertices(),
@@ -2115,7 +2115,7 @@ namespace parallel
 				       // now create a forest out of the
 				       // connectivity data structure
       parallel_forest
-	= internal::p4est::functions<2>::
+	= dealii::internal::p4est::functions<2>::
 	new_forest (mpi_communicator,
 		    connectivity,
 		    /* minimum initial number of quadrants per tree */ 0,
@@ -2130,7 +2130,7 @@ namespace parallel
 
     template <>
     void
-    Triangulation<3,3>::copy_new_triangulation_to_p4est (internal::int2type<3>)
+    Triangulation<3,3>::copy_new_triangulation_to_p4est (dealii::internal::int2type<3>)
     {
       const int dim = 3, spacedim = 3;
       Assert (this->n_cells(0) > 0, ExcInternalError());
@@ -2151,7 +2151,7 @@ namespace parallel
       get_vertex_to_cell_mappings (*this,
 				   vertex_touch_count,
 				   vertex_to_cell);
-      const internal::p4est::types<2>::locidx
+      const dealii::internal::p4est::types<2>::locidx
 	num_vtt = std::accumulate (vertex_touch_count.begin(),
 				   vertex_touch_count.end(),
 				   0);
@@ -2165,7 +2165,7 @@ namespace parallel
       get_edge_to_cell_mappings (*this,
 				 edge_touch_count,
 				 edge_to_cell);
-      const internal::p4est::types<2>::locidx
+      const dealii::internal::p4est::types<2>::locidx
 	num_ett = std::accumulate (edge_touch_count.begin(),
 				   edge_touch_count.end(),
 				   0);
@@ -2181,7 +2181,7 @@ namespace parallel
 	;
 
       connectivity
-	= internal::p4est::functions<3>::
+	= dealii::internal::p4est::functions<3>::
 	connectivity_new ((set_vertex_info == true ? this->n_vertices() : 0),
 			  this->n_cells(0),
 			  this->n_active_lines(),
@@ -2271,7 +2271,7 @@ namespace parallel
 				       // now create a forest out of the
 				       // connectivity data structure
       parallel_forest
-	= internal::p4est::functions<3>::
+	= dealii::internal::p4est::functions<3>::
 	new_forest (mpi_communicator,
 		    connectivity,
 		    /* minimum initial number of quadrants per tree */ 0,
@@ -2303,14 +2303,14 @@ namespace parallel
       bool mesh_changed = false;
 
 				       // query p4est for the ghost cells
-      typename internal::p4est::types<dim>::ghost * ghostlayer;
-      ghostlayer = internal::p4est::functions<dim>::ghost_new (parallel_forest,
+      typename dealii::internal::p4est::types<dim>::ghost * ghostlayer;
+      ghostlayer = dealii::internal::p4est::functions<dim>::ghost_new (parallel_forest,
 							       (dim == 2
 								?
-								typename internal::p4est::types<dim>::
+								typename dealii::internal::p4est::types<dim>::
 								balance_type(P4EST_BALANCE_CORNER)
 								:
-								typename internal::p4est::types<dim>::
+								typename dealii::internal::p4est::types<dim>::
 								balance_type(P8EST_BALANCE_CORNER)));
 
       Assert (ghostlayer, ExcInternalError());
@@ -2356,11 +2356,11 @@ namespace parallel
 						   // this cell.
 
 
-		  typename internal::p4est::types<dim>::quadrant p4est_coarse_cell;
-		  typename internal::p4est::types<dim>::tree *tree =
+		  typename dealii::internal::p4est::types<dim>::quadrant p4est_coarse_cell;
+		  typename dealii::internal::p4est::types<dim>::tree *tree =
 		    init_tree(cell->index());
 
-		  internal::p4est::init_coarse_quadrant<dim>(p4est_coarse_cell);
+		  dealii::internal::p4est::init_coarse_quadrant<dim>(p4est_coarse_cell);
 
 		  match_tree_recursively<dim,spacedim> (*tree, cell,
 							p4est_coarse_cell,
@@ -2374,7 +2374,7 @@ namespace parallel
 					   // iterate over every ghostquadrant,
 					   // find corresponding deal coarsecell
 					   // and recurse.
-	  typename internal::p4est::types<dim>::quadrant * quadr;
+	  typename dealii::internal::p4est::types<dim>::quadrant * quadr;
 	  unsigned int ghost_owner=0;
 
 	  for (unsigned int g_idx=0;g_idx<ghostlayer->ghosts.elem_count;++g_idx)
@@ -2382,7 +2382,7 @@ namespace parallel
 	      while (g_idx >= (unsigned int)ghostlayer->proc_offsets[ghost_owner+1])
 		++ghost_owner;
 
-	      quadr = static_cast<typename internal::p4est::types<dim>::quadrant *>
+	      quadr = static_cast<typename dealii::internal::p4est::types<dim>::quadrant *>
 		      ( sc_array_index(&ghostlayer->ghosts, g_idx) );
 
 	      unsigned int coarse_cell_index =
@@ -2391,8 +2391,8 @@ namespace parallel
 	      const typename Triangulation<dim,spacedim>::cell_iterator
 		cell (this, 0U, coarse_cell_index);
 
-	      typename internal::p4est::types<dim>::quadrant p4est_coarse_cell;
-	      internal::p4est::init_coarse_quadrant<dim> (p4est_coarse_cell);
+	      typename dealii::internal::p4est::types<dim>::quadrant p4est_coarse_cell;
+	      dealii::internal::p4est::init_coarse_quadrant<dim> (p4est_coarse_cell);
 
 	      match_quadrant_recursively<dim,spacedim> (p4est_coarse_cell, cell, *quadr,
 							*parallel_forest, ghost_owner);
@@ -2474,7 +2474,7 @@ namespace parallel
       {
 	const unsigned int total_local_cells = this->n_active_cells();
 
-	if (Utilities::System::get_n_mpi_processes (mpi_communicator) == 1)
+	if (Utilities::MPI::n_mpi_processes (mpi_communicator) == 1)
 	  Assert (static_cast<unsigned int>(parallel_forest->local_num_quadrants) ==
 		  total_local_cells,
 		  ExcInternalError())
@@ -2499,7 +2499,7 @@ namespace parallel
 
       }
 
-      internal::p4est::functions<dim>::ghost_destroy (ghostlayer);
+      dealii::internal::p4est::functions<dim>::ghost_destroy (ghostlayer);
 
       this->smooth_grid = save_smooth;
     }
@@ -2570,11 +2570,11 @@ namespace parallel
 	      ExcInternalError());
       parallel_forest->user_pointer = &refine_and_coarsen_list;
 
-      internal::p4est::functions<dim>::
+      dealii::internal::p4est::functions<dim>::
 	refine (parallel_forest, /* refine_recursive */ false,
 		&RefineAndCoarsenList<dim,spacedim>::refine_callback,
 		/*init_callback=*/NULL);
-      internal::p4est::functions<dim>::
+      dealii::internal::p4est::functions<dim>::
 	coarsen (parallel_forest, /* coarsen_recursive */ false,
 		 &RefineAndCoarsenList<dim,spacedim>::coarsen_callback,
 		 /*init_callback=*/NULL);
@@ -2588,15 +2588,15 @@ namespace parallel
       parallel_forest->user_pointer = this;
 
 				       // enforce 2:1 hanging node condition
-      internal::p4est::functions<dim>::
+      dealii::internal::p4est::functions<dim>::
 	balance (parallel_forest,
 						  /* face and corner balance */
 		 (dim == 2
 		  ?
-		  typename internal::p4est::types<dim>::
+		  typename dealii::internal::p4est::types<dim>::
 		  balance_type(P4EST_BALANCE_FULL)
 		  :
-		  typename internal::p4est::types<dim>::
+		  typename dealii::internal::p4est::types<dim>::
 		  balance_type(P8EST_BALANCE_FULL)),
 		 /*init_callback=*/NULL);
 
@@ -2609,7 +2609,7 @@ namespace parallel
 
 				       // partition the new mesh between
 				       // all processors
-      internal::p4est::functions<dim>::
+      dealii::internal::p4est::functions<dim>::
 	partition (parallel_forest,
                    /* prepare coarsening */ 1,
 		   /* weight_callback */ NULL);
@@ -2655,7 +2655,7 @@ namespace parallel
     {
       Assert (number_cache.n_locally_owned_active_cells.size()
 	      ==
-	      Utilities::System::get_n_mpi_processes (mpi_communicator),
+	      Utilities::MPI::n_mpi_processes (mpi_communicator),
 	      ExcInternalError());
 
       std::fill (number_cache.n_locally_owned_active_cells.begin(),
@@ -2726,7 +2726,7 @@ namespace parallel
     template <int dim, int spacedim>
     unsigned int
     Triangulation<dim,spacedim>::
-    register_data_attach (const size_t size,
+    register_data_attach (const std::size_t size,
 			  const std_cxx1x::function<void(const cell_iterator&,
 							 const CellStatus,
 							 void*)> & pack_callback)
@@ -2769,11 +2769,11 @@ namespace parallel
 		  == false)
 	    continue;
 
-	  typename internal::p4est::types<dim>::quadrant p4est_coarse_cell;
-	  typename internal::p4est::types<dim>::tree *tree =
+	  typename dealii::internal::p4est::types<dim>::quadrant p4est_coarse_cell;
+	  typename dealii::internal::p4est::types<dim>::tree *tree =
 	    init_tree(cell->index());
 
-	  internal::p4est::init_coarse_quadrant<dim> (p4est_coarse_cell);
+	  dealii::internal::p4est::init_coarse_quadrant<dim> (p4est_coarse_cell);
 
 					   // parent_cell is not correct here,
 					   // but is only used in a refined
@@ -2796,13 +2796,22 @@ namespace parallel
 
 					   // and release the data
 	  void * userptr = parallel_forest->user_pointer;
-	  internal::p4est::functions<dim>::reset_data (parallel_forest, 0, NULL, NULL);
+	  dealii::internal::p4est::functions<dim>::reset_data (parallel_forest, 0, NULL, NULL);
 	  parallel_forest->user_pointer = userptr;
 	}
     }
 
 
 
+	template <int dim, int spacedim>
+	const std::vector<unsigned int> &
+	Triangulation<dim,spacedim>::get_p4est_tree_to_coarse_cell_permutation() const
+	{
+	  return p4est_tree_to_coarse_cell_permutation;
+	}
+
+
+
     template <int dim, int spacedim>
     MPI_Comm
     Triangulation<dim,spacedim>::get_communicator () const
@@ -2813,10 +2822,10 @@ namespace parallel
 
 
     template <int dim, int spacedim>
-    unsigned int
+    std::size_t
     Triangulation<dim,spacedim>::memory_consumption () const
     {
-      unsigned int mem=
+      std::size_t mem=
 	this->dealii::Triangulation<dim,spacedim>::memory_consumption()
 	+ MemoryConsumption::memory_consumption(mpi_communicator)
 	+ MemoryConsumption::memory_consumption(my_subdomain)
@@ -2831,8 +2840,7 @@ namespace parallel
 //	+ MemoryConsumption::memory_consumption(attached_data_pack_callbacks) //TODO[TH]: how?
 	+ MemoryConsumption::memory_consumption(coarse_cell_to_p4est_tree_permutation)
 	+ MemoryConsumption::memory_consumption(p4est_tree_to_coarse_cell_permutation)
-	+ memory_consumption_p4est()
-	;
+	+ memory_consumption_p4est();
 
       return mem;
     }
@@ -2840,11 +2848,11 @@ namespace parallel
 
 
     template <int dim, int spacedim>
-    unsigned int
+    std::size_t
     Triangulation<dim,spacedim>::memory_consumption_p4est () const
     {
-      return internal::p4est::functions<dim>::forest_memory_used(parallel_forest)
-	+ internal::p4est::functions<dim>::connectivity_memory_used(connectivity);
+      return dealii::internal::p4est::functions<dim>::forest_memory_used(parallel_forest)
+	+ dealii::internal::p4est::functions<dim>::connectivity_memory_used(connectivity);
     }
 
 
@@ -2876,7 +2884,7 @@ namespace parallel
 
 				       // realloc user_data in p4est
       void * userptr = parallel_forest->user_pointer;
-      internal::p4est::functions<dim>::reset_data (parallel_forest,
+      dealii::internal::p4est::functions<dim>::reset_data (parallel_forest,
 						   attached_data_size+sizeof(CellStatus),
 						   NULL, NULL);
       parallel_forest->user_pointer = userptr;
@@ -2896,11 +2904,11 @@ namespace parallel
 		  == false)
 	    continue;
 
-	  typename internal::p4est::types<dim>::quadrant p4est_coarse_cell;
-	  typename internal::p4est::types<dim>::tree *tree =
+	  typename dealii::internal::p4est::types<dim>::quadrant p4est_coarse_cell;
+	  typename dealii::internal::p4est::types<dim>::tree *tree =
 	    init_tree(cell->index());
 
-	  internal::p4est::init_coarse_quadrant<dim>(p4est_coarse_cell);
+	  dealii::internal::p4est::init_coarse_quadrant<dim>(p4est_coarse_cell);
 
 	  attach_mesh_data_recursively<dim,spacedim>(*tree,
 						     cell,
@@ -2937,6 +2945,15 @@ namespace parallel
     }
 
 
+    template <>
+    unsigned int
+    Triangulation<1,1>::n_global_active_cells () const
+    {
+      Assert (false, ExcNotImplemented());
+      return 0;
+    }
+
+
     template <>
     MPI_Comm
     Triangulation<1,1>::get_communicator () const
@@ -2944,6 +2961,13 @@ namespace parallel
       return MPI_COMM_WORLD;
     }
 
+	template <>
+	const std::vector<unsigned int> &
+	Triangulation<1,1>::get_p4est_tree_to_coarse_cell_permutation() const
+	{
+	  static std::vector<unsigned int> a;
+	  return a;
+	}
 
 
     template <>
@@ -2970,6 +2994,15 @@ namespace parallel
     }
 
 
+    template <>
+    unsigned int
+    Triangulation<1,2>::n_global_active_cells () const
+    {
+      Assert (false, ExcNotImplemented());
+      return 0;
+    }
+
+
     template <>
     MPI_Comm
     Triangulation<1,2>::get_communicator () const
@@ -3008,6 +3041,17 @@ namespace parallel
       Assert (false, ExcNotImplemented());
       return 0;
     }
+
+
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+    template <int dim, int spacedim>
+    MPI_Comm
+    Triangulation<dim,spacedim>::get_communicator () const
+    {
+      Assert (false, ExcNotImplemented());
+      return MPI_COMM_WORLD;
+    }
+#endif
   }
 }
 
