@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_faces.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: dof_faces.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2006, 2010 by the deal.II authors
+//    Copyright (C) 2006, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,9 +12,9 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/exceptions.h>
-#include <base/memory_consumption.h>
-#include <dofs/dof_faces.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/dofs/dof_faces.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -23,21 +23,21 @@ namespace internal
 {
   namespace DoFHandler
   {
-    unsigned int
+    std::size_t
     DoFFaces<1>::memory_consumption () const
     {
       return 0;
     }
 
 
-    unsigned int
+    std::size_t
     DoFFaces<2>::memory_consumption () const
     {
       return MemoryConsumption::memory_consumption (lines);
     }
 
 
-    unsigned int
+    std::size_t
     DoFFaces<3>::memory_consumption () const
     {
       return (MemoryConsumption::memory_consumption (quads) +
