@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_handler.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: dof_handler.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,18 +12,18 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/memory_consumption.h>
-#include <dofs/dof_handler.h>
-#include <dofs/dof_handler_policy.h>
-#include <dofs/dof_levels.h>
-#include <dofs/dof_faces.h>
-#include <dofs/dof_accessor.h>
-#include <grid/tria_accessor.h>
-#include <grid/tria_iterator.h>
-#include <grid/tria_levels.h>
-#include <grid/tria.h>
-#include <base/geometry_info.h>
-#include <fe/fe.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/dofs/dof_handler_policy.h>
+#include <deal.II/dofs/dof_levels.h>
+#include <deal.II/dofs/dof_faces.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/grid/tria_levels.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/base/geometry_info.h>
+#include <deal.II/fe/fe.h>
 
 #include <set>
 #include <algorithm>
@@ -1630,17 +1630,17 @@ DoFHandler<dim,spacedim>::n_boundary_dofs (const std::set<unsigned char> &bounda
 
 
 template<int dim, int spacedim>
-unsigned int
+std::size_t
 DoFHandler<dim,spacedim>::memory_consumption () const
 {
-  unsigned int mem = (MemoryConsumption::memory_consumption (tria) +
-		      MemoryConsumption::memory_consumption (selected_fe) +
-		      MemoryConsumption::memory_consumption (block_info_object) +
-		      MemoryConsumption::memory_consumption (levels) +
-		      MemoryConsumption::memory_consumption (*faces) +
-		      MemoryConsumption::memory_consumption (faces) +
-		      sizeof (number_cache) +
-		      MemoryConsumption::memory_consumption (vertex_dofs));
+  std::size_t mem = (MemoryConsumption::memory_consumption (tria) +
+		     MemoryConsumption::memory_consumption (selected_fe) +
+		     MemoryConsumption::memory_consumption (block_info_object) +
+		     MemoryConsumption::memory_consumption (levels) +
+		     MemoryConsumption::memory_consumption (*faces) +
+		     MemoryConsumption::memory_consumption (faces) +
+		     sizeof (number_cache) +
+		     MemoryConsumption::memory_consumption (vertex_dofs));
   for (unsigned int i=0; i<levels.size(); ++i)
     mem += MemoryConsumption::memory_consumption (*levels[i]);
 
