@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_handler_policy.cc 23058 2010-12-21 15:30:55Z kronbichler $
+//    $Id: dof_handler_policy.cc 24435 2011-09-26 18:50:03Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,15 +12,15 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/geometry_info.h>
-#include <base/utilities.h>
-#include <base/memory_consumption.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_handler.h>
-#include <dofs/dof_accessor.h>
-#include <dofs/dof_handler_policy.h>
-#include <fe/fe.h>
+#include <deal.II/base/geometry_info.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/dofs/dof_handler_policy.h>
+#include <deal.II/fe/fe.h>
 
 #include <set>
 #include <algorithm>
@@ -226,7 +226,7 @@ namespace internal
 	  static
 	  unsigned int
 	  distribute_dofs (const unsigned int        offset,
-			   const unsigned int        subdomain_id,
+			   const types::subdomain_id_t subdomain_id,
 			   DoFHandler<dim,spacedim> &dof_handler)
 	    {
 	      const dealii::Triangulation<dim,spacedim> & tria
@@ -622,7 +622,7 @@ namespace internal
 				dofs.size() * sizeof(unsigned int));
 		    ptr += dofs.size() * sizeof(unsigned int);
 
-		    Assert (ptr == &buffer[buffer.size()],
+		    Assert (ptr == &buffer[0]+buffer.size(),
 			    ExcInternalError());
 
 		  }
@@ -993,7 +993,8 @@ namespace internal
 
 					   // complete all sends, so that we can
 					   // safely destroy the buffers.
-	  MPI_Waitall(requests.size(), &requests[0], MPI_STATUSES_IGNORE);
+	  if (requests.size() > 0)
+	    MPI_Waitall(requests.size(), &requests[0], MPI_STATUSES_IGNORE);
 
 
 #ifdef DEBUG
@@ -1004,8 +1005,8 @@ namespace internal
 	    unsigned int sent=needs_to_get_cells.size();
 	    unsigned int recv=senders.size();
 
-	    MPI_Reduce(&sent, &sum_send, 1, MPI_INT, MPI_SUM, 0, tr->get_communicator());
-	    MPI_Reduce(&recv, &sum_recv, 1, MPI_INT, MPI_SUM, 0, tr->get_communicator());
+	    MPI_Reduce(&sent, &sum_send, 1, MPI_UNSIGNED, MPI_SUM, 0, tr->get_communicator());
+	    MPI_Reduce(&recv, &sum_recv, 1, MPI_UNSIGNED, MPI_SUM, 0, tr->get_communicator());
 	    Assert(sum_send==sum_recv, ExcInternalError());
 	  }
 #endif
@@ -1067,7 +1068,7 @@ namespace internal
 	Assert (tr != 0, ExcInternalError());
 
 	const unsigned int
-	  n_cpus = Utilities::System::get_n_mpi_processes (tr->get_communicator());
+	  n_cpus = Utilities::MPI::n_mpi_processes (tr->get_communicator());
 
 					 //* 1. distribute on own
 					 //* subdomain
@@ -1127,18 +1128,18 @@ namespace internal
 	number_cache.n_locally_owned_dofs_per_processor.resize(n_cpus);
 
 	MPI_Allgather ( &number_cache.n_locally_owned_dofs,
-			1, MPI_INT,
+			1, MPI_UNSIGNED,
 			&number_cache.n_locally_owned_dofs_per_processor[0],
-			1, MPI_INT,
+			1, MPI_UNSIGNED,
 			tr->get_communicator());
 
 	const unsigned int
-	  shift = std::accumulate( &number_cache
-				   .n_locally_owned_dofs_per_processor[0],
-				   &number_cache
-				   .n_locally_owned_dofs_per_processor
-				   [tr->locally_owned_subdomain()],
-				   0 );
+	  shift = std::accumulate (number_cache
+				   .n_locally_owned_dofs_per_processor.begin(),
+				   number_cache
+				   .n_locally_owned_dofs_per_processor.begin()
+				   + tr->locally_owned_subdomain(),
+				   0);
 	for (std::vector<unsigned int>::iterator it=renumbering.begin();
 	     it!=renumbering.end(); ++it)
 	  if (*it != DoFHandler<dim,spacedim>::invalid_dof_index)
@@ -1155,11 +1156,11 @@ namespace internal
 					 // now a little bit of
 					 // housekeeping
 	number_cache.n_global_dofs
-	  = std::accumulate( &number_cache
-			     .n_locally_owned_dofs_per_processor[0],
-			     &number_cache
-			     .n_locally_owned_dofs_per_processor[n_cpus],
-			     0 );
+	  = std::accumulate (number_cache
+			     .n_locally_owned_dofs_per_processor.begin(),
+			     number_cache
+			     .n_locally_owned_dofs_per_processor.end(),
+			     0);
 
 	number_cache.locally_owned_dofs = IndexSet(number_cache.n_global_dofs);
 	number_cache.locally_owned_dofs
@@ -1501,12 +1502,11 @@ namespace internal
 	  number_cache.locally_owned_dofs.block_write(oss);
 	  std::string oss_str=oss.str();
 	  std::vector<char> my_data(oss_str.begin(), oss_str.end());
-	  unsigned int my_size = my_data.size();
+	  unsigned int my_size = oss_str.size();
 
 					   // determine maximum size of IndexSet
-	  unsigned int max_size;
-
-	  MPI_Allreduce(&my_size, &max_size, 1, MPI_INT, MPI_MAX, tr->get_communicator());
+	  const unsigned int max_size
+	    = Utilities::MPI::max (my_size, tr->get_communicator());
 
 					   // as we are reading past the end, we
 					   // need to increase the size of the
@@ -1539,11 +1539,11 @@ namespace internal
 	    }
 
 	  number_cache.n_global_dofs
-	    = std::accumulate( &number_cache
-			       .n_locally_owned_dofs_per_processor[0],
-			       &number_cache
-			       .n_locally_owned_dofs_per_processor[n_cpus],
-			       0 );
+	    = std::accumulate (number_cache
+			       .n_locally_owned_dofs_per_processor.begin(),
+			       number_cache
+			       .n_locally_owned_dofs_per_processor.end(),
+			       0);
 
 	  tr->load_user_flags(user_flags);
 	}
