@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_renumbering.cc 22920 2010-12-05 22:50:08Z bangerth $
+//    $Id: dof_renumbering.cc 24418 2011-09-26 12:52:19Z bangerth $
 //    Version: $name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,31 +12,31 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/thread_management.h>
-#include <base/utilities.h>
-#include <base/quadrature_lib.h>
+#include <deal.II/base/thread_management.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/base/quadrature_lib.h>
 
-#include <lac/sparsity_pattern.h>
-#include <lac/sparsity_tools.h>
-#include <lac/compressed_simple_sparsity_pattern.h>
-#include <lac/constraint_matrix.h>
+#include <deal.II/lac/sparsity_pattern.h>
+#include <deal.II/lac/sparsity_tools.h>
+#include <deal.II/lac/compressed_simple_sparsity_pattern.h>
+#include <deal.II/lac/constraint_matrix.h>
 
-#include <dofs/dof_accessor.h>
-#include <dofs/dof_handler.h>
-#include <dofs/dof_tools.h>
-#include <dofs/dof_renumbering.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/dofs/dof_tools.h>
+#include <deal.II/dofs/dof_renumbering.h>
 
-#include <grid/tria_iterator.h>
-#include <grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/grid/tria.h>
 
-#include <fe/fe.h>
-#include <hp/dof_handler.h>
-#include <hp/fe_collection.h>
-#include <hp/fe_values.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/hp/dof_handler.h>
+#include <deal.II/hp/fe_collection.h>
+#include <deal.II/hp/fe_values.h>
 
-#include <multigrid/mg_dof_handler.h>
-#include <multigrid/mg_dof_accessor.h>
-#include <multigrid/mg_tools.h>
+#include <deal.II/multigrid/mg_dof_handler.h>
+#include <deal.II/multigrid/mg_dof_accessor.h>
+#include <deal.II/multigrid/mg_tools.h>
 
 #include <boost/config.hpp>
 #include <boost/graph/adjacency_list.hpp>
@@ -92,10 +92,10 @@ namespace DoFRenumbering
 	  {
 	    return (! (T::operator==(i)));
 	  }
-					 // Allow access to these operators of T
-	T::operator->;
-	T::operator++;
-	T::operator==;
+					 // Allow access to these private operators of T
+	using T::operator->;
+	using T::operator++;
+	using T::operator==;
     };
 
 
@@ -117,10 +117,11 @@ namespace DoFRenumbering
 	  {
 	    return (! (T::operator==(i)));
 	  }
-					 // Allow access to these operators of T
-	T::operator->;
-	T::operator++;
-	T::operator==;
+					 // Allow access to these
+					 // private operators of T
+	using T::operator->;
+	using T::operator++;
+	using T::operator==;
     };
   }
 
@@ -410,7 +411,7 @@ namespace DoFRenumbering
       for (int i=0; i<n; ++i)
 	{
 	  Assert (std::find (perm.begin(), perm.end(), i)
-		  != inverse_perm.end(),
+		  != perm.end(),
 		  ExcInternalError());
 	  Assert (std::find (inverse_perm.begin(), inverse_perm.end(), i)
 		  != inverse_perm.end(),
@@ -764,7 +765,7 @@ namespace DoFRenumbering
     std::vector<std::vector<unsigned int> >
       component_to_dof_map (fe_collection.n_components());
     for (ITERATOR cell=start; cell!=end; ++cell)
-      if (!cell->is_artificial() && !cell->is_ghost())
+      if (cell->is_locally_owned())
 	{
 					 // on each cell: get dof indices
 					 // and insert them into the global
@@ -832,10 +833,10 @@ namespace DoFRenumbering
 					 // gather information from all CPUs
 	std::vector<unsigned int>
 	  all_dof_counts(fe_collection.n_components() *
-			 Utilities::System::get_n_mpi_processes (tria->get_communicator()));
+			 Utilities::MPI::n_mpi_processes (tria->get_communicator()));
 
-	MPI_Allgather ( &local_dof_count[0], n_buckets, MPI_INT, &all_dof_counts[0],
-			n_buckets, MPI_INT, tria->get_communicator());
+	MPI_Allgather ( &local_dof_count[0], n_buckets, MPI_UNSIGNED, &all_dof_counts[0],
+			n_buckets, MPI_UNSIGNED, tria->get_communicator());
 
 	for (unsigned int i=0; i<n_buckets; ++i)
 	  Assert (all_dof_counts[n_buckets*tria->locally_owned_subdomain()+i]
@@ -848,9 +849,9 @@ namespace DoFRenumbering
 	for (unsigned int c=0; c<n_buckets; ++c)
 	  {
 	    shifts[c]=cumulated;
-	    for (unsigned int i=0; i<tria->locally_owned_subdomain(); ++i)
+	    for (types::subdomain_id_t i=0; i<tria->locally_owned_subdomain(); ++i)
 	      shifts[c] += all_dof_counts[c+n_buckets*i];
-	    for (unsigned int i=0; i<Utilities::System::get_n_mpi_processes (tria->get_communicator()); ++i)
+	    for (unsigned int i=0; i<Utilities::MPI::n_mpi_processes (tria->get_communicator()); ++i)
 	      cumulated += all_dof_counts[c+n_buckets*i];
 	  }
 #else
@@ -898,6 +899,128 @@ namespace DoFRenumbering
     return next_free_index;
   }
 
+  namespace
+  {
+	// helper function for hierarchical()
+    template <int dim, class iterator>
+    unsigned int
+    compute_hierarchical_recursive (
+      unsigned int next_free,
+      std::vector<unsigned int>& new_indices,
+      const iterator & cell,
+      const IndexSet & locally_owned)
+    {
+      if (cell->has_children())
+        {
+		  //recursion
+          for (unsigned int c = 0;c < GeometryInfo<dim>::max_children_per_cell; ++c)
+            next_free = compute_hierarchical_recursive<dim> (
+                          next_free,
+                          new_indices,
+                          cell->child (c),
+                          locally_owned);
+        }
+      else
+        {
+          if (cell->is_locally_owned())
+            {
+              const unsigned int dofs_per_cell = cell->get_fe().dofs_per_cell;
+              std::vector<unsigned int> local_dof_indices (dofs_per_cell);
+              cell->get_dof_indices (local_dof_indices);
+
+              for (unsigned int i = 0;i < dofs_per_cell;++i)
+                {
+                  if (locally_owned.is_element (local_dof_indices[i]))
+                    {
+                      // this is a locally owned DoF, assign new number if not assigned a number yet
+                      unsigned int idx = locally_owned.index_within_set (local_dof_indices[i]);
+                      if (new_indices[idx] == DoFHandler<dim>::invalid_dof_index)
+                        {
+                          new_indices[idx] = locally_owned.nth_index_in_set (next_free);
+                          next_free++;
+                        }
+                    }
+                }
+            }
+        }
+      return next_free;
+    }
+  }
+
+
+
+  template <int dim>
+  void
+  hierarchical (DoFHandler<dim> &dof_handler)
+  {
+    std::vector<unsigned int> renumbering (dof_handler.n_locally_owned_dofs(),
+					   DoFHandler<dim>::invalid_dof_index);
+
+	typename DoFHandler<dim>::cell_iterator cell;
+
+	unsigned int next_free = 0;
+	const IndexSet locally_owned = dof_handler.locally_owned_dofs();
+
+    const parallel::distributed::Triangulation<dim> * tria
+	= dynamic_cast<const parallel::distributed::Triangulation<dim>*>
+	   (&dof_handler.get_tria());
+
+	if (tria)
+      {
+#ifdef DEAL_II_USE_P4EST
+        //this is a distributed Triangulation. We need to traverse the coarse
+        //cells in the order p4est does
+        for (unsigned int c = 0; c < tria->n_cells (0); ++c)
+          {
+            unsigned int coarse_cell_index =
+              tria->get_p4est_tree_to_coarse_cell_permutation() [c];
+
+            const typename DoFHandler<dim>::cell_iterator
+            cell (tria, 0, coarse_cell_index, &dof_handler);
+
+            next_free = compute_hierarchical_recursive<dim> (next_free,
+                        renumbering,
+                        cell,
+                        locally_owned);
+          }
+#else
+        Assert (false, ExcNotImplemented());
+#endif
+      }
+    else
+      {
+        //this is not a distributed Triangulation. Traverse coarse cells in the
+        //normal order
+        for (cell = dof_handler.begin (0); cell != dof_handler.end (0); ++cell)
+          next_free = compute_hierarchical_recursive<dim> (next_free,
+                      renumbering,
+                      cell,
+                      locally_owned);
+      }
+
+				     // verify that the last numbered
+				     // degree of freedom is either
+				     // equal to the number of degrees
+				     // of freedom in total (the
+				     // sequential case) or in the
+				     // distributed case at least
+				     // makes sense
+    Assert ((next_free == dof_handler.n_locally_owned_dofs())
+	    ||
+	    ((dof_handler.n_locally_owned_dofs() < dof_handler.n_dofs())
+	     &&
+	     (next_free <= dof_handler.n_dofs())),
+	    ExcRenumberingIncomplete());
+
+	// make sure that all local DoFs got new numbers assigned
+	Assert (std::find (renumbering.begin(), renumbering.end(),
+		       numbers::invalid_unsigned_int)
+	    == renumbering.end(),
+	    ExcInternalError());
+
+	dof_handler.renumber_dofs(renumbering);
+  }
+
 
 
   template <class DH>
@@ -1360,12 +1483,12 @@ namespace DoFRenumbering
 
 	const hp::FECollection<dim> fe_collection (dof.get_fe ());
 	Assert (fe_collection[0].has_support_points(),
-		DoFTools::ExcFEHasNoSupportPoints());
+		typename FiniteElement<dim>::ExcFEHasNoSupportPoints());
 	hp::QCollection<dim> quadrature_collection;
 	for (unsigned int comp=0; comp<fe_collection.size(); ++comp)
 	  {
 	    Assert (fe_collection[comp].has_support_points(),
-		    DoFTools::ExcFEHasNoSupportPoints());
+		    typename FiniteElement<dim>::ExcFEHasNoSupportPoints());
 	    quadrature_collection.push_back
 	      (Quadrature<DH::dimension> (fe_collection[comp].
 					  get_unit_support_points()));
@@ -1491,7 +1614,7 @@ namespace DoFRenumbering
     else
       {
 	Assert (dof.get_fe().has_support_points(),
-		DoFTools::ExcFEHasNoSupportPoints());
+		typename FiniteElement<dim>::ExcFEHasNoSupportPoints());
 	const unsigned int n_dofs = dof.n_dofs(level);
 	std::vector<std::pair<Point<dim>,unsigned int> > support_point_list
 	  (n_dofs);
