@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: fe.cc 22793 2010-11-17 17:02:40Z kanschat $
+//    $Id: fe.cc 24072 2011-08-15 12:19:02Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,16 +12,16 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/memory_consumption.h>
-#include <fe/mapping.h>
-#include <fe/fe.h>
-#include <fe/fe_values.h>
-#include <base/quadrature.h>
-#include <base/qprojector.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_accessor.h>
-#include <grid/tria_boundary.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/fe/mapping.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/grid/tria_boundary.h>
 
 #include <algorithm>
 #include <functional>
@@ -204,7 +204,7 @@ FiniteElement<dim,spacedim>::FiniteElement (
 				   // Fill with default value; may be
 				   // changed by constructor of
 				   // derived class.
-  first_block_of_base_table.resize(1,0);
+  base_to_block_indices.reinit(1,1);
 
 				   // initialize the restriction and
 				   // prolongation matrices. the default
@@ -909,7 +909,7 @@ FiniteElement<dim,spacedim>::interpolate(
 
 
 template <int dim, int spacedim>
-unsigned int
+std::size_t
 FiniteElement<dim,spacedim>::memory_consumption () const
 {
   return (sizeof(FiniteElementData<dim>) +
@@ -1150,6 +1150,9 @@ const FiniteElement<dim,spacedim>&
 FiniteElement<dim,spacedim>::base_element(const unsigned index) const
 {
   Assert (index==0, ExcIndexRange(index,0,1));
+				   // This function should not be
+				   // called for a system element
+  Assert (base_to_block_indices.size() == 1, ExcInternalError());
   return *this;
 }
 
