@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_abf.cc 22678 2010-11-10 15:35:22Z kronbichler $
+//    $Id: fe_abf.cc 24072 2011-08-15 12:19:02Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,19 +11,19 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/utilities.h>
-#include <base/quadrature.h>
-#include <base/quadrature_lib.h>
-#include <base/qprojector.h>
-#include <base/table.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_accessor.h>
-#include <fe/fe.h>
-#include <fe/mapping.h>
-#include <fe/fe_abf.h>
-#include <fe/fe_values.h>
-#include <fe/fe_tools.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/base/table.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/fe/mapping.h>
+#include <deal.II/fe/fe_abf.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/fe/fe_tools.h>
 
 #ifdef HAVE_STD_STRINGSTREAM
 #  include <sstream>
@@ -511,38 +511,6 @@ FE_ABF<dim>::update_each (const UpdateFlags flags) const
 // Data field initialization
 //---------------------------------------------------------------------------
 
-
-
-
-template <int dim>
-unsigned int
-FE_ABF<dim>::n_base_elements () const
-{
-  return 1;
-}
-
-
-
-template <int dim>
-const FiniteElement<dim> &
-FE_ABF<dim>::base_element (const unsigned int index) const
-{
-  Assert (index==0, ExcIndexRange(index, 0, 1));
-  return *this;
-}
-
-
-
-template <int dim>
-unsigned int
-FE_ABF<dim>::element_multiplicity (const unsigned int index) const
-{
-  Assert (index==0, ExcIndexRange(index, 0, 1));
-  return 1;
-}
-
-
-
 template <int dim>
 bool
 FE_ABF<dim>::has_support_on_face (const unsigned int shape_index,
@@ -720,7 +688,7 @@ FE_ABF<dim>::interpolate(
 
 
 template <int dim>
-unsigned int
+std::size_t
 FE_ABF<dim>::memory_consumption () const
 {
   Assert (false, ExcNotImplemented ());
