@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_dgp_nonparametric.cc 22676 2010-11-10 15:21:40Z kronbichler $
+//    $Id: fe_dgp_nonparametric.cc 24072 2011-08-15 12:19:02Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,14 +11,14 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/quadrature.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_accessor.h>
-#include <fe/fe.h>
-#include <fe/mapping.h>
-#include <fe/fe_dgp_nonparametric.h>
-#include <fe/fe_values.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/fe/mapping.h>
+#include <deal.II/fe/fe_dgp_nonparametric.h>
+#include <deal.II/fe/fe_values.h>
 
 #include <sstream>
 
@@ -413,35 +413,6 @@ FE_DGPNonparametric<dim,spacedim>::fill_fe_subface_values (
 
 
 
-template <int dim, int spacedim>
-unsigned int
-FE_DGPNonparametric<dim,spacedim>::n_base_elements () const
-{
-  return 1;
-}
-
-
-
-template <int dim, int spacedim>
-const FiniteElement<dim,spacedim> &
-FE_DGPNonparametric<dim,spacedim>::base_element (const unsigned int index) const
-{
-  Assert (index==0, ExcIndexRange(index, 0, 1));
-  return *this;
-}
-
-
-
-template <int dim, int spacedim>
-unsigned int
-FE_DGPNonparametric<dim,spacedim>::element_multiplicity (const unsigned int index) const
-{
-  Assert (index==0, ExcIndexRange(index, 0, 1));
-  return 1;
-}
-
-
-
 template <int dim, int spacedim>
 void
 FE_DGPNonparametric<dim,spacedim>::
@@ -573,11 +544,10 @@ FE_DGPNonparametric<dim,spacedim>::
 compare_for_face_domination (const FiniteElement<dim,spacedim> &fe_other) const
 {
 				   // check whether both are discontinuous
-				   // elements and both could dominate, see
-				   // the description of
+				   // elements, see the description of
 				   // FiniteElementDomination::Domination
   if (dynamic_cast<const FE_DGPNonparametric<dim,spacedim>*>(&fe_other) != 0)
-    return FiniteElementDomination::either_element_can_dominate;
+    return FiniteElementDomination::no_requirements;
 
   Assert (false, ExcNotImplemented());
   return FiniteElementDomination::neither_element_dominates;
@@ -596,7 +566,7 @@ FE_DGPNonparametric<dim,spacedim>::has_support_on_face (const unsigned int,
 
 
 template <int dim, int spacedim>
-unsigned int
+std::size_t
 FE_DGPNonparametric<dim,spacedim>::memory_consumption () const
 {
   Assert (false, ExcNotImplemented ());
