@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_dgq.cc 22676 2010-11-10 15:21:40Z kronbichler $
+//    $Id: fe_dgq.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010 by the deal.II authors
+//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,11 +11,11 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/quadrature.h>
-#include <base/quadrature_lib.h>
-#include <base/template_constraints.h>
-#include <fe/fe_dgq.h>
-#include <fe/fe_tools.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/template_constraints.h>
+#include <deal.II/fe/fe_dgq.h>
+#include <deal.II/fe/fe_tools.h>
 
 
 #include <iostream>
@@ -142,12 +142,32 @@ FE_DGQ<dim, spacedim>::FE_DGQ (const unsigned int degree)
 				   // restriction and prolongation
 				   // matrices to the right sizes
   this->reinit_restriction_and_prolongation_matrices();
-				   // Fill prolongation matrices with embedding operators
-  if (dim ==spacedim)
-  FETools::compute_embedding_matrices (*this, this->prolongation);
-				   // Fill restriction matrices with L2-projection
-  if (dim ==spacedim)
-  FETools::compute_projection_matrices (*this, this->restriction);
+
+				   // Fill prolongation matrices with
+				   // embedding operators and
+				   // restriction with L2 projection
+				   //
+				   // we can call the respective
+				   // functions in the case
+				   // dim==spacedim, but they are not
+				   // currently implemented for the
+				   // codim-1 case. there's no harm
+				   // here, though, since these
+				   // matrices are independent of the
+				   // space dimension and so we can
+				   // copy them from the respective
+				   // elements (not cheap, but works)
+  if (dim == spacedim)
+    {
+      FETools::compute_embedding_matrices (*this, this->prolongation);
+      FETools::compute_projection_matrices (*this, this->restriction);
+    }
+  else
+    {
+      FE_DGQ<dim> tmp (degree);
+      this->prolongation = tmp.prolongation;
+      this->restriction  = tmp.restriction;
+    }
 
 
 				   // finally fill in support points
@@ -556,11 +576,10 @@ FiniteElementDomination::Domination
 FE_DGQ<dim, spacedim>::compare_for_face_domination (const FiniteElement<dim, spacedim> &fe_other) const
 {
 				   // check whether both are discontinuous
-				   // elements and both could dominate, see
-				   // the description of
+				   // elements, see the description of
 				   // FiniteElementDomination::Domination
   if (dynamic_cast<const FE_DGQ<dim, spacedim>*>(&fe_other) != 0)
-    return FiniteElementDomination::either_element_can_dominate;
+    return FiniteElementDomination::no_requirements;
 
   Assert (false, ExcNotImplemented());
   return FiniteElementDomination::neither_element_dominates;
@@ -649,7 +668,7 @@ FE_DGQ<dim, spacedim>::has_support_on_face (const unsigned int shape_index,
 
 
 template <int dim, int spacedim>
-unsigned int
+std::size_t
 FE_DGQ<dim, spacedim>::memory_consumption () const
 {
   Assert (false, ExcNotImplemented ());
