@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_poly_tensor.cc 22828 2010-11-20 00:08:58Z bangerth $
+//    $Id: fe_poly_tensor.cc 24072 2011-08-15 12:19:02Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,14 +11,14 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/qprojector.h>
-#include <base/polynomials_bdm.h>
-#include <base/polynomials_raviart_thomas.h>
-#include <base/polynomials_abf.h>
-#include <base/polynomials_nedelec.h>
-#include <fe/fe_poly_tensor.h>
-#include <fe/fe_values.h>
-#include <fe/mapping_cartesian.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/base/polynomials_bdm.h>
+#include <deal.II/base/polynomials_raviart_thomas.h>
+#include <deal.II/base/polynomials_abf.h>
+#include <deal.II/base/polynomials_nedelec.h>
+#include <deal.II/fe/fe_poly_tensor.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/fe/mapping_cartesian.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -969,34 +969,6 @@ FE_PolyTensor<POLY,dim,spacedim>::fill_fe_subface_values (
 
 
 
-template <class POLY, int dim, int spacedim>
-unsigned int
-FE_PolyTensor<POLY,dim,spacedim>::n_base_elements () const
-{
-  return 1;
-}
-
-
-
-template <class POLY, int dim, int spacedim>
-const FiniteElement<dim,spacedim> &
-FE_PolyTensor<POLY,dim,spacedim>::base_element (const unsigned int index) const
-{
-  Assert (index==0, ExcIndexRange(index, 0, 1));
-  return *this;
-}
-
-
-
-template <class POLY, int dim, int spacedim>
-unsigned int
-FE_PolyTensor<POLY,dim,spacedim>::element_multiplicity (const unsigned int index) const
-{
-  Assert (index==0, ExcIndexRange(index, 0, 1));
-  return 1;
-}
-
-
 template <class POLY, int dim, int spacedim>
 UpdateFlags
 FE_PolyTensor<POLY,dim,spacedim>::update_once (const UpdateFlags flags) const
