@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_q.cc 22694 2010-11-11 20:46:11Z kanschat $
+//    $Id: fe_q.cc 24131 2011-08-21 00:16:16Z heister $
 //    Version: $Name$
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,16 +11,15 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/quadrature.h>
-#include <base/qprojector.h>
-#include <base/template_constraints.h>
-#include <fe/fe_q.h>
-#include <fe/fe_nothing.h>
-#include <fe/fe_tools.h>
-#include <base/quadrature_lib.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/base/template_constraints.h>
+#include <deal.II/fe/fe_q.h>
+#include <deal.II/fe/fe_nothing.h>
+#include <deal.II/fe/fe_tools.h>
+#include <deal.II/base/quadrature_lib.h>
 
 #include <vector>
-#include <iostream>
 #include <sstream>
 
 DEAL_II_NAMESPACE_OPEN
@@ -71,105 +70,81 @@ namespace FE_Q_Helper
     }
 
 
-				     // given an integer N, compute its
-				     // integer square root (if it
-				     // exists, otherwise give up)
-#if defined(DEAL_II_ANON_NAMESPACE_BUG) && defined(DEAL_II_ANON_NAMESPACE_LINKAGE_BUG)
+
+				// in initialize_embedding() and
+				// initialize_restriction(), want to undo
+				// tensorization on inner loops for
+				// performance reasons. this clears a
+				// dim-array
+    template <int dim>
+#ifdef DEAL_II_ANON_NAMESPACE_BUG
     static
 #endif
     inline
-    unsigned int int_sqrt (const unsigned int N)
+    void
+    zero_indices (unsigned int indices[dim])
     {
-      for (unsigned int i=0; i<=N; ++i)
-	if (i*i == N)
-	  return i;
-      Assert (false, ExcInternalError());
-      return numbers::invalid_unsigned_int;
-    }
-
-
-				     // given an integer N, compute its
-				     // integer cube root (if it
-				     // exists, otherwise give up)
-#if defined(DEAL_II_ANON_NAMESPACE_BUG) && defined(DEAL_II_ANON_NAMESPACE_LINKAGE_BUG)
-    static
-#endif
-    inline
-    unsigned int int_cuberoot (const unsigned int N)
-    {
-      for (unsigned int i=0; i<=N; ++i)
-	if (i*i*i == N)
-	  return i;
-      Assert (false, ExcInternalError());
-      return numbers::invalid_unsigned_int;
-    }
-
-
-				     // given N, generate i=0...N-1
-				     // equidistant points in the
-				     // interior of the interval [0,1]
-#if defined(DEAL_II_ANON_NAMESPACE_BUG) && defined(DEAL_II_ANON_NAMESPACE_LINKAGE_BUG)
-    static
-#endif
-    inline
-    Point<1>
-    generate_unit_point (const unsigned int i,
-			 const unsigned int N,
-			 const dealii::internal::int2type<1>  )
-    {
-      Assert (i<N, ExcInternalError());
-      const double h = 1./(N-1);
-      return Point<1>(i*h);
-    }
-
-
-				     // given N, generate i=0...N-1
-				     // equidistant points in the domain
-				     // [0,1]^2
-#if defined(DEAL_II_ANON_NAMESPACE_BUG) && defined(DEAL_II_ANON_NAMESPACE_LINKAGE_BUG)
-    static
-#endif
-    inline
-    Point<2>
-    generate_unit_point (const unsigned int i,
-			 const unsigned int N,
-			 const dealii::internal::int2type<2>  )
-    {
-      Assert (i<N, ExcInternalError());
-      Assert (N>=4, ExcInternalError());
-
-      const unsigned int N1d = int_sqrt(N);
-      const double h = 1./(N1d-1);
-
-      return Point<2> (i%N1d * h,
-		       i/N1d * h);
+      for (unsigned int d=0; d<dim; ++d)
+	indices[d] = 0;
     }
 
 
 
-				     // given N, generate i=0...N-1
-				     // equidistant points in the domain
-				     // [0,1]^3
-#if defined(DEAL_II_ANON_NAMESPACE_BUG) && defined(DEAL_II_ANON_NAMESPACE_LINKAGE_BUG)
+				// in initialize_embedding() and
+				// initialize_restriction(), want to undo
+				// tensorization on inner loops for
+				// performance reasons. this increments tensor
+				// product indices
+    template <int dim>
+#ifdef DEAL_II_ANON_NAMESPACE_BUG
     static
 #endif
     inline
-    Point<3>
-    generate_unit_point (const unsigned int i,
-			 const unsigned int N,
-			 const dealii::internal::int2type<3>  )
+    void
+    increment_indices (unsigned int       indices[dim],
+		       const unsigned int dofs1d)
     {
-      Assert (i<N, ExcInternalError());
-      Assert (N>=8, ExcInternalError());
-
-      const unsigned int N1d = int_cuberoot(N);
-      const double h = 1./(N1d-1);
-
-      return Point<3> (i%N1d * h,
-		       (i/N1d)%N1d * h,
-		       i/(N1d*N1d) * h);
+      ++indices[0];
+      for (int d=0; d<dim-1; ++d)
+	if (indices[d]==dofs1d)
+	  {
+	    indices[d] = 0;
+	    indices[d+1]++;
+	  }
     }
 
+
+
+				// in initialize_embedding() and
+				// initialize_restriction(), want to undo
+				// tensorization on inner loops for
+				// performance reasons, and we need to again
+				// access 1D polynomials. This function
+				// creates them from dim-dimensional support
+				// points.
+    template <int dim>
+#ifdef DEAL_II_ANON_NAMESPACE_BUG
+    static
+#endif
+    inline
+    std::vector<Polynomials::Polynomial<double> >
+    generate_poly_space1d (const std::vector<Point<dim> >  &unit_support_points,
+			   const std::vector<unsigned int> &index_map_inverse,
+			   const unsigned int               dofs1d)
+    {
+      AssertDimension (Utilities::fixed_power<dim> (dofs1d),
+		       unit_support_points.size());
+      std::vector<Point<1> > points1d (dofs1d);
+      for (unsigned int i=0; i<dofs1d; ++i)
+	{
+	  const unsigned int j = index_map_inverse[i];
+	  points1d[i] = Point<1>(unit_support_points[j](0));
+	  for (unsigned int d=1; d<dim; ++d)
+	    Assert (unit_support_points[j][d] == 0.,
+		    ExcInternalError());
+	}
+      return Polynomials::generate_complete_Lagrange_basis (points1d);
+    }
   }
 }
 
@@ -571,12 +546,15 @@ FE_Q<dim,spacedim>::FE_Q (const unsigned int degree)
   initialize_unit_support_points ();
   initialize_unit_face_support_points ();
 
-				   // compute constraint, embedding
-				   // and restriction matrices
+				   // reinit constraints
   initialize_constraints ();
+
+				   // Reinit the vectors of restriction and
+				   // prolongation matrices to the right sizes
+				   // and compute the matrices
   this->reinit_restriction_and_prolongation_matrices();
-  initialize_embedding ();
-  initialize_restriction ();
+  initialize_embedding();
+  initialize_restriction();
 
   initialize_quad_dof_index_permutation();
 }
@@ -595,7 +573,7 @@ FE_Q<dim,spacedim>::FE_Q (const Quadrature<1> &points)
 		  std::vector<std::vector<bool> >(1, std::vector<bool>(1,true))),
 		face_index_map(FE_Q_Helper::invert_numbering(face_lexicographic_to_hierarchic_numbering (points.size()-1)))
 {
-  const unsigned int degree = points.size()-1;
+  const int degree = points.size()-1;
 
   Assert (degree > 0,
           ExcMessage ("This element can only be used for polynomial degrees "
@@ -614,20 +592,14 @@ FE_Q<dim,spacedim>::FE_Q (const Quadrature<1> &points)
   initialize_unit_support_points (points);
   initialize_unit_face_support_points (points);
 
-				   // compute constraint, embedding
-				   // and restriction matrices
+				   // reinit constraints
   Implementation::initialize_constraints (points, *this);
 
-				   // Reinit the vectors of
-				   // restriction and prolongation
-				   // matrices to the right sizes
+				   // Reinit the vectors of restriction and
+				   // prolongation matrices to the right sizes
+				   // and compute the matrices
   this->reinit_restriction_and_prolongation_matrices();
-
-				   // Fill prolongation matrices with
-				   // embedding operators
-  FETools::compute_embedding_matrices (*this, this->prolongation);
-
-				   // Fill restriction matrices
+  initialize_embedding();
   initialize_restriction();
 
   initialize_quad_dof_index_permutation();
@@ -806,12 +778,10 @@ get_interpolation_matrix (const FiniteElement<dim,spacedim> &x_source_fe,
 template <>
 void
 FE_Q<1>::
-get_face_interpolation_matrix (const FiniteElement<1> &/*x_source_fe*/,
+get_face_interpolation_matrix (const FiniteElement<1,1> &/*x_source_fe*/,
 			       FullMatrix<double>     &/*interpolation_matrix*/) const
 {
-  Assert (false,
-	  FiniteElement<1>::
-	  ExcInterpolationNotImplemented ());
+  Assert (false, ExcImpossibleInDim(1));
 }
 
 
@@ -819,13 +789,11 @@ get_face_interpolation_matrix (const FiniteElement<1> &/*x_source_fe*/,
 template <>
 void
 FE_Q<1>::
-get_subface_interpolation_matrix (const FiniteElement<1> &/*x_source_fe*/,
+get_subface_interpolation_matrix (const FiniteElement<1,1> &/*x_source_fe*/,
 				  const unsigned int      /*subface*/,
 				  FullMatrix<double>     &/*interpolation_matrix*/) const
 {
-  Assert (false,
-	  FiniteElement<1>::
-	  ExcInterpolationNotImplemented ());
+  Assert (false, ExcImpossibleInDim(1));
 }
 
 
@@ -973,103 +941,106 @@ get_subface_interpolation_matrix (const FiniteElement<dim,spacedim> &x_source_fe
 				  const unsigned int        subface,
 				  FullMatrix<double>       &interpolation_matrix) const
 {
-				   // this is only implemented, if the
-				   // source FE is also a
-				   // Q element
-  typedef FE_Q<dim,spacedim> FEQ;
-  typedef FiniteElement<dim,spacedim> FEL;
-  AssertThrow ((x_source_fe.get_name().find ("FE_Q<") == 0)
-               ||
-               (dynamic_cast<const FEQ*>(&x_source_fe) != 0),
-               typename FEL::
-               ExcInterpolationNotImplemented());
-
-  Assert (interpolation_matrix.n() == this->dofs_per_face,
-	  ExcDimensionMismatch (interpolation_matrix.n(),
-				this->dofs_per_face));
   Assert (interpolation_matrix.m() == x_source_fe.dofs_per_face,
 	  ExcDimensionMismatch (interpolation_matrix.m(),
 				x_source_fe.dofs_per_face));
 
-				   // ok, source is a Q element, so
-				   // we will be able to do the work
-  const FE_Q<dim,spacedim> &source_fe
-    = dynamic_cast<const FE_Q<dim,spacedim>&>(x_source_fe);
+				   // see if source is a Q element
+  if (const FE_Q<dim,spacedim> *source_fe
+      = dynamic_cast<const FE_Q<dim,spacedim> *>(&x_source_fe))
+    {
+				       // have this test in here since
+				       // a table of size 2x0 reports
+				       // its size as 0x0
+      Assert (interpolation_matrix.n() == this->dofs_per_face,
+	      ExcDimensionMismatch (interpolation_matrix.n(),
+				    this->dofs_per_face));
 
-				   // Make sure, that the element,
-                                   // for which the DoFs should be
-                                   // constrained is the one with
-                                   // the higher polynomial degree.
-                                   // Actually the procedure will work
-                                   // also if this assertion is not
-                                   // satisfied. But the matrices
-                                   // produced in that case might
-				   // lead to problems in the
-                                   // hp procedures, which use this
-				   // method.
-  Assert (this->dofs_per_face <= source_fe.dofs_per_face,
-	  typename FEL::
-	  ExcInterpolationNotImplemented ());
+				       // Make sure, that the element,
+				       // for which the DoFs should be
+				       // constrained is the one with
+				       // the higher polynomial degree.
+				       // Actually the procedure will work
+				       // also if this assertion is not
+				       // satisfied. But the matrices
+				       // produced in that case might
+				       // lead to problems in the
+				       // hp procedures, which use this
+				       // method.
+      Assert (this->dofs_per_face <= source_fe->dofs_per_face,
+	      (typename FiniteElement<dim,spacedim>::
+	       ExcInterpolationNotImplemented ()));
 
-                                   // generate a point on this
-                                   // cell and evaluate the
-                                   // shape functions there
-  const Quadrature<dim-1>
-    quad_face_support (source_fe.get_unit_face_support_points ());
+				       // generate a point on this
+				       // cell and evaluate the
+				       // shape functions there
+      const Quadrature<dim-1>
+	quad_face_support (source_fe->get_unit_face_support_points ());
 
-				   // Rule of thumb for FP accuracy,
-				   // that can be expected for a
-				   // given polynomial degree.
-				   // This value is used to cut
-				   // off values close to zero.
-  double eps = 2e-13*this->degree*(dim-1);
+				       // Rule of thumb for FP accuracy,
+				       // that can be expected for a
+				       // given polynomial degree.
+				       // This value is used to cut
+				       // off values close to zero.
+      double eps = 2e-13*this->degree*(dim-1);
 
-				   // compute the interpolation
-				   // matrix by simply taking the
-                                   // value at the support points.
+				       // compute the interpolation
+				       // matrix by simply taking the
+				       // value at the support points.
 //TODO: Verify that all faces are the same with respect to
 // these support points. Furthermore, check if something has to
 // be done for the face orientation flag in 3D.
-  const Quadrature<dim> subface_quadrature
-    = QProjector<dim>::project_to_subface (quad_face_support, 0, subface);
-  for (unsigned int i=0; i<source_fe.dofs_per_face; ++i)
-    {
-      const Point<dim> &p = subface_quadrature.point (i);
-
-      for (unsigned int j=0; j<this->dofs_per_face; ++j)
+      const Quadrature<dim> subface_quadrature
+	= QProjector<dim>::project_to_subface (quad_face_support, 0, subface);
+      for (unsigned int i=0; i<source_fe->dofs_per_face; ++i)
 	{
-	  double matrix_entry = this->shape_value (this->face_to_cell_index(j, 0), p);
+	  const Point<dim> &p = subface_quadrature.point (i);
 
-					   // Correct the interpolated
-					   // value. I.e. if it is close
-					   // to 1 or 0, make it exactly
-					   // 1 or 0. Unfortunately, this
-					   // is required to avoid problems
-					   // with higher order elements.
-	  if (std::fabs (matrix_entry - 1.0) < eps)
-	    matrix_entry = 1.0;
-	  if (std::fabs (matrix_entry) < eps)
-	    matrix_entry = 0.0;
+	  for (unsigned int j=0; j<this->dofs_per_face; ++j)
+	    {
+	      double matrix_entry = this->shape_value (this->face_to_cell_index(j, 0), p);
 
-	  interpolation_matrix(i,j) = matrix_entry;
+					       // Correct the interpolated
+					       // value. I.e. if it is close
+					       // to 1 or 0, make it exactly
+					       // 1 or 0. Unfortunately, this
+					       // is required to avoid problems
+					       // with higher order elements.
+	      if (std::fabs (matrix_entry - 1.0) < eps)
+		matrix_entry = 1.0;
+	      if (std::fabs (matrix_entry) < eps)
+		matrix_entry = 0.0;
+
+	      interpolation_matrix(i,j) = matrix_entry;
+	    }
+	}
+
+				       // make sure that the row sum of
+				       // each of the matrices is 1 at
+				       // this point. this must be so
+				       // since the shape functions sum up
+				       // to 1
+      for (unsigned int j=0; j<source_fe->dofs_per_face; ++j)
+	{
+	  double sum = 0.;
+
+	  for (unsigned int i=0; i<this->dofs_per_face; ++i)
+	    sum += interpolation_matrix(j,i);
+
+	  Assert (std::fabs(sum-1) < 2e-13*this->degree*dim,
+		  ExcInternalError());
 	}
     }
-
-				   // make sure that the row sum of
-				   // each of the matrices is 1 at
-				   // this point. this must be so
-				   // since the shape functions sum up
-				   // to 1
-  for (unsigned int j=0; j<source_fe.dofs_per_face; ++j)
+  else if (dynamic_cast<const FE_Nothing<dim> *>(&x_source_fe) != 0)
     {
-      double sum = 0.;
-
-      for (unsigned int i=0; i<this->dofs_per_face; ++i)
-        sum += interpolation_matrix(j,i);
-
-      Assert (std::fabs(sum-1) < 2e-13*this->degree*this->degree*dim,
-              ExcInternalError());
+				       // nothing to do here, the
+				       // FE_Nothing has no degrees of
+				       // freedom anyway
     }
+  else
+    AssertThrow (false,
+		 (typename FiniteElement<dim,spacedim>::
+		  ExcInterpolationNotImplemented()));
 }
 
 
@@ -1131,26 +1102,37 @@ hp_line_dof_identities (const FiniteElement<dim,spacedim> &fe_other) const
 				   // FE_Nothing
   if (const FE_Q<dim,spacedim> *fe_q_other = dynamic_cast<const FE_Q<dim,spacedim>*>(&fe_other))
     {
-				       // dofs are located along
-				       // lines, so two dofs are
-				       // identical if they are
-				       // located at identical
-				       // positions. note that for
-				       // elements of orders p and q,
-				       // nodes are located on edges
-				       // at locations (i+1)/p and
-				       // (j+1)/q, so we need to find
-				       // those combinations i,j for
-				       // which (i+1)/p == (j+1)/q,
-				       // i.e. (i+1)*q == (j+1)*p
+				       // dofs are located along lines, so two
+				       // dofs are identical if they are
+				       // located at identical positions. if
+				       // we had only equidistant points, we
+				       // could simple check for similarity
+				       // like (i+1)*q == (j+1)*p, but we
+				       // might have other support points
+				       // (e.g. Gauss-Lobatto
+				       // points). Therefore, read the points
+				       // in unit_support_points for the first
+				       // coordinate direction. We take the
+				       // lexicographic ordering of the points
+				       // in the first direction (i.e.,
+				       // x-direction), which we access
+				       // between index 1 and p-1 (index 0 and
+				       // p are vertex dofs).
       const unsigned int p = this->degree;
       const unsigned int q = fe_q_other->degree;
 
       std::vector<std::pair<unsigned int, unsigned int> > identities;
 
+      const std::vector<unsigned int> &index_map_inverse=
+	this->poly_space.get_numbering_inverse();
+      const std::vector<unsigned int> &index_map_inverse_other=
+	fe_q_other->poly_space.get_numbering_inverse();
+
       for (unsigned int i=0; i<p-1; ++i)
 	for (unsigned int j=0; j<q-1; ++j)
-	  if ((i+1)*q == (j+1)*p)
+	  if (std::fabs(this->unit_support_points[index_map_inverse[i+1]][0]-
+			fe_q_other->unit_support_points[index_map_inverse_other[j+1]][0])
+	      < 1e-14)
 	    identities.push_back (std::make_pair(i,j));
 
       return identities;
@@ -1184,26 +1166,35 @@ hp_quad_dof_identities (const FiniteElement<dim,spacedim>        &fe_other) cons
   if (const FE_Q<dim,spacedim> *fe_q_other = dynamic_cast<const FE_Q<dim,spacedim>*>(&fe_other))
     {
 				       // this works exactly like the line
-				       // case above, except that now we
-				       // have to have two indices i1, i2
-				       // and j1, j2 to characterize the
-				       // dofs on the face of each of the
-				       // finite elements. since they are
-				       // ordered in lexicographic order,
-				       // the rest is rather
-				       // straightforward
+				       // case above, except that now we have
+				       // to have two indices i1, i2 and j1,
+				       // j2 to characterize the dofs on the
+				       // face of each of the finite
+				       // elements. since they are ordered
+				       // lexicographically along the first
+				       // line and we have a tensor product,
+				       // the rest is rather straightforward
       const unsigned int p = this->degree;
       const unsigned int q = fe_q_other->degree;
 
       std::vector<std::pair<unsigned int, unsigned int> > identities;
 
+      const std::vector<unsigned int> &index_map_inverse=
+	this->poly_space.get_numbering_inverse();
+      const std::vector<unsigned int> &index_map_inverse_other=
+	fe_q_other->poly_space.get_numbering_inverse();
+
       for (unsigned int i1=0; i1<p-1; ++i1)
 	for (unsigned int i2=0; i2<p-1; ++i2)
 	  for (unsigned int j1=0; j1<q-1; ++j1)
 	    for (unsigned int j2=0; j2<q-1; ++j2)
-	      if (((i1+1)*q == (j1+1)*p)
+	      if ((std::fabs(this->unit_support_points[index_map_inverse[i1+1]][0]-
+			     fe_q_other->unit_support_points[index_map_inverse_other[j1+1]][0])
+		   < 1e-14)
 		  &&
-		  ((i2+1)*q == (j2+1)*p))
+		  (std::fabs(this->unit_support_points[index_map_inverse[i2+1]][0]-
+			     fe_q_other->unit_support_points[index_map_inverse_other[j2+1]][0])
+		   < 1e-14))
 		identities.push_back (std::make_pair(i1*(p-1)+i2,
 						     j1*(q-1)+j2));
 
@@ -1244,16 +1235,12 @@ compare_for_face_domination (const FiniteElement<dim,spacedim> &fe_other) const
   else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != 0)
     {
 				       // the FE_Nothing has no
-				       // degrees of
-				       // freedom. nevertheless, we
-				       // say that the FE_Q element
-				       // dominates so that we don't
-				       // have to force the FE_Q side
-				       // to become a zero function
-				       // and rather allow the
-				       // function to be discontinuous
-				       // along the interface
-      return FiniteElementDomination::other_element_dominates;
+				       // degrees of freedom and it is
+				       // typically used in a context
+				       // where we don't require any
+				       // continuity along the
+				       // interface
+      return FiniteElementDomination::no_requirements;
     }
 
   Assert (false, ExcNotImplemented());
@@ -1565,59 +1552,54 @@ FE_Q<dim,spacedim>::initialize_embedding ()
 				   // value eps is used a threshold to
 				   // decide when certain evaluations of the
 				   // Lagrange polynomials are zero or one.
-  const std::vector<unsigned int> &index_map=
-    this->poly_space.get_numbering();
+  const double eps = 1e-15*this->degree*dim;
 
-  const double eps = 1e-13*this->degree*this->degree*this->degree*this->degree*dim;
-
-  unsigned n_ones = 0;
-				   // precompute subcell interpolation
-				   // information, which will give us a
-				   // vector of permutations. it actually is
-				   // a matrix (the inverse of which we'd
-				   // need to multiply the celL
-				   // interpolation matrix with), but since
-				   // we use Lagrangian basis functions
-				   // here, we know that each basis function
-				   // will just one at one node and zero on
-				   // all the others. this makes this
-				   // process much cheaper.
-  std::vector<unsigned int> subcell_permutations (this->dofs_per_cell,
-						  deal_II_numbers::invalid_unsigned_int);
+#ifdef DEBUG
+				// in DEBUG mode, check that the evaluation of
+				// support points in the current numbering
+				// gives the identity operation
   for (unsigned int i=0; i<this->dofs_per_cell; ++i)
-    for (unsigned int j=0; j<this->dofs_per_cell; ++j)
-      {
-	const Point<dim> p_subcell
-	  = FE_Q_Helper::generate_unit_point (index_map[j], this->dofs_per_cell,
-					      dealii::internal::int2type<dim>());
-	const double
-	  subcell_value = this->poly_space.compute_value (i, p_subcell);
-
-	if (std::fabs(subcell_value-1) < eps)
-	  {
-	    subcell_permutations[i] = j;
-				   // in debug mode, still want to check
-				   // whether we're not getting any strange
-				   // results with more than one 1 per row.
-#ifndef DEBUG
-	    break;
-#else
-	    n_ones++;
-#endif
-	  }
-	else
-	  Assert (std::fabs(subcell_value) < eps,
+    {
+      Assert (std::fabs (1.-this->poly_space.compute_value
+			 (i, this->unit_support_points[i])) < eps,
+	      ExcInternalError());
+      for (unsigned int j=0; j<this->dofs_per_cell; ++j)
+	if (j!=i)
+	  Assert (std::fabs (this->poly_space.compute_value
+			     (i, this->unit_support_points[j])) < eps,
 		  ExcInternalError());
+    }
+#endif
+
+				// to efficiently evaluate the polynomial at
+				// the subcell, make use of the tensor product
+				// structure of this element and only evaluate
+				// 1D information from the polynomial. This
+				// makes the cost of this function almost
+				// negligible also for high order elements
+  const unsigned int dofs1d = this->degree+1;
+  std::vector<Table<2,double> >
+    subcell_evaluations (dim, Table<2,double>(dofs1d, dofs1d));
+  const std::vector<unsigned int> &index_map_inverse =
+    this->poly_space.get_numbering_inverse();
+
+				//  recreate 1D polynomials
+  std::vector<Polynomials::Polynomial<double> > poly_space1d =
+    FE_Q_Helper::generate_poly_space1d (this->unit_support_points,
+                                        index_map_inverse, dofs1d);
+
+				// helper value: step size how to walk through
+				// diagonal and how many points we have left
+				// apart from the first dimension
+  unsigned int step_size_diag = 0;
+  {
+    unsigned int factor = 1;
+    for (unsigned int d=0; d<dim; ++d)
+      {
+	step_size_diag += factor;
+	factor *= dofs1d;
       }
-					     // make sure that we only
-					     // extracted a single one per
-					     // row, and that each row
-					     // actually got one value
-  Assert (n_ones == this->dofs_per_cell,
-	  ExcDimensionMismatch(n_ones, this->dofs_per_cell));
-  for (unsigned int i=0; i<this->dofs_per_cell; ++i)
-    Assert (subcell_permutations[i] < this->dofs_per_cell,
-	    ExcInternalError());
+  }
 
 					     // next evaluate the functions
 					     // for the different refinement
@@ -1625,22 +1607,20 @@ FE_Q<dim,spacedim>::initialize_embedding ()
   for (unsigned int ref=0; ref<RefinementCase<dim>::isotropic_refinement; ++ref)
     for (unsigned int child=0; child<GeometryInfo<dim>::n_children(RefinementCase<dim>(ref+1)); ++child)
       {
-	for (unsigned int j=0; j<this->dofs_per_cell; ++j)
+				// go through the points in diagonal to
+				// capture variation in all directions
+				// simultaneously
+	for (unsigned int j=0; j<dofs1d; ++j)
 	  {
-					     // generate a point on the
-					     // child cell and evaluate the
-					     // shape functions there
-	    const Point<dim> p_subcell
-	      = FE_Q_Helper::generate_unit_point (index_map[j], this->dofs_per_cell,
-						  dealii::internal::int2type<dim>());
+	    const unsigned int diag_comp = index_map_inverse[j*step_size_diag];
+	    const Point<dim> p_subcell = this->unit_support_points[diag_comp];
 	    const Point<dim> p_cell =
 	      GeometryInfo<dim>::child_to_cell_coordinates (p_subcell, child,
 							    RefinementCase<dim>(ref+1));
-
-	    for (unsigned int i=0; i<this->dofs_per_cell; ++i)
-	      {
-		const double
-		  cell_value    = this->poly_space.compute_value (i, p_cell);
+	    for (unsigned int i=0; i<dofs1d; ++i)
+	      for (unsigned int d=0; d<dim; ++d)
+		{
+		  const double cell_value = poly_space1d[i].value (p_cell[d]);
 
 				   // cut off values that are too
 				   // small. note that we have here Lagrange
@@ -1652,7 +1632,7 @@ FE_Q<dim,spacedim>::initialize_embedding ()
 				   //
 				   // the actual cut-off value is somewhat
 				   // fuzzy, but it works for
-				   // 2e-13*degree^2*dim (see above), which
+				   // 2e-13*degree*dim (see above), which
 				   // is kind of reasonable given that we
 				   // compute the values of the polynomials
 				   // via an degree-step recursion and then
@@ -1668,25 +1648,63 @@ FE_Q<dim,spacedim>::initialize_embedding ()
 				   // all we need to do is to switch the
 				   // rows we write the data into. moreover,
 				   // cut off very small values here
-		if (std::fabs(cell_value) < eps)
-		  this->prolongation[ref][child](subcell_permutations[j],i) = 0;
-		else
-		  this->prolongation[ref][child](subcell_permutations[j],i) =
-		    cell_value;
+		  if (std::fabs(cell_value) < eps)
+		    subcell_evaluations[d](j,i) = 0;
+		  else
+		    subcell_evaluations[d](j,i) = cell_value;
+		}
+	  }
+
+				// now expand from 1D info. block innermost
+				// dimension (x_0) in order to avoid difficult
+				// checks at innermost loop
+	unsigned int j_indices[dim];
+	FE_Q_Helper::zero_indices<dim> (j_indices);
+	for (unsigned int j=0; j<this->dofs_per_cell; j+=dofs1d)
+	  {
+	    unsigned int i_indices[dim];
+	    FE_Q_Helper::zero_indices<dim> (i_indices);
+	    for (unsigned int i=0; i<this->dofs_per_cell; i+=dofs1d)
+	      {
+		double val_extra_dim = 1.;
+		for (unsigned int d=1; d<dim; ++d)
+		  val_extra_dim *= subcell_evaluations[d](j_indices[d-1],
+							  i_indices[d-1]);
+
+				// innermost sum where we actually
+				// compute. the same as
+				// this->prolongation[ref][child](j,i) =
+				// this->poly_space.compute_value (i, p_cell);
+		for (unsigned int jj=0; jj<dofs1d; ++jj)
+		  {
+		    const unsigned int j_ind = index_map_inverse[j+jj];
+		    for (unsigned int ii=0; ii<dofs1d; ++ii)
+		      this->prolongation[ref][child](j_ind,index_map_inverse[i+ii])
+			= val_extra_dim * subcell_evaluations[0](jj,ii);
+		  }
+
+				// update indices that denote the tensor
+				// product position. a bit fuzzy and therefore
+				// not done for innermost x_0 direction
+		FE_Q_Helper::increment_indices<dim> (i_indices, dofs1d);
 	      }
+	    Assert (i_indices[dim-1] == 1, ExcInternalError());
+	    FE_Q_Helper::increment_indices<dim> (j_indices, dofs1d);
 	  }
 
 				   // and make sure that the row sum is
 				   // 1. this must be so since for this
 				   // element, the shape functions add up to
 				   // one
+#ifdef DEBUG
 	for (unsigned int row=0; row<this->dofs_per_cell; ++row)
 	  {
 	    double sum = 0;
 	    for (unsigned int col=0; col<this->dofs_per_cell; ++col)
 	      sum += this->prolongation[ref][child](row,col);
-	    Assert (std::fabs(sum-1.) < this->degree*eps, ExcInternalError());
+	    Assert (std::fabs(sum-1.) < eps, ExcInternalError());
 	  }
+#endif
       }
 }
 
@@ -1719,10 +1737,7 @@ FE_Q<dim,spacedim>::initialize_restriction ()
                                    // function there. rather than
                                    // doing it for the shape functions
                                    // on the mother cell, we take the
-                                   // interpolation points there are
-                                   // also search which shape function
-                                   // corresponds to it (too lazy to
-                                   // do this mapping by hand)
+                                   // interpolation points there
                                    //
                                    // note that the interpolation
                                    // point of a shape function can be
@@ -1742,33 +1757,22 @@ FE_Q<dim,spacedim>::initialize_restriction ()
                                    // (compute on a later child), so
                                    // we don't have to care about this
 
-  const double eps = 1e-13*this->degree*this->degree*this->degree*this->degree*dim;
-  const std::vector<unsigned int> &index_map_inverse=
+  const double eps = 1e-15*this->degree*dim;
+  const std::vector<unsigned int> &index_map_inverse =
     this->poly_space.get_numbering_inverse();
+
+				//  recreate 1D polynomials for faster
+				//  evaluation of polynomial
+  const unsigned int dofs1d = this->degree+1;
+  std::vector<Polynomials::Polynomial<double> > poly_space1d =
+    FE_Q_Helper::generate_poly_space1d (this->unit_support_points,
+                                        index_map_inverse, dofs1d);
+  std::vector<Tensor<1,dim> > evaluations1d (dofs1d);
+
   for (unsigned int i=0; i<this->dofs_per_cell; ++i)
     {
-      const Point<dim> p_cell = this->unit_support_points[index_map_inverse[i]];
-      unsigned int mother_dof = 0;
-      for (; mother_dof<this->dofs_per_cell; ++mother_dof)
-        {
-          const double val
-            = this->poly_space.compute_value(mother_dof, p_cell);
-          if (std::fabs (val-1.) < eps)
-                                             // ok, this is the right
-                                             // dof
-            break;
-          else
-                                             // make sure that all
-                                             // other shape functions
-                                             // are zero there
-            Assert (std::fabs(val) < eps, ExcInternalError());
-        }
-                                       // check also the shape
-                                       // functions after that
-      for (unsigned int j=mother_dof+1; j<this->dofs_per_cell; ++j)
-        Assert (std::fabs (this->poly_space.compute_value(j, p_cell))
-                < eps,
-                ExcInternalError());
+      unsigned int mother_dof = index_map_inverse[i];
+      const Point<dim> p_cell = this->unit_support_points[mother_dof];
 
                                        // then find the children on
                                        // which the interpolation
@@ -1776,18 +1780,33 @@ FE_Q<dim,spacedim>::initialize_restriction ()
       for (unsigned int ref=RefinementCase<dim>::cut_x; ref<=RefinementCase<dim>::isotropic_refinement; ++ref)
 	for (unsigned int child=0; child<GeometryInfo<dim>::n_children(RefinementCase<dim>(ref)); ++child)
 	  {
-					     // first initialize this
-					     // column of the matrix
-	    for (unsigned int j=0; j<this->dofs_per_cell; ++j)
-	      this->restriction[ref-1][child](mother_dof, j) = 0.;
-
-					     // then check whether this
+					     // check whether this
 					     // interpolation point is
 					     // inside this child cell
 	    const Point<dim> p_subcell
 	      = GeometryInfo<dim>::cell_to_child_coordinates (p_cell, child, RefinementCase<dim>(ref));
 	    if (GeometryInfo<dim>::is_inside_unit_cell (p_subcell))
 	      {
+				// same logic as in initialize_embedding to
+				// evaluate the polynomial faster than from
+				// the tensor product: since we evaluate all
+				// polynomials, it is much faster to just
+				// compute the 1D values for all polynomials
+				// before and then get the dim-data.
+		for (unsigned int j=0; j<dofs1d; ++j)
+		  for (unsigned int d=0; d<dim; ++d)
+		    evaluations1d[j][d] = poly_space1d[j].value (p_subcell[d]);
+		unsigned int j_indices[dim];
+		FE_Q_Helper::zero_indices<dim> (j_indices);
+		double sum_check = 0;
+		for (unsigned int j = 0; j<this->dofs_per_cell; j += dofs1d)
+		  {
+		    double val_extra_dim = 1.;
+		    for (unsigned int d=1; d<dim; ++d)
+		      val_extra_dim *= evaluations1d[j_indices[d-1]][d];
+		    for (unsigned int jj=0; jj<dofs1d; ++jj)
+		      {
+
 						 // find the child shape
 						 // function(s) corresponding
 						 // to this point. Usually
@@ -1800,20 +1819,19 @@ FE_Q<dim,spacedim>::initialize_restriction ()
 						 // get more than one nonzero
 						 // value per row. Still, the
 						 // values should sum up to 1
-		double sum_check = 0;
-		for (unsigned int child_dof = 0; child_dof<this->dofs_per_cell;
-		     ++child_dof)
-		  {
-		    const double val
-		      = this->poly_space.compute_value(child_dof, p_subcell);
-		    if (std::fabs (val-1.) < eps)
-		      this-> restriction[ref-1][child](mother_dof,child_dof)=1.;
-		    else if(std::fabs(val) > eps)
-		      this->restriction[ref-1][child](mother_dof,child_dof)= val;
-
-		    sum_check += val;
+			const double val
+			  = val_extra_dim * evaluations1d[jj][0];
+			const unsigned int child_dof =
+			  index_map_inverse[j+jj];
+			if (std::fabs (val-1.) < eps)
+			  this->restriction[ref-1][child](mother_dof,child_dof)=1.;
+			else if(std::fabs(val) > eps)
+			  this->restriction[ref-1][child](mother_dof,child_dof)=val;
+			sum_check += val;
+		      }
+		    FE_Q_Helper::increment_indices<dim> (j_indices, dofs1d);
 		  }
-		Assert (std::fabs(sum_check-1) < this->degree*eps,
+		Assert (std::fabs(sum_check-1) < eps,
 			ExcInternalError());
 	      }
 	  }
@@ -1981,7 +1999,7 @@ FE_Q<dim,spacedim>::has_support_on_face (const unsigned int shape_index,
 
 
 template <int dim, int spacedim>
-unsigned int
+std::size_t
 FE_Q<dim,spacedim>::memory_consumption () const
 {
   Assert (false, ExcNotImplemented ());
