@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_q_hierarchical.cc 22676 2010-11-10 15:21:40Z kronbichler $
+//    $Id: fe_q_hierarchical.cc 24131 2011-08-21 00:16:16Z heister $
 //    Version: $Name$
 //
-//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2010 by the deal.II authors
+//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,8 +11,8 @@
 //
 //---------------------------------------------------------------------------
 
-#include <fe/fe_q_hierarchical.h>
-#include <fe/fe_nothing.h>
+#include <deal.II/fe/fe_q_hierarchical.h>
+#include <deal.II/fe/fe_nothing.h>
 
 #include <cmath>
 #include <sstream>
@@ -44,7 +44,8 @@ FE_Q_Hierarchical<dim>::FE_Q_Hierarchical (const unsigned int degree)
 		:
 		FE_Poly<TensorProductPolynomials<dim>, dim> (
 		  Polynomials::Hierarchical::generate_complete_basis(degree),
-		  FiniteElementData<dim>(get_dpo_vector(degree),1, degree, FiniteElementData<dim>::H1),
+		  FiniteElementData<dim>(get_dpo_vector(degree),1, degree,
+					 FiniteElementData<dim>::H1),
 		  std::vector<bool> (FiniteElementData<dim>(
 				       get_dpo_vector(degree),1, degree).dofs_per_cell, false),
 		  std::vector<std::vector<bool> >(FiniteElementData<dim>(
@@ -164,6 +165,38 @@ hp_vertex_dof_identities (const FiniteElement<dim> &fe_other) const
     }
 }
 
+
+template <int dim>
+FiniteElementDomination::Domination
+FE_Q_Hierarchical<dim>::
+compare_for_face_domination (const FiniteElement<dim> &fe_other) const
+{
+  if (const FE_Q_Hierarchical<dim> *fe_q_other
+      = dynamic_cast<const FE_Q_Hierarchical<dim>*>(&fe_other))
+    {
+      if (this->degree < fe_q_other->degree)
+	return FiniteElementDomination::this_element_dominates;
+      else if (this->degree == fe_q_other->degree)
+	return FiniteElementDomination::either_element_can_dominate;
+      else
+	return FiniteElementDomination::other_element_dominates;
+    }
+  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != 0)
+    {
+				       // the FE_Nothing has no
+				       // degrees of freedom and it is
+				       // typically used in a context
+				       // where we don't require any
+				       // continuity along the
+				       // interface
+      return FiniteElementDomination::no_requirements;
+    }
+
+  Assert (false, ExcNotImplemented());
+  return FiniteElementDomination::neither_element_dominates;
+}
+
+
 //---------------------------------------------------------------------------
 // Auxiliary functions
 //---------------------------------------------------------------------------
@@ -554,25 +587,21 @@ void FE_Q_Hierarchical<1>::initialize_unit_face_support_points ()
 
 template <>
 void FE_Q_Hierarchical<1>::
-get_face_interpolation_matrix (const FiniteElement<1> &/*x_source_fe*/,
+get_face_interpolation_matrix (const FiniteElement<1,1> &/*x_source_fe*/,
 			       FullMatrix<double>     &/*interpolation_matrix*/) const
 {
-  Assert (false,
-	  FiniteElement<1>::
-	  ExcInterpolationNotImplemented ());
+  Assert (false, ExcImpossibleInDim(1));
 }
 
 
 template <>
 void
 FE_Q_Hierarchical<1>::
-get_subface_interpolation_matrix (const FiniteElement<1> &/*x_source_fe*/,
+get_subface_interpolation_matrix (const FiniteElement<1,1> &/*x_source_fe*/,
 				  const unsigned int      /*subface*/,
 				  FullMatrix<double>     &/*interpolation_matrix*/) const
 {
-  Assert (false,
-	  FiniteElement<1>::
-	  ExcInterpolationNotImplemented ());
+  Assert (false, ExcImpossibleInDim(1));
 }
 
 
@@ -1824,7 +1853,7 @@ FE_Q_Hierarchical<dim>::get_embedding_dofs (const unsigned int sub_degree) const
 
 
 template <int dim>
-unsigned int
+std::size_t
 FE_Q_Hierarchical<dim>::memory_consumption () const
 {
   Assert (false, ExcNotImplemented ());
