@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_raviart_thomas.cc 22676 2010-11-10 15:21:40Z kronbichler $
+//    $Id: fe_raviart_thomas.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,19 +11,19 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/utilities.h>
-#include <base/quadrature.h>
-#include <base/quadrature_lib.h>
-#include <base/qprojector.h>
-#include <base/table.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_accessor.h>
-#include <fe/fe.h>
-#include <fe/mapping.h>
-#include <fe/fe_raviart_thomas.h>
-#include <fe/fe_values.h>
-#include <fe/fe_tools.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/base/table.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/fe/mapping.h>
+#include <deal.II/fe/fe_raviart_thomas.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/fe/fe_tools.h>
 
 #include <sstream>
 #include <iostream>
@@ -529,7 +529,7 @@ FE_RaviartThomas<dim>::interpolate(
 
 
 template <int dim>
-unsigned int
+std::size_t
 FE_RaviartThomas<dim>::memory_consumption () const
 {
   Assert (false, ExcNotImplemented ());
