@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_raviart_thomas_nodal.cc 22676 2010-11-10 15:21:40Z kronbichler $
+//    $Id: fe_raviart_thomas_nodal.cc 24131 2011-08-21 00:16:16Z heister $
 //    Version: $Name$
 //
 //    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
@@ -11,17 +11,17 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/quadrature_lib.h>
-#include <base/qprojector.h>
-#include <base/table.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_accessor.h>
-#include <fe/fe.h>
-#include <fe/mapping.h>
-#include <fe/fe_raviart_thomas.h>
-#include <fe/fe_values.h>
-#include <fe/fe_tools.h>
+#include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/base/table.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/fe/mapping.h>
+#include <deal.II/fe/fe_raviart_thomas.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/fe/fe_tools.h>
 
 #include <sstream>
 
@@ -542,7 +542,7 @@ FE_RaviartThomasNodal<dim>::compare_for_face_domination (
 template <>
 void
 FE_RaviartThomasNodal<1>::get_face_interpolation_matrix (
-  const FiniteElement<1> &/*x_source_fe*/,
+  const FiniteElement<1,1> &/*x_source_fe*/,
   FullMatrix<double>     &/*interpolation_matrix*/) const
 {
   Assert (false, ExcImpossibleInDim(1));
@@ -552,7 +552,7 @@ FE_RaviartThomasNodal<1>::get_face_interpolation_matrix (
 template <>
 void
 FE_RaviartThomasNodal<1>::get_subface_interpolation_matrix (
-  const FiniteElement<1> &/*x_source_fe*/,
+  const FiniteElement<1,1> &/*x_source_fe*/,
   const unsigned int      /*subface*/,
   FullMatrix<double>     &/*interpolation_matrix*/) const
 {
