@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_tools.cc 22861 2010-11-26 14:36:05Z kronbichler $
+//    $Id: fe_tools.cc 23989 2011-08-02 09:47:19Z kronbichler $
 //    Version: $Name$
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,45 +11,100 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/quadrature_lib.h>
-#include <base/qprojector.h>
-#include <base/thread_management.h>
-#include <base/utilities.h>
-#include <lac/full_matrix.h>
-#include <lac/householder.h>
-#include <lac/vector.h>
-#include <lac/block_vector.h>
-#include <lac/trilinos_vector.h>
-#include <lac/trilinos_block_vector.h>
-#include <lac/constraint_matrix.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <grid/grid_generator.h>
-#include <fe/fe_tools.h>
-#include <fe/fe.h>
-#include <fe/fe_q.h>
-#include <fe/fe_q_hierarchical.h>
-#include <fe/fe_dgq.h>
-#include <fe/fe_dgp.h>
-#include <fe/fe_dgp_monomial.h>
-#include <fe/fe_dgp_nonparametric.h>
-#include <fe/fe_nedelec.h>
-#include <fe/fe_raviart_thomas.h>
-#include <fe/fe_system.h>
-#include <fe/fe_values.h>
-#include <fe/mapping_cartesian.h>
-#include <dofs/dof_handler.h>
-#include <dofs/dof_accessor.h>
-#include <dofs/dof_tools.h>
-#include <hp/dof_handler.h>
+#include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/base/thread_management.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/householder.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/petsc_parallel_vector.h>
+#include <deal.II/lac/trilinos_vector.h>
+#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/constraint_matrix.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/grid/grid_generator.h>
+#include <deal.II/fe/fe_tools.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/fe/fe_q.h>
+#include <deal.II/fe/fe_q_hierarchical.h>
+#include <deal.II/fe/fe_dgq.h>
+#include <deal.II/fe/fe_dgp.h>
+#include <deal.II/fe/fe_dgp_monomial.h>
+#include <deal.II/fe/fe_dgp_nonparametric.h>
+#include <deal.II/fe/fe_nedelec.h>
+#include <deal.II/fe/fe_raviart_thomas.h>
+#include <deal.II/fe/fe_system.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/fe/mapping_cartesian.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/dofs/dof_tools.h>
+#include <deal.II/hp/dof_handler.h>
 
-#include <base/std_cxx1x/shared_ptr.h>
+#include <deal.II/base/std_cxx1x/shared_ptr.h>
+
+#include <deal.II/base/index_set.h>
 
 #include <iostream>
 
 
 DEAL_II_NAMESPACE_OPEN
 
+namespace FETools
+{
+				// Not implemented in the general case.
+  template <class FE>
+  FiniteElement<FE::dimension, FE::dimension>*
+  FEFactory<FE>::get (const Quadrature<1> &) const
+  {
+    Assert(false, ExcNotImplemented());
+    return 0;
+  }
+
+				// Specializations for FE_Q.
+  template <>
+  FiniteElement<1, 1>*
+  FEFactory<FE_Q<1, 1> >::get (const Quadrature<1> &quad) const
+  {
+    return new FE_Q<1>(quad);
+  }
+  template <>
+  FiniteElement<2, 2>*
+  FEFactory<FE_Q<2, 2> >::get (const Quadrature<1> &quad) const
+  {
+    return new FE_Q<2>(quad);
+  }
+  template <>
+  FiniteElement<3, 3>*
+  FEFactory<FE_Q<3, 3> >::get (const Quadrature<1> &quad) const
+  {
+    return new FE_Q<3>(quad);
+  }
+
+
+				// Specializations for FE_DGQArbitraryNodes.
+  template <>
+  FiniteElement<1, 1>*
+  FEFactory<FE_DGQ<1> >::get (const Quadrature<1> &quad) const
+  {
+    return new FE_DGQArbitraryNodes<1>(quad);
+  }
+  template <>
+  FiniteElement<2, 2>*
+  FEFactory<FE_DGQ<2> >::get (const Quadrature<1> &quad) const
+  {
+    return new FE_DGQArbitraryNodes<2>(quad);
+  }
+  template <>
+  FiniteElement<3, 3>*
+  FEFactory<FE_DGQ<3> >::get (const Quadrature<1> &quad) const
+  {
+    return new FE_DGQArbitraryNodes<3>(quad);
+  }
+}
 
 namespace
 {
@@ -622,8 +677,7 @@ namespace FETools
       const FiniteElement<dim, spacedim>& fe,
       const FEValues<dim>& coarse,
       const Householder<double>& H,
-      FullMatrix<number>& this_matrix,
-      Threads::Mutex& /*mutex*/)
+      FullMatrix<number>& this_matrix)
     {
       const unsigned int n  = fe.dofs_per_cell;
       const unsigned int nd = fe.n_components ();
@@ -664,18 +718,8 @@ namespace FETools
 				       // function to a fine grid
 				       // function, the columns
 				       // are fine grid.
-
-//TODO: This function is called in parallel for different sets of
-//matrices, thus there should be no overlap and the mutex should not
-//be necessary. Remove this TODO and the mutex if everybody agrees.
-
-
-//      mutex.acquire ();
-
       for (unsigned int j = 0; j < n; ++j)
 	this_matrix(j, i) = v_fine(j);
-
-//      mutex.release ();
     }
 
 
@@ -733,7 +777,6 @@ namespace FETools
 	  for (unsigned int k = 0; k < nq; ++k)
 	    A (k * nd + d, j) = fine.shape_value_component (j, k, d);
 
-      static Threads::Mutex mutex;
       Householder<double> H (A);
       unsigned int cell_number = 0;
 
@@ -771,7 +814,7 @@ namespace FETools
 		{
 		  task_group +=
 		    Threads::new_task (&compute_embedding_for_shape_function<dim, number, spacedim>,
-				       i, fe, coarse, H, this_matrix, mutex);
+				       i, fe, coarse, H, this_matrix);
 		}
 	      task_group.join_all();
 	    }
@@ -780,24 +823,16 @@ namespace FETools
 	      for (unsigned int i = 0; i < n; ++i)
 		{
 		  compute_embedding_for_shape_function<dim, number, spacedim>
-		    (i, fe, coarse, H, this_matrix, mutex);
+		    (i, fe, coarse, H, this_matrix);
 		}
 	    }
 
-//TODO: This function is called in parallel for different sets of
-//matrices, thus there should be no overlap and the mutex should not
-//be necessary. Remove this TODO and the mutex if everybody agrees.
-
-
-//	  mutex.acquire ();
 					   // Remove small entries from
 					   // the matrix
 	  for (unsigned int i = 0; i < this_matrix.m (); ++i)
 	    for (unsigned int j = 0; j < this_matrix.n (); ++j)
 	      if (std::fabs (this_matrix (i, j)) < 1e-12)
 		this_matrix (i, j) = 0.;
-
-//	  mutex.release ();
 	}
 
       Assert (cell_number == GeometryInfo<dim>::n_children (RefinementCase<dim> (ref_case)),
@@ -806,7 +841,6 @@ namespace FETools
   }
 
 
-// This function is tested by tests/fe/internals, since it produces the matrices printed there
   template <int dim, typename number, int spacedim>
   void
   compute_embedding_matrices(const FiniteElement<dim,spacedim>& fe,
@@ -828,7 +862,7 @@ namespace FETools
   }
 
 
-// This function is tested by tests/fe/internals, since it produces the matrices printed there
+
   template <int dim, typename number, int spacedim>
   void
   compute_face_embedding_matrices(const FiniteElement<dim,spacedim>& fe,
@@ -1218,55 +1252,85 @@ namespace FETools
 
 
 
-template <int dim, int spacedim,
-          template <int, int> class DH1,
-          template <int, int> class DH2,
-          class InVector, class OutVector>
-void
-interpolate (const DH1<dim, spacedim> &dof1,
-	     const InVector           &u1,
-	     const DH2<dim, spacedim> &dof2,
-	     const ConstraintMatrix   &constraints,
-	     OutVector                &u2)
-{
-  Assert(&dof1.get_tria()==&dof2.get_tria(), ExcTriangulationMismatch());
+  template <int dim, int spacedim,
+            template <int, int> class DH1,
+            template <int, int> class DH2,
+            class InVector, class OutVector>
+  void
+  interpolate (const DH1<dim, spacedim> &dof1,
+	       const InVector           &u1,
+	       const DH2<dim, spacedim> &dof2,
+	       const ConstraintMatrix   &constraints,
+	       OutVector                &u2)
+  {
+    Assert(&dof1.get_tria()==&dof2.get_tria(), ExcTriangulationMismatch());
 
-  Assert(u1.size()==dof1.n_dofs(),
-         ExcDimensionMismatch(u1.size(), dof1.n_dofs()));
-  Assert(u2.size()==dof2.n_dofs(),
-         ExcDimensionMismatch(u2.size(), dof2.n_dofs()));
+    Assert(u1.size()==dof1.n_dofs(),
+           ExcDimensionMismatch(u1.size(), dof1.n_dofs()));
+    Assert(u2.size()==dof2.n_dofs(),
+           ExcDimensionMismatch(u2.size(), dof2.n_dofs()));
 
+#ifdef DEAL_II_USE_PETSC
+    if (dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1) != 0)
+      if (dynamic_cast<const DoFHandler<dim>*>(&dof1) != 0)
+        {
+                                   // if u1 is a parallel distributed
+                                   // PETSc vector, we check the local
+                                   // size of u1 for safety
+          Assert(dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1)->local_size() == dof1.locally_owned_dofs().n_elements(),
+                 ExcDimensionMismatch(dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1)->local_size(), dof1.locally_owned_dofs().n_elements()));
+        };
+
+    if (dynamic_cast<PETScWrappers::MPI::Vector *>(&u2) != 0)
+      if (dynamic_cast<const DoFHandler<dim>*>(&dof2) != 0)
+        {
+          Assert(dynamic_cast<PETScWrappers::MPI::Vector *>(&u2)->local_size() == dof2.locally_owned_dofs().n_elements(),
+                 ExcDimensionMismatch(dynamic_cast<PETScWrappers::MPI::Vector *>(&u2)->local_size(), dof2.locally_owned_dofs().n_elements()));
+        };
+#endif
                                    // allocate vectors at maximal
                                    // size. will be reinited in inner
                                    // cell, but Vector makes sure that
                                    // this does not lead to
                                    // reallocation of memory
-  Vector<typename OutVector::value_type> u1_local(DoFTools::max_dofs_per_cell(dof1));
-  Vector<typename OutVector::value_type> u2_local(DoFTools::max_dofs_per_cell(dof2));
+    Vector<typename OutVector::value_type> u1_local(DoFTools::max_dofs_per_cell(dof1));
+    Vector<typename OutVector::value_type> u2_local(DoFTools::max_dofs_per_cell(dof2));
 
                                    // have a map for interpolation
                                    // matrices. shared_ptr make sure
                                    // that memory is released again
-  std::map<const FiniteElement<dim,spacedim> *,
     std::map<const FiniteElement<dim,spacedim> *,
-    std_cxx1x::shared_ptr<FullMatrix<double> > > >
-    interpolation_matrices;
+      std::map<const FiniteElement<dim,spacedim> *,
+      std_cxx1x::shared_ptr<FullMatrix<double> > > >
+      interpolation_matrices;
 
-  typename DH1<dim,spacedim>::active_cell_iterator cell1 = dof1.begin_active(),
-                                                   endc1 = dof1.end();
-  typename DH2<dim,spacedim>::active_cell_iterator cell2 = dof2.begin_active(),
-                                                   endc2 = dof2.end();
+    typename DH1<dim,spacedim>::active_cell_iterator cell1 = dof1.begin_active(),
+                                                     endc1 = dof1.end();
+    typename DH2<dim,spacedim>::active_cell_iterator cell2 = dof2.begin_active(),
+                                                     endc2 = dof2.end();
 
-  std::vector<unsigned int> touch_count(dof2.n_dofs(),0);
-  std::vector<unsigned int> dofs;
-  dofs.reserve (DoFTools::max_dofs_per_cell (dof2));
-  u2 = 0;
+    std::vector<unsigned int> dofs;
+    dofs.reserve (DoFTools::max_dofs_per_cell (dof2));
 
-  for (; cell1!=endc1; ++cell1, ++cell2)
-    {
-      Assert(cell1->get_fe().n_components() == cell2->get_fe().n_components(),
-             ExcDimensionMismatch (cell1->get_fe().n_components(),
-                                   cell2->get_fe().n_components()));
+    u2 = 0;
+    OutVector touch_count(u2);
+    touch_count = 0;
+
+                                   // for distributed triangulations,
+                                   // we can only interpolate u1 on
+                                   // a cell, which this processor owns,
+				   // so we have to know the subdomain_id
+    const types::subdomain_id_t subdomain_id =
+      dof1.get_tria().locally_owned_subdomain();
+
+    for (; cell1!=endc1; ++cell1, ++cell2)
+      if ((cell1->subdomain_id() == subdomain_id)
+          ||
+          (subdomain_id == types::invalid_subdomain_id))
+        {
+          Assert(cell1->get_fe().n_components() == cell2->get_fe().n_components(),
+                 ExcDimensionMismatch (cell1->get_fe().n_components(),
+                                       cell2->get_fe().n_components()));
 
                                        // for continuous elements on
                                        // grids with hanging nodes we
@@ -1275,69 +1339,85 @@ interpolate (const DH1<dim, spacedim> &dof1,
                                        // if there are no constraints
                                        // then hanging nodes are not
                                        // allowed.
-      const bool hanging_nodes_not_allowed
-        = ((cell2->get_fe().dofs_per_vertex != 0) &&
-           (constraints.n_constraints() == 0));
+          const bool hanging_nodes_not_allowed
+            = ((cell2->get_fe().dofs_per_vertex != 0) &&
+               (constraints.n_constraints() == 0));
 
-      if (hanging_nodes_not_allowed)
-	for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
-	  Assert (cell1->at_boundary(face) ||
-		  cell1->neighbor(face)->level() == cell1->level(),
-		  ExcHangingNodesNotAllowed(0));
+          if (hanging_nodes_not_allowed)
+	    for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
+	      Assert (cell1->at_boundary(face) ||
+		      cell1->neighbor(face)->level() == cell1->level(),
+		      ExcHangingNodesNotAllowed(0));
 
 
-      const unsigned int dofs_per_cell1 = cell1->get_fe().dofs_per_cell;
-      const unsigned int dofs_per_cell2 = cell2->get_fe().dofs_per_cell;
-      u1_local.reinit (dofs_per_cell1);
-      u2_local.reinit (dofs_per_cell2);
+          const unsigned int dofs_per_cell1 = cell1->get_fe().dofs_per_cell;
+          const unsigned int dofs_per_cell2 = cell2->get_fe().dofs_per_cell;
+          u1_local.reinit (dofs_per_cell1);
+          u2_local.reinit (dofs_per_cell2);
 
                                        // check if interpolation
                                        // matrix for this particular
                                        // pair of elements is already
                                        // there
-      if (interpolation_matrices[&cell1->get_fe()][&cell2->get_fe()].get() ==
-	  0)
-        {
-          std_cxx1x::shared_ptr<FullMatrix<double> >
-            interpolation_matrix (new FullMatrix<double> (dofs_per_cell2,
-                                                          dofs_per_cell1));
+          if (interpolation_matrices[&cell1->get_fe()][&cell2->get_fe()].get() == 0)
+            {
+              std_cxx1x::shared_ptr<FullMatrix<double> >
+                interpolation_matrix (new FullMatrix<double> (dofs_per_cell2,
+                                                              dofs_per_cell1));
+              interpolation_matrices[&cell1->get_fe()][&cell2->get_fe()]
+                = interpolation_matrix;
+
+              get_interpolation_matrix(cell1->get_fe(),
+				       cell2->get_fe(),
+				       *interpolation_matrix);
+            }
+
+          cell1->get_dof_values(u1, u1_local);
           interpolation_matrices[&cell1->get_fe()][&cell2->get_fe()]
-            = interpolation_matrix;
+            ->vmult(u2_local, u1_local);
 
-          get_interpolation_matrix(cell1->get_fe(),
-				   cell2->get_fe(),
-				   *interpolation_matrix);
+          dofs.resize (dofs_per_cell2);
+          cell2->get_dof_indices(dofs);
+
+          for (unsigned int i=0; i<dofs_per_cell2; ++i)
+            {
+              u2(dofs[i])+=u2_local(i);
+              touch_count(dofs[i]) += 1;
+            }
         }
-
-      cell1->get_dof_values(u1, u1_local);
-      interpolation_matrices[&cell1->get_fe()][&cell2->get_fe()]
-        ->vmult(u2_local, u1_local);
-
-      dofs.resize (dofs_per_cell2);
-      cell2->get_dof_indices(dofs);
-
-      for (unsigned int i=0; i<dofs_per_cell2; ++i)
-	{
-	  u2(dofs[i])+=u2_local(i);
-	  ++touch_count[dofs[i]];
-	}
-    }
                                    // cell1 is at the end, so should
                                    // be cell2
-  Assert (cell2 == endc2, ExcInternalError());
+    Assert (cell2 == endc2, ExcInternalError());
+
+    u2.compress();
+    touch_count.compress();
+
+                                   // if we work on parallel distributed
+                                   // vectors, we have to ensure, that we only
+                                   // work on dofs this processor owns.
+    IndexSet  locally_owned_dofs = dof2.locally_owned_dofs();
 
 				   // when a discontinuous element is
 				   // interpolated to a continuous
 				   // one, we take the mean values.
-  for (unsigned int i=0; i<dof2.n_dofs(); ++i)
-    {
-      Assert(touch_count[i]!=0, ExcInternalError());
-      u2(i) /= touch_count[i];
-    }
+				   // for parallel vectors check,
+				   // if this component is owned by
+				   // this processor.
+    for (unsigned int i=0; i<dof2.n_dofs(); ++i)
+      if (locally_owned_dofs.is_element(i))
+        {
+          Assert(touch_count(i)!=0, ExcInternalError());
+          u2(i) /= touch_count(i);
+        }
 
+				   // finish the work on parallel vectors
+    u2.compress();
 				   // Apply hanging node constraints.
-  constraints.distribute(u2);
-}
+    constraints.distribute(u2);
+
+				   // and finally update ghost values
+    u2.update_ghost_values();
+  }
 
 
 
@@ -1354,6 +1434,25 @@ interpolate (const DH1<dim, spacedim> &dof1,
     Assert(u1_interpolated.size()==dof1.n_dofs(),
 	   ExcDimensionMismatch(u1_interpolated.size(), dof1.n_dofs()));
 
+#ifdef DEAL_II_USE_PETSC
+    if (dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1) != 0)
+      if (dynamic_cast<const DoFHandler<dim>*>(&dof1) != 0)
+        {
+                                   // if u1 is a parallel distributed
+                                   // PETSc vector, we check the local
+                                   // size of u1 for safety
+          Assert(dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1)->local_size() == dof1.locally_owned_dofs().n_elements(),
+                 ExcDimensionMismatch(dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1)->local_size(), dof1.locally_owned_dofs().n_elements()));
+        };
+
+    if (dynamic_cast<PETScWrappers::MPI::Vector *>(&u1_interpolated) != 0)
+      if (dynamic_cast<const DoFHandler<dim>*>(&dof1) != 0)
+        {
+          Assert(dynamic_cast<PETScWrappers::MPI::Vector *>(&u1_interpolated)->local_size() == dof1.locally_owned_dofs().n_elements(),
+                 ExcDimensionMismatch(dynamic_cast<PETScWrappers::MPI::Vector *>(&u1_interpolated)->local_size(), dof1.locally_owned_dofs().n_elements()));
+        };
+#endif
+
 				     // For continuous elements on grids
 				     // with hanging nodes we need
 				     // hanging node
@@ -1369,6 +1468,9 @@ interpolate (const DH1<dim, spacedim> &dof1,
     Vector<typename OutVector::value_type> u1_local(dofs_per_cell1);
     Vector<typename OutVector::value_type> u1_int_local(dofs_per_cell1);
 
+    const types::subdomain_id_t subdomain_id =
+      dof1.get_tria().locally_owned_subdomain();
+
     typename DoFHandler<dim,spacedim>::active_cell_iterator cell = dof1.begin_active(),
 							    endc = dof1.end();
 
@@ -1376,17 +1478,26 @@ interpolate (const DH1<dim, spacedim> &dof1,
     get_back_interpolation_matrix(dof1.get_fe(), fe2,
 				  interpolation_matrix);
     for (; cell!=endc; ++cell)
-      {
-	if (hanging_nodes_not_allowed)
-	  for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
-	    Assert (cell->at_boundary(face) ||
-		    cell->neighbor(face)->level() == cell->level(),
-		    ExcHangingNodesNotAllowed(0));
+      if ((cell->subdomain_id() == subdomain_id)
+          ||
+          (subdomain_id == types::invalid_subdomain_id))
+        {
+	  if (hanging_nodes_not_allowed)
+	    for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
+	      Assert (cell->at_boundary(face) ||
+		      cell->neighbor(face)->level() == cell->level(),
+		      ExcHangingNodesNotAllowed(0));
 
-	cell->get_dof_values(u1, u1_local);
-	interpolation_matrix.vmult(u1_int_local, u1_local);
-	cell->set_dof_values(u1_int_local, u1_interpolated);
-      }
+	  cell->get_dof_values(u1, u1_local);
+	  interpolation_matrix.vmult(u1_int_local, u1_local);
+	  cell->set_dof_values(u1_int_local, u1_interpolated);
+        }
+
+				   // if we work on a parallel PETSc vector
+				   // we have to finish the work and
+				   // update ghost values
+    u1_interpolated.compress();
+    u1_interpolated.update_ghost_values();
   }
 
 
@@ -1399,29 +1510,54 @@ interpolate (const DH1<dim, spacedim> &dof1,
 		   const InVector           &u1,
 		   const FiniteElement<dim,spacedim> &fe2,
 		   OutVector                &u1_interpolated)
-	    {
-	      Assert(u1.size() == dof1.n_dofs(),
-		     ExcDimensionMismatch(u1.size(), dof1.n_dofs()));
-	      Assert(u1_interpolated.size() == dof1.n_dofs(),
-		     ExcDimensionMismatch(u1_interpolated.size(), dof1.n_dofs()));
+  {
+    Assert(u1.size() == dof1.n_dofs(),
+           ExcDimensionMismatch(u1.size(), dof1.n_dofs()));
+    Assert(u1_interpolated.size() == dof1.n_dofs(),
+           ExcDimensionMismatch(u1_interpolated.size(), dof1.n_dofs()));
 
-	      Vector<typename OutVector::value_type> u1_local(DoFTools::max_dofs_per_cell(dof1));
-	      Vector<typename OutVector::value_type> u1_int_local(DoFTools::max_dofs_per_cell(dof1));
+#ifdef DEAL_II_USE_PETSC
+    if (dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1) != 0)
+      if (dynamic_cast<const DoFHandler<dim>*>(&dof1) != 0)
+        {
+                                   // if u1 is a parallel distributed
+                                   // PETSc vector, we check the local
+                                   // size of u1 for safety
+          Assert(dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1)->local_size() == dof1.locally_owned_dofs().n_elements(),
+                 ExcDimensionMismatch(dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1)->local_size(), dof1.locally_owned_dofs().n_elements()));
+        };
 
-	      typename DH<dim>::active_cell_iterator cell = dof1.begin_active(),
-						     endc = dof1.end();
+    if (dynamic_cast<PETScWrappers::MPI::Vector *>(&u1_interpolated) != 0)
+      if (dynamic_cast<const DoFHandler<dim>*>(&dof1) != 0)
+        {
+          Assert(dynamic_cast<PETScWrappers::MPI::Vector *>(&u1_interpolated)->local_size() == dof1.locally_owned_dofs().n_elements(),
+                 ExcDimensionMismatch(dynamic_cast<PETScWrappers::MPI::Vector *>(&u1_interpolated)->local_size(), dof1.locally_owned_dofs().n_elements()));
+        };
+#endif
+
+    Vector<typename OutVector::value_type> u1_local(DoFTools::max_dofs_per_cell(dof1));
+    Vector<typename OutVector::value_type> u1_int_local(DoFTools::max_dofs_per_cell(dof1));
+
+    const types::subdomain_id_t subdomain_id =
+      dof1.get_tria().locally_owned_subdomain();
+
+    typename DH<dim>::active_cell_iterator cell = dof1.begin_active(),
+                                           endc = dof1.end();
 
 					       // map from possible fe objects in
 					       // dof1 to the back_interpolation
 					       // matrices
-	      std::map<const FiniteElement<dim> *,
-		std_cxx1x::shared_ptr<FullMatrix<double> > > interpolation_matrices;
+    std::map<const FiniteElement<dim> *,
+       std_cxx1x::shared_ptr<FullMatrix<double> > > interpolation_matrices;
 
-for (; cell!=endc; ++cell)
-  {
-    Assert(cell->get_fe().n_components() == fe2.n_components(),
-	   ExcDimensionMismatch(cell->get_fe().n_components(),
-				fe2.n_components()));
+    for (; cell!=endc; ++cell)
+      if ((cell->subdomain_id() == subdomain_id)
+          ||
+          (subdomain_id == types::invalid_subdomain_id))
+        {
+          Assert(cell->get_fe().n_components() == fe2.n_components(),
+                 ExcDimensionMismatch(cell->get_fe().n_components(),
+                 fe2.n_components()));
 
 				     // For continuous elements on
 				     // grids with hanging nodes we
@@ -1430,36 +1566,42 @@ for (; cell!=endc; ++cell)
 				     // when the elements are
 				     // continuous no hanging node
 				     // constraints are allowed.
-    const bool hanging_nodes_not_allowed=
-      (cell->get_fe().dofs_per_vertex != 0) || (fe2.dofs_per_vertex != 0);
+          const bool hanging_nodes_not_allowed=
+            (cell->get_fe().dofs_per_vertex != 0) || (fe2.dofs_per_vertex != 0);
 
-    if (hanging_nodes_not_allowed)
-      for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
-	Assert (cell->at_boundary(face) ||
-		cell->neighbor(face)->level() == cell->level(),
-		ExcHangingNodesNotAllowed(0));
+          if (hanging_nodes_not_allowed)
+            for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
+              Assert (cell->at_boundary(face) ||
+                      cell->neighbor(face)->level() == cell->level(),
+                      ExcHangingNodesNotAllowed(0));
 
-    const unsigned int dofs_per_cell1 = cell->get_fe().dofs_per_cell;
+          const unsigned int dofs_per_cell1 = cell->get_fe().dofs_per_cell;
 
 				     // make sure back_interpolation
 				     // matrix is available
-    if (interpolation_matrices[&cell->get_fe()] != 0)
-      {
-	interpolation_matrices[&cell->get_fe()] =
-	  std_cxx1x::shared_ptr<FullMatrix<double> >
-	  (new FullMatrix<double>(dofs_per_cell1, dofs_per_cell1));
-	get_back_interpolation_matrix(dof1.get_fe(), fe2,
-				      *interpolation_matrices[&cell->get_fe()]);
-      }
+          if (interpolation_matrices[&cell->get_fe()] != 0)
+            {
+              interpolation_matrices[&cell->get_fe()] =
+                std_cxx1x::shared_ptr<FullMatrix<double> >
+                  (new FullMatrix<double>(dofs_per_cell1, dofs_per_cell1));
+              get_back_interpolation_matrix(dof1.get_fe(), fe2,
+                                            *interpolation_matrices[&cell->get_fe()]);
+            }
 
-    u1_local.reinit (dofs_per_cell1);
-    u1_int_local.reinit (dofs_per_cell1);
+          u1_local.reinit (dofs_per_cell1);
+          u1_int_local.reinit (dofs_per_cell1);
 
-    cell->get_dof_values(u1, u1_local);
-    interpolation_matrices[&cell->get_fe()]->vmult(u1_int_local, u1_local);
-    cell->set_dof_values(u1_int_local, u1_interpolated);
+          cell->get_dof_values(u1, u1_local);
+          interpolation_matrices[&cell->get_fe()]->vmult(u1_int_local, u1_local);
+          cell->set_dof_values(u1_int_local, u1_interpolated);
+        };
+
+				   // if we work on a parallel PETSc vector
+				   // we have to finish the work and
+				   // update ghost values
+    u1_interpolated.compress();
+    u1_interpolated.update_ghost_values();
   }
-}
 
 
 
@@ -1493,9 +1635,34 @@ for (; cell!=endc; ++cell)
 					 // interpolate back to dof1
 					 // taking into account
 					 // constraints1
-	Vector<typename OutVector::value_type> u2(dof2.n_dofs());
-	interpolate(dof1, u1, dof2, constraints2, u2);
-	interpolate(dof2, u2, dof1, constraints1, u1_interpolated);
+#ifdef DEAL_II_USE_PETSC
+        if (dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1) != 0)
+          {
+            AssertThrow (dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1_interpolated) != 0,
+                         ExcMessage("Interpolation can only be done between vectors of same type!"));
+
+					 // if u1 is a parallel distributed
+					 // PETSc vector, we create a vector u2
+					 // with based on the sets of locally owned
+					 // and relevant dofs of dof2
+            IndexSet  dof2_locally_owned_dofs = dof2.locally_owned_dofs();
+            IndexSet  dof2_locally_relevant_dofs;
+            DoFTools::extract_locally_relevant_dofs (dof2,
+                                                     dof2_locally_relevant_dofs);
+
+            PETScWrappers::MPI::Vector  u2 (dynamic_cast<const PETScWrappers::MPI::Vector *> (&u1)->get_mpi_communicator(),
+                                            dof2_locally_owned_dofs,
+                                            dof2_locally_relevant_dofs);
+            interpolate(dof1, u1, dof2, constraints2, u2);
+            interpolate(dof2, u2, dof1, constraints1, u1_interpolated);
+          }
+        else
+#endif
+          {
+            Vector<typename OutVector::value_type> u2(dof2.n_dofs());
+            interpolate(dof1, u1, dof2, constraints2, u2);
+            interpolate(dof2, u2, dof1, constraints1, u1_interpolated);
+          }
       }
   }
 
@@ -1513,9 +1680,28 @@ for (; cell!=endc; ++cell)
     Assert(u1_difference.size()==dof1.n_dofs(),
 	   ExcDimensionMismatch(u1_difference.size(), dof1.n_dofs()));
 
+#ifdef DEAL_II_USE_PETSC
+    if (dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1) != 0)
+      if (dynamic_cast<const DoFHandler<dim>*>(&dof1) != 0)
+        {
+                                   // if u1 is a parallel distributed
+                                   // PETSc vector, we check the local
+                                   // size of u1 for safety
+          Assert(dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1)->local_size() == dof1.locally_owned_dofs().n_elements(),
+                 ExcDimensionMismatch(dynamic_cast<const PETScWrappers::MPI::Vector *>(&u1)->local_size(), dof1.locally_owned_dofs().n_elements()));
+        };
+
+    if (dynamic_cast<PETScWrappers::MPI::Vector *>(&u1_difference) != 0)
+      if (dynamic_cast<const DoFHandler<dim>*>(&dof1) != 0)
+        {
+          Assert(dynamic_cast<PETScWrappers::MPI::Vector *>(&u1_difference)->local_size() == dof1.locally_owned_dofs().n_elements(),
+                 ExcDimensionMismatch(dynamic_cast<PETScWrappers::MPI::Vector *>(&u1_difference)->local_size(), dof1.locally_owned_dofs().n_elements()));
+        };
+#endif
+
 				     // For continuous elements on grids
 				     // with hanging nodes we need
-				     // hnaging node
+				     // hanging node
 				     // constraints. Consequently, when
 				     // the elements are continuous no
 				     // hanging node constraints are
@@ -1528,6 +1714,9 @@ for (; cell!=endc; ++cell)
     Vector<typename OutVector::value_type> u1_local(dofs_per_cell);
     Vector<typename OutVector::value_type> u1_diff_local(dofs_per_cell);
 
+    const types::subdomain_id_t subdomain_id =
+      dof1.get_tria().locally_owned_subdomain();
+
     FullMatrix<double> difference_matrix(dofs_per_cell, dofs_per_cell);
     get_interpolation_difference_matrix(dof1.get_fe(), fe2,
 					difference_matrix);
@@ -1536,17 +1725,26 @@ for (; cell!=endc; ++cell)
 							    endc = dof1.end();
 
     for (; cell!=endc; ++cell)
-      {
-	if (hanging_nodes_not_allowed)
-	  for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
-	    Assert (cell->at_boundary(face) ||
-		    cell->neighbor(face)->level() == cell->level(),
-		    ExcHangingNodesNotAllowed(0));
+      if ((cell->subdomain_id() == subdomain_id)
+          ||
+          (subdomain_id == types::invalid_subdomain_id))
+        {
+	  if (hanging_nodes_not_allowed)
+	    for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
+	      Assert (cell->at_boundary(face) ||
+		      cell->neighbor(face)->level() == cell->level(),
+		      ExcHangingNodesNotAllowed(0));
 
-	cell->get_dof_values(u1, u1_local);
-	difference_matrix.vmult(u1_diff_local, u1_local);
-	cell->set_dof_values(u1_diff_local, u1_difference);
-      }
+	  cell->get_dof_values(u1, u1_local);
+	  difference_matrix.vmult(u1_diff_local, u1_local);
+	  cell->set_dof_values(u1_diff_local, u1_difference);
+        }
+
+				   // if we work on a parallel PETSc vector
+				   // we have to finish the work and
+				   // update ghost values
+      u1_difference.compress();
+      u1_difference.update_ghost_values();
   }
 
 
@@ -1570,10 +1768,17 @@ for (; cell!=endc; ++cell)
       {
 	back_interpolate(dof1, constraints1, u1, dof2, constraints2, u1_difference);
 	u1_difference.sadd(-1, u1);
+
+				   // if we work on a parallel PETSc vector
+				   // we have to finish the work and
+				   // update ghost values
+        u1_difference.compress();
+        u1_difference.update_ghost_values();
       }
   }
 
 
+
   template <int dim, class InVector, class OutVector, int spacedim>
   void project_dg(const DoFHandler<dim,spacedim> &dof1,
 		  const InVector &u1,
@@ -1996,16 +2201,15 @@ for (; cell!=endc; ++cell)
 	    else
 	      {
 		unsigned int position = name.find('(');
-		const std::string quadrature_name(name, 0, position-1);
-		name.erase(0,position);
+		const std::string quadrature_name(name, 0, position);
+		name.erase(0,position+1);
 		if (quadrature_name.compare("QGaussLobatto") == 0)
 		  {
 		    const std::pair<int,unsigned int> tmp
 		      = Utilities::get_integer_at_position (name, 0);
-		    name.erase(0, tmp.second+1);
-//TODO: Implement a get function taking Quadrature<1> in fe_tools.h.
-//return fe_name_map.find(name_part)->second->get(QGaussLobatto<1>(tmp.first));
-		    AssertThrow (false, ExcNotImplemented());
+				// delete "))"
+		    name.erase(0, tmp.second+2);
+		    return fe_name_map.find(name_part)->second->get(QGaussLobatto<1>(tmp.first));
 		  }
 		else
 		  {
