@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: mapping_cartesian.cc 22682 2010-11-10 16:24:30Z kanschat $
+//    $Id: mapping_cartesian.cc 24056 2011-08-12 23:09:22Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,16 +11,16 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/tensor.h>
-#include <base/quadrature.h>
-#include <base/qprojector.h>
-#include <base/memory_consumption.h>
-#include <lac/full_matrix.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_accessor.h>
-#include <fe/mapping_cartesian.h>
-#include <fe/fe_values.h>
+#include <deal.II/base/tensor.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/fe/mapping_cartesian.h>
+#include <deal.II/fe/fe_values.h>
 
 #include <cmath>
 #include <algorithm>
@@ -43,7 +43,7 @@ MappingCartesian<dim, spacedim>::InternalData::InternalData (const Quadrature<di
 
 
 template<int dim, int spacedim>
-unsigned int
+std::size_t
 MappingCartesian<dim, spacedim>::InternalData::memory_consumption () const
 {
   return (Mapping<dim, spacedim>::InternalDataBase::memory_consumption() +
@@ -258,6 +258,18 @@ MappingCartesian<dim, spacedim>::compute_fill (const typename Triangulation<dim,
 
       switch (dim)
         {
+	  case 1:
+	  {
+            static const Point<dim>
+              normals[GeometryInfo<1>::faces_per_cell]
+              = { Point<dim>(-1.),
+                  Point<dim>( 1.) };
+            std::fill (normal_vectors.begin(),
+                       normal_vectors.end(),
+                       normals[face_no]);
+            break;
+          }
+	    
           case 2:
           {
             static const Point<dim>
