@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: mapping_q.cc 22676 2010-11-10 15:21:40Z kronbichler $
+//    $Id: mapping_q.cc 24133 2011-08-21 00:22:17Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,19 +12,19 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/utilities.h>
-#include <base/polynomial.h>
-#include <base/quadrature.h>
-#include <base/quadrature_lib.h>
-#include <base/memory_consumption.h>
-#include <base/tensor_product_polynomials.h>
-#include <lac/full_matrix.h>
-#include <grid/tria_iterator.h>
-#include <grid/tria_boundary.h>
-#include <dofs/dof_accessor.h>
-#include <fe/fe_tools.h>
-#include <fe/mapping_q.h>
-#include <fe/fe_q.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/base/polynomial.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/base/tensor_product_polynomials.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/grid/tria_boundary.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/fe/fe_tools.h>
+#include <deal.II/fe/mapping_q.h>
+#include <deal.II/fe/fe_q.h>
 
 #include <numeric>
 #include <memory>
@@ -45,7 +45,7 @@ MappingQ<dim,spacedim>::InternalData::InternalData (const unsigned int n_shape_f
 
 
 template<int dim, int spacedim>
-unsigned int
+std::size_t
 MappingQ<dim,spacedim>::InternalData::memory_consumption () const
 {
   return (MappingQ1<dim,spacedim>::InternalData::memory_consumption () +
@@ -115,8 +115,9 @@ MappingQ<dim,spacedim>::MappingQ (const unsigned int p,
 		degree(p),
 		n_inner(Utilities::fixed_power<dim>(degree-1)),
 		n_outer((dim==1) ? 2 :
-			(dim==2) ? 4+4*(degree-1)
-			:8+12*(degree-1)+6*(degree-1)*(degree-1)),
+			((dim==2) ?
+			 4+4*(degree-1) :
+			 8+12*(degree-1)+6*(degree-1)*(degree-1))),
 		tensor_pols(0),
 		n_shape_functions(Utilities::fixed_power<dim>(degree+1)),
 		renumber(FETools::
@@ -157,7 +158,7 @@ MappingQ<dim,spacedim>::MappingQ (const MappingQ<dim,spacedim> &mapping)
 		MappingQ1<dim,spacedim>(),
 		degree(mapping.degree),
 		n_inner(mapping.n_inner),
-		n_outer(n_outer),
+		n_outer(mapping.n_outer),
 		tensor_pols(0),
 		n_shape_functions(mapping.n_shape_functions),
 		renumber(mapping.renumber),
@@ -1355,6 +1356,12 @@ transform_unit_to_real_cell (const typename Triangulation<dim,spacedim>::cell_it
                &*mdata);
 
   compute_mapping_support_points(cell, p_data->mapping_support_points);
+				   // If this should be Q1, ignore all
+				   // other support points.
+  if(p_data->shape_values.size()<p_data->mapping_support_points.size())
+    p_data->mapping_support_points.resize
+      (GeometryInfo<dim>::vertices_per_cell);
+
 
   return this->transform_unit_to_real_cell_internal(*p_data);
 }
@@ -1389,6 +1396,12 @@ transform_real_to_unit_cell (const typename Triangulation<dim,spacedim>::cell_it
 
       std::vector<Point<spacedim> > &points = mdata->mapping_support_points;
       compute_mapping_support_points (cell, points);
+				       // If this is a q1 mapping,
+				       // then only use the support
+				       // points on the vertices.
+      if(mdata->shape_values.size()<points.size())
+	points.resize(GeometryInfo<dim>::vertices_per_cell);
+
 
       this->transform_real_to_unit_cell_internal(cell, p, *mdata, p_unit);
     }
