@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: mapping_q1.cc 23028 2010-12-20 16:38:36Z bangerth $
+//    $Id: mapping_q1.cc 24090 2011-08-16 12:51:27Z kronbichler $
 //    Version: $Name$
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,16 +11,17 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/tensor.h>
-#include <base/quadrature.h>
-#include <base/qprojector.h>
-#include <base/memory_consumption.h>
-#include <lac/full_matrix.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_accessor.h>
-#include <fe/fe_values.h>
-#include <fe/mapping_q1.h>
+#include <deal.II/base/tensor.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/fe/mapping_q1.h>
+#include <deal.II/fe/mapping_q1_eulerian.h>
 
 #include <cmath>
 #include <algorithm>
@@ -46,7 +47,7 @@ MappingQ1<dim,spacedim>::InternalData::InternalData (const unsigned int n_shape_
 
 
 template<int dim, int spacedim>
-unsigned int
+std::size_t
 MappingQ1<dim,spacedim>::InternalData::memory_consumption () const
 {
   return (Mapping<dim,spacedim>::InternalDataBase::memory_consumption() +
@@ -910,73 +911,6 @@ MappingQ1<dim,spacedim>::fill_fe_values (
 
 
 
-template <>
-void
-MappingQ1<1,1>::fill_fe_face_values (
-  const Triangulation<1,1>::cell_iterator &,
-  const unsigned,
-  const Quadrature<0>&,
-  Mapping<1,1>::InternalDataBase&,
-  std::vector<Point<1> >&,
-  std::vector<double>&,
-  std::vector<Tensor<1,1> >&,
-  std::vector<Point<1> >&) const
-{
-  Assert(false, ExcNotImplemented());
-}
-
-
-
-template <>
-void
-MappingQ1<1,2>::fill_fe_face_values (
-  const Triangulation<1,2>::cell_iterator &,
-  const unsigned,
-  const Quadrature<0>&,
-  Mapping<1,2>::InternalDataBase&,
-  std::vector<Point<2> >&,
-  std::vector<double>&,
-  std::vector<Tensor<1,2> >&,
-  std::vector<Point<2> >&) const
-{
-  Assert(false, ExcNotImplemented());
-}
-
-
-
-template <>
-void
-MappingQ1<1,1>::fill_fe_subface_values (
-  const Triangulation<1,1>::cell_iterator &,
-  const unsigned,
-  const unsigned,
-  const Quadrature<0>&,
-  Mapping<1,1>::InternalDataBase&,
-  std::vector<Point<1> >&,
-  std::vector<double>&,
-  std::vector<Tensor<1,1> >&,
-  std::vector<Point<1> >&) const
-{
-  Assert(false, ExcNotImplemented());
-}
-
-
-
-template <>
-void
-MappingQ1<1,2>::fill_fe_subface_values (
-  const Triangulation<1,2>::cell_iterator &,
-  const unsigned,
-  const unsigned,
-  const Quadrature<0>&,
-  Mapping<1,2>::InternalDataBase&,
-  std::vector<Point<2> >&,
-  std::vector<double>&,
-  std::vector<Tensor<1,2> >&,
-  std::vector<Point<2> >&) const
-{
-  Assert(false, ExcNotImplemented());
-}
 
 
 
@@ -984,24 +918,6 @@ namespace internal
 {
   namespace
   {
-    template <int spacedim>
-    void
-    compute_fill_face (const dealii::MappingQ1<1,spacedim> &,
-		       const typename dealii::Triangulation<1,spacedim>::cell_iterator &,
-		       const unsigned int,
-		       const unsigned int,
-		       const unsigned int,
-		       const std::vector<double> &,
-		       typename dealii::MappingQ1<1,spacedim>::InternalData &,
-		       std::vector<double> &,
-		       std::vector<Tensor<1,spacedim> > &,
-		       std::vector<Point<spacedim> > &)
-    {
-      Assert(false, ExcNotImplemented());
-    }
-
-
-
     template <int dim, int spacedim>
     void
     compute_fill_face (const dealii::MappingQ1<dim,spacedim> &mapping,
@@ -1029,8 +945,11 @@ namespace internal
       		    ExcDimensionMismatch(JxW_values.size(), n_q_points));
 
 					   // map the unit tangentials to the
-					   // real cell
-	  for (unsigned int d=0; d<dim-1; ++d)
+					   // real cell. checking for d!=dim-1
+					   // eliminates compiler warnings
+					   // regarding unsigned expressions <
+					   // 0.
+	  for (unsigned int d=0; d!=dim-1; ++d)
 	    {
 	      Assert (face_no+GeometryInfo<dim>::faces_per_cell*d <
 		      data.unit_tangentials.size(),
@@ -1052,12 +971,35 @@ namespace internal
 	  if (dim == spacedim)
 	    {
 	      for (unsigned int i=0; i<n_q_points; ++i)
-		if (dim == 2)
-      	  	  cross_product (boundary_forms[i], data.aux[0][i]);
-		else if (dim == 3)
-      	  	  cross_product (boundary_forms[i], data.aux[0][i], data.aux[1][i]);
-		else
-		  Assert(false, ExcNotImplemented());
+		switch (dim)
+		  {
+		    case 1:
+							   // in 1d, we don't
+							   // have access to
+							   // any of the
+							   // data.aux fields
+							   // (because it has
+							   // only dim-1
+							   // components), but
+							   // we can still
+							   // compute the
+							   // boundary form
+							   // simply by simply
+							   // looking at the
+							   // number of the
+							   // face
+			  boundary_forms[i][0] = (face_no == 0 ?
+						  -1 : +1);
+			  break;
+		    case 2:
+			  cross_product (boundary_forms[i], data.aux[0][i]);
+			  break;
+		    case 3:
+			  cross_product (boundary_forms[i], data.aux[0][i], data.aux[1][i]);
+			  break;
+		    default:
+			  Assert(false, ExcNotImplemented());
+		  }
       	    }
 	  else
 	    {
@@ -1117,7 +1059,7 @@ namespace internal
 						       // FEFaceValues in 1d
 		      Assert (false, ExcNotImplemented());
 		      cross_product (boundary_forms[point],
-				     (face_no == 0 ? 1 : -1) * cell_normal);
+				     (face_no == 0 ? 1. : -1.) * cell_normal);
 		    }
 		  else if ( (dim==2) && (spacedim==3) )
 		    cross_product (boundary_forms[point],
@@ -1139,7 +1081,8 @@ namespace internal
       	      {
       	  	if (update_flags & update_JxW_values)
       	  	  {
-      	  	    JxW_values[i] = boundary_forms[i].norm() * weights[i];
+		    JxW_values[i] = boundary_forms[i].norm() * weights[i];
+
       	  	    if (subface_no!=deal_II_numbers::invalid_unsigned_int)
       	  	      {
       	  		const double area_ratio=GeometryInfo<dim>::subface_ratio(
@@ -1458,6 +1401,15 @@ MappingQ1<dim,spacedim>::transform_unit_to_real_cell (
 				   // points
   compute_mapping_support_points(cell, mdata->mapping_support_points);
 
+				   // Mapping support points can be
+				   // bigger than necessary. If this
+				   // is the case, force them to be
+				   // Q1.
+  if(mdata->mapping_support_points.size() > mdata->shape_values.size())
+    mdata->mapping_support_points.resize
+      (GeometryInfo<dim>::vertices_per_cell);
+
+
   return transform_unit_to_real_cell_internal(*mdata);
 }
 
@@ -1505,14 +1457,18 @@ transform_real_to_unit_cell (const typename Triangulation<dim,spacedim>::cell_it
   std::auto_ptr<InternalData>
     mdata(dynamic_cast<InternalData *> (
             MappingQ1<dim,spacedim>::get_data(update_transformation_values
-                                     | update_transformation_gradients,
-                                     point_quadrature)));
+					      | update_transformation_gradients,
+					      point_quadrature)));
 
-  MappingQ1<dim,spacedim>::compute_mapping_support_points (cell,
-							   mdata->mapping_support_points);
-  Assert(mdata->mapping_support_points.size() ==
-         GeometryInfo<dim>::vertices_per_cell,
+  compute_mapping_support_points (cell, mdata->mapping_support_points);
+				   // The support points have to be at
+				   // least as many as there are
+				   // vertices.
+  Assert(mdata->mapping_support_points.size() >=
+	 GeometryInfo<dim>::vertices_per_cell,
 	 ExcInternalError());
+				   // Ignore non vertex support points.
+  mdata->mapping_support_points.resize(GeometryInfo<dim>::vertices_per_cell);
 
 				   // perform the newton iteration.
   transform_real_to_unit_cell_internal(cell, p, *mdata, p_unit);
@@ -1521,35 +1477,6 @@ transform_real_to_unit_cell (const typename Triangulation<dim,spacedim>::cell_it
 }
 
 
-
-template<>
-void
-MappingQ1<1,2>::
-transform_real_to_unit_cell_internal
-(const Triangulation<1,2>::cell_iterator &,
- const Point<2>                            &,
- InternalData                                     &,
- Point<1>                                       &) const
-{
-	Assert(false, ExcNotImplemented());
-}
-
-
-
-template<>
-void
-MappingQ1<2,3>::
-transform_real_to_unit_cell_internal
-(const Triangulation<2,3>::cell_iterator &,
- const Point<3>                            &,
- InternalData                                     &,
- Point<2>                                       &) const
-{
-	Assert(false, ExcNotImplemented());
-}
-
-
-
 template<int dim, int spacedim>
 void
 MappingQ1<dim,spacedim>::
@@ -1566,6 +1493,7 @@ transform_real_to_unit_cell_internal
   std::vector<Point<spacedim> > &points=mdata.mapping_support_points;
   Assert(points.size()==n_shapes, ExcInternalError());
 
+
 				   // Newton iteration to solve
 				   // f(x)=p(x)-p=0
 				   // x_{n+1}=x_n-[f'(x)]^{-1}f(x)
@@ -1577,6 +1505,14 @@ transform_real_to_unit_cell_internal
 				   // of the mapping at this point are
 				   // previously computed.
 
+				   // In the codimension one case, the
+				   // f' is rectangular, and it is
+				   // augmented with the normal to the
+				   // cell. The tranformation from
+				   // real to unit is intented as a
+				   // projection to the surface (or
+				   // curve) generated by the given
+				   // cell.
 
 				   // f(x)
   Point<spacedim> p_real(transform_unit_to_real_cell_internal(mdata));
@@ -1587,26 +1523,55 @@ transform_real_to_unit_cell_internal
   while (f.square()>eps*eps && loop++<10)
     {
 				       // f'(x)
-      Tensor<2,dim> df;
+      Tensor<2,spacedim> df;
       for (unsigned int k=0; k<mdata.n_shape_functions; ++k)
 	{
 	  const Tensor<1,dim> &grad_transform=mdata.derivative(0,k);
 	  const Point<spacedim> &point=points[k];
 
-	  for (unsigned int i=0; i<dim; ++i)
+	  for (unsigned int i=0; i<spacedim; ++i)
 	    for (unsigned int j=0; j<dim; ++j)
 	      df[i][j]+=point[i]*grad_transform[j];
+	  if(dim<spacedim)
+	    {
+					       // Temporarily
+					       // transpose df, to
+					       // access gradients as
+					       // a single index, and
+					       // put in the dimth row
+					       // the normal to this
+					       // cell.
+	      df = transpose(df);
+	      if ( (dim==1) && (spacedim==2) )
+		cross_product(df[1],-df[0]);
+	      else if ( (dim==2) && (spacedim==3) )
+		cross_product(df[2], df[0], df[1]);
+	      else
+		Assert (false, ExcNotImplemented());
+					       // Transpose back
+	      df = transpose(df);
+	    }
 	}
 
 				       // Solve  [f'(x)]d=f(x)
-      Tensor<1,dim> d;
-      Tensor<2,dim> df_1;
+      Tensor<1,spacedim> d;
+      Tensor<2,spacedim> df_1;
 
       df_1 = invert(df);
-      contract (d, df_1, static_cast<const Tensor<1,dim>&>(f));
+      contract (d, df_1, static_cast<const Tensor<1,spacedim>&>(f));
+
+				       // update of p_unit. The
+				       // spacedimth component of
+				       // transformed point is simply
+				       // ignored in codimension one
+				       // case. When this component is
+				       // not zero, then we are
+				       // projecting the point to the
+				       // surface or curve identified
+				       // by the cell.
+      for(unsigned int i=0;i<dim; ++i)
+	p_unit[i] -= d[i];
 
-				       // update of p_unit
-      p_unit -= d;
 				       // shape values and derivatives
 				       // at new p_unit point
       compute_shapes(std::vector<Point<dim> > (1, p_unit), mdata);
@@ -1614,6 +1579,13 @@ transform_real_to_unit_cell_internal
 				       // f(x)
       p_real = transform_unit_to_real_cell_internal(mdata);
       f = p_real-p;
+				       // In the codimension one case,
+				       // we only project on the
+				       // surface, i.e., we remove the
+				       // normal component of the
+				       // difference.
+      if(dim<spacedim)
+	f -= (transpose(df)[dim]*f)*f;
     }
 }
 
