@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: grid_generator.cc 22691 2010-11-11 18:22:14Z kanschat $
+//    $Id: grid_generator.cc 24288 2011-09-09 02:23:06Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,27 +12,28 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/quadrature_lib.h>
-#include <base/thread_management.h>
-#include <lac/vector.h>
-#include <lac/vector_memory.h>
-#include <lac/filtered_matrix.h>
-#include <lac/precondition.h>
-#include <lac/solver_cg.h>
-#include <lac/sparse_matrix.h>
-#include <lac/constraint_matrix.h>
-#include <grid/grid_generator.h>
-#include <grid/grid_reordering.h>
-#include <grid/grid_tools.h>
-#include <grid/tria.h>
-#include <grid/tria_accessor.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_handler.h>
-#include <dofs/dof_accessor.h>
-#include <dofs/dof_tools.h>
-#include <fe/mapping_q1.h>
-#include <fe/fe_q.h>
-#include <numerics/matrices.h>
+#include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/thread_management.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/vector_memory.h>
+#include <deal.II/lac/filtered_matrix.h>
+#include <deal.II/lac/precondition.h>
+#include <deal.II/lac/solver_cg.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/constraint_matrix.h>
+#include <deal.II/grid/grid_generator.h>
+#include <deal.II/grid/grid_reordering.h>
+#include <deal.II/grid/grid_tools.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/grid/tria_boundary_lib.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/dofs/dof_tools.h>
+#include <deal.II/fe/mapping_q1.h>
+#include <deal.II/fe/fe_q.h>
+#include <deal.II/numerics/matrices.h>
 
 #include <iostream>
 #include <cmath>
@@ -231,6 +232,141 @@ GridGenerator::moebius (
 
 
 
+void
+GridGenerator::torus (Triangulation<2,3>&  tria,
+		      const double         R,
+		      const double         r)
+{
+  Assert (R>r, ExcMessage("Outer radius must be greater than inner radius."));
+
+  const unsigned int dim=2;
+  const unsigned int spacedim=3;
+  std::vector<Point<spacedim> > vertices (16);
+
+  vertices[0]=Point<spacedim>(R-r,0,0);
+  vertices[1]=Point<spacedim>(R,-r,0);
+  vertices[2]=Point<spacedim>(R+r,0,0);
+  vertices[3]=Point<spacedim>(R, r,0);
+  vertices[4]=Point<spacedim>(0,0,R-r);
+  vertices[5]=Point<spacedim>(0,-r,R);
+  vertices[6]=Point<spacedim>(0,0,R+r);
+  vertices[7]=Point<spacedim>(0,r,R);
+  vertices[8]=Point<spacedim>(-(R-r),0,0);
+  vertices[9]=Point<spacedim>(-R,-r,0);
+  vertices[10]=Point<spacedim>(-(R+r),0,0);
+  vertices[11]=Point<spacedim>(-R, r,0);
+  vertices[12]=Point<spacedim>(0,0,-(R-r));
+  vertices[13]=Point<spacedim>(0,-r,-R);
+  vertices[14]=Point<spacedim>(0,0,-(R+r));
+  vertices[15]=Point<spacedim>(0,r,-R);
+
+  std::vector<CellData<dim> > cells (16);
+				   //Right Hand Orientation
+  cells[0].vertices[0] =  0;
+  cells[0].vertices[1] =  4;
+  cells[0].vertices[2] =  7;
+  cells[0].vertices[3] =  3;
+  cells[0].material_id = 0;
+
+  cells[1].vertices[0] =  1;
+  cells[1].vertices[1] =  5;
+  cells[1].vertices[2] =  4;
+  cells[1].vertices[3] =  0;
+  cells[1].material_id = 0;
+
+  cells[2].vertices[0] =  2;
+  cells[2].vertices[1] =  6;
+  cells[2].vertices[2] =  5;
+  cells[2].vertices[3] =  1;
+  cells[2].material_id = 0;
+
+  cells[3].vertices[0] =  3;
+  cells[3].vertices[1] =  7;
+  cells[3].vertices[2] =  6;
+  cells[3].vertices[3] =  2;
+  cells[3].material_id = 0;
+
+  cells[4].vertices[0] =  4;
+  cells[4].vertices[1] =  8;
+  cells[4].vertices[2] =  11;
+  cells[4].vertices[3] =  7;
+  cells[4].material_id = 0;
+
+  cells[5].vertices[0] =  5;
+  cells[5].vertices[1] =  9;
+  cells[5].vertices[2] =  8;
+  cells[5].vertices[3] =  4;
+  cells[5].material_id = 0;
+
+  cells[6].vertices[0] =  6;
+  cells[6].vertices[1] =  10;
+  cells[6].vertices[2] =  9;
+  cells[6].vertices[3] =  5;
+  cells[6].material_id = 0;
+
+  cells[7].vertices[0] =  7;
+  cells[7].vertices[1] =  11;
+  cells[7].vertices[2] =  10;
+  cells[7].vertices[3] =  6;
+  cells[7].material_id = 0;
+
+  cells[8].vertices[0] =  8;
+  cells[8].vertices[1] =  12;
+  cells[8].vertices[2] =  15;
+  cells[8].vertices[3] =  11;
+  cells[8].material_id = 0;
+
+  cells[9].vertices[0] =  9;
+  cells[9].vertices[1] =  13;
+  cells[9].vertices[2] =  12;
+  cells[9].vertices[3] =  8;
+  cells[9].material_id = 0;
+
+  cells[10].vertices[0] =  10;
+  cells[10].vertices[1] =  14;
+  cells[10].vertices[2] =  13;
+  cells[10].vertices[3] =  9;
+  cells[10].material_id = 0;
+
+  cells[11].vertices[0] =  11;
+  cells[11].vertices[1] =  15;
+  cells[11].vertices[2] =  14;
+  cells[11].vertices[3] =  10;
+  cells[11].material_id = 0;
+
+  cells[12].vertices[0] =  12;
+  cells[12].vertices[1] =  0;
+  cells[12].vertices[2] =  3;
+  cells[12].vertices[3] =  15;
+  cells[12].material_id = 0;
+
+  cells[13].vertices[0] =  13;
+  cells[13].vertices[1] =  1;
+  cells[13].vertices[2] =  0;
+  cells[13].vertices[3] =  12;
+  cells[13].material_id = 0;
+
+  cells[14].vertices[0] =  14;
+  cells[14].vertices[1] =  2;
+  cells[14].vertices[2] =  1;
+  cells[14].vertices[3] =  13;
+  cells[14].material_id = 0;
+
+  cells[15].vertices[0] =  15;
+  cells[15].vertices[1] =  3;
+  cells[15].vertices[2] =  2;
+  cells[15].vertices[3] =  14;
+  cells[15].material_id = 0;
+
+				   // Must call this to be able to create a
+				   // correct triangulation in dealii, read
+				   // GridReordering<> doc
+  GridReordering<dim,spacedim>::reorder_cells (cells);
+  tria.create_triangulation_compatibility (vertices, cells, SubCellData());
+}
+
+
+
 
 // Implementation for 2D only
 template<>
@@ -1207,12 +1343,22 @@ GridGenerator::half_hyper_shell (Triangulation<1>&,
 				 const Point<1>&,
 				 const double,
 				 const double,
-				 const unsigned int)
+				 const unsigned int,
+				 const bool)
 {
   Assert (false, ExcNotImplemented());
 }
 
-
+template <>
+void GridGenerator::quarter_hyper_shell (Triangulation<1>&,
+					 const Point<1>&,
+					 const double,
+					 const double,
+					 const unsigned int,
+					 const bool)
+{
+  Assert (false, ExcNotImplemented());
+}
 
 template <>
 void GridGenerator::enclosed_hyper_cube (Triangulation<2> &tria,
@@ -1441,6 +1587,7 @@ GridGenerator::colorize_hyper_shell (
   for (Triangulation<2>::cell_iterator cell = tria.begin();
        cell != tria.end(); ++cell)
     {
+      Assert (cell->face(2)->at_boundary(), ExcInternalError());
       cell->face(2)->set_boundary_indicator(1);
     }
 }
@@ -1624,7 +1771,8 @@ GridGenerator::half_hyper_shell (Triangulation<2>   &tria,
 				 const Point<2>     &center,
 				 const double        inner_radius,
 				 const double        outer_radius,
-				 const unsigned int  n_cells)
+				 const unsigned int  n_cells,
+				 const bool colorize)
 {
   Assert ((inner_radius > 0) && (inner_radius < outer_radius),
 	  ExcInvalidRadii ());
@@ -1684,9 +1832,100 @@ GridGenerator::half_hyper_shell (Triangulation<2>   &tria,
     };
 
   tria.create_triangulation (vertices, cells, SubCellData());
+
+  if (colorize)
+  {
+      Triangulation<2>::cell_iterator cell = tria.begin();
+      for (;cell!=tria.end();++cell)
+      {
+	cell->face(2)->set_boundary_indicator(1);
+      }
+      tria.begin()->face(0)->set_boundary_indicator(3);
+
+      tria.last()->face(1)->set_boundary_indicator(2);
+  }
 }
 
 
+template <>
+void GridGenerator::quarter_hyper_shell (Triangulation<2>   &tria,
+				  const Point<2>     &center,
+				  const double        inner_radius,
+				  const double        outer_radius,
+				  const unsigned int  n_cells,
+				  const bool colorize)
+    {
+  Assert ((inner_radius > 0) && (inner_radius < outer_radius),
+	  ExcInvalidRadii ());
+
+  const double pi     = numbers::PI;
+				   // determine the number of cells
+				   // for the grid. if not provided by
+				   // the user determine it such that
+				   // the length of each cell on the
+				   // median (in the middle between
+				   // the two circles) is equal to its
+				   // radial extent (which is the
+				   // difference between the two
+				   // radii)
+  const unsigned int N = (n_cells == 0 ?
+			  static_cast<unsigned int>
+			  (std::ceil((pi* (outer_radius + inner_radius)/4) /
+				     (outer_radius - inner_radius))) :
+			  n_cells);
+
+				   // set up N+1 vertices on the
+				   // outer and N+1 vertices on
+				   // the inner circle. the
+				   // first N+1 ones are on the
+				   // outer one, and all are
+				   // numbered counter-clockwise
+  std::vector<Point<2> > vertices(2*(N+1));
+  for (unsigned int i=0; i<=N; ++i)
+    {
+				       // enforce that the x-coordinates
+				       // of the last point is exactly
+				       // zero (contrary to what we may
+				       // compute using the imprecise
+				       // value of pi)
+      vertices[i] =  Point<2>( ( (i==N) ?
+				 0 :
+				 std::cos(pi * i/N/2) ),
+			       std::sin(pi * i/N/2)) * outer_radius;
+      vertices[i+N+1] = vertices[i] * (inner_radius/outer_radius);
+
+      vertices[i]     += center;
+      vertices[i+N+1] += center;
+    };
+
+
+  std::vector<CellData<2> > cells (N, CellData<2>());
+
+  for (unsigned int i=0; i<N; ++i)
+    {
+      cells[i].vertices[0] = i;
+      cells[i].vertices[1] = (i+1)%(N+1);
+      cells[i].vertices[2] = N+1+i;
+      cells[i].vertices[3] = N+1+((i+1)%(N+1));
+
+      cells[i].material_id = 0;
+    };
+
+  tria.create_triangulation (vertices, cells, SubCellData());
+
+  if (colorize)
+  {
+      Triangulation<2>::cell_iterator cell = tria.begin();
+      for (;cell!=tria.end();++cell)
+      {
+	cell->face(2)->set_boundary_indicator(1);
+      }
+      tria.begin()->face(0)->set_boundary_indicator(3);
+
+      tria.last()->face(1)->set_boundary_indicator(2);
+  }
+}
+
 
 
 // Implementation for 3D only
@@ -1818,10 +2057,11 @@ void GridGenerator::truncated_cone (Triangulation<3> &triangulation,
 				    const double half_length)
 {
 				   // Determine number of cells and vertices
-  const unsigned int n_cells = static_cast<unsigned int>(std::floor (half_length /
-								     std::max (radius_0,
-									       radius_1) +
-								     0.5));
+  const unsigned int
+    n_cells = static_cast<unsigned int>(std::floor (half_length /
+						    std::max (radius_0,
+							      radius_1) +
+						    0.5));
   const unsigned int n_vertices = 4 * (n_cells + 1);
   std::vector<Point<3> > vertices_tmp(n_vertices);
 
@@ -1847,7 +2087,8 @@ void GridGenerator::truncated_cone (Triangulation<3> &triangulation,
       Point<3> (dx, 0, 0.5 * (radius_1 - radius_0) * dx / half_length);
   }
 
-  const std::vector<Point<3> > vertices (&vertices_tmp[0], &vertices_tmp[n_vertices]);
+  const std::vector<Point<3> > vertices (vertices_tmp.begin(),
+					 vertices_tmp.end());
   Table<2,unsigned int> cell_vertices(n_cells,GeometryInfo<3>::vertices_per_cell);
 
   for (unsigned int i = 0; i < n_cells; ++i)
@@ -2229,44 +2470,86 @@ GridGenerator::half_hyper_ball (Triangulation<3>& tria,
 // Implementation for 3D only
 template<>
 void
-GridGenerator::colorize_hyper_shell (
-  Triangulation<3>& tria,
-  const Point<3>&, const double, const double)
+GridGenerator::
+colorize_hyper_shell (Triangulation<3>& tria,
+		      const Point<3>&,
+		      const double,
+		      const double)
 {
-				   // Inspite of receiving geometrical
-				   // data, we do this only based on
-				   // topology.
-
-				   // For the mesh based on  cube,
-				   // this is highly irregular
+				   // the following uses a good amount
+				   // of knowledge about the
+				   // orientation of cells. this is
+				   // probably not good style...
   if (tria.n_cells() == 6)
     {
       Triangulation<3>::cell_iterator cell = tria.begin();
+
       cell->face(4)->set_boundary_indicator(1);
+      Assert (cell->face(4)->at_boundary(), ExcInternalError());
+
       (++cell)->face(2)->set_boundary_indicator(1);
+      Assert (cell->face(2)->at_boundary(), ExcInternalError());
+
       (++cell)->face(2)->set_boundary_indicator(1);
+      Assert (cell->face(2)->at_boundary(), ExcInternalError());
+
       (++cell)->face(0)->set_boundary_indicator(1);
+      Assert (cell->face(0)->at_boundary(), ExcInternalError());
+
       (++cell)->face(2)->set_boundary_indicator(1);
+      Assert (cell->face(2)->at_boundary(), ExcInternalError());
+
       (++cell)->face(0)->set_boundary_indicator(1);
+      Assert (cell->face(0)->at_boundary(), ExcInternalError());
     }
-  else
-				     // For higher polyhedra, this is regular.
+  else if (tria.n_cells() == 12)
     {
+				       // again use some internal
+				       // knowledge
       for (Triangulation<3>::cell_iterator cell = tria.begin();
 	   cell != tria.end(); ++cell)
-	cell->face(5)->set_boundary_indicator(1);
+	{
+	  Assert (cell->face(5)->at_boundary(), ExcInternalError());
+	  cell->face(5)->set_boundary_indicator(1);
+	}
     }
+  else if (tria.n_cells() == 96)
+    {
+				       // the 96-cell hypershell is
+				       // based on a once refined
+				       // 12-cell mesh. consequently,
+				       // since the outer faces all
+				       // are face_no==5 above, so
+				       // they are here (unless they
+				       // are in the interior). Use
+				       // this to assign boundary
+				       // indicators, but also make
+				       // sure that we encounter
+				       // exactly 48 such faces
+      unsigned int count = 0;
+      for (Triangulation<3>::cell_iterator cell = tria.begin();
+	   cell != tria.end(); ++cell)
+	if (cell->face(5)->at_boundary())
+	  {
+	    cell->face(5)->set_boundary_indicator(1);
+	    ++count;
+	  }
+      Assert (count == 48, ExcInternalError());
+    }
+  else
+    Assert (false, ExcNotImplemented());
 }
 
 
 
 template <>
-void GridGenerator::hyper_shell (Triangulation<3>& tria,
-				 const Point<3>& p,
-				 const double inner_radius,
-				 const double outer_radius,
-				 const unsigned int n,
-				 const bool colorize)
+void
+GridGenerator::hyper_shell (Triangulation<3>& tria,
+			    const Point<3>& p,
+			    const double inner_radius,
+			    const double outer_radius,
+			    const unsigned int n,
+			    const bool colorize)
 {
   Assert ((inner_radius > 0) && (inner_radius < outer_radius),
 	  ExcInvalidRadii ());
@@ -2278,7 +2561,7 @@ void GridGenerator::hyper_shell (Triangulation<3>& tria,
 
 				   // Start with the shell bounded by
 				   // two nested cubes
-  if (n <= 6)
+  if (n == 6)
     {
       for (unsigned int i=0;i<8;++i)
 	vertices.push_back(p+hexahedron[i]*irad);
@@ -2302,11 +2585,13 @@ void GridGenerator::hyper_shell (Triangulation<3>& tria,
 	    cells[i].vertices[j] = cell_vertices[i][j];
 	  cells[i].material_id = 0;
 	}
+
+      tria.create_triangulation (vertices, cells, SubCellData());
     }
 				   // A more regular subdivision can
 				   // be obtained by two nested
 				   // rhombic dodecahedra
-  else if (n <= 12)
+  else if (n == 12)
     {
       for (unsigned int i=0;i<8;++i)
 	vertices.push_back(p+hexahedron[i]*irad);
@@ -2343,16 +2628,137 @@ void GridGenerator::hyper_shell (Triangulation<3>& tria,
 	    }
 	  cells[i].material_id = 0;
 	}
+
+      tria.create_triangulation (vertices, cells, SubCellData());
+    }
+  else if (n == 96)
+    {
+				       // create a triangulation based on the
+				       // 12-cell one where we refine the mesh
+				       // once and then re-arrange all
+				       // interior nodes so that the mesh is
+				       // the least distorted
+      HyperShellBoundary<3> boundary (p);
+      Triangulation<3> tmp;
+      GridGenerator::hyper_shell (tmp, p, inner_radius, outer_radius, 12);
+      tmp.set_boundary(0, boundary);
+      tmp.set_boundary(1, boundary);
+      tmp.refine_global (1);
+
+				       // let's determine the distance at
+				       // which the interior nodes should be
+				       // from the center. let's say we
+				       // measure distances in multiples of
+				       // outer_radius and call
+				       // r=inner_radius.
+				       //
+				       // then note
+				       // that we now have 48 faces on the
+				       // inner and 48 on the outer sphere,
+				       // each with an area of approximately
+				       // 4*pi/48*r^2 and 4*pi/48, for
+				       // a face edge length of approximately
+				       // sqrt(pi/12)*r and sqrt(pi/12)
+				       //
+				       // let's say we put the interior nodes
+				       // at a distance rho, then a measure of
+				       // deformation for the inner cells
+				       // would be
+				       //   di=max(sqrt(pi/12)*r/(rho-r),
+				       //          (rho-r)/sqrt(pi/12)/r)
+				       // and for the outer cells
+				       //   do=max(sqrt(pi/12)/(1-rho),
+				       //          (1-rho)/sqrt(pi/12))
+				       //
+				       // we now seek a rho so that the
+				       // deformation of cells on the inside
+				       // and outside is equal. there are in
+				       // principle four possibilities for one
+				       // of the branches of do== one of the
+				       // branches of di, though not all of
+				       // them satisfy do==di, of
+				       // course. however, we are not
+				       // interested in cases where the inner
+				       // cell is long and skinny and the
+				       // outer one tall -- yes, they have the
+				       // same aspect ratio, but in different
+				       // space directions.
+				       //
+				       // so it only boils down to the
+				       // following two possibilities: the
+				       // first branch of each max(.,.)
+				       // functions are equal, or the second
+				       // one are. on the other hand, since
+				       // they two branches are reciprocals of
+				       // each other, if one pair of branches
+				       // is equal, so is the other
+				       //
+				       // this yields the following equation
+				       // for rho:
+				       //   sqrt(pi/12)*r/(rho-r)
+				       //   == sqrt(pi/12)/(1-rho)
+				       // with solution rho=2r/(1+r)
+      const double r = inner_radius / outer_radius;
+      const double rho = 2*r/(1+r);
+
+				       // then this is the distance of the
+				       // interior nodes from the center:
+      const double middle_radius = rho * outer_radius;
+
+				       // mark vertices we've already moved or
+				       // that we want to ignore: we don't
+				       // want to move vertices at the inner
+				       // or outer boundaries
+      std::vector<bool> vertex_already_treated (tmp.n_vertices(), false);
+      for (Triangulation<3>::active_cell_iterator cell = tmp.begin_active();
+	   cell != tmp.end(); ++cell)
+	for (unsigned int f=0; f<GeometryInfo<3>::faces_per_cell; ++f)
+	  if (cell->at_boundary(f))
+	    for (unsigned int v=0; v<GeometryInfo<3>::vertices_per_face; ++v)
+	      vertex_already_treated[cell->face(f)->vertex_index(v)] = true;
+
+				       // now move the remaining vertices
+      for (Triangulation<3>::active_cell_iterator cell = tmp.begin_active();
+	   cell != tmp.end(); ++cell)
+	for (unsigned int v=0; v<GeometryInfo<3>::vertices_per_cell; ++v)
+	  if (vertex_already_treated[cell->vertex_index(v)] == false)
+	    {
+					       // this is a new interior
+					       // vertex. mesh refinement may
+					       // have placed it at a number
+					       // of places in radial
+					       // direction and oftentimes not
+					       // in a particularly good
+					       // one. move it to halfway
+					       // between inner and outer
+					       // sphere
+	      const Point<3> old_distance = cell->vertex(v) - p;
+	      const double old_radius = cell->vertex(v).distance(p);
+	      cell->vertex(v) = p + old_distance * (middle_radius / old_radius);
+
+	      vertex_already_treated[cell->vertex_index(v)] = true;
+	    }
+
+				       // now copy the resulting level 1 cells
+				       // into the new triangulation,
+      cells.resize(tmp.n_active_cells(), CellData<3>());
+
+      unsigned int index = 0;
+      for (Triangulation<3>::active_cell_iterator cell = tmp.begin_active();
+	   cell != tmp.end(); ++cell, ++index)
+	{
+	  for (unsigned int v=0; v<GeometryInfo<3>::vertices_per_cell; ++v)
+	    cells[index].vertices[v] = cell->vertex_index(v);
+      	  cells[index].material_id = 0;
+	}
+
+      tria.create_triangulation (tmp.get_vertices(), cells, SubCellData());
     }
   else
     {
-      Assert(false, ExcIndexRange(n, 1, 7));
+      Assert(false, ExcMessage ("Invalid number of coarse mesh cells."));
     }
 
-  tria.create_triangulation (vertices, cells,
-			     SubCellData());       // no boundary
-						   // information
-
   if (colorize)
     colorize_hyper_shell(tria, p, inner_radius, outer_radius);
 }
@@ -2367,10 +2773,12 @@ GridGenerator::half_hyper_shell (Triangulation<3>& tria,
 				 const Point<3>& center,
 				 const double inner_radius,
 				 const double outer_radius,
-				 const unsigned int n)
+				 const unsigned int n,
+				 const bool colorize)
 {
   Assert ((inner_radius > 0) && (inner_radius < outer_radius),
 	  ExcInvalidRadii ());
+  Assert(colorize == false, ExcNotImplemented());
 
   if (n <= 5)
     {
@@ -2434,6 +2842,17 @@ GridGenerator::half_hyper_shell (Triangulation<3>& tria,
 }
 
 
+template <>
+void GridGenerator::quarter_hyper_shell (Triangulation<3> &,
+					 const Point<3>&,
+					 const double,
+					 const double,
+					 const unsigned int,
+					 const bool)
+{
+  Assert (false, ExcNotImplemented());
+}
+
 
 // Implementation for 3D only
 template <>
@@ -2518,6 +2937,59 @@ void GridGenerator::cylinder_shell (Triangulation<3>   &tria,
 
 
 
+template <int dim, int spacedim>
+void
+GridGenerator::
+merge_triangulations (const Triangulation<dim, spacedim> &triangulation_1,
+		      const Triangulation<dim, spacedim> &triangulation_2,
+		      Triangulation<dim, spacedim>       &result)
+{
+  Assert (triangulation_1.n_levels() == 1,
+	  ExcMessage ("The input triangulations must be coarse meshes."));
+  Assert (triangulation_2.n_levels() == 1,
+	  ExcMessage ("The input triangulations must be coarse meshes."));
+
+  // get the union of the set of vertices
+  std::vector<Point<spacedim> > vertices = triangulation_1.get_vertices();
+  vertices.insert (vertices.end(),
+		   triangulation_2.get_vertices().begin(),
+		   triangulation_2.get_vertices().end());
+
+  // now form the union of the set of cells
+  std::vector<CellData<dim> > cells;
+  cells.reserve (triangulation_1.n_cells() + triangulation_2.n_cells());
+  for (typename Triangulation<dim,spacedim>::cell_iterator
+    cell = triangulation_1.begin(); cell != triangulation_1.end(); ++cell)
+  {
+    CellData<dim> this_cell;
+    for (unsigned int v=0; v<GeometryInfo<dim>::vertices_per_cell; ++v)
+      this_cell.vertices[v] = cell->vertex_index(v);
+    this_cell.material_id = cell->material_id();
+    cells.push_back (this_cell);
+  }
+
+  // now do the same for the other other mesh. note that we have to
+  // translate the vertex indices
+  for (typename Triangulation<dim,spacedim>::cell_iterator
+    cell = triangulation_2.begin(); cell != triangulation_2.end(); ++cell)
+  {
+    CellData<dim> this_cell;
+    for (unsigned int v=0; v<GeometryInfo<dim>::vertices_per_cell; ++v)
+      this_cell.vertices[v] = cell->vertex_index(v) + triangulation_1.n_vertices();
+    this_cell.material_id = cell->material_id();
+    cells.push_back (this_cell);
+  }
+
+  // throw out duplicated vertices from the two meshes
+  // and create the triangulation
+  SubCellData subcell_data;
+  std::vector<unsigned int> considered_vertices;
+  GridTools::delete_duplicated_vertices (vertices, cells, subcell_data, considered_vertices);
+  result.clear ();
+  result.create_triangulation (vertices, cells, subcell_data);
+}
+
+
 
 // make the following function inline. this is so that it becomes marked
 // internal/weak for the linker and we don't get multiply defined errors
