@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: grid_in.cc 22655 2010-11-09 19:27:22Z bangerth $
+//    $Id: grid_in.cc 23904 2011-07-01 14:36:09Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,13 +11,13 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/path_search.h>
-#include <base/utilities.h>
+#include <deal.II/base/path_search.h>
+#include <deal.II/base/utilities.h>
 
-#include <grid/grid_in.h>
-#include <grid/tria.h>
-#include <grid/grid_reordering.h>
-#include <grid/grid_tools.h>
+#include <deal.II/grid/grid_in.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/grid_reordering.h>
+#include <deal.II/grid/grid_tools.h>
 
 #include <map>
 #include <algorithm>
@@ -1666,7 +1666,7 @@ void GridIn<2>::read_tecplot (std::istream &in)
 	  if (next_index==dim && structured)
 	    break;
 
-	  if (i==tecplot2deal[next_index])
+	  if ((next_index<dim) && (i==tecplot2deal[next_index]))
 	    {
 					       // we need this line, read it in
 	      for (unsigned int j=1; j<n_vertices+1; ++j)
