@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: grid_out.cc 22656 2010-11-09 19:55:31Z kanschat $
+//    $Id: grid_out.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2008, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2008, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,15 +12,15 @@
 //---------------------------------------------------------------------------
 
 
-#include <grid/grid_out.h>
-#include <base/parameter_handler.h>
-#include <base/point.h>
-#include <base/quadrature.h>
-#include <base/qprojector.h>
-#include <grid/tria.h>
-#include <grid/tria_accessor.h>
-#include <grid/tria_iterator.h>
-#include <fe/mapping.h>
+#include <deal.II/grid/grid_out.h>
+#include <deal.II/base/parameter_handler.h>
+#include <deal.II/base/point.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/base/qprojector.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/fe/mapping.h>
 
 #include <cstring>
 #include <iomanip>
@@ -549,7 +549,7 @@ GridOut::parse_parameters(ParameterHandler& param)
 
 
 
-unsigned int
+std::size_t
 GridOut::memory_consumption () const
 {
   return (sizeof(dx_flags) +
