@@ -2,7 +2,7 @@
 //   grid_reordering.cc,v 1.27 2002/05/28 07:43:22 wolf Exp
 //   Version:
 //
-//   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2010 by the deal.II authors
+//   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2010, 2011 by the deal.II authors
 //
 //   This file is subject to QPL and may not be  distributed
 //   without copyright and license information. Please refer
@@ -11,9 +11,10 @@
 //
 //---------------------------------------------------------------------------
 
-#include <grid/grid_reordering.h>
-#include <grid/grid_reordering_internal.h>
-#include <grid/grid_tools.h>
+#include <deal.II/grid/grid_reordering.h>
+#include <deal.II/grid/grid_reordering_internal.h>
+#include <deal.II/grid/grid_tools.h>
+#include <deal.II/base/utilities.h>
 
 #include <algorithm>
 #include <set>
@@ -266,39 +267,41 @@ namespace internal
     }
 
 
-    struct MQuad::MakeQuad : public std::binary_function<CellData<2>,
-		  std::vector<MSide>,
-		  MQuad>
+    namespace
     {
-	MQuad operator()(const CellData<2> &q,
-			 const std::vector<MSide> &elist) const
-	  {
-					     //Assumes that the sides
-					     //are in the vector.. Bad
-					     //things will happen if
-					     //they are not!
-	    return MQuad(q.vertices[0],q.vertices[1], q.vertices[2], q.vertices[3],
-			 std::distance(elist.begin(),
-				       std::lower_bound(elist.begin(), elist.end(),
-							quadside(q,0),
-							MSide::SideSortLess() )),
-			 std::distance(elist.begin(),
-				       std::lower_bound(elist.begin(), elist.end(),
-							quadside(q,1),
-							MSide::SideSortLess() )),
-			 std::distance(elist.begin(),
-				       std::lower_bound(elist.begin(), elist.end(),
-							quadside(q,2),
-							MSide::SideSortLess() )),
-			 std::distance(elist.begin(),
-				       std::lower_bound(elist.begin(), elist.end(),
-							quadside(q,3),
-							MSide::SideSortLess() )),
-			 q);
-	  }
+				       /**
+					* Create an MQuad object from the
+					* indices of the four vertices by
+					* looking up the indices of the four
+					* sides.
+					*/
+      MQuad build_quad_from_vertices(const CellData<2> &q,
+				     const std::vector<MSide> &elist)
+      {
+					 // compute the indices of the four
+					 // sides that bound this quad. note
+					 // that the incoming list elist is
+					 // sorted with regard to the
+					 // MSide::SideSortLess criterion
+	unsigned int edges[4] = { numbers::invalid_unsigned_int,
+				  numbers::invalid_unsigned_int,
+				  numbers::invalid_unsigned_int,
+				  numbers::invalid_unsigned_int };
 
-    };
+	for (unsigned int i=0; i<4; ++i)
+	  edges[i] = (Utilities::lower_bound (elist.begin(),
+							elist.end(),
+							quadside(q,i),
+							MSide::SideSortLess())
+		      -
+		      elist.begin());
 
+	return MQuad(q.vertices[0],q.vertices[1], q.vertices[2], q.vertices[3],
+		     edges[0], edges[1], edges[2], edges[3],
+		     q);
+      }
+    }
+    
 
 
     void
@@ -315,7 +318,6 @@ namespace internal
     {
 				       //Reserve some space
       sides.reserve(4*inquads.size());
-      mquads.reserve(inquads.size());
 
 				       //Insert all the sides into the side vector
       for (int i = 0;i<4;++i)
@@ -339,11 +341,16 @@ namespace internal
 				       // Swap trick to shrink the
 				       // side vector
       std::vector<MSide>(sides).swap(sides);
-
-				       //Assigns the correct sides to
-				       //each quads
-      std::transform(inquads.begin(),inquads.end(), std::back_inserter(mquads),
-		     std::bind2nd(MQuad::MakeQuad(),sides) );
+	
+				       // Now assign the correct sides to
+				       // each quads
+      mquads.reserve(inquads.size());
+      std::transform(inquads.begin(),
+		     inquads.end(),
+		     std::back_inserter(mquads),
+		     std_cxx1x::bind(build_quad_from_vertices,
+				     std_cxx1x::_1,
+				     std_cxx1x::cref(sides)) );
 
 				       // Assign the quads to their sides also.
       int qctr = 0;
@@ -595,7 +602,8 @@ GridReordering<2>::reorder_cells (std::vector<CellData<2> > &original_cells)
 
 template<>
 void
-GridReordering<2,3>::reorder_cells (std::vector<CellData<2> > &original_cells) {
+GridReordering<2,3>::reorder_cells (std::vector<CellData<2> > &original_cells)
+{
     GridReordering<2>::reorder_cells(original_cells);
 }
 
