@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: tria_accessor.cc 22971 2010-12-15 15:44:51Z bangerth $
+//    $Id: tria_accessor.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,15 +12,15 @@
 //---------------------------------------------------------------------------
 
 
-#include <grid/tria.h>
-#include <grid/tria_levels.h>
-#include <grid/tria_iterator.h>
-#include <grid/tria_accessor.h>
-#include <grid/tria_accessor.templates.h>
-#include <grid/tria_iterator.templates.h>
-#include <base/geometry_info.h>
-#include <grid/grid_tools.h>
-#include <fe/mapping_q1.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_levels.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/grid/tria_accessor.templates.h>
+#include <deal.II/grid/tria_iterator.templates.h>
+#include <deal.II/base/geometry_info.h>
+#include <deal.II/grid/grid_tools.h>
+#include <deal.II/fe/mapping_q1.h>
 
 #include <cmath>
 
@@ -1018,6 +1018,15 @@ double TriaAccessor<1,1,1>::extent_in_direction(const unsigned int axis) const
 }
 
 
+template <>
+double TriaAccessor<1,1,2>::extent_in_direction(const unsigned int axis) const
+{
+  Assert (axis == 0, ExcIndexRange (axis, 0, 1));
+
+  return this->diameter();
+}
+
+
 template <>
 double TriaAccessor<2,2,2>::extent_in_direction(const unsigned int axis) const
 {
@@ -1030,6 +1039,18 @@ double TriaAccessor<2,2,2>::extent_in_direction(const unsigned int axis) const
 		  this->line(lines[axis][1])->diameter());
 }
 
+template <>
+double TriaAccessor<2,2,3>::extent_in_direction(const unsigned int axis) const
+{
+  const unsigned int lines[2][2] = {{2,3}, /// Lines along x-axis, see GeometryInfo
+				    {0,1}};/// Lines along y-axis
+
+  Assert (axis < 2, ExcIndexRange (axis, 0, 2));
+
+  return std::max(this->line(lines[axis][0])->diameter(),
+		  this->line(lines[axis][1])->diameter());
+}
+
 
 template <>
 double TriaAccessor<3,3,3>::extent_in_direction(const unsigned int axis) const
