@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: tria_faces.cc 22655 2010-11-09 19:27:22Z bangerth $
+//    $Id: tria_faces.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2006, 2010 by the deal.II authors
+//    Copyright (C) 2006, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,8 +11,8 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/memory_consumption.h>
-#include <grid/tria_faces.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/grid/tria_faces.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -22,21 +22,21 @@ namespace internal
   namespace Triangulation
   {
 
-    unsigned int
+    std::size_t
     TriaFaces<1>::memory_consumption () const
     {
       return 0;
     }
 
 
-    unsigned int
+    std::size_t
     TriaFaces<2>::memory_consumption () const
     {
       return MemoryConsumption::memory_consumption (lines);
     }
 
 
-    unsigned int
+    std::size_t
     TriaFaces<3>::memory_consumption () const
     {
       return (MemoryConsumption::memory_consumption (quads) +
