@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: tria_objects.cc 22655 2010-11-09 19:27:22Z bangerth $
+//    $Id: tria_objects.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2006, 2007, 2010 by the deal.II authors
+//    Copyright (C) 2006, 2007, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,11 +12,11 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/memory_consumption.h>
-#include <grid/tria_objects.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <grid/tria_accessor.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/grid/tria_objects.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/grid/tria_accessor.h>
 
 #include <algorithm>
 #include <functional>
@@ -786,7 +786,7 @@ namespace internal
 
 
     template<typename G>
-    unsigned int
+    std::size_t
     TriaObjects<G>::memory_consumption () const
     {
       return (MemoryConsumption::memory_consumption (cells) +
@@ -799,7 +799,7 @@ namespace internal
     }
 
 
-    unsigned int
+    std::size_t
     TriaObjectsHex::memory_consumption () const
     {
       return (MemoryConsumption::memory_consumption (face_orientations) +
@@ -809,7 +809,7 @@ namespace internal
     }
 
 
-    unsigned int
+    std::size_t
     TriaObjectsQuad3D::memory_consumption () const
     {
       return (MemoryConsumption::memory_consumption (line_orientations) +
