@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_faces.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: dof_faces.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2006 by the deal.II authors
+//    Copyright (C) 2006, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,8 +12,8 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/memory_consumption.h>
-#include <hp/dof_faces.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/hp/dof_faces.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -22,7 +22,7 @@ namespace internal
   namespace hp
   {
 
-    unsigned int
+    std::size_t
     DoFFaces<1>::memory_consumption () const
     {
       return 0;
@@ -30,7 +30,7 @@ namespace internal
 
 
 
-    unsigned int
+    std::size_t
     DoFFaces<2>::memory_consumption () const
     {
       return MemoryConsumption::memory_consumption (lines);
@@ -38,7 +38,7 @@ namespace internal
 
 
 
-    unsigned int
+    std::size_t
     DoFFaces<3>::memory_consumption () const
     {
       return (MemoryConsumption::memory_consumption (quads) +
