@@ -1,8 +1,8 @@
 //----------------------------  hp_dof_levels.cc  ---------------------------
-//    $Id: dof_levels.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: dof_levels.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2003, 2006 by the deal.II authors
+//    Copyright (C) 2003, 2006, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,8 +12,8 @@
 //----------------------------  hp_dof_levels.cc  ------------------------
 
 
-#include <base/memory_consumption.h>
-#include <hp/dof_levels.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/hp/dof_levels.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -22,7 +22,7 @@ namespace internal
   namespace hp
   {
 
-    unsigned int
+    std::size_t
     DoFLevel<1>::memory_consumption () const
     {
       return (DoFLevel<0>::memory_consumption() +
@@ -31,7 +31,7 @@ namespace internal
     
 
 
-    unsigned int
+    std::size_t
     DoFLevel<2>::memory_consumption () const
     {
       return (DoFLevel<0>::memory_consumption () +
@@ -40,7 +40,7 @@ namespace internal
 
 
 
-    unsigned int
+    std::size_t
     DoFLevel<3>::memory_consumption () const
     {
       return (DoFLevel<0>::memory_consumption () +
@@ -49,7 +49,7 @@ namespace internal
 
 
 
-    unsigned int
+    std::size_t
     DoFLevel<0>::memory_consumption () const
     {
       return MemoryConsumption::memory_consumption (active_fe_indices);
