@@ -1,5 +1,5 @@
 //----------------------------  hp_fe_values.cc  ---------------------------
-//    $Id: fe_values.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: fe_values.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2003, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
@@ -12,8 +12,8 @@
 //----------------------------  hp_fe_values.cc  ---------------------------
 
 
-#include <hp/fe_values.h>
-#include <fe/mapping_q1.h>
+#include <deal.II/hp/fe_values.h>
+#include <deal.II/fe/mapping_q1.h>
 
 DEAL_II_NAMESPACE_OPEN
 
