@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: block_sparsity_pattern.cc 22430 2010-10-23 13:41:09Z heister $
+//    $Id: block_sparsity_pattern.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,9 +11,9 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/vector_slice.h>
-#include <base/memory_consumption.h>
-#include <lac/block_sparsity_pattern.h>
+#include <deal.II/base/vector_slice.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/lac/block_sparsity_pattern.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -399,10 +399,10 @@ BlockSparsityPattern::is_compressed () const
 }
 
 
-unsigned int
+std::size_t
 BlockSparsityPattern::memory_consumption () const
 {
-  unsigned int mem = 0;
+  std::size_t mem = 0;
   mem += (MemoryConsumption::memory_consumption (rows) +
 	  MemoryConsumption::memory_consumption (columns) +
 	  MemoryConsumption::memory_consumption (sub_objects) +
