@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: constraint_matrix.cc 22671 2010-11-10 14:42:37Z kronbichler $
+//    $Id: constraint_matrix.cc 24214 2011-08-30 14:43:11Z heister $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,30 +12,30 @@
 //---------------------------------------------------------------------------
 
 
-#include <lac/constraint_matrix.h>
-#include <lac/constraint_matrix.templates.h>
+#include <deal.II/lac/constraint_matrix.h>
+#include <deal.II/lac/constraint_matrix.templates.h>
 
-#include <base/memory_consumption.h>
-#include <lac/compressed_sparsity_pattern.h>
-#include <lac/compressed_set_sparsity_pattern.h>
-#include <lac/compressed_simple_sparsity_pattern.h>
-#include <lac/block_vector.h>
-#include <lac/block_sparse_matrix.h>
-#include <lac/sparse_matrix_ez.h>
-#include <lac/block_sparse_matrix_ez.h>
-#include <lac/petsc_vector.h>
-#include <lac/petsc_block_vector.h>
-#include <lac/petsc_sparse_matrix.h>
-#include <lac/petsc_block_sparse_matrix.h>
-#include <lac/petsc_parallel_vector.h>
-#include <lac/petsc_parallel_block_vector.h>
-#include <lac/petsc_parallel_sparse_matrix.h>
-#include <lac/petsc_parallel_block_sparse_matrix.h>
-#include <lac/trilinos_vector.h>
-#include <lac/trilinos_block_vector.h>
-#include <lac/trilinos_sparse_matrix.h>
-#include <lac/trilinos_block_sparse_matrix.h>
-#include <lac/matrix_block.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/lac/compressed_sparsity_pattern.h>
+#include <deal.II/lac/compressed_set_sparsity_pattern.h>
+#include <deal.II/lac/compressed_simple_sparsity_pattern.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/block_sparse_matrix.h>
+#include <deal.II/lac/sparse_matrix_ez.h>
+#include <deal.II/lac/block_sparse_matrix_ez.h>
+#include <deal.II/lac/petsc_vector.h>
+#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/petsc_sparse_matrix.h>
+#include <deal.II/lac/petsc_block_sparse_matrix.h>
+#include <deal.II/lac/petsc_parallel_vector.h>
+#include <deal.II/lac/petsc_parallel_block_vector.h>
+#include <deal.II/lac/petsc_parallel_sparse_matrix.h>
+#include <deal.II/lac/petsc_parallel_block_sparse_matrix.h>
+#include <deal.II/lac/trilinos_vector.h>
+#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_sparse_matrix.h>
+#include <deal.II/lac/trilinos_block_sparse_matrix.h>
+#include <deal.II/lac/matrix_block.h>
 
 #include <algorithm>
 #include <numeric>
@@ -82,7 +82,7 @@ ConstraintMatrix::ConstraintLine::operator == (const ConstraintLine &a) const
 
 
 
-unsigned int
+std::size_t
 ConstraintMatrix::ConstraintLine::memory_consumption () const
 {
   return (MemoryConsumption::memory_consumption (line) +
@@ -171,6 +171,9 @@ void ConstraintMatrix::add_selected_constraints
   (const ConstraintMatrix &constraints,
    const IndexSet         &filter)
 {
+  if (constraints.n_constraints() == 0)
+    return;
+
   Assert (filter.size() > constraints.lines.back().line,
 	  ExcMessage ("Filter needs to be larger than constraint matrix size."));
   for (std::vector<ConstraintLine>::const_iterator line=constraints.lines.begin();
@@ -545,9 +548,8 @@ void
 ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
 			 const MergeConflictBehavior merge_conflict_behavior)
 {
-				   //TODO: this doesn't work with IndexSets yet. [TH]
-  AssertThrow(local_lines.size()==0, ExcNotImplemented());
-  AssertThrow(other_constraints.local_lines.size()==0, ExcNotImplemented());
+  AssertThrow(local_lines == other_constraints.local_lines,
+	      ExcNotImplemented());
 
 				   // store the previous state with
 				   // respect to sorting
@@ -584,7 +586,7 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
 					   // constrained, or if we won't take
 					   // the constraint from the other
 					   // object, then simply copy it over
-	  if (!other_constraints.is_constrained(line->entries[i].first)
+	  if (other_constraints.is_constrained(line->entries[i].first) == false
 	      ||
 	      ((merge_conflict_behavior != right_object_wins)
 	       &&
@@ -615,7 +617,7 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
 							      j->second*weight));
 
 	      line->inhomogeneity += other_constraints.get_inhomogeneity(line->entries[i].first) *
-				     line->entries[i].second;
+				     weight;
 	    }
 	}
 				       // finally exchange old and
@@ -630,7 +632,7 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
   for (std::vector<ConstraintLine>::const_iterator
 	 line=other_constraints.lines.begin();
        line!=other_constraints.lines.end(); ++line)
-    if (!is_constrained(line->line))
+    if (is_constrained(line->line) == false)
       lines.push_back (*line);
     else
       {
@@ -665,15 +667,17 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
 	  }
       }
 
-				   // if the object was sorted before,
-				   // then make sure it is so
-				   // afterwards as well. otherwise
-				   // leave everything in the unsorted
-				   // state
+				// update the lines cache
   unsigned int counter = 0;
   for (std::vector<ConstraintLine>::const_iterator line=lines.begin();
        line!=lines.end(); ++line, ++counter)
-    lines_cache[line->line] = counter;
+    lines_cache[calculate_line_index(line->line)] = counter;
+
+				   // if the object was sorted before,
+				   // then make sure it is so
+				   // afterward as well. otherwise
+				   // leave everything in the unsorted
+				   // state
   if (object_was_sorted == true)
     close ();
 }
@@ -1839,6 +1843,8 @@ template<>
 void
 ConstraintMatrix::distribute (TrilinosWrappers::MPI::Vector &vec) const
 {
+  Assert (sorted==true, ExcMatrixIsClosed());
+
 				   //TODO: not implemented yet, we need to fix
 				   //LocalRange() first to only include
 				   //"owned" indices. For this we need to keep
@@ -1848,16 +1854,15 @@ ConstraintMatrix::distribute (TrilinosWrappers::MPI::Vector &vec) const
   AssertThrow (vec.vector_partitioner().IsOneToOne(),
 	       ExcMessage ("Distribute does not work on vectors with overlapping parallel partitioning."));
 
-
   typedef std::vector<ConstraintLine>::const_iterator constraint_iterator;
   ConstraintLine index_comparison;
   index_comparison.line = vec.local_range().first;
   const constraint_iterator begin_my_constraints =
-    std::lower_bound (lines.begin(),lines.end(),index_comparison);
+    Utilities::lower_bound (lines.begin(),lines.end(),index_comparison);
 
   index_comparison.line = vec.local_range().second;
   const constraint_iterator end_my_constraints
-    = std::lower_bound(lines.begin(),lines.end(),index_comparison);
+    = Utilities::lower_bound(lines.begin(),lines.end(),index_comparison);
 
 				   // Here we search all the indices that we
 				   // need to have read-access to - the
@@ -1925,6 +1930,8 @@ template<>
 void
 ConstraintMatrix::distribute (TrilinosWrappers::MPI::BlockVector &vec) const
 {
+  Assert (sorted==true, ExcMatrixIsClosed());
+
   IndexSet my_indices (vec.size());
   for (unsigned int block=0; block<vec.n_blocks(); ++block)
     {
@@ -1933,13 +1940,13 @@ ConstraintMatrix::distribute (TrilinosWrappers::MPI::BlockVector &vec) const
       index_comparison.line = vec.block(block).local_range().first
 	+vec.get_block_indices().block_start(block);
       const constraint_iterator begin_my_constraints =
-	std::lower_bound (lines.begin(),lines.end(),index_comparison);
+	Utilities::lower_bound (lines.begin(),lines.end(),index_comparison);
 
       index_comparison.line = vec.block(block).local_range().second
 	+vec.get_block_indices().block_start(block);
 
       const constraint_iterator end_my_constraints
-	= std::lower_bound(lines.begin(),lines.end(),index_comparison);
+	= Utilities::lower_bound(lines.begin(),lines.end(),index_comparison);
 
 				   // Here we search all the indices that we
 				   // need to have read-access to - the local
@@ -1988,13 +1995,13 @@ ConstraintMatrix::distribute (TrilinosWrappers::MPI::BlockVector &vec) const
       index_comparison.line = vec.block(block).local_range().first
 	+vec.get_block_indices().block_start(block);
       const constraint_iterator begin_my_constraints =
-	std::lower_bound (lines.begin(),lines.end(),index_comparison);
+	Utilities::lower_bound (lines.begin(),lines.end(),index_comparison);
 
       index_comparison.line = vec.block(block).local_range().second
 	+vec.get_block_indices().block_start(block);
 
       const constraint_iterator end_my_constraints
-	= std::lower_bound(lines.begin(),lines.end(),index_comparison);
+	= Utilities::lower_bound(lines.begin(),lines.end(),index_comparison);
 
       for (constraint_iterator it = begin_my_constraints;
 	   it != end_my_constraints; ++it)
@@ -2008,12 +2015,8 @@ ConstraintMatrix::distribute (TrilinosWrappers::MPI::BlockVector &vec) const
 			  it->entries[i].second);
 	  vec(it->line) = new_value;
 	}
+      vec.block(block).compress(Insert);
     }
-
-				   // force every processor to write something
-  unsigned int idx = vec.block(0).local_range().first;
-  vec(idx) = vec(idx);
-  vec.compress ();
 }
 
 #endif
@@ -2031,15 +2034,17 @@ template<>
 void
 ConstraintMatrix::distribute (PETScWrappers::MPI::Vector &vec) const
 {
+  Assert (sorted==true, ExcMatrixIsClosed());
+
   typedef std::vector<ConstraintLine>::const_iterator constraint_iterator;
   ConstraintLine index_comparison;
   index_comparison.line = vec.local_range().first;
   const constraint_iterator begin_my_constraints =
-    std::lower_bound (lines.begin(),lines.end(),index_comparison);
+    Utilities::lower_bound (lines.begin(),lines.end(),index_comparison);
 
   index_comparison.line = vec.local_range().second;
   const constraint_iterator end_my_constraints
-    = std::lower_bound(lines.begin(),lines.end(),index_comparison);
+    = Utilities::lower_bound(lines.begin(),lines.end(),index_comparison);
 
 				   // all indices we need to read from
   IndexSet my_indices (vec.size());
@@ -2087,9 +2092,6 @@ ConstraintMatrix::distribute (PETScWrappers::MPI::Vector &vec) const
       vec(it->line) = new_value;
     }
 
-				   // force every processor to write something
-  vec(local_range.first) = vec(local_range.first);
-
   vec.compress ();
 }
 
@@ -2098,6 +2100,7 @@ template<>
 void
 ConstraintMatrix::distribute (PETScWrappers::MPI::BlockVector &/*vec*/) const
 {
+  Assert (sorted==true, ExcMatrixIsClosed());
   AssertThrow (false, ExcNotImplemented());
 }
 
@@ -2135,8 +2138,8 @@ unsigned int ConstraintMatrix::max_constraint_indirections () const
   for (std::vector<ConstraintLine>::const_iterator i=lines.begin();
        i!=lines.end(); ++i)
 				     // use static cast, since
-				     // typeof(size)==size_t, which is
-				     // != unsigned int on AIX
+				     // typeof(size)==std::size_t, which is !=
+				     // unsigned int on AIX
     return_value = std::max(return_value,
 			    static_cast<unsigned int>(i->entries.size()));
 
@@ -2220,7 +2223,7 @@ ConstraintMatrix::write_dot (std::ostream &out) const
 
 
 
-unsigned int
+std::size_t
 ConstraintMatrix::memory_consumption () const
 {
   return (MemoryConsumption::memory_consumption (lines) +
@@ -2467,4 +2470,16 @@ ONLY_MATRIX_FUNCTIONS(SparseMatrix<double>);
 ONLY_MATRIX_FUNCTIONS(MatrixBlock<SparseMatrix<float> >);
 ONLY_MATRIX_FUNCTIONS(MatrixBlock<SparseMatrix<double> >);
 
+#ifdef DEAL_II_USE_TRILINOS
+ONLY_MATRIX_FUNCTIONS(TrilinosWrappers::SparseMatrix);
+ONLY_MATRIX_FUNCTIONS(TrilinosWrappers::BlockSparseMatrix);
+#endif
+
+#ifdef DEAL_II_USE_PETSC
+ONLY_MATRIX_FUNCTIONS(PETScWrappers::SparseMatrix);
+ONLY_MATRIX_FUNCTIONS(PETScWrappers::BlockSparseMatrix);
+ONLY_MATRIX_FUNCTIONS(PETScWrappers::MPI::SparseMatrix);
+ONLY_MATRIX_FUNCTIONS(PETScWrappers::MPI::BlockSparseMatrix);
+#endif
+
 DEAL_II_NAMESPACE_CLOSE
