@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: lapack_full_matrix.cc 22672 2010-11-10 14:43:23Z kronbichler $
+//    $Id: lapack_full_matrix.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2005, 2006, 2008, 2010 by the deal.II authors
@@ -11,12 +11,12 @@
 //
 //---------------------------------------------------------------------------
 
-#include <lac/lapack_full_matrix.h>
-#include <lac/lapack_templates.h>
-#include <lac/lapack_support.h>
-#include <lac/full_matrix.h>
-#include <lac/vector.h>
-#include <lac/block_vector.h>
+#include <deal.II/lac/lapack_full_matrix.h>
+#include <deal.II/lac/lapack_templates.h>
+#include <deal.II/lac/lapack_support.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
 
 #include <iostream>
 #include <iomanip>
