@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_parallel_sparse_matrix.cc 22430 2010-10-23 13:41:09Z heister $
+//    $Id: petsc_parallel_sparse_matrix.cc 24320 2011-09-13 09:41:50Z young $
 //    Version: $Name$
 //
 //    Copyright (C) 2004, 2005, 2006, 2008, 2009, 2010 by the deal.II authors
@@ -12,14 +12,14 @@
 //---------------------------------------------------------------------------
 
 
-#include <lac/petsc_parallel_sparse_matrix.h>
+#include <deal.II/lac/petsc_parallel_sparse_matrix.h>
 
 #ifdef DEAL_II_USE_PETSC
 
-#  include <lac/petsc_vector.h>
-#  include <lac/sparsity_pattern.h>
-#  include <lac/compressed_sparsity_pattern.h>
-#  include <lac/compressed_simple_sparsity_pattern.h>
+#  include <deal.II/lac/petsc_vector.h>
+#  include <deal.II/lac/sparsity_pattern.h>
+#  include <deal.II/lac/compressed_sparsity_pattern.h>
+#  include <deal.II/lac/compressed_simple_sparsity_pattern.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -114,7 +114,11 @@ namespace PETScWrappers
 
                                        // get rid of old matrix and generate a
                                        // new one
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
       const int ierr = MatDestroy (matrix);
+#else
+      const int ierr = MatDestroy (&matrix);
+#endif
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       do_reinit (m, n, local_rows, local_columns,
@@ -136,7 +140,11 @@ namespace PETScWrappers
 
                                        // get rid of old matrix and generate a
                                        // new one
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
       const int ierr = MatDestroy (matrix);
+#else
+      const int ierr = MatDestroy (&matrix);
+#endif
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       do_reinit (m, n, local_rows, local_columns, row_lengths, is_symmetric);
@@ -158,7 +166,11 @@ namespace PETScWrappers
 
                                        // get rid of old matrix and generate a
                                        // new one
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
       const int ierr = MatDestroy (matrix);
+#else
+      const int ierr = MatDestroy (&matrix);
+#endif
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       do_reinit (sparsity_pattern, local_rows_per_process,
@@ -530,9 +542,9 @@ namespace PETScWrappers
 				           // Tell PETSc that we are not
 				           // planning on adding new entries
 				           // to the matrix. Generate errors
-				           // in debugmode.
-#if DEAL_II_PETSC_VERSION_LT(3,0,0)
+				           // in debug mode.
           int ierr;
+#if DEAL_II_PETSC_VERSION_LT(3,0,0)
 #ifdef DEBUG
 	  ierr = MatSetOption (matrix, MAT_NEW_NONZERO_LOCATION_ERR);
 	  AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -541,7 +553,6 @@ namespace PETScWrappers
 	  AssertThrow (ierr == 0, ExcPETScError(ierr));
 #endif  
 #else
-          int ierr;
 #ifdef DEBUG
 	  ierr = MatSetOption (matrix, MAT_NEW_NONZERO_LOCATION_ERR, PETSC_TRUE);
 	  AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -550,6 +561,25 @@ namespace PETScWrappers
 	  AssertThrow (ierr == 0, ExcPETScError(ierr));
 #endif
 #endif
+
+					   // Tell PETSc to keep the
+					   // SparsityPattern entries even if
+					   // we delete a row with
+					   // clear_rows() which calls
+					   // MatZeroRows(). Otherwise one can
+					   // not write into that row
+					   // afterwards.
+#if DEAL_II_PETSC_VERSION_LT(3,0,0)
+	  ierr = MatSetOption (matrix, MAT_KEEP_ZEROED_ROWS);
+	  AssertThrow (ierr == 0, ExcPETScError(ierr));
+#elif DEAL_II_PETSC_VERSION_LT(3,1,0)
+	  ierr = MatSetOption (matrix, MAT_KEEP_ZEROED_ROWS, PETSC_TRUE);
+	  AssertThrow (ierr == 0, ExcPETScError(ierr));
+#else
+	  ierr = MatSetOption (matrix, MAT_KEEP_NONZERO_PATTERN, PETSC_TRUE);
+	  AssertThrow (ierr == 0, ExcPETScError(ierr));
+#endif
+	  
         }
     }
 
