@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_solver.cc 21563 2010-07-23 18:24:21Z young $
+//    $Id: petsc_solver.cc 24320 2011-09-13 09:41:50Z young $
 //    Version: $Name$
 //
 //    Copyright (C) 2004, 2006, 2008, 2009, 2010 by the deal.II authors
@@ -12,13 +12,13 @@
 //---------------------------------------------------------------------------
 
 
-#include <lac/petsc_solver.h>
+#include <deal.II/lac/petsc_solver.h>
 
 #ifdef DEAL_II_USE_PETSC
 
-#  include <lac/petsc_matrix_base.h>
-#  include <lac/petsc_vector_base.h>
-#  include <lac/petsc_precondition.h>
+#  include <deal.II/lac/petsc_matrix_base.h>
+#  include <deal.II/lac/petsc_vector_base.h>
+#  include <deal.II/lac/petsc_precondition.h>
 #  include <cmath>
 
 #if DEAL_II_PETSC_VERSION_LT(2,2,0)
@@ -34,7 +34,12 @@ namespace PETScWrappers
   SolverBase::SolverData::~SolverData ()
   {
                                      // destroy the solver object
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
     int ierr = KSPDestroy (ksp);
+#else
+    int ierr = KSPDestroy (&ksp);
+#endif
+    
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
                                      // and destroy the solver object if we
@@ -68,7 +73,6 @@ namespace PETScWrappers
                      const PreconditionerBase &preconditioner)
   {
     int ierr;
-
                                      // first create a solver object if this
                                      // is necessary
     if (solver_data.get() == 0)
@@ -92,10 +96,8 @@ namespace PETScWrappers
                                          // preconditioner
         set_solver_type (solver_data->ksp);
 
-        ierr = KSPGetPC (solver_data->ksp, &solver_data->pc);
-        AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-        preconditioner.set_preconditioner_type (solver_data->pc);
+	ierr = KSPSetPC (solver_data->ksp, preconditioner.get_pc());
+	AssertThrow (ierr == 0, ExcPETScError(ierr));
 
                                          // then a convergence monitor
                                          // function. that function simply
@@ -400,7 +402,12 @@ namespace PETScWrappers
 				     // right
     if (additional_data.right_preconditioning)
       {
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
 	ierr = KSPSetPreconditionerSide(ksp, PC_RIGHT);
+#else
+	ierr = KSPSetPCSide(ksp, PC_RIGHT);
+#endif
+
 	AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
 
