@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_vector_base.cc 23092 2010-12-31 13:25:01Z bangerth $
+//    $Id: petsc_vector_base.cc 24320 2011-09-13 09:41:50Z young $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,13 +12,13 @@
 //---------------------------------------------------------------------------
 
 
-#include <lac/petsc_vector_base.h>
+#include <deal.II/lac/petsc_vector_base.h>
 
 #ifdef DEAL_II_USE_PETSC
 
-#  include <base/memory_consumption.h>
-#  include <lac/petsc_vector.h>
-#  include <lac/petsc_parallel_vector.h>
+#  include <deal.II/base/memory_consumption.h>
+#  include <deal.II/lac/petsc_vector.h>
+#  include <deal.II/lac/petsc_parallel_vector.h>
 #  include <cmath>
 
 DEAL_II_NAMESPACE_OPEN
@@ -215,7 +215,11 @@ namespace PETScWrappers
   {
     if (attained_ownership)
       {	
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
 	const int ierr = VecDestroy (vector);
+#else
+	const int ierr = VecDestroy (&vector);
+#endif
 	AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
   }
@@ -253,7 +257,12 @@ namespace PETScWrappers
     Assert (size() == v.size(),
             ExcDimensionMismatch(size(), v.size()));
 
-    PetscTruth flag;
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
+    PetscTruth 
+#else
+      PetscBool 
+#endif
+      flag;
 
     const int ierr = VecEqual (vector, v.vector, &flag);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -269,7 +278,12 @@ namespace PETScWrappers
     Assert (size() == v.size(),
             ExcDimensionMismatch(size(), v.size()));
 
-    PetscTruth flag;
+#if DEAL_II_PETSC_VERSION_LT(3,2,0)
+    PetscTruth 
+#else
+      PetscBool 
+#endif
+      flag;
 
     const int ierr = VecEqual (vector, v.vector, &flag);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -393,11 +407,22 @@ namespace PETScWrappers
   void
   VectorBase::compress ()
   {
+    // note that one may think that we only need to do something
+    // if in fact the state is anything but LastAction::none. but
+    // that's not true: one frequently gets into situations where
+    // only one processor (or a subset of processors) actually writes
+    // something into a vector, but we still need to call 
+    // VecAssemblyBegin/End on all processors, even those that are
+    // still in LastAction::none state
     int ierr;
     ierr = VecAssemblyBegin(vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
     ierr = VecAssemblyEnd(vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
+    
+    // reset the last action field to indicate that we're back to
+    // a pristine state
+    last_action = VectorBase::LastAction::none;
   }
 
 
@@ -414,13 +439,21 @@ namespace PETScWrappers
   PetscScalar
   VectorBase::mean_value () const
   {
-    Assert(dynamic_cast<const PETScWrappers::MPI::Vector *>(this) == 0,
-	   ExcMessage	("::mean_value() not implemented for MPI::Vector."));
+    int ierr;
+				     // We can only use our more efficient
+				     // routine in the serial case.
+    if (dynamic_cast<const PETScWrappers::MPI::Vector *>(this) != 0)
+      {
+	PetscScalar sum;
+	ierr = VecSum( vector, &sum);
+	AssertThrow (ierr == 0, ExcPETScError(ierr));
+	return sum/size();
+      }
     
                                      // get a representation of the vector and
                                      // loop over all the elements
     PetscScalar *start_ptr;
-    int ierr = VecGetArray (vector, &start_ptr);
+    ierr = VecGetArray (vector, &start_ptr);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     PetscScalar mean = 0;
@@ -1074,15 +1107,15 @@ namespace PETScWrappers
   }
 
 
-  unsigned int
+  std::size_t
   VectorBase::memory_consumption () const
   {    
-    unsigned int mem = sizeof(Vec)+sizeof(LastAction::Values)
+    std::size_t mem = sizeof(Vec)+sizeof(LastAction::Values)
       +MemoryConsumption::memory_consumption(ghosted)
       +MemoryConsumption::memory_consumption(ghost_indices);
 
 				     // TH: I am relatively sure that PETSc is
-				     // storing the local data in a contigious
+				     // storing the local data in a contiguous
 				     // block without indices:
     mem += local_size()*sizeof(PetscScalar);
 				     // assume that PETSc is storing one index
@@ -1102,15 +1135,15 @@ namespace PETScWrappers
 				    const PetscScalar  *values,
 				    const bool          add_values)
   {
-    if (last_action != VectorBase::LastAction::insert)
-      {
-        int ierr;
-        ierr = VecAssemblyBegin (vector);
-        AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-        ierr = VecAssemblyEnd (vector);
-        AssertThrow (ierr == 0, ExcPETScError(ierr));
-      }
+    Assert ((last_action == (add_values ? 
+                             LastAction::add :
+                             LastAction::insert))
+            ||
+            (last_action == LastAction::none),
+	    internal::VectorReference::ExcWrongMode ((add_values ? 
+	                                              LastAction::add :
+	                                              LastAction::insert),
+				                     last_action));
 
 				     // VecSetValues complains if we
 				     // come with an empty
@@ -1129,16 +1162,18 @@ namespace PETScWrappers
 	const int * petsc_indices = (const int*)indices;
 #endif
 
-	InsertMode mode = ADD_VALUES;
-	if (!add_values)
-	  mode = INSERT_VALUES;
+	const InsertMode mode = (add_values ? ADD_VALUES : INSERT_VALUES);
 	const int ierr
 	  = VecSetValues (vector, n_elements, petsc_indices, values,
 			  mode);
 	AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
 
-    last_action = LastAction::insert;
+    // set the mode here, independent of whether we have actually
+    // written elements or whether the list was empty
+    last_action = (add_values ? 
+                   VectorBase::LastAction::add :
+                   VectorBase::LastAction::insert);
   }
 
 
@@ -1148,7 +1183,7 @@ namespace PETScWrappers
 				     // generate an error for not ghosted
 				     // vectors
     if (!ghosted)
-	throw ExcInternalError();
+      AssertThrow (false, ExcInternalError());
 
     int ierr;
 
