@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: relaxation_block.cc 22387 2010-10-19 19:20:22Z kanschat $
+//    $Id: relaxation_block.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2010 by the deal.II authors
@@ -12,8 +12,8 @@
 //---------------------------------------------------------------------------
 
 
-#include <lac/relaxation_block.templates.h>
-#include <lac/sparse_matrix.h>
+#include <deal.II/lac/relaxation_block.templates.h>
+#include <deal.II/lac/sparse_matrix.h>
 
 DEAL_II_NAMESPACE_OPEN
 #include "relaxation_block.inst"
