@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: sparse_direct.cc 22199 2010-09-29 21:17:46Z kanschat $
+//    $Id: sparse_direct.cc 24418 2011-09-26 12:52:19Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,12 +12,12 @@
 //---------------------------------------------------------------------------
 
 
-#include <lac/sparse_direct.h>
-#include <base/memory_consumption.h>
-#include <base/thread_management.h>
-#include <lac/sparse_matrix.h>
-#include <lac/block_sparse_matrix.h>
-#include <lac/vector.h>
+#include <deal.II/lac/sparse_direct.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/base/thread_management.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/block_sparse_matrix.h>
+#include <deal.II/lac/vector.h>
 
 #include <iostream>
 #include <list>
@@ -315,7 +315,7 @@ struct SparseDirectMA27::DetachedModeData
                                       */
     template <typename T>
     void put (const T *t,
-              const size_t N,
+              const std::size_t N,
 	      const char * /*debug_info*/) const
       {
         unsigned int count = 0;
@@ -349,7 +349,7 @@ struct SparseDirectMA27::DetachedModeData
                                       */
     template <typename T>
     void get (T *t,
-	      const size_t N,
+	      const std::size_t N,
 	      const char * /*debug_info*/) const
       {
         unsigned int count = 0;
@@ -408,7 +408,7 @@ SparseDirectMA27::~SparseDirectMA27()
 					 // avoid compiler warnings
 //TODO:[WB] Should t be used to trace errors?
         ssize_t t = write (detached_mode_data->server_client_pipe[1], "7", 1);
-	t = 0;
+	(void)t;
                                          // then also delete data
         delete detached_mode_data;
         detached_mode_data = 0;
@@ -462,7 +462,7 @@ SparseDirectMA27::initialize (const SparsityPattern &sp)
 				       // warnings
 //TODO:[WB] Use t to trace errors?      
       int t = pipe(detached_mode_data->server_client_pipe);
-      t = pipe(detached_mode_data->client_server_pipe);      
+      (void)t;      
 
                                        // fflush(NULL) is said to be a
                                        // good idea before fork()
@@ -823,7 +823,7 @@ SparseDirectMA27::solve (const SparseMatrix<number> &matrix,
 
 
 
-unsigned int
+std::size_t
 SparseDirectMA27::memory_consumption () const
 {
   return (sizeof(*this) +
@@ -1346,7 +1346,7 @@ SparseDirectMA47::solve (const SparseMatrix<double> &matrix,
 
 
 
-unsigned int
+std::size_t
 SparseDirectMA47::memory_consumption () const
 {
   return (sizeof(*this) +
@@ -1600,6 +1600,27 @@ sort_arrays (const SparseMatrix<number> &matrix)
 
 
 
+template <typename number>
+void
+SparseDirectUMFPACK::
+sort_arrays (const SparseMatrixEZ<number> &matrix)
+{
+                                   //same thing for SparseMatrixEZ
+  for (unsigned int row=0; row<matrix.m(); ++row)
+    {
+      long int cursor = Ap[row];
+      while ((cursor < Ap[row+1]-1) &&
+             (Ai[cursor] > Ai[cursor+1]))
+        {
+          std::swap (Ai[cursor], Ai[cursor+1]);
+          std::swap (Ax[cursor], Ax[cursor+1]);
+          ++cursor;
+        }
+    }
+}
+
+
+
 template <typename number>
 void
 SparseDirectUMFPACK::
@@ -1691,13 +1712,13 @@ factorize (const Matrix &matrix)
                                    // anyway. people are supposed to provide
                                    // accurate sparsity patterns.
   Ap.resize (N+1);
-  Ai.resize (matrix.get_sparsity_pattern().n_nonzero_elements());
-  Ax.resize (matrix.get_sparsity_pattern().n_nonzero_elements());
+  Ai.resize (matrix.n_nonzero_elements());
+  Ax.resize (matrix.n_nonzero_elements());
 
                                    // first fill row lengths array
   Ap[0] = 0;
   for (unsigned int row=1; row<=N; ++row)
-    Ap[row] = Ap[row-1] + matrix.get_sparsity_pattern().row_length(row-1);
+    Ap[row] = Ap[row-1] + matrix.get_row_length(row-1);
   Assert (static_cast<unsigned int>(Ap.back()) == Ai.size(),
           ExcInternalError());
   
@@ -1819,14 +1840,14 @@ SparseDirectUMFPACK::clear ()
 template <class Matrix>
 void SparseDirectUMFPACK::factorize (const Matrix &)
 {
-  Assert(false, ExcNeedsUMFPACK());
+  AssertThrow(false, ExcNeedsUMFPACK());
 }
 
 
 void
 SparseDirectUMFPACK::solve (Vector<double> &) const
 {
-  Assert(false, ExcNeedsUMFPACK());
+  AssertThrow(false, ExcNeedsUMFPACK());
 }
 
 
@@ -1835,7 +1856,7 @@ void
 SparseDirectUMFPACK::solve (const Matrix   &,
                             Vector<double> &)
 {
-  Assert(false, ExcNeedsUMFPACK());
+  AssertThrow(false, ExcNeedsUMFPACK());
 }
 
 
@@ -1913,7 +1934,7 @@ void SparseDirectMUMPS::initialize (const Matrix& matrix,
   dmumps_c (&id);
    
                          // Hand over matrix and right-hand side
-  if (Utilities::System::get_this_mpi_process (MPI_COMM_WORLD) == 0) 
+  if (Utilities::MPI::this_mpi_process (MPI_COMM_WORLD) == 0) 
     {
 
                          // Objects denoting a MUMPS data structure:
@@ -1986,7 +2007,7 @@ void SparseDirectMUMPS::solve (Vector<double>& vector)
   dmumps_c (&id);
 
                         // Copy solution into the given vector
-  if (Utilities::System::get_this_mpi_process (MPI_COMM_WORLD) == 0) 
+  if (Utilities::MPI::this_mpi_process (MPI_COMM_WORLD) == 0) 
     {
       for (unsigned int i=0; i<n; ++i)
 	vector(i) = rhs[i];
@@ -2028,6 +2049,8 @@ void SparseDirectMA27::solve (const SparseMatrix<float>  &matrix,
 
 InstantiateUMFPACK(SparseMatrix<double>);
 InstantiateUMFPACK(SparseMatrix<float>);
+InstantiateUMFPACK(SparseMatrixEZ<double>);
+InstantiateUMFPACK(SparseMatrixEZ<float>);
 InstantiateUMFPACK(BlockSparseMatrix<double>);
 InstantiateUMFPACK(BlockSparseMatrix<float>);
 
