@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: sparsity_pattern.cc 21669 2010-08-17 08:58:48Z kanschat $
+//    $Id: sparsity_pattern.cc 24428 2011-09-26 15:06:23Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,13 +11,14 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/vector_slice.h>
-#include <lac/sparsity_pattern.h>
-#include <lac/sparsity_tools.h>
-#include <lac/full_matrix.h>
-#include <lac/compressed_sparsity_pattern.h>
-#include <lac/compressed_set_sparsity_pattern.h>
-#include <lac/compressed_simple_sparsity_pattern.h>
+#include <deal.II/base/vector_slice.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/lac/sparsity_pattern.h>
+#include <deal.II/lac/sparsity_tools.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/compressed_sparsity_pattern.h>
+#include <deal.II/lac/compressed_set_sparsity_pattern.h>
+#include <deal.II/lac/compressed_simple_sparsity_pattern.h>
 
 #include <iostream>
 #include <iomanip>
@@ -183,7 +184,7 @@ SparsityPattern::SparsityPattern (const SparsityPattern &original,
       const unsigned int * const
 	original_last_before_side_diagonals
 	= (row > extra_off_diagonals ?
-	   std::lower_bound (original_row_start,
+	   Utilities::lower_bound (original_row_start,
 			     original_row_end,
 			     row-extra_off_diagonals) :
 	   original_row_start);
@@ -734,13 +735,13 @@ SparsityPattern::operator () (const unsigned int i,
 				   // top of this function, so it may
 				   // not be called for noncompressed
 				   // structures.
-  const unsigned int * const sorted_region_start = (diagonal_optimized ?
-						    &colnums[rowstart[i]+1] :
-						    &colnums[rowstart[i]]);
+  const unsigned int * sorted_region_start = (diagonal_optimized ?
+					      &colnums[rowstart[i]+1] :
+					      &colnums[rowstart[i]]);
   const unsigned int * const p
-    = internals::SparsityPatternTools::optimized_lower_bound (sorted_region_start,
-							      &colnums[rowstart[i+1]],
-							      j);
+    = Utilities::lower_bound<const unsigned int *> (sorted_region_start,
+					&colnums[rowstart[i+1]],
+					j);
   if ((p != &colnums[rowstart[i+1]])  &&  (*p == j))
     return (p - &colnums[0]);
   else
@@ -793,8 +794,7 @@ SparsityPattern::add_entries (const unsigned int row,
 
 
 bool
-SparsityPattern::exists (const unsigned int i,
-			 const unsigned int j) const
+SparsityPattern::exists (const unsigned int i, const unsigned int j) const
 {
   Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
   Assert (i<rows, ExcIndexRange(i,0,rows));
@@ -810,6 +810,23 @@ SparsityPattern::exists (const unsigned int i,
 
 
 
+unsigned int
+SparsityPattern::row_position (const unsigned int i, const unsigned int j) const
+{
+  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
+  Assert (i<rows, ExcIndexRange(i,0,rows));
+  Assert (j<cols, ExcIndexRange(j,0,cols));
+
+  for (unsigned int k=rowstart[i]; k<rowstart[i+1]; k++)
+    {
+				       // entry exists
+      if (colnums[k] == j) return k-rowstart[i];
+    }
+  return numbers::invalid_unsigned_int;
+}
+
+
+
 std::pair<unsigned int, unsigned int>
 SparsityPattern::matrix_position (const unsigned int global_index) const
 {
@@ -1030,7 +1047,7 @@ SparsityPattern::block_read (std::istream &in)
 std::size_t
 SparsityPattern::memory_consumption () const
 {
-  return (max_dim * static_cast<size_t>(sizeof(unsigned int)) +
+  return (max_dim * sizeof(unsigned int) +
 	  sizeof(*this) +
 	  max_vec_len * sizeof(unsigned int));
 }
