@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: sparsity_tools.cc 22430 2010-10-23 13:41:09Z heister $
+//    $Id: sparsity_tools.cc 24418 2011-09-26 12:52:19Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,18 +11,18 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/exceptions.h>
-#include <lac/exceptions.h>
-#include <lac/sparsity_pattern.h>
-#include <lac/sparsity_tools.h>
+#include <deal.II/base/exceptions.h>
+#include <deal.II/lac/exceptions.h>
+#include <deal.II/lac/sparsity_pattern.h>
+#include <deal.II/lac/sparsity_tools.h>
 
 #include <algorithm>
 
 #ifdef DEAL_II_COMPILER_SUPPORTS_MPI
-#include <base/utilities.h>
-#include <lac/compressed_sparsity_pattern.h>
-#include <lac/compressed_set_sparsity_pattern.h>
-#include <lac/compressed_simple_sparsity_pattern.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/lac/compressed_sparsity_pattern.h>
+#include <deal.II/lac/compressed_set_sparsity_pattern.h>
+#include <deal.II/lac/compressed_simple_sparsity_pattern.h>
 #endif
 
 #ifdef DEAL_II_USE_METIS
@@ -421,7 +421,7 @@ namespace SparsityTools
 				   const MPI_Comm & mpi_comm,
 				   const IndexSet & myrange)
   {
-    unsigned int myid = Utilities::System::get_this_mpi_process(mpi_comm);
+    unsigned int myid = Utilities::MPI::this_mpi_process(mpi_comm);
     std::vector<unsigned int> start_index(rows_per_cpu.size()+1);
     start_index[0]=0;
     for (unsigned int i=0;i<rows_per_cpu.size();++i)
@@ -479,7 +479,7 @@ namespace SparsityTools
 	  send_to.push_back(it->first);
 
       num_receive =
-	Utilities::System::
+	Utilities::MPI::
 	compute_point_to_point_communication_pattern(mpi_comm, send_to).size();
     }
 
