@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_precondition.cc 21406 2010-06-29 14:59:33Z bangerth $
+//    $Id: trilinos_precondition.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be distributed
 //    without copyright and license information. Please refer
@@ -12,13 +12,13 @@
 //---------------------------------------------------------------------------
 
 
-#include <lac/trilinos_precondition.h>
+#include <deal.II/lac/trilinos_precondition.h>
 
 #ifdef DEAL_II_USE_TRILINOS
 
-#  include <lac/vector.h>
-#  include <lac/sparse_matrix.h>
-#  include <lac/trilinos_sparse_matrix.h>
+#  include <deal.II/lac/vector.h>
+#  include <deal.II/lac/sparse_matrix.h>
+#  include <deal.II/lac/trilinos_sparse_matrix.h>
 
 #  include <Ifpack.h>
 #  include <Ifpack_Chebyshev.h>
@@ -578,6 +578,7 @@ namespace TrilinosWrappers
       additional_data.constant_modes.size();
     Epetra_MultiVector distributed_constant_modes (domain_map,
 						   constant_modes_dimension);
+    std::vector<double> dummy (constant_modes_dimension);
 
     if (constant_modes_dimension > 1)
       {
@@ -609,8 +610,16 @@ namespace TrilinosWrappers
 	parameter_list.set("null space: type", "pre-computed");
 	parameter_list.set("null space: dimension",
 			   distributed_constant_modes.NumVectors());
-	parameter_list.set("null space: vectors",
-			   distributed_constant_modes.Values());
+	if (my_size > 0)
+	  parameter_list.set("null space: vectors",
+			     distributed_constant_modes.Values());
+				// We need to set a valid pointer to data even
+				// if there is no data on the current
+				// processor. Therefore, pass a dummy in that
+				// case
+	else
+	  parameter_list.set("null space: vectors",
+			     &dummy[0]);
       }
 
     initialize (matrix, parameter_list);
@@ -684,7 +693,8 @@ namespace TrilinosWrappers
 
 
 
-  unsigned int PreconditionAMG::memory_consumption() const
+  std::size_t
+  PreconditionAMG::memory_consumption() const
   {
     unsigned int memory = sizeof(this);
 
