@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_sparse_matrix.cc 22927 2010-12-06 13:55:13Z kronbichler $
+//    $Id: trilinos_sparse_matrix.cc 23986 2011-08-02 08:55:36Z kronbichler $
 //    Version: $Name$
 //
-//    Copyright (C) 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,17 +12,17 @@
 //---------------------------------------------------------------------------
 
 
-#include <lac/trilinos_sparse_matrix.h>
+#include <deal.II/lac/trilinos_sparse_matrix.h>
 
 #ifdef DEAL_II_USE_TRILINOS
 
-#  include <base/utilities.h>
-#  include <lac/sparse_matrix.h>
-#  include <lac/trilinos_sparsity_pattern.h>
-#  include <lac/sparsity_pattern.h>
-#  include <lac/compressed_sparsity_pattern.h>
-#  include <lac/compressed_set_sparsity_pattern.h>
-#  include <lac/compressed_simple_sparsity_pattern.h>
+#  include <deal.II/base/utilities.h>
+#  include <deal.II/lac/sparse_matrix.h>
+#  include <deal.II/lac/trilinos_sparsity_pattern.h>
+#  include <deal.II/lac/sparsity_pattern.h>
+#  include <deal.II/lac/compressed_sparsity_pattern.h>
+#  include <deal.II/lac/compressed_set_sparsity_pattern.h>
+#  include <deal.II/lac/compressed_simple_sparsity_pattern.h>
 
 #  include <ml_epetra_utils.h>
 #  include <ml_struct.h>
@@ -770,7 +770,7 @@ namespace TrilinosWrappers
   {
 				      // Extract local indices in
 				      // the matrix.
-    int trilinos_i = matrix->LRID(i), trilinos_j = matrix->LRID(j);
+    int trilinos_i = matrix->LRID(i), trilinos_j = matrix->LCID(j);
     TrilinosScalar value = 0.;
 
 				      // If the data is not on the
@@ -844,23 +844,18 @@ namespace TrilinosWrappers
   {
 				      // Extract local indices in
 				      // the matrix.
-    int trilinos_i = matrix->LRID(i), trilinos_j = matrix->LRID(j);
+    int trilinos_i = matrix->LRID(i), trilinos_j = matrix->LCID(j);
     TrilinosScalar value = 0.;
 
 				      // If the data is not on the
 				      // present processor, we can't
-				      // continue. Just print out
-				      // zero.
-
-				      // TODO: Is this reasonable? Or
-				      // should we retain the assert
-				      // call?
+				      // continue. Just print out zero
+				      // as discussed in the
+				      // documentation of this
+				      // function. if you want error
+				      // checking, use operator().
     if ((trilinos_i == -1 ) || (trilinos_j == -1))
-      {
-	return 0.;
-	//Assert (false, ExcAccessToNonLocalElement(i, j, local_range().first,
-	//				  local_range().second));
-      }
+      return 0.;
     else
     {
 				      // Check whether the matrix
@@ -958,7 +953,7 @@ namespace TrilinosWrappers
 			const VectorBase   &V,
 			const bool          transpose_left)
     {
-      const bool use_vector = V.size() == inputright.m() ? true : false;
+      const bool use_vector = (V.size() == inputright.m() ? true : false);
       if (transpose_left == false)
 	{
 	  Assert (inputleft.n() == inputright.m(),
@@ -1020,10 +1015,15 @@ namespace TrilinosWrappers
 				   // transposed matrix, let ML know it
       ML_Comm* comm;
       ML_Comm_Create(&comm);
+#ifdef ML_MPI
+      const Epetra_MpiComm *epcomm = dynamic_cast<const Epetra_MpiComm*>(&(inputleft.trilinos_matrix().Comm()));
+      // Get the MPI communicator, as it may not be MPI_COMM_W0RLD, and update the ML comm object
+      if (epcomm) ML_Comm_Set_UsrComm(comm,epcomm->Comm());
+#endif
       ML_Operator *A_ = ML_Operator_Create(comm);
-      ML_Operator *Anotrans_ = ML_Operator_Create(comm);
       ML_Operator *B_ = ML_Operator_Create(comm);
       ML_Operator *C_ = ML_Operator_Create(comm);
+      SparseMatrix transposed_mat;
 
       if (transpose_left == false)
 	ML_Operator_WrapEpetraCrsMatrix
@@ -1031,12 +1031,42 @@ namespace TrilinosWrappers
 	   false);
       else
 	{
-	  ML_Operator_WrapEpetraCrsMatrix
-	    (const_cast<Epetra_CrsMatrix*>(&inputleft.trilinos_matrix()),
-	     Anotrans_,false);
-	  ML_Operator_Transpose_byrow(Anotrans_,A_);
-	}
+				// create transposed matrix
+	  SparsityPattern sparsity_transposed (inputleft.domain_partitioner(),
+					       inputleft.range_partitioner());
+	  Assert (inputleft.domain_partitioner().LinearMap() == true,
+		  ExcMessage("Matrix must be partitioned contiguously between procs."));
+	  for (unsigned int i=0; i<inputleft.local_size(); ++i)
+	    {
+	      int num_entries, * indices;
+	      inputleft.trilinos_sparsity_pattern().ExtractMyRowView(i, num_entries,
+								     indices);
+	      Assert (num_entries >= 0, ExcInternalError());
+	      const unsigned int GID = inputleft.row_partitioner().GID(i);
+	      for (int j=0; j<num_entries; ++j)
+		sparsity_transposed.add (inputleft.col_partitioner().GID(indices[j]),
+					 GID);
+	    }
 
+	  sparsity_transposed.compress();
+	  transposed_mat.reinit (sparsity_transposed);
+	  for (unsigned int i=0; i<inputleft.local_size(); ++i)
+	    {
+	      int num_entries, * indices;
+	      double * values;
+	      inputleft.trilinos_matrix().ExtractMyRowView(i, num_entries,
+							   values, indices);
+	      Assert (num_entries >= 0, ExcInternalError());
+	      const unsigned int GID = inputleft.row_partitioner().GID(i);
+	      for (int j=0; j<num_entries; ++j)
+		transposed_mat.set (inputleft.col_partitioner().GID(indices[j]),
+				    GID, values[j]);
+	    }
+	  transposed_mat.compress();
+	  ML_Operator_WrapEpetraCrsMatrix
+	    (const_cast<Epetra_CrsMatrix*>(&transposed_mat.trilinos_matrix()),
+	     A_,false);
+	}
       ML_Operator_WrapEpetraCrsMatrix(mod_B.get(),B_,false);
 
 				   // We implement the multiplication by
@@ -1111,7 +1141,6 @@ namespace TrilinosWrappers
 				   // destroy allocated memory
       delete C_mat;
       ML_Operator_Destroy (&A_);
-      ML_Operator_Destroy (&Anotrans_);
       ML_Operator_Destroy (&B_);
       ML_Operator_Destroy (&C_);
       ML_Comm_Destroy (&comm);
@@ -1330,7 +1359,7 @@ namespace TrilinosWrappers
 	  {
 	    matrix->ExtractMyRowView (i, num_entries, values, indices);
 	    for (int j=0; j<num_entries; ++j)
-	      out << "(" << matrix->GRID(i) << "," << indices[matrix->GCID(j)] << ") "
+	      out << "(" << matrix->GRID(i) << "," << matrix->GCID(indices[j]) << ") "
 		  << values[j] << std::endl;
 	  }
       }
@@ -1340,7 +1369,7 @@ namespace TrilinosWrappers
 
 
 
-  unsigned int
+  std::size_t
   SparseMatrix::memory_consumption () const
   {
     unsigned int static_memory = sizeof(this) + sizeof (*matrix)
