@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_sparsity_pattern.cc 21751 2010-08-27 14:59:27Z kronbichler $
+//    $Id: trilinos_sparsity_pattern.cc 23748 2011-05-27 22:07:07Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,15 +12,15 @@
 //---------------------------------------------------------------------------
 
 
-#include <lac/trilinos_sparsity_pattern.h>
+#include <deal.II/lac/trilinos_sparsity_pattern.h>
 
 #ifdef DEAL_II_USE_TRILINOS
 
-#  include <base/utilities.h>
-#  include <lac/sparsity_pattern.h>
-#  include <lac/compressed_sparsity_pattern.h>
-#  include <lac/compressed_set_sparsity_pattern.h>
-#  include <lac/compressed_simple_sparsity_pattern.h>
+#  include <deal.II/base/utilities.h>
+#  include <deal.II/lac/sparsity_pattern.h>
+#  include <deal.II/lac/compressed_sparsity_pattern.h>
+#  include <deal.II/lac/compressed_set_sparsity_pattern.h>
+#  include <deal.II/lac/compressed_simple_sparsity_pattern.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -417,7 +417,7 @@ namespace TrilinosWrappers
   {
 				      // Extract local indices in
 				      // the matrix.
-    int trilinos_i = graph->LRID(i), trilinos_j = graph->LRID(j);
+    int trilinos_i = graph->LRID(i), trilinos_j = graph->LCID(j);
 
 				      // If the data is not on the
 				      // present processor, we throw
