@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: mg_dof_handler.cc 22870 2010-11-28 23:20:00Z kanschat $
+//    $Id: mg_dof_handler.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,20 +11,20 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/memory_consumption.h>
-#include <lac/sparse_matrix.h>
-#include <lac/constraint_matrix.h>
-#include <dofs/dof_levels.h>
-#include <dofs/dof_faces.h>
-#include <multigrid/mg_dof_handler.h>
-#include <multigrid/mg_dof_accessor.h>
-#include <grid/tria_levels.h>
-#include <grid/tria_accessor.h>
-#include <grid/tria_iterator.h>
-#include <grid/tria.h>
-#include <base/geometry_info.h>
-#include <fe/fe.h>
-#include <base/exceptions.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/constraint_matrix.h>
+#include <deal.II/dofs/dof_levels.h>
+#include <deal.II/dofs/dof_faces.h>
+#include <deal.II/multigrid/mg_dof_handler.h>
+#include <deal.II/multigrid/mg_dof_accessor.h>
+#include <deal.II/grid/tria_levels.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/base/geometry_info.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/base/exceptions.h>
 
 #include <algorithm>
 
@@ -868,10 +868,10 @@ MGDoFHandler<dim,spacedim>::~MGDoFHandler ()
 
 
 template <int dim, int spacedim>
-unsigned int
+std::size_t
 MGDoFHandler<dim,spacedim>::memory_consumption() const
 {
-  unsigned int mem = DoFHandler<dim,spacedim>::memory_consumption();
+  std::size_t mem = DoFHandler<dim,spacedim>::memory_consumption();
   for (unsigned int l=0;l<mg_levels.size();++l)
     mem += mg_levels[l]->memory_consumption();
 
