@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: mg_transfer_block.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: mg_transfer_block.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2010 by the deal.II authors
@@ -11,21 +11,21 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/logstream.h>
+#include <deal.II/base/logstream.h>
 
-#include <lac/vector.h>
-#include <lac/block_vector.h>
-#include <lac/sparse_matrix.h>
-#include <lac/block_sparse_matrix.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_tools.h>
-#include <fe/fe.h>
-#include <multigrid/mg_dof_handler.h>
-#include <multigrid/mg_dof_accessor.h>
-#include <multigrid/mg_transfer_block.h>
-#include <multigrid/mg_transfer_block.templates.h>
-#include <multigrid/mg_tools.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/block_sparse_matrix.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_tools.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/multigrid/mg_dof_handler.h>
+#include <deal.II/multigrid/mg_dof_accessor.h>
+#include <deal.II/multigrid/mg_transfer_block.h>
+#include <deal.II/multigrid/mg_transfer_block.templates.h>
+#include <deal.II/multigrid/mg_tools.h>
 
 #include <algorithm>
 #include <numeric>
