@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: mg_transfer_component.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: mg_transfer_component.cc 24324 2011-09-14 23:56:36Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
@@ -11,23 +11,23 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/logstream.h>
-#include <base/function.h>
+#include <deal.II/base/logstream.h>
+#include <deal.II/base/function.h>
 
-#include <lac/vector.h>
-#include <lac/block_vector.h>
-#include <lac/block_indices.h>
-#include <lac/sparse_matrix.h>
-#include <lac/block_sparse_matrix.h>
-#include <grid/tria.h>
-#include <grid/tria_iterator.h>
-#include <dofs/dof_tools.h>
-#include <fe/fe.h>
-#include <multigrid/mg_dof_handler.h>
-#include <multigrid/mg_dof_accessor.h>
-#include <multigrid/mg_transfer_component.h>
-#include <multigrid/mg_transfer_component.templates.h>
-#include <multigrid/mg_tools.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/block_indices.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/block_sparse_matrix.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/dofs/dof_tools.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/multigrid/mg_dof_handler.h>
+#include <deal.II/multigrid/mg_dof_accessor.h>
+#include <deal.II/multigrid/mg_transfer_component.h>
+#include <deal.II/multigrid/mg_transfer_component.templates.h>
+#include <deal.II/multigrid/mg_tools.h>
 
 #include <algorithm>
 #include <numeric>
@@ -650,7 +650,7 @@ void MGTransferSelect<number>::build_matrices (
 		    = mg_component_start[level][mg_target_component[component]];
 		  const unsigned int global_start
 		    = component_start[target_component[component]];
-		  temp_copy_indices[level_dof_indices[i]-level_start] = 
+		  temp_copy_indices[level_dof_indices[i]-level_start] =
 		    global_dof_indices[i] - global_start;
 		}
 	    }
