@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: data_out.cc 22976 2010-12-16 18:56:32Z bangerth $
+//    $Id: data_out.cc 24304 2011-09-11 05:44:26Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,25 +12,25 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/quadrature_lib.h>
-#include <base/work_stream.h>
-#include <base/memory_consumption.h>
-#include <lac/vector.h>
-#include <lac/block_vector.h>
-#include <lac/petsc_vector.h>
-#include <lac/petsc_block_vector.h>
-#include <lac/trilinos_vector.h>
-#include <lac/trilinos_block_vector.h>
-#include <numerics/data_out.h>
-#include <grid/tria.h>
-#include <dofs/dof_handler.h>
-#include <dofs/dof_accessor.h>
-#include <hp/dof_handler.h>
-#include <grid/tria_iterator.h>
-#include <fe/fe.h>
-#include <fe/fe_values.h>
-#include <hp/fe_values.h>
-#include <fe/mapping_q1.h>
+#include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/work_stream.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/petsc_vector.h>
+#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/trilinos_vector.h>
+#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/numerics/data_out.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/hp/dof_handler.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/hp/fe_values.h>
+#include <deal.II/fe/mapping_q1.h>
 
 #include <sstream>
 
@@ -286,7 +286,7 @@ get_function_hessians (const FEValuesBase<DH::dimension,DH::space_dimension> &fe
 template <class DH,
 	  int patch_dim, int patch_space_dim>
 template <typename VectorType>
-unsigned int
+std::size_t
 DataOut_DoFData<DH,patch_dim,patch_space_dim>::
 DataEntry<VectorType>::memory_consumption () const
 {
@@ -660,7 +660,7 @@ DataOut_DoFData<DH,patch_dim,patch_space_dim>::get_patches () const
 
 template <class DH,
 	  int patch_dim, int patch_space_dim>
-unsigned int
+std::size_t
 DataOut_DoFData<DH,patch_dim,patch_space_dim>::memory_consumption () const
 {
   return (DataOutInterface<patch_dim,patch_space_dim>::memory_consumption () +
@@ -708,14 +708,22 @@ build_one_patch (const std::pair<cell_iterator, unsigned int> *cell_and_index,
 				       // of curved cells, if necessary
 				       // append the quadrature points to
 				       // the last rows of the patch.data
-				       // member. THis is the case if we
+				       // member. This is the case if we
 				       // want to produce curved cells at
 				       // the boundary and this cell
 				       // actually is at the boundary, or
 				       // else if we want to produce curved
 				       // cells everywhere
-      if (curved_cell_region==curved_inner_cells ||
-	  (curved_cell_region==curved_boundary && cell_and_index->first->at_boundary()))
+				       //
+				       // note: a cell is *always* at
+				       // the boundary if dim<spacedim
+      if (curved_cell_region==curved_inner_cells
+	  ||
+	  (curved_cell_region==curved_boundary
+	   &&
+	   (cell_and_index->first->at_boundary()
+	    ||
+	    (DH::dimension != DH::space_dimension))))
 	{
 	  Assert(patch.space_dim==DH::space_dimension, ExcInternalError());
 	  const std::vector<Point<DH::space_dimension> > & q_points=fe_patch_values.get_quadrature_points();
@@ -768,11 +776,11 @@ build_one_patch (const std::pair<cell_iterator, unsigned int> *cell_and_index,
 		  if (update_flags & update_hessians)
 		    this->dof_data[dataset]->get_function_hessians (fe_patch_values,
 								    data.patch_hessians);
-								    
+
 		  if (update_flags & update_quadrature_points)
 		    data.patch_evaluation_points = fe_patch_values.get_quadrature_points();
-		    
-		    
+
+
 		  std::vector<Point<DH::space_dimension> > dummy_normals;
 		  postprocessor->
 		    compute_derived_quantities_scalar(data.patch_values,
@@ -797,12 +805,12 @@ build_one_patch (const std::pair<cell_iterator, unsigned int> *cell_and_index,
 		  if (update_flags & update_hessians)
 		    this->dof_data[dataset]->get_function_hessians (fe_patch_values,
 								    data.patch_hessians_system);
-		  
+
 		  if (update_flags & update_quadrature_points)
 		    data.patch_evaluation_points = fe_patch_values.get_quadrature_points();
-		  
+
 		  std::vector<Point<DH::space_dimension> > dummy_normals;
-		  
+
 		  postprocessor->
 		    compute_derived_quantities_vector(data.patch_values_system,
 						      data.patch_gradients_system,
@@ -1086,15 +1094,16 @@ void DataOut<dim,DH>::build_patches (const Mapping<DH::dimension,DH::space_dimen
 
 
 				   // now build the patches in parallel
-  WorkStream::run (&all_cells[0],
-		   &all_cells[0]+all_cells.size(),
-		   std_cxx1x::bind(&DataOut<dim,DH>::build_one_patch,
-				   *this, _1, _2, _3,
-				   curved_cell_region),
-		   std_cxx1x::bind(&internal::DataOut::append_patch_to_list<dim,DH::space_dimension>,
-				   _1, std_cxx1x::ref(this->patches)),
-		   thread_data,
-		   sample_patch);
+  if (all_cells.size() > 0)
+    WorkStream::run (&all_cells[0],
+		     &all_cells[0]+all_cells.size(),
+		     std_cxx1x::bind(&DataOut<dim,DH>::build_one_patch,
+				     *this, std_cxx1x::_1, std_cxx1x::_2, std_cxx1x::_3,
+				     curved_cell_region),
+		     std_cxx1x::bind(&internal::DataOut::append_patch_to_list<dim,DH::space_dimension>,
+				     std_cxx1x::_1, std_cxx1x::ref(this->patches)),
+		     thread_data,
+		     sample_patch);
 }
 
 
@@ -1134,7 +1143,7 @@ DataOut<dim,DH>::first_locally_owned_cell ()
 				   // is a ghost or artificial cell
   while ((cell != this->dofs->end()) &&
 	 (cell->has_children() == false) &&
-	 (cell->is_ghost() || cell->is_artificial()))
+	 !cell->is_locally_owned())
     cell = next_cell(cell);
 
   return cell;
@@ -1150,7 +1159,7 @@ DataOut<dim,DH>::next_locally_owned_cell (const typename DataOut<dim,DH>::cell_i
     cell = next_cell(old_cell);
   while ((cell != this->dofs->end()) &&
 	 (cell->has_children() == false) &&
-	 (cell->is_ghost() || cell->is_artificial()))
+	 !cell->is_locally_owned())
     cell = next_cell(cell);
   return cell;
 }
