@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: error_estimator.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: error_estimator.cc 23752 2011-05-30 00:16:00Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,30 +12,30 @@
 //---------------------------------------------------------------------------
 
 
-#include <base/thread_management.h>
-#include <base/quadrature.h>
-#include <base/quadrature_lib.h>
-#include <base/work_stream.h>
-#include <lac/vector.h>
-#include <lac/block_vector.h>
-#include <lac/petsc_vector.h>
-#include <lac/petsc_block_vector.h>
-#include <lac/trilinos_vector.h>
-#include <lac/trilinos_block_vector.h>
-#include <grid/tria_iterator.h>
-#include <base/geometry_info.h>
-#include <dofs/dof_handler.h>
-#include <dofs/dof_accessor.h>
-#include <fe/fe.h>
-#include <fe/fe_values.h>
-#include <hp/fe_values.h>
-#include <fe/fe_update_flags.h>
-#include <fe/mapping_q1.h>
-#include <hp/q_collection.h>
-#include <hp/mapping_collection.h>
-#include <numerics/error_estimator.h>
+#include <deal.II/base/thread_management.h>
+#include <deal.II/base/quadrature.h>
+#include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/work_stream.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/petsc_vector.h>
+#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/trilinos_vector.h>
+#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/base/geometry_info.h>
+#include <deal.II/dofs/dof_handler.h>
+#include <deal.II/dofs/dof_accessor.h>
+#include <deal.II/fe/fe.h>
+#include <deal.II/fe/fe_values.h>
+#include <deal.II/hp/fe_values.h>
+#include <deal.II/fe/fe_update_flags.h>
+#include <deal.II/fe/mapping_q1.h>
+#include <deal.II/hp/q_collection.h>
+#include <deal.II/hp/mapping_collection.h>
+#include <deal.II/numerics/error_estimator.h>
 
-#include <base/std_cxx1x/bind.h>
+#include <deal.II/base/std_cxx1x/bind.h>
 
 #include <numeric>
 #include <algorithm>
@@ -154,9 +154,9 @@ namespace internal
 					  * the faces of the current and
 					  * potentially of neighbor cells.
 					  */
-	dealii::hp::FEFaceValues<dim>    fe_face_values_cell;
-	dealii::hp::FEFaceValues<dim>    fe_face_values_neighbor;
-	dealii::hp::FESubfaceValues<dim> fe_subface_values;
+	dealii::hp::FEFaceValues<dim,spacedim>    fe_face_values_cell;
+	dealii::hp::FEFaceValues<dim,spacedim>    fe_face_values_neighbor;
+	dealii::hp::FESubfaceValues<dim,spacedim> fe_subface_values;
 
 					 /**
 					  * A vector to store the jump
@@ -192,18 +192,18 @@ namespace internal
 					  * index of the solution
 					  * vector.
 					  */
-	std::vector<std::vector<std::vector<Tensor<1,dim> > > > psi;
+	std::vector<std::vector<std::vector<Tensor<1,spacedim> > > > psi;
 
 					 /**
 					  * The same vector for a neighbor cell
 					  */
-	std::vector<std::vector<std::vector<Tensor<1,dim> > > > neighbor_psi;
+	std::vector<std::vector<std::vector<Tensor<1,spacedim> > > > neighbor_psi;
 
 					 /**
 					  * The normal vectors of the finite
 					  * element function on one face
 					  */
-	std::vector<Point<dim> > normal_vectors;
+	std::vector<Point<spacedim> > normal_vectors;
 
 					 /**
 					  * Two arrays needed for the
@@ -238,9 +238,9 @@ namespace internal
 					  * the KellyErrorEstimator::estimate()
 					  * function.
 					  */
-	const typename FunctionMap<dim>::type *neumann_bc;
+	const typename FunctionMap<spacedim>::type *neumann_bc;
 	const std::vector<bool>                component_mask;
-	const Function<dim>                   *coefficients;
+	const Function<spacedim>                   *coefficients;
 
 					 /**
 					  * Constructor.
@@ -253,9 +253,9 @@ namespace internal
 		      const unsigned int n_solution_vectors,
 		      const types::subdomain_id_t subdomain_id,
 		      const unsigned int material_id,
-		      const typename FunctionMap<DH::dimension>::type *neumann_bc,
+		      const typename FunctionMap<spacedim>::type *neumann_bc,
 		      const std::vector<bool>                component_mask,
-		      const Function<DH::dimension>                   *coefficients);
+		      const Function<spacedim>                   *coefficients);
 
 					 /**
 					  * Resize the arrays so that they fit the
@@ -277,9 +277,9 @@ namespace internal
 		  const unsigned int n_solution_vectors,
 		  const types::subdomain_id_t subdomain_id,
 		  const unsigned int material_id,
-		  const typename FunctionMap<DH::dimension>::type *neumann_bc,
+		  const typename FunctionMap<spacedim>::type *neumann_bc,
 		  const std::vector<bool>                component_mask,
-		  const Function<DH::dimension>         *coefficients)
+		  const Function<spacedim>         *coefficients)
 		    :
 		    finite_element (fe),
 		    face_quadratures (face_quadratures),
@@ -305,13 +305,13 @@ namespace internal
 			 (face_quadratures.max_n_quadrature_points(),
 			  std::vector<double> (fe.n_components()))),
 		    psi (n_solution_vectors,
-			 std::vector<std::vector<Tensor<1,dim> > >
+			 std::vector<std::vector<Tensor<1,spacedim> > >
 			 (face_quadratures.max_n_quadrature_points(),
-			  std::vector<Tensor<1,dim> > (fe.n_components()))),
+			  std::vector<Tensor<1,spacedim> > (fe.n_components()))),
 		    neighbor_psi (n_solution_vectors,
-				  std::vector<std::vector<Tensor<1,dim> > >
+				  std::vector<std::vector<Tensor<1,spacedim> > >
 				  (face_quadratures.max_n_quadrature_points(),
-				   std::vector<Tensor<1,dim> > (fe.n_components()))),
+				   std::vector<Tensor<1,spacedim> > (fe.n_components()))),
 		    normal_vectors (face_quadratures.max_n_quadrature_points()),
 		    coefficient_values1 (face_quadratures.max_n_quadrature_points()),
 		    coefficient_values (face_quadratures.max_n_quadrature_points(),
@@ -414,8 +414,8 @@ namespace internal
 				 std::map<typename DH::face_iterator,std::vector<double> > &local_face_integrals,
 				 const typename DH::active_cell_iterator &cell,
 				 const unsigned int                       face_no,
-				 dealii::hp::FEFaceValues<DH::dimension> &fe_face_values_cell,
-				 dealii::hp::FEFaceValues<DH::dimension> &fe_face_values_neighbor)
+				 dealii::hp::FEFaceValues<DH::dimension, DH::space_dimension> &fe_face_values_cell,
+				 dealii::hp::FEFaceValues<DH::dimension, DH::space_dimension> &fe_face_values_neighbor)
     {
       const unsigned int dim = DH::dimension;
 
@@ -625,8 +625,8 @@ namespace internal
 				   std::map<typename DH::face_iterator,std::vector<double> > &local_face_integrals,
 				   const typename DH::active_cell_iterator    &cell,
 				   const unsigned int                          face_no,
-				   dealii::hp::FEFaceValues<DH::dimension>    &fe_face_values,
-				   dealii::hp::FESubfaceValues<DH::dimension> &fe_subface_values)
+				   dealii::hp::FEFaceValues<DH::dimension,DH::space_dimension>    &fe_face_values,
+				   dealii::hp::FESubfaceValues<DH::dimension, DH::space_dimension> &fe_subface_values)
     {
       const unsigned int dim = DH::dimension;
 
@@ -871,7 +871,7 @@ namespace internal
 					   // then do nothing since we
 					   // integrate over the subfaces when
 					   // we visit the coarse cells.
-	  if (cell->at_boundary(face_no) == false)
+	  if (face->at_boundary() == false)
 	    if (cell->neighbor_is_coarser(face_no))
 	      continue;
 
@@ -884,11 +884,9 @@ namespace internal
 					   // cells, we enter this face into
 					   // the list of faces with
 					   // contribution zero.
-	  const unsigned char boundary_indicator
-	    = face->boundary_indicator();
 	  if (face->at_boundary()
 	      &&
-	      (parallel_data.neumann_bc->find(boundary_indicator) ==
+	      (parallel_data.neumann_bc->find(face->boundary_indicator()) ==
 	       parallel_data.neumann_bc->end()))
 	    {
 	      local_face_integrals[face]
@@ -1010,11 +1008,11 @@ KellyErrorEstimator<1,spacedim>::
 estimate (const Mapping<1,spacedim>      &mapping,
           const DH   &dof_handler,
           const Quadrature<0> &quadrature,
-          const FunctionMap<1>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const InputVector       &solution,
           Vector<float>           &error,
           const std::vector<bool> &component_mask,
-          const Function<1>     *coefficients,
+          const Function<spacedim>     *coefficients,
           const unsigned int       n_threads,
           const types::subdomain_id_t subdomain_id,
           const unsigned int       material_id)
@@ -1034,11 +1032,11 @@ void
 KellyErrorEstimator<1,spacedim>::
 estimate (const DH   &dof_handler,
           const Quadrature<0> &quadrature,
-          const FunctionMap<1>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const InputVector       &solution,
           Vector<float>           &error,
           const std::vector<bool> &component_mask,
-          const Function<1>     *coefficients,
+          const Function<spacedim>     *coefficients,
           const unsigned int       n_threads,
           const types::subdomain_id_t subdomain_id,
           const unsigned int       material_id)
@@ -1056,11 +1054,11 @@ void
 KellyErrorEstimator<1,spacedim>::
 estimate (const DH   &dof_handler,
           const Quadrature<0> &quadrature,
-          const FunctionMap<1>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const std::vector<const InputVector*> &solutions,
           std::vector<Vector<float>*> &errors,
           const std::vector<bool> &component_mask,
-          const Function<1>     *coefficients,
+          const Function<spacedim>     *coefficients,
           const unsigned int       n_threads,
           const types::subdomain_id_t subdomain_id,
           const unsigned int       material_id)
@@ -1079,11 +1077,11 @@ KellyErrorEstimator<1,spacedim>::
 estimate (const Mapping<1,spacedim>      &mapping,
           const DH   &dof_handler,
           const hp::QCollection<0> &quadrature,
-          const FunctionMap<1>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const InputVector       &solution,
           Vector<float>           &error,
           const std::vector<bool> &component_mask,
-          const Function<1>     *coefficients,
+          const Function<spacedim>     *coefficients,
           const unsigned int       n_threads,
           const types::subdomain_id_t subdomain_id,
           const unsigned int       material_id)
@@ -1102,11 +1100,11 @@ void
 KellyErrorEstimator<1,spacedim>::
 estimate (const DH   &dof_handler,
           const hp::QCollection<0> &quadrature,
-          const FunctionMap<1>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const InputVector       &solution,
           Vector<float>           &error,
           const std::vector<bool> &component_mask,
-          const Function<1>     *coefficients,
+          const Function<spacedim>     *coefficients,
           const unsigned int       n_threads,
           const types::subdomain_id_t subdomain_id,
           const unsigned int       material_id)
@@ -1124,11 +1122,11 @@ void
 KellyErrorEstimator<1,spacedim>::
 estimate (const DH   &dof_handler,
           const hp::QCollection<0> &quadrature,
-          const FunctionMap<1>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const std::vector<const InputVector*> &solutions,
           std::vector<Vector<float>*> &errors,
           const std::vector<bool> &component_mask,
-          const Function<1>     *coefficients,
+          const Function<spacedim>     *coefficients,
           const unsigned int       n_threads,
           const types::subdomain_id_t subdomain_id,
           const unsigned int       material_id)
@@ -1147,11 +1145,11 @@ void KellyErrorEstimator<1,spacedim>::
 estimate (const Mapping<1,spacedim>                    &/*mapping*/,
           const DH                            &/*dof_handler*/,
           const hp::QCollection<0>            &,
-          const FunctionMap<1>::type          &/*neumann_bc*/,
+          const typename FunctionMap<spacedim>::type          &/*neumann_bc*/,
           const std::vector<const InputVector *> &/*solutions*/,
           std::vector<Vector<float>*>            &/*errors*/,
           const std::vector<bool>                &/*component_mask_*/,
-          const Function<1>                   */*coefficient*/,
+          const Function<spacedim>                   */*coefficient*/,
           const unsigned int,
           const types::subdomain_id_t          /*subdomain_id*/,
           const unsigned int                   /*material_id*/)
@@ -1167,11 +1165,11 @@ void KellyErrorEstimator<1,spacedim>::
 estimate (const Mapping<1,spacedim>                    &mapping,
           const DH                 &dof_handler,
           const Quadrature<0>                 &,
-          const FunctionMap<1>::type          &neumann_bc,
+          const typename FunctionMap<spacedim>::type          &neumann_bc,
           const std::vector<const InputVector *> &solutions,
           std::vector<Vector<float>*>              &errors,
           const std::vector<bool>                  &component_mask_,
-          const Function<1>                   *coefficient,
+          const Function<spacedim>                   *coefficient,
           const unsigned int,
           const types::subdomain_id_t         subdomain_id_,
           const unsigned int                  material_id)
@@ -1210,7 +1208,7 @@ estimate (const Mapping<1,spacedim>                    &mapping,
   Assert (neumann_bc.find(255) == neumann_bc.end(),
 	  ExcInvalidBoundaryIndicator());
 
-  for (FunctionMap<1>::type::const_iterator i=neumann_bc.begin();
+  for (typename FunctionMap<spacedim>::type::const_iterator i=neumann_bc.begin();
        i!=neumann_bc.end(); ++i)
     Assert (i->second->n_components == n_components, ExcInvalidBoundaryFunction());
 
@@ -1247,7 +1245,7 @@ estimate (const Mapping<1,spacedim>                    &mapping,
 	  (coefficient->n_components == 1),
 	  ExcInvalidCoefficient());
 
-  for (FunctionMap<1>::type::const_iterator i=neumann_bc.begin();
+  for (typename FunctionMap<spacedim>::type::const_iterator i=neumann_bc.begin();
        i!=neumann_bc.end(); ++i)
     Assert (i->second->n_components == n_components,
             ExcInvalidBoundaryFunction());
@@ -1264,10 +1262,10 @@ estimate (const Mapping<1,spacedim>                    &mapping,
 				   // need several auxiliary fields,
 				   // depending on the way we get it
 				   // (see below)
-  std::vector<std::vector<std::vector<Tensor<1,1> > > >
+  std::vector<std::vector<std::vector<Tensor<1,spacedim> > > >
     gradients_here (n_solution_vectors,
-		    std::vector<std::vector<Tensor<1,1> > >(2, std::vector<Tensor<1,1> >(n_components)));
-  std::vector<std::vector<std::vector<Tensor<1,1> > > >
+		    std::vector<std::vector<Tensor<1,spacedim> > >(2, std::vector<Tensor<1,spacedim> >(n_components)));
+  std::vector<std::vector<std::vector<Tensor<1,spacedim> > > >
     gradients_neighbor (gradients_here);
   std::vector<Vector<double> >
     grad_neighbor (n_solution_vectors, Vector<double>(n_components));
@@ -1437,11 +1435,11 @@ KellyErrorEstimator<dim, spacedim>::
 estimate (const Mapping<dim, spacedim>      &mapping,
           const DH                &dof_handler,
           const Quadrature<dim-1> &quadrature,
-          const typename FunctionMap<dim>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const InputVector       &solution,
           Vector<float>           &error,
           const std::vector<bool> &component_mask,
-          const Function<dim>     *coefficients,
+          const Function<spacedim>     *coefficients,
           const unsigned int       n_threads,
           const types::subdomain_id_t subdomain_id,
           const unsigned int       material_id)
@@ -1460,17 +1458,17 @@ void
 KellyErrorEstimator<dim,spacedim>::
 estimate (const DH                &dof_handler,
           const Quadrature<dim-1> &quadrature,
-          const typename FunctionMap<dim>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const InputVector       &solution,
           Vector<float>           &error,
           const std::vector<bool> &component_mask,
-          const Function<dim>     *coefficients,
+          const Function<spacedim>     *coefficients,
           const unsigned int       n_threads,
           const types::subdomain_id_t subdomain_id,
           const unsigned int       material_id)
 {
   Assert (DEAL_II_COMPAT_MAPPING, ExcCompatibility("mapping"));
-  estimate(StaticMappingQ1<dim>::mapping, dof_handler, quadrature, neumann_bc, solution,
+  estimate(StaticMappingQ1<dim,spacedim>::mapping, dof_handler, quadrature, neumann_bc, solution,
 	   error, component_mask, coefficients, n_threads,
            subdomain_id, material_id);
 }
@@ -1483,11 +1481,11 @@ KellyErrorEstimator<dim, spacedim>::
 estimate (const Mapping<dim, spacedim>      &mapping,
           const DH                &dof_handler,
           const hp::QCollection<dim-1> &quadrature,
-          const typename FunctionMap<dim>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const InputVector       &solution,
           Vector<float>           &error,
           const std::vector<bool> &component_mask,
-          const Function<dim>     *coefficients,
+          const Function<spacedim>     *coefficients,
           const unsigned int       n_threads,
           const types::subdomain_id_t subdomain_id,
           const unsigned int       material_id)
@@ -1506,17 +1504,17 @@ void
 KellyErrorEstimator<dim, spacedim>::
 estimate (const DH                &dof_handler,
           const hp::QCollection<dim-1> &quadrature,
-          const typename FunctionMap<dim>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const InputVector       &solution,
           Vector<float>           &error,
           const std::vector<bool> &component_mask,
-          const Function<dim>     *coefficients,
+          const Function<spacedim>     *coefficients,
           const unsigned int       n_threads,
           const types::subdomain_id_t subdomain_id,
           const unsigned int       material_id)
 {
   Assert (DEAL_II_COMPAT_MAPPING, ExcCompatibility("mapping"));
-  estimate(StaticMappingQ1<dim>::mapping, dof_handler, quadrature, neumann_bc, solution,
+  estimate(StaticMappingQ1<dim, spacedim>::mapping, dof_handler, quadrature, neumann_bc, solution,
 	   error, component_mask, coefficients, n_threads,
            subdomain_id, material_id);
 }
@@ -1531,11 +1529,11 @@ KellyErrorEstimator<dim, spacedim>::
 estimate (const Mapping<dim, spacedim>                  &mapping,
           const DH                            &dof_handler,
           const hp::QCollection<dim-1>        &face_quadratures,
-          const typename FunctionMap<dim>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const std::vector<const InputVector *> &solutions,
           std::vector<Vector<float>*>              &errors,
           const std::vector<bool>                  &component_mask_,
-          const Function<dim>                 *coefficients,
+          const Function<spacedim>                 *coefficients,
           const unsigned int                   ,
           const types::subdomain_id_t          subdomain_id_,
           const unsigned int                   material_id)
@@ -1575,7 +1573,7 @@ estimate (const Mapping<dim, spacedim>                  &mapping,
   Assert (solutions.size() == errors.size(),
 	  ExcIncompatibleNumberOfElements(solutions.size(), errors.size()));
 
-  for (typename FunctionMap<dim>::type::const_iterator i=neumann_bc.begin();
+  for (typename FunctionMap<spacedim>::type::const_iterator i=neumann_bc.begin();
        i!=neumann_bc.end(); ++i)
     Assert (i->second->n_components == n_components,
             ExcInvalidBoundaryFunction());
@@ -1638,9 +1636,9 @@ estimate (const Mapping<dim, spacedim>                  &mapping,
   WorkStream::run (dof_handler.begin_active(),
 		   static_cast<typename DH::active_cell_iterator>(dof_handler.end()),
 		   std_cxx1x::bind (&internal::estimate_one_cell<dim,spacedim,InputVector,DH>,
-				    _1, _2, _3, std_cxx1x::ref(solutions)),
+				    std_cxx1x::_1, std_cxx1x::_2, std_cxx1x::_3, std_cxx1x::ref(solutions)),
 		   std_cxx1x::bind (&internal::copy_local_to_global<DH>,
-				    _1, std_cxx1x::ref(face_integrals)),
+				    std_cxx1x::_1, std_cxx1x::ref(face_integrals)),
 		   parallel_data,
 		   sample_local_face_integrals);
 
@@ -1708,11 +1706,11 @@ KellyErrorEstimator<dim, spacedim>::
 estimate (const Mapping<dim, spacedim>                  &mapping,
           const DH                            &dof_handler,
           const Quadrature<dim-1>             &quadrature,
-          const typename FunctionMap<dim>::type &neumann_bc,
+          const typename FunctionMap<spacedim>::type &neumann_bc,
           const std::vector<const InputVector *> &solutions,
           std::vector<Vector<float>*>              &errors,
           const std::vector<bool>                  &component_mask,
-          const Function<dim>                 *coefficients,
+          const Function<spacedim>                 *coefficients,
           const unsigned int                   n_threads,
           const types::subdomain_id_t          subdomain_id,
           const unsigned int                   material_id)
@@ -1730,17 +1728,17 @@ template <int dim, int spacedim>
 template <typename InputVector, class DH>
 void KellyErrorEstimator<dim, spacedim>::estimate (const DH                            &dof_handler,
 					 const Quadrature<dim-1>             &quadrature,
-					 const typename FunctionMap<dim>::type &neumann_bc,
+					 const typename FunctionMap<spacedim>::type &neumann_bc,
 					 const std::vector<const InputVector *> &solutions,
 					 std::vector<Vector<float>*>              &errors,
 					 const std::vector<bool>                  &component_mask,
-					 const Function<dim>                 *coefficients,
+					 const Function<spacedim>                 *coefficients,
 					 const unsigned int                   n_threads,
                                          const types::subdomain_id_t subdomain_id,
                                          const unsigned int       material_id)
 {
   Assert (DEAL_II_COMPAT_MAPPING, ExcCompatibility("mapping"));
-  estimate(StaticMappingQ1<dim>::mapping, dof_handler, quadrature, neumann_bc, solutions,
+  estimate(StaticMappingQ1<dim, spacedim>::mapping, dof_handler, quadrature, neumann_bc, solutions,
 	   errors, component_mask, coefficients, n_threads,
            subdomain_id, material_id);
 }
@@ -1751,17 +1749,17 @@ template <int dim, int spacedim>
 template <typename InputVector, class DH>
 void KellyErrorEstimator<dim, spacedim>::estimate (const DH                            &dof_handler,
 					 const hp::QCollection<dim-1>             &quadrature,
-					 const typename FunctionMap<dim>::type &neumann_bc,
+					 const typename FunctionMap<spacedim>::type &neumann_bc,
 					 const std::vector<const InputVector *> &solutions,
 					 std::vector<Vector<float>*>              &errors,
 					 const std::vector<bool>                  &component_mask,
-					 const Function<dim>                 *coefficients,
+					 const Function<spacedim>                 *coefficients,
 					 const unsigned int                   n_threads,
                                          const types::subdomain_id_t subdomain_id,
                                          const unsigned int       material_id)
 {
   Assert (DEAL_II_COMPAT_MAPPING, ExcCompatibility("mapping"));
-  estimate(StaticMappingQ1<dim>::mapping, dof_handler, quadrature, neumann_bc, solutions,
+  estimate(StaticMappingQ1<dim, spacedim>::mapping, dof_handler, quadrature, neumann_bc, solutions,
 	   errors, component_mask, coefficients, n_threads,
            subdomain_id, material_id);
 }
