@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: mesh_worker_info.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: mesh_worker_info.cc 24272 2011-09-06 16:01:55Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 2009, 2010 by the deal.II authors
+//    Copyright (C) 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -11,14 +11,15 @@
 //
 //---------------------------------------------------------------------------
 
-#include <lac/vector.h>
-#include <lac/block_vector.h>
-#include <lac/petsc_vector.h>
-#include <lac/petsc_block_vector.h>
-#include <lac/trilinos_vector.h>
-#include <lac/trilinos_block_vector.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/petsc_vector.h>
+#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/trilinos_vector.h>
+#include <deal.II/lac/trilinos_block_vector.h>
 
-#include <numerics/mesh_worker_info.templates.h>
+#include <deal.II/meshworker/dof_info.templates.h>
+#include <deal.II/meshworker/integration_info.templates.h>
 
 DEAL_II_NAMESPACE_OPEN
 
