@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: mesh_worker_vector_selector.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: mesh_worker_vector_selector.cc 24247 2011-09-04 17:20:00Z young $
 //    Version: $Name$
 //
 //    Copyright (C) 2009, 2010 by the deal.II authors
@@ -11,14 +11,14 @@
 //
 //---------------------------------------------------------------------------
 
-#include <lac/vector.h>
-#include <lac/block_vector.h>
-#include <lac/petsc_vector.h>
-#include <lac/petsc_block_vector.h>
-#include <lac/trilinos_vector.h>
-#include <lac/trilinos_block_vector.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/petsc_vector.h>
+#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/trilinos_vector.h>
+#include <deal.II/lac/trilinos_block_vector.h>
 
-#include <numerics/mesh_worker_vector_selector.templates.h>
+#include <deal.II/meshworker/vector_selector.templates.h>
 
 DEAL_II_NAMESPACE_OPEN
 
