@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: operator.cc 21513 2010-07-18 04:00:43Z kanschat $
+//    $Id: operator.cc 23750 2011-05-29 23:39:28Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2010 by the deal.II authors
@@ -11,18 +11,18 @@
 //
 //---------------------------------------------------------------------------
 
-#include <base/parameter_handler.h>
-#include <base/logstream.h>
-#include <lac/vector_memory.h>
+#include <deal.II/base/parameter_handler.h>
+#include <deal.II/base/logstream.h>
+#include <deal.II/lac/vector_memory.h>
 
-#include <numerics/dof_output_operator.templates.h>
+#include <deal.II/numerics/dof_output_operator.templates.h>
 
-#include <lac/vector.h>
-#include <lac/block_vector.h>
-#include <lac/petsc_vector.h>
-#include <lac/petsc_block_vector.h>
-#include <lac/trilinos_vector.h>
-#include <lac/trilinos_block_vector.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/petsc_vector.h>
+#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/trilinos_vector.h>
+#include <deal.II/lac/trilinos_block_vector.h>
 
 
 DEAL_II_NAMESPACE_OPEN
