@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: point_value_history.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: point_value_history.cc 23709 2011-05-17 04:34:08Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2009, 2010 by Michael Rapson and the deal.II authors
@@ -11,14 +11,14 @@
 //
 //---------------------------------------------------------------------------
 
-#include <lac/vector.h>
-#include <lac/block_vector.h>
-#include <lac/petsc_vector.h>
-#include <lac/petsc_block_vector.h>
-#include <lac/trilinos_vector.h>
-#include <lac/trilinos_block_vector.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/petsc_vector.h>
+#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/trilinos_vector.h>
+#include <deal.II/lac/trilinos_block_vector.h>
 
-#include <numerics/point_value_history.h>
+#include <deal.II/numerics/point_value_history.h>
 
 
 DEAL_II_NAMESPACE_OPEN
