@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: time_dependent.cc 22749 2010-11-16 13:50:47Z kronbichler $
+//    $Id: time_dependent.cc 23944 2011-07-12 12:23:28Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2008, 2009, 2010 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2008, 2009, 2010, 2011 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -12,14 +12,15 @@
 //---------------------------------------------------------------------------
 
 
-#include <numerics/time_dependent.h>
-#include <base/memory_consumption.h>
-#include <base/thread_management.h>
-#include <grid/tria.h>
-#include <grid/tria_accessor.h>
-#include <grid/tria_iterator.h>
-#include <grid/grid_refinement.h>
-#include <lac/vector.h>
+#include <deal.II/numerics/time_dependent.h>
+#include <deal.II/base/memory_consumption.h>
+#include <deal.II/base/thread_management.h>
+#include <deal.II/base/utilities.h>
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/tria_accessor.h>
+#include <deal.II/grid/tria_iterator.h>
+#include <deal.II/grid/grid_refinement.h>
+#include <deal.II/lac/vector.h>
 
 #include <functional>
 #include <algorithm>
@@ -234,13 +235,14 @@ void TimeDependent::end_sweep (const unsigned int begin,
 
 
 
-unsigned int TimeDependent::memory_consumption () const
+std::size_t
+TimeDependent::memory_consumption () const
 {
-  unsigned int mem = (MemoryConsumption::memory_consumption (timesteps) +
-		      MemoryConsumption::memory_consumption (sweep_no) +
-		      sizeof(timestepping_data_primal) +
-		      sizeof(timestepping_data_dual) +
-		      sizeof(timestepping_data_postprocess));
+  std::size_t mem = (MemoryConsumption::memory_consumption (timesteps) +
+		     MemoryConsumption::memory_consumption (sweep_no) +
+		     sizeof(timestepping_data_primal) +
+		     sizeof(timestepping_data_dual) +
+		     sizeof(timestepping_data_postprocess));
   for (unsigned int i=0; i<timesteps.size(); ++i)
     mem += MemoryConsumption::memory_consumption (*timesteps[i]);
 
@@ -397,7 +399,7 @@ TimeStepBase::set_sweep_no (const unsigned int sweep)
 
 
 
-unsigned int
+std::size_t
 TimeStepBase::memory_consumption () const
 {
 				   // only simple data types
@@ -765,7 +767,7 @@ void TimeStepBase_Tria<dim>::refine_grid (const RefinementData refinement_data)
       sorted_criteria = criteria;
       std::sort (sorted_criteria.begin(),
 		 sorted_criteria.end());
-      p_refinement_threshold = std::lower_bound (sorted_criteria.begin(),
+      p_refinement_threshold = Utilities::lower_bound (sorted_criteria.begin(),
 						 sorted_criteria.end(),
 						 static_cast<float>(refinement_threshold));
       p_coarsening_threshold = std::upper_bound (sorted_criteria.begin(),
@@ -1125,7 +1127,7 @@ void TimeStepBase_Tria<dim>::init_for_refinement ()
 
 
 template <int dim>
-unsigned int
+std::size_t
 TimeStepBase_Tria<dim>::memory_consumption () const
 {
   return (TimeStepBase::memory_consumption () +
